﻿(function asyncOperationsModule()
{
    "use strict";
    var anonymousOperationName = "Anonymous";
    var AsyncOperation = WinJS.Class.define(function constructor(operationName)
        {
            if(!(this instanceof BingApp.Classes.AsyncOperation))
            {
                BingApp.traceWarning("AsyncOperation.ctor: Attempted using AsyncOperation ctor as function; redirecting to use 'new AsyncOperation()'.");
                return new BingApp.Classes.AsyncOperation(operationName)
            }
            this.operationName = operationName || anonymousOperationName
        },{
            run: function(options)
            {
                if(this.isRunning())
                {
                    if(this.shouldContinue(this.currentOptions,options))
                        return this.currentPromise;
                    this.cancel()
                }
                this.currentOptions = options;
                var that = this;
                this.currentPromise = new WinJS.Promise(function init(complete, error)
                {
                    that.handleStart(options);
                    that.currentOperation = that.handleExecute(options);
                    that.currentOperation.done(function onComplete(result)
                    {
                        that.handleComplete(options,result);
                        that.currentOperation = null;
                        complete(result)
                    },function onError(err)
                    {
                        if(BingApp.Utilities.isPromiseCancellationError(err))
                            that.handleComplete(options,undefined,true);
                        else
                            that.handleComplete(options,undefined,false,err);
                        that.currentOperation = null;
                        error(err)
                    })
                },this.cancel);
                return this.currentPromise
            },
            cancel: function()
            {
                if(this.currentOperation)
                {
                    BingApp.traceInfo("AsyncOperation.cancel: cancelling current <{0}> operation upon request.",this.operationName);
                    this.currentOperation.cancel()
                }
            },
            isRunning: function()
            {
                return this.currentOperation ? true : false
            },
            then: function(handleComplete, onError)
            {
                var promise = this.currentPromise || WinJS.Promise.as();
                return promise.then.apply(promise,arguments)
            },
            done: function(handleComplete, onError)
            {
                var promise = this.currentPromise || WinJS.Promise.as();
                promise.done.apply(promise,arguments)
            },
            shouldContinue: function(oldOptions, newOptions)
            {
                return this.matchOptions(oldOptions,newOptions)
            },
            matchOptions: function(options1, options2)
            {
                return options1 === options2
            },
            handleStart: function(options)
            {
                BingApp.traceInfo("AsyncOperation.handleStart: starting <{0}> operation.",this.operationName)
            },
            handleExecute: function(options)
            {
                var methodName = "AsyncOperation.handleExecute";
                return WinJS.Promise.wrapError(new BingApp.Classes.ErrorInvalidOperation(methodName,BingApp.Utilities.format(WinJS.Resources.getString("error_must_override").value,methodName)))
            },
            handleComplete: function(options, result, canceled, error)
            {
                if(error)
                    BingApp.traceInfo("AsyncOperation.handleComplete: <{0}> operation completed with error.",this.operationName);
                else if(canceled)
                    BingApp.traceInfo("AsyncOperation.handleComplete: <{0}> operation was cancelled.",this.operationName);
                else
                    BingApp.traceInfo("AsyncOperation.handleComplete: <{0}> operation completed.",this.operationName)
            }
        });
    WinJS.Namespace.define("BingApp.Classes",{AsyncOperation: AsyncOperation});
    var LoadJsonOperation = WinJS.Class.derive(BingApp.Classes.AsyncOperation,function constructor()
        {
            if(!(this instanceof BingApp.Classes.LoadJsonOperation))
            {
                BingApp.traceWarning("LoadJsonOperation.ctor: Attempted using LoadJsonOperation ctor as function; redirecting to use 'new LoadJsonOperation()'.");
                return new BingApp.Classes.LoadJsonOperation
            }
            BingApp.Classes.AsyncOperation.call(this,"LoadJsonOperation")
        },{
            matchOptions: function(options1, options2)
            {
                var fileLocation1 = this._getFileLocationFromOptions(options1);
                var fileLocation2 = this._getFileLocationFromOptions(options2);
                return fileLocation1 === fileLocation2
            },
            handleStart: function(options)
            {
                var fileLocation = this._getFileLocationFromOptions(options);
                BingApp.traceInfo("LoadJsonOperation.handleStart: started loading object from {0} file.",fileLocation)
            },
            handleExecute: function(options)
            {
                var fileLocation = this._getFileLocationFromOptions(options);
                return BingApp.Utilities.jsonLoader(fileLocation)
            },
            handleComplete: function(options, result, canceled, error)
            {
                var fileLocation = this._getFileLocationFromOptions(options);
                if(error)
                    BingApp.traceInfo("LoadJsonOperation.handleComplete: failed to load object from {0} file.",fileLocation);
                else if(canceled)
                    BingApp.traceInfo("LoadJsonOperation.handleComplete: cancelled loading from {0} file.",fileLocation);
                else
                    BingApp.traceInfo("LoadJsonOperation.handleComplete: finished loading from {0} file.",fileLocation)
            },
            _getFileLocationFromOptions: function(options)
            {
                if(BingApp.Utilities.isNullOrUndefined(options))
                    throw new BingApp.Classes.ErrorArgumentNullOrUndefined("options");
                var fileLocation = options.fileLocation;
                if(BingApp.Utilities.isNullOrUndefined(fileLocation))
                    throw new BingApp.Classes.ErrorArgument("options");
                return fileLocation
            }
        });
    WinJS.Namespace.define("BingApp.Classes",{LoadJsonOperation: LoadJsonOperation})
})()