﻿(function eventSourceModule()
{
    "use strict";
    var EventSource = WinJS.Class.define(function constructor()
        {
            if(!(this instanceof BingApp.Classes.EventSource))
            {
                BingApp.traceWarning("EventSource.ctor: Attempted using EventSource ctor as function; redirecting to use 'new EventSource()'.");
                return new BingApp.Classes.EventSource
            }
            Object.defineProperties(this,{eventRelay: {
                    value: new BingApp.Classes.EventRelay,
                    writable: false,
                    enumerable: false,
                    configurable: false
                }})
        },{
            registerIframe: function(id, iframe, targetOrigin)
            {
                this.eventRelay.registerIframe(id,iframe,targetOrigin)
            },
            unregisterIframe: function(id)
            {
                this.eventRelay.unregisterIframe(id)
            },
            getRegisteredIframeInfo: function(id)
            {
                return this.eventRelay.getRegisteredIframeInfo(id)
            },
            fireEvent: function(eventName, args)
            {
                this.eventRelay.fireEvent(eventName,args,this)
            },
            addEventListener: function(eventName, listener)
            {
                var extendedListener = this._createExtendedListener(listener);
                this.eventRelay.addEventListener(eventName,extendedListener)
            },
            removeEventListener: function(eventName, listener)
            {
                var extendedListener = this._createExtendedListener(listener);
                this.eventRelay.removeEventListener(eventName,extendedListener)
            },
            isRegisteredEventListener: function(eventName, listener)
            {
                var extendedListener = this._createExtendedListener(listener);
                return this.eventRelay.isRegisteredEventListener(eventName,extendedListener)
            },
            _createExtendedListener: function(listener)
            {
                if(BingApp.Utilities.isNullOrUndefined(listener))
                    throw new BingApp.Classes.ErrorArgumentNullOrUndefined("listener");
                if(!BingApp.Utilities.isNullOrUndefined(listener.source))
                    throw new BingApp.Classes.ErrorArgument("listener",WinJS.Resources.getString("error_listener_must_not_have_explicit_source").value);
                listener = BingApp.Classes.EventRelay.normalizeEventListener(listener);
                var extendedListener = Object.create(listener);
                Object.defineProperty(extendedListener,"source",{value: this});
                return extendedListener
            }
        });
    WinJS.Namespace.define("BingApp.Classes",{EventSource: EventSource})
})()