﻿(function imageExportTool()
{
    "use strict";
    function saveImageAs(uriText, suggestedName)
    {
        if(!uriText)
            return WinJS.Promise.wrapError(new BingApp.Classes.ErrorArgumentNullOrUndefined("uriText"));
        var uri = new Windows.Foundation.Uri(uriText);
        var extension = uri.extension;
        var dotLessExtension = extension;
        if(dotLessExtension && dotLessExtension[0] === ".")
            dotLessExtension = dotLessExtension.substr(1);
        var streamRef = Windows.Storage.Streams.RandomAccessStreamReference.createFromUri(uri);
        var filePicker = new Windows.Storage.Pickers.FileSavePicker;
        filePicker.suggestedStartLocation = Windows.Storage.Pickers.PickerLocationId.picturesLibrary;
        if(!suggestedName)
        {
            suggestedName = WinJS.Resources.getString("saveas_default_filename").value;
            if(dotLessExtension)
                suggestedName = suggestedName + "." + dotLessExtension
        }
        filePicker.suggestedFileName = suggestedName;
        if(dotLessExtension)
            filePicker.fileTypeChoices.insert(dotLessExtension,[extension]);
        else
            filePicker.fileTypeChoices.insert("Jpeg",[".jpg"]);
        var tempFile,
            pickedFile,
            inputStream;
        function cleanupTempFile()
        {
            if(!tempFile)
                return WinJS.Promise.as();
            else
                return tempFile.deleteAsync().then(function resetVariable()
                    {
                        tempFile = null
                    },function ignoreError(error)
                    {
                        tempFile = null;
                        BingApp.traceError("{0}: {1}","ImageExportTool.cleanupTempFile",error && error.message)
                    })
        }
        return streamRef.openReadAsync().then(function gotInputStream(stream)
            {
                inputStream = stream
            }).then(function createTempFile()
            {
                var tempFolder = Windows.Storage.ApplicationData.current.temporaryFolder;
                return tempFolder.createFileAsync("tmpimage.bin",Windows.Storage.CreationCollisionOption.generateUniqueName)
            }).then(function tempFileCreated(file)
            {
                tempFile = file;
                return tempFile.openAsync(Windows.Storage.FileAccessMode.readWrite)
            }).then(function gotOutputStream(stream)
            {
                return Windows.Storage.Streams.RandomAccessStream.copyAndCloseAsync(inputStream,stream)
            }).then(function pickFile()
            {
                return filePicker.pickSaveFileAsync()
            }).then(function filePicked(file)
            {
                if(file)
                    pickedFile = file;
                else
                    return WinJS.Promise.wrapError(new WinJS.ErrorFromName("Canceled","Canceled"))
            }).then(function copyTempToPickedFile()
            {
                return tempFile.copyAndReplaceAsync(pickedFile)
            }).then(cleanupTempFile,function cleanupAndPropagateError(error)
            {
                return cleanupTempFile().then(function propagateError()
                    {
                        return WinJS.Promise.wrapError(error)
                    })
            })
    }
    function setLockScreen(uriText)
    {
        if(!uriText)
            return WinJS.Promise.wrapError(new BingApp.Classes.ErrorArgumentNullOrUndefined("uriText"));
        var uri = new Windows.Foundation.Uri(uriText);
        var streamRef = Windows.Storage.Streams.RandomAccessStreamReference.createFromUri(uri);
        return streamRef.openReadAsync().then(function gotInput(stream)
            {
                return Windows.System.UserProfile.LockScreen.setImageStreamAsync(stream)
            }).then(function sendTraceToAutomation()
            {
                BingApp.traceVerbose("LockScreenSetImageStreamSuccess")
            })
    }
    function createDataPackage(title)
    {
        var dataPackage = new Windows.ApplicationModel.DataTransfer.DataPackage;
        dataPackage.requestedOperation = Windows.ApplicationModel.DataTransfer.DataPackageOperation.copy;
        dataPackage.properties.applicationName = WinJS.Resources.getString("app_name").value;
        if(title)
            dataPackage.properties.title = title;
        return dataPackage
    }
    function copyImageToClipboard(uriText, title)
    {
        if(!uriText)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("uriText");
        var uri = new Windows.Foundation.Uri(uriText);
        var streamRef = Windows.Storage.Streams.RandomAccessStreamReference.createFromUri(uri);
        var dataPackage = createDataPackage(title);
        dataPackage.setBitmap(streamRef);
        Windows.ApplicationModel.DataTransfer.Clipboard.setContent(dataPackage)
    }
    function copyLinkToClipboard(uriText, title)
    {
        if(!uriText)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("uriText");
        var uri = new Windows.Foundation.Uri(uriText);
        var dataPackage = createDataPackage(title);
        dataPackage.setText(uriText);
        dataPackage.setUri(uri);
        Windows.ApplicationModel.DataTransfer.Clipboard.setContent(dataPackage)
    }
    WinJS.Namespace.define("BingApp.ImageExportTool",{
        saveImageAs: saveImageAs,
        setLockScreen: setLockScreen,
        copyImageToClipboard: copyImageToClipboard,
        copyLinkToClipboard: copyLinkToClipboard
    })
})()