﻿(function utilitiesModule()
{
    "use strict";
    function isNullOrUndefined(value)
    {
        return value === null || typeof value === "undefined"
    }
    function isNotNullOrUndefined(value)
    {
        return isNullOrUndefined(value) ? false : true
    }
    function isPromiseCancellationError(error)
    {
        var canceledName = "Canceled";
        return error instanceof Error && error.name === canceledName && error.message === canceledName
    }
    function format(formatText)
    {
        if(typeof formatText !== "string")
            throw new BingApp.Classes.ErrorArgument("formatText");
        var thatArguments = arguments;
        formatText = formatText.replace(/\{(0|[1-9]\d*)\}/g,function getArgument(fullMatch, extractedIndex)
        {
            extractedIndex = parseInt(extractedIndex) + 1;
            return thatArguments.hasOwnProperty(extractedIndex) ? thatArguments[extractedIndex] : fullMatch
        });
        return formatText
    }
    function jsonLoader(fileLocation)
    {
        if(BingApp.Utilities.isNullOrUndefined(fileLocation))
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("fileLocation");
        var xhrPromise;
        return new WinJS.Promise(function init(complete, error)
            {
                BingApp.traceInfo("Utilities.jsonLoader: started loading from {0} file.",fileLocation);
                xhrPromise = WinJS.xhr({url: fileLocation});
                xhrPromise.done(function done(response)
                {
                    try
                    {
                        var loadedObject = JSON.parse(response.responseText)
                    }
                    catch(err)
                    {
                        BingApp.traceError("Utilities.jsonLoader: failed to parse JSON retrieved from {0} file.",fileLocation);
                        error(err);
                        return
                    }
                    complete(loadedObject)
                },error)
            },function cancel()
            {
                BingApp.traceInfo("Utilities.jsonLoader: cancelling loading from {0} file.",fileLocation);
                if(xhrPromise)
                    xhrPromise.cancel()
            })
    }
    function isColdStart(executionState)
    {
        var stateNamespace = Windows.ApplicationModel.Activation.ApplicationExecutionState;
        return executionState !== stateNamespace.suspended && executionState !== stateNamespace.running
    }
    function areEqualIgnoreCase(left, right)
    {
        if(BingApp.Utilities.isNullOrUndefined(left))
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("left");
        if(BingApp.Utilities.isNullOrUndefined(right))
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("right");
        return left.toUpperCase() === right.toUpperCase()
    }
    function invokeURI(uri)
    {
        if(!uri)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("uri");
        var appUri;
        try
        {
            appUri = new Windows.Foundation.Uri(uri)
        }
        catch(err)
        {
            BingApp.traceError("Utilities.invokeURI: Error creating a URI from '{0}'; fails with message = {1}.",uri,err.message);
            return WinJS.Promise.as()
        }
        try
        {
            return Windows.System.Launcher.launchUriAsync(appUri)
        }
        catch(e)
        {
            BingApp.traceError("Utilities.invokeURI: Error occurred during launching '{0}' URI; failed with message = {1}.",uri,e.message);
            return WinJS.Promise.wrapError(e)
        }
    }
    var formMatchExpression = /[\?&]form=[^&]*\b/gi;
    function upsertFormCode(uri, formCode)
    {
        if(typeof uri === "string")
            uri = new Windows.Foundation.Uri(uri);
        if(!uri)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("uri");
        if(!formCode)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("formCode");
        var updatedQuery = uri.query.replace(formMatchExpression,"");
        updatedQuery = updatedQuery.charAt(0) === "?" ? updatedQuery + "&form=" + formCode : "?form=" + formCode + updatedQuery;
        var originalUri = uri.absoluteUri;
        var updatedUri = originalUri.substring(0,originalUri.length - uri.query.length - uri.fragment.length) + updatedQuery + originalUri.substring(originalUri.length - uri.fragment.length);
        return updatedUri
    }
    function isWellFormedDomain(domain)
    {
        if(BingApp.Utilities.isNullOrUndefined(domain))
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("domain");
        var uri;
        try
        {
            uri = new Windows.Foundation.Uri(domain)
        }
        catch(error)
        {
            BingApp.traceError("isWellFormedDomain: Error creating a URI from {0} : fails with message = {1}.",domain,error.message);
            return false
        }
        return!uri.suspicious
    }
    function getPageDimensions()
    {
        return{
                width: window.innerWidth,
                height: window.innerHeight
            }
    }
    function getArchitecture()
    {
        var arch = Windows.ApplicationModel.Package.current.id.architecture;
        var architectureString = "";
        switch(arch)
        {
            case Windows.System.ProcessorArchitecture.x86:
                architectureString = "X86";
                break;
            case Windows.System.ProcessorArchitecture.arm:
                architectureString = "Arm";
                break;
            case Windows.System.ProcessorArchitecture.x64:
                architectureString = "X64";
                break;
            case Windows.System.ProcessorArchitecture.neutral:
                architectureString = "Neutral";
                break;
            case Windows.System.ProcessorArchitecture.unknown:
                architectureString = "Unknown";
                break;
            default:
                architectureString = arch.toString();
                break
        }
        return architectureString
    }
    function getTimeStamp()
    {
        return(new Date).getTime()
    }
    function ellipsify(domEl, maxHeight, text, ellipsis, staticHTML)
    {
        if(domEl && domEl.clientHeight > maxHeight)
        {
            var threshold = 1.5;
            if(domEl.clientHeight > maxHeight * threshold)
                text = text.substring(0,Math.floor(text.length * maxHeight * threshold / domEl.clientHeight));
            var words = text.split(/ |\r\n|\r|\n/);
            do
            {
                words.pop();
                if(staticHTML)
                    domEl.innerHTML = staticHTML + window.toStaticHTML(words.join(" ").trim() + ellipsis);
                else
                    domEl.innerHTML = window.toStaticHTML(words.join(" ").trim() + ellipsis)
            } while(domEl.clientHeight > maxHeight && words.length > 0)
        }
    }
    function trim(text)
    {
        return text.replace(/^\s+|\s+$/g,"")
    }
    function isKeyDownAndSpaceOrEnterKey(eventData)
    {
        if(eventData && eventData.type === "keydown" && (eventData.which === WinJS.Utilities.Key.space || eventData.which === WinJS.Utilities.Key.enter))
            return true;
        return false
    }
    function createClosure(object, functionRef)
    {
        return function()
            {
                return functionRef.apply(object,arguments)
            }
    }
    function showErrorDialog(resourceNoArgument, resourceWithArgument, traceTag, message)
    {
        var retryCount = 0;
        var retryDelay = 2e3;
        function waitAndRetry()
        {
            retryCount++;
            if(retryCount >= 3)
                return;
            WinJS.Promise.timeout(retryDelay).done(displayDialog,displayDialog)
        }
        function displayDialog()
        {
            showErrorDialogAsync(resourceNoArgument,resourceWithArgument,traceTag,message).done(null,waitAndRetry)
        }
        displayDialog()
    }
    function showErrorDialogAsync(resourceNoArgument, resourceWithArgument, traceTag, message)
    {
        BingApp.traceError("{0}: {1}",traceTag,message ? message : "(no details)");
        var fullMessage;
        if(resourceWithArgument && message)
            fullMessage = BingApp.Utilities.format(WinJS.Resources.getString(resourceWithArgument).value,message.trim());
        else
            fullMessage = WinJS.Resources.getString(resourceNoArgument).value;
        try
        {
            var dialog = new Windows.UI.Popups.MessageDialog(fullMessage);
            return dialog.showAsync()
        }
        catch(e)
        {
            return WinJS.Promise.wrapError(e)
        }
    }
    function displayOverlayMessage(message)
    {
        if(!message)
            return WinJS.Promise.wrapError(new BingApp.Classes.ErrorArgumentNullOrUndefined("message"));
        var overlayMessageElement = document.querySelector("#overlayMessage");
        var overlayMessageTextElement = document.querySelector("#overlayMessageText");
        if(!overlayMessageElement || typeof overlayMessageTextElement === "undefined" || !overlayMessageTextElement)
            return WinJS.Promise.wrapError(new BingApp.Classes.ErrorInvalidOperation("BingApp.Utilities.displayOverlaymessage","Missing overlay node object"));
        overlayMessageElement.style.opacity = "0";
        overlayMessageElement.style.display = "block";
        overlayMessageTextElement.innerText = message;
        return WinJS.UI.Animation.fadeIn(overlayMessageElement)
    }
    function removeOverlayMessage()
    {
        var overlayMessageElement = document.querySelector("#overlayMessage");
        var overlayMessageTextElement = document.querySelector("#overlayMessageText");
        if(!overlayMessageElement || typeof overlayMessageTextElement === "undefined" || !overlayMessageTextElement)
            return WinJS.Promise.wrapError(new BingApp.Classes.ErrorInvalidOperation("BingApp.Utilities.removeOverlayMessage","Missing overlay node object"));
        function animationDone()
        {
            overlayMessageElement.style.display = "none";
            overlayMessageTextElement.innerText = ""
        }
        return WinJS.UI.Animation.fadeOut(overlayMessageElement).then(animationDone,animationDone)
    }
    function onMSPointerDown(eventData)
    {
        switch(eventData.pointerType)
        {
            case eventData.MSPOINTER_TYPE_TOUCH:
                eventData.currentTarget.evtSourceOrigin = "Touch";
                break;
            case eventData.MSPOINTER_TYPE_MOUSE:
                eventData.currentTarget.evtSourceOrigin = "Mouse";
                break;
            case eventData.MSPOINTER_TYPE_PEN:
                eventData.currentTarget.evtSourceOrigin = "Pen";
                break;
            default:
                eventData.currentTarget.evtSourceOrigin = "Unknown";
                break
        }
    }
    function getViewMode(state)
    {
        var mode = state || Windows.UI.ViewManagement.ApplicationView.value;
        var viewMode = "";
        switch(mode)
        {
            case Windows.UI.ViewManagement.ApplicationViewState.fullScreenLandscape:
                viewMode = "Landscape";
                break;
            case Windows.UI.ViewManagement.ApplicationViewState.filled:
                viewMode = "Fill";
                break;
            case Windows.UI.ViewManagement.ApplicationViewState.snapped:
                viewMode = "Snap";
                break;
            case Windows.UI.ViewManagement.ApplicationViewState.fullScreenPortrait:
                viewMode = "Portrait";
                break;
            default:
                BingApp.traceError("BingApp.Utilities.getViewMode: Unknown Mode");
                viewMode = "Unknown";
                break
        }
        return viewMode
    }
    function compareVersions(leftVersionString, rightVersionString)
    {
        if(!leftVersionString)
            return-1;
        if(!rightVersionString)
            return 1;
        if(leftVersionString === rightVersionString)
            return 0;
        try
        {
            var leftVersion = leftVersionString.split(".");
            var rightVersion = rightVersionString.split(".");
            var minLength = Math.min(leftVersion.length,rightVersion.length);
            for(var i = 0; i < minLength; i++)
            {
                var leftPart = parseInt(leftVersion[i],10);
                var rightPart = parseInt(rightVersion[i],10);
                if(isNaN(leftPart) || isNaN(rightPart))
                    return-1;
                if(leftPart > rightPart)
                    return 1;
                if(leftPart < rightPart)
                    return-1
            }
            return leftVersion.length - rightVersion.length
        }
        catch(e)
        {
            BingApp.traceError("BingApp.Utilities.compareVersion: Error comparing version numbers '{0}' and '{1}' fails with message = {2}.",leftVersionString,rightVersionString,e.message);
            return
        }
    }
    function setLightDismiss(flyoutControl, enabled)
    {
        if(flyoutControl && flyoutControl.winControl && typeof flyoutControl.winControl._sticky !== "undefined")
        {
            flyoutControl.winControl._sticky = !enabled;
            return true
        }
        else
        {
            BingApp.traceError("Utilities.setLightDismiss: Failed to set _sticky property on invalid flyout control.");
            return false
        }
    }
    function createTraceErrorHandler(errorPrefixMessage)
    {
        return function errorHandler(error)
            {
                if(!error)
                    error = {};
                BingApp.traceError('{0}; error name="{1}" message="{2}"',errorPrefixMessage,error.name,error.message)
            }
    }
    WinJS.Namespace.define("BingApp.Utilities",{
        format: format,
        jsonLoader: jsonLoader,
        isPromiseCancellationError: isPromiseCancellationError,
        isNullOrUndefined: isNullOrUndefined,
        isNotNullOrUndefined: isNotNullOrUndefined,
        isColdStart: isColdStart,
        areEqualIgnoreCase: areEqualIgnoreCase,
        invokeURI: invokeURI,
        isWellFormedDomain: isWellFormedDomain,
        getPageDimensions: getPageDimensions,
        getArchitecture: getArchitecture,
        getTimeStamp: getTimeStamp,
        ellipsify: ellipsify,
        trim: trim,
        isKeyDownAndSpaceOrEnterKey: isKeyDownAndSpaceOrEnterKey,
        upsertFormCode: upsertFormCode,
        createClosure: createClosure,
        showErrorDialog: showErrorDialog,
        showErrorDialogAsync: showErrorDialogAsync,
        displayOverlayMessage: displayOverlayMessage,
        removeOverlayMessage: removeOverlayMessage,
        onMSPointerDown: onMSPointerDown,
        getViewMode: getViewMode,
        compareVersions: compareVersions,
        setLightDismiss: setLightDismiss,
        createTraceErrorHandler: createTraceErrorHandler
    })
})()