﻿(function arrayDataProviderModule()
{
    "use strict";
    var ArrayDataProvider = WinJS.Class.define(function constructor(identity, key, initialItems)
        {
            if(!(this instanceof BingApp.Classes.ArrayDataProvider))
            {
                BingApp.traceWarning("ArrayDataProvider.ctor: Attempted using ArrayDataProvider ctor as function; redirecting to use 'new ArrayDataProvider()'.");
                return new BingApp.Classes.ArrayDataProvider(identity,key,initialItems)
            }
            if(!identity)
                throw new BingApp.Classes.ErrorArgumentNullOrUndefined("identity");
            if(!key)
                throw new BingApp.Classes.ErrorArgumentNullOrUndefined("key");
            Object.defineProperties(this,{
                identity: {
                    value: identity,
                    writable: false,
                    enumerable: false,
                    configurable: false
                },
                key: {
                    value: key,
                    writable: false,
                    enumerable: false,
                    configurable: false
                },
                items: {
                    value: [],
                    writable: false,
                    enumerable: false,
                    configurable: false
                },
                keys: {
                    value: {},
                    writable: false,
                    enumerable: false,
                    configurable: false
                },
                lastGeneratedId: {
                    value: 0,
                    writable: true,
                    enumerable: false,
                    configurable: false
                }
            });
            var that = this;
            if(initialItems)
                initialItems.forEach(function addItem(item)
                {
                    that.addItem(item)
                })
        },{
            getIdentity: function()
            {
                return this.identity
            },
            getCount: function()
            {
                return WinJS.Promise.as(this.items.length)
            },
            getItems: function(from, count, processItem)
            {
                if(from < 0)
                    return WinJS.Promise.as(WinJS.UI.FetchError.doesNotExist);
                var itemsCount = this.items.length;
                if(from >= itemsCount)
                    return WinJS.Promise.as(WinJS.UI.FetchError.doesNotExist);
                var to = from + count;
                if(to > itemsCount)
                    to = itemsCount;
                var itemsSet = this.items.slice(from,to);
                if(processItem)
                    for(var indexItem = 0; indexItem < itemsSet.length; indexItem++)
                    {
                        var processedItem = processItem(itemsSet[indexItem]);
                        if(!BingApp.Utilities.isNullOrUndefined(processedItem))
                            itemsSet[indexItem] = processedItem
                    }
                return WinJS.Promise.as({
                        items: itemsSet,
                        totalCount: itemsCount
                    })
            },
            addItem: function(item)
            {
                var keyValue = this._getOrCreateItemKeyValue(item);
                if(this.keys.hasOwnProperty(keyValue))
                    throw new BingApp.Classes.ErrorArgument("item",WinJS.Resources.getString("error_array_data_provider_item_must_have_unique_key_value").value);
                this.keys[keyValue] = this.items.length;
                this.items.push(item);
                return WinJS.Promise.as(item)
            },
            updateItem: function(item)
            {
                var index = this._getItemIndex(item);
                this.items[index] = item;
                return WinJS.Promise.as(item)
            },
            removeItem: function(item)
            {
                var keyValue = this._getItemKeyValue(item);
                var indexRemove = this._getItemIndex(item);
                var itemsCount = this.items.length;
                for(var indexItem = indexRemove; indexItem < itemsCount - 1; indexItem++)
                {
                    var existingItem = this.items[indexItem + 1];
                    var existingKeyValue = existingItem[this.key];
                    this.keys[existingKeyValue] = indexItem
                }
                var removedItem = this.items.splice(indexRemove,1)[0];
                delete this.keys[keyValue];
                return WinJS.Promise.as(removedItem)
            },
            _getItemKeyValue: function(item)
            {
                if(!item)
                    throw new BingApp.Classes.ErrorArgumentNullOrUndefined("item");
                return item[this.key]
            },
            _getOrCreateItemKeyValue: function(item)
            {
                if(!item)
                    throw new BingApp.Classes.ErrorArgumentNullOrUndefined("item");
                var keyValue = item[this.key];
                if(BingApp.Utilities.isNullOrUndefined(keyValue))
                {
                    keyValue = "generatedId." + this.lastGeneratedId;
                    item[this.key] = keyValue;
                    this.lastGeneratedId++
                }
                else if(typeof keyValue !== "string")
                    throw new BingApp.Classes.ErrorArgument("item",WinJS.Resources.getString("error_array_data_provider_item_must_have_string_key_value").value);
                return keyValue
            },
            _getItemIndex: function(item)
            {
                var keyValue = this._getItemKeyValue(item);
                var index = this.keys[keyValue];
                if(typeof index === "undefined")
                    throw new BingApp.Classes.ErrorArgument("item",BingApp.Utilities.format(WinJS.Resources.getString("error_array_data_provider_item_must_be_inside_array").value,keyValue));
                return index
            }
        });
    WinJS.Namespace.define("BingApp.Classes",{ArrayDataProvider: ArrayDataProvider})
})()