﻿(function dataUtilitiesModule()
{
    "use strict";
    function getAllItems(dataProvider, options)
    {
        if(!dataProvider)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("dataProvider");
        var providerIdentity = dataProvider.getIdentity();
        var maxCount;
        var processItem;
        var initialIndex = 0;
        if(options)
        {
            maxCount = options.count;
            processItem = options.processItem;
            if(options.from)
                initialIndex = options.from
        }
        var hasLimit = !!maxCount;
        var allItems = [];
        var getCount = dataProvider.getCount || function noCountIsAvailable()
            {
                return WinJS.Promise.as(WinJS.UI.CountResult.unknown)
            };
        return getCount.call(dataProvider).then(function onGetCountCallComplete(initialCount)
            {
                if(initialCount === WinJS.UI.CountResult.unknown || initialCount === WinJS.UI.CountError.noResponse)
                    initialCount = Infinity;
                BingApp.traceVerbose("BingApp.Data.getAllItems: data provider '{0}' returned '{1}' as its initial count for items.",providerIdentity,initialCount);
                var from = initialIndex;
                if(initialCount <= from)
                {
                    BingApp.traceWarning("BingApp.Data.getAllItems: returning empty array of items because specified initial index {0} is larger than or equal to the number of items {1} that can be returned by data provider '{2}'",from,initialCount,providerIdentity);
                    return allItems
                }
                var totalCount = initialCount;
                var nextBatchSize = hasLimit ? Math.min(maxCount,initialCount) : initialCount;
                function collectItems(promise)
                {
                    return promise.then(function onCollectItemsComplete(result)
                        {
                            if(!result.items)
                            {
                                BingApp.traceWarning("BingApp.Data.getAllItems: data provider '{0}' did not return any items in the result object; function will abort and will return items that has been collected so far (count={1}).",providerIdentity,allItems.length);
                                return allItems
                            }
                            else
                                BingApp.traceVerbose("BingApp.Data.getAllItems: data provider '{0}' returned {1} items on current iteration.",providerIdentity,result.items.length);
                            allItems.push.apply(allItems,result.items);
                            if(!!result.totalCount)
                            {
                                BingApp.traceVerbose("BingApp.Data.getAllItems: data provider '{0}' indicated new total count value: {1}.",providerIdentity,result.totalCount);
                                totalCount = result.totalCount
                            }
                            nextBatchSize = (hasLimit ? Math.min(maxCount,totalCount) : totalCount) - allItems.length;
                            if(nextBatchSize <= 0)
                            {
                                BingApp.traceVerbose("BingApp.Data.getAllItems: data provider '{0}' completed fetching items; overall number of returned items is {1}.",providerIdentity,allItems.length);
                                return allItems
                            }
                            else
                            {
                                from = initialIndex + allItems.length;
                                return collectItems(getItemsInternal(dataProvider,from,nextBatchSize,processItem))
                            }
                        })
                }
                return collectItems(getItemsInternal(dataProvider,from,nextBatchSize,processItem))
            })
    }
    function getItemsInternal(dataProvider, from, count, processItem)
    {
        BingApp.traceVerbose("BingApp.Data.getAllItems: asking data provider '{0}' to fetch {1} items starting from index {2}",dataProvider.getIdentity(),count,from);
        return dataProvider.getItems(from,count,processItem)
    }
    WinJS.Namespace.define("BingApp.Data",{getAllItems: getAllItems})
})()