﻿(function localStorageCacheProviderModule()
{
    "use strict";
    var LocalStorageCacheProvider = WinJS.Class.derive(BingApp.Classes.LocalStorageDataProvider,function constructor(identity, key)
        {
            if(!(this instanceof BingApp.Classes.LocalStorageCacheProvider))
            {
                BingApp.traceWarning("LocalStorageCacheProvider.ctor: Attempted using LocalStorageCacheProvider ctor as function; redirecting to use 'new LocalStorageCacheProvider()'.");
                return new BingApp.Classes.LocalStorageCacheProvider(identity,key)
            }
            BingApp.Classes.LocalStorageDataProvider.call(this,identity,key)
        },{
            clear: function()
            {
                if(!this._isDisabled())
                {
                    for(var indexStorageKey = window.localStorage.length - 1; indexStorageKey >= 0; indexStorageKey--)
                    {
                        var storageKey = window.localStorage.key(indexStorageKey);
                        if(storageKey.indexOf(this.localStoragePrefix) === 0)
                            window.localStorage.removeItem(storageKey)
                    }
                    this.items.length = 0;
                    this._setCountInternal(0);
                    this.lastGeneratedId = 0;
                    window.localStorage.setItem(this._getSlotForLastGeneratedId(),this.lastGeneratedId.toString())
                }
                return WinJS.Promise.as()
            },
            addItems: function(items)
            {
                if(!items)
                    throw new BingApp.Classes.ErrorArgumentNullOrUndefined("items");
                if(!this._isDisabled())
                {
                    var currentCount = this._getCountInternal();
                    for(var indexItem = 0; indexItem < items.length; indexItem++)
                    {
                        var item = items[indexItem];
                        this._ensureItemHasKeyValue(item);
                        var indexSlot = this._getSlotForIndexFromItemKey(item);
                        if(window.localStorage.hasOwnProperty(indexSlot))
                            throw new BingApp.Classes.ErrorArgument("item",WinJS.Resources.getString("error_local_storage_data_provider_item_must_have_unique_key_value").value);
                        var indexInStorage = indexItem + currentCount;
                        var itemSlot = this._getSlotForItemFromIndex(indexInStorage);
                        window.localStorage.setItem(itemSlot,JSON.stringify(item));
                        window.localStorage.setItem(indexSlot,indexInStorage);
                        this.items[indexInStorage] = item
                    }
                    this._setCountInternal(indexItem + currentCount)
                }
                return WinJS.Promise.as(items)
            }
        });
    WinJS.Namespace.define("BingApp.Classes",{LocalStorageCacheProvider: LocalStorageCacheProvider})
})()