﻿(function priorityAggregateDataProviderModule()
{
    "use strict";
    var PriorityAggregateDataProvider = WinJS.Class.derive(BingApp.Classes.AggregateDataProvider,function constructor(identity, dataProviders, criticalDataProviders)
        {
            if(!(this instanceof BingApp.Classes.PriorityAggregateDataProvider))
            {
                BingApp.traceWarning("PriorityAggregateDataProvider.ctor: Attempted using PriorityAggregateDataProvider ctor as function; redirecting to use 'new AggregateDataProvider()'.");
                return new BingApp.Classes.PriorityAggregateDataProvider(identity,dataProviders,criticalDataProviders)
            }
            if(!criticalDataProviders)
                throw new BingApp.Classes.ErrorArgumentNullOrUndefined("criticalDataProviders");
            Object.defineProperties(this,{criticalDataProviders: {
                    value: [],
                    writable: false
                }});
            var that = this;
            criticalDataProviders.forEach(function setCriticalDataProviderIdentity(criticalDataProvider)
            {
                that.criticalDataProviders[criticalDataProvider.getIdentity()] = true
            });
            BingApp.Classes.AggregateDataProvider.call(this,identity,dataProviders)
        },{
            getCount: function()
            {
                var that = this;
                return BingApp.Classes.AggregateDataProvider.prototype.getCount.call(this).then(function onCompleted(count)
                    {
                        if(count === 0)
                            return count;
                        else
                            return that._processGetCountResults()
                    })
            },
            getItems: function(from, count, processItem)
            {
                var that = this;
                return BingApp.Classes.AggregateDataProvider.prototype.getItems.call(this,from,count,processItem).then(function onCompleted(result)
                    {
                        return that._processGetItemsResults(result)
                    })
            },
            _processGetCountResults: function()
            {
                var that = this;
                var success = this.dataProviderDescriptors.every(function processGetCountResult(descriptor, index, array)
                    {
                        var providerIdentity = descriptor.dataProvider.getIdentity();
                        if(that.criticalDataProviders[providerIdentity])
                            if(!descriptor.count)
                            {
                                BingApp.traceInfo("PriorityAggregateDataProvider._processGetCountResults: count not returned for provider with identity '{0}'.",providerIdentity);
                                return false
                            }
                            else
                                return true;
                        else
                            return true
                    });
                if(!success)
                    this.aggregateCount = 0;
                return this.aggregateCount
            },
            _processGetItemsResults: function(result)
            {
                var that = this;
                var success = this.dataProviderDescriptors.every(function processGetItemsResult(descriptor, index, array)
                    {
                        var providerIdentity = descriptor.dataProvider.getIdentity();
                        if(that.criticalDataProviders[providerIdentity])
                            if(!descriptor.items || descriptor.items.length === 0)
                            {
                                BingApp.traceInfo("PriorityAggregateDataProvider._processGetItemsResults: items could not be returned for provider with identity '{0}'.",providerIdentity);
                                return false
                            }
                            else
                                return true;
                        else
                            return true
                    });
                return success ? result : WinJS.UI.FetchError.doesNotExist
            }
        });
    WinJS.Namespace.define("BingApp.Classes",{PriorityAggregateDataProvider: PriorityAggregateDataProvider})
})()