﻿(function xhrDataProviderModule()
{
    "use strict";
    function getItems(from, count, processItem, options)
    {
        if(BingApp.Utilities.isNullOrUndefined(from))
            from = 0;
        if(from !== 0)
            throw new BingApp.Classes.ErrorArgument("from","From should be zero for this data provider");
        var urlOptions = options.urlOptions;
        var parser = options.parser;
        var request;
        return new WinJS.Promise(function init(complete, error)
            {
                var xhrOptions;
                try
                {
                    xhrOptions = urlOptions()
                }
                catch(err)
                {
                    BingApp.traceError("XhrDataProvider: url options are invalid - error name: {0}, error message: {1}",err.name || "",err.message || "");
                    complete(WinJS.UI.FetchError.doesNotExist);
                    return
                }
                if(!xhrOptions.url)
                {
                    BingApp.traceError("XhrDataProvider: url is undefined in urlOptions");
                    complete(WinJS.UI.FetchError.doesNotExist);
                    return
                }
                request = WinJS.xhr(xhrOptions);
                request.done(function onComplete(response)
                {
                    try
                    {
                        var results = parser(response.responseText)
                    }
                    catch(err)
                    {
                        BingApp.traceError("XhrDataProvider: parser was not able to parse response from: {0} - error name: {1}, error message: {2}",xhrOptions.url,err.name || "",err.message || "");
                        complete(WinJS.UI.FetchError.doesNotExist);
                        return
                    }
                    if(results.length === 0)
                    {
                        BingApp.traceError("XhrDataProvider: parser did not return any valid data");
                        complete(WinJS.UI.FetchError.doesNotExist);
                        return
                    }
                    if(processItem)
                        for(var indexItem = 0; indexItem < results.length; indexItem++)
                        {
                            var processedItem = processItem(results[indexItem]);
                            if(!BingApp.Utilities.isNullOrUndefined(processedItem))
                                results[indexItem] = processedItem
                        }
                    complete({
                        items: results,
                        totalCount: results.length
                    })
                },function onError(err)
                {
                    var xhrErrorMessage = BingApp.Utilities.format("XhrDataProvider: xhr request failed - error name: {0}, error message: {1}",err.name || "",err.message || "");
                    if(!BingApp.Utilities.isPromiseCancellationError(err))
                    {
                        var instObject = {
                                url: xhrOptions.url,
                                ei: xhrErrorMessage
                            };
                        BingApp.Instrumentation.logError(instObject)
                    }
                    BingApp.traceError(xhrErrorMessage);
                    complete(WinJS.UI.FetchError.doesNotExist)
                })
            },function cancel()
            {
                request && request.cancel()
            })
    }
    var XhrDataProvider = WinJS.Class.define(function constructor(identity, count, options)
        {
            if(!(this instanceof BingApp.Classes.XhrDataProvider))
            {
                BingApp.traceWarning("XhrDataProvider.ctor: Attempted using XhrDataProvider ctor as function; redirecting to use 'new XhrDataProvider()'.");
                return new BingApp.Classes.XhrDataProvider(identity,count,options)
            }
            if(!identity)
                throw new BingApp.Classes.ErrorArgumentNullOrUndefined("identity");
            if(BingApp.Utilities.isNullOrUndefined(count))
                throw new BingApp.Classes.ErrorArgumentNullOrUndefined("count");
            if(count === 0)
                throw new BingApp.Classes.ErrorArgument("count","Count cannot be zero");
            if(!options)
                throw new BingApp.Classes.ErrorArgumentNullOrUndefined("options");
            if(!options.urlOptions)
                throw new BingApp.Classes.ErrorArgumentNullOrUndefined("urlOptions");
            if(!options.parser)
                throw new BingApp.Classes.ErrorArgumentNullOrUndefined("parser");
            if(typeof options.urlOptions !== "function")
                throw new BingApp.Classes.ErrorArgument("urlOptions");
            if(typeof options.parser !== "function")
                throw new BingApp.Classes.ErrorArgument("parser");
            Object.defineProperties(this,{
                identity: {
                    value: identity,
                    writable: false,
                    enumerable: false,
                    configurable: false
                },
                count: {
                    value: count,
                    writable: false,
                    enumerable: false,
                    configurable: false
                }
            });
            this.getItems = function getItemsWrapper(from, numberOfItems, processItem)
            {
                return getItems(from,numberOfItems,processItem,options)
            }
        },{
            getIdentity: function()
            {
                return this.identity
            },
            getCount: function()
            {
                return WinJS.Promise.as(this.count)
            }
        });
    WinJS.Namespace.define("BingApp.Classes",{XhrDataProvider: XhrDataProvider})
})()