﻿(function filePickerViewModelModule()
{
    "use strict";
    var showSafeSearchUri = "bingsearch://settings/safesearch";
    var defaultFileName = "image";
    var noBackgroundImage = "none";
    var localStorageLastSearchKey = "FilePicker.lastQuery";
    var selectedFilterStyle = "fontMedium fontWeightRegular";
    var unselectedFilterStyle = "fontMedium fontWeightRegular unselectedFilter";
    var unfocusableTabIndex = "-1";
    var focusableTabIndex = "0";
    var backgroundImageCount = 7;
    var fileNameMaxLength = 200;
    var supportedExtensions = Object.freeze({
            ".gif": true,
            ".tif": true,
            ".tiff": true,
            ".png": true,
            ".bmp": true,
            ".jpg": true,
            ".jpeg": true
        });
    function constructor(fileOpenPickerUI, direction, windowObject, options)
    {
        if(!fileOpenPickerUI)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("fileOpenPickerUI");
        if(!windowObject)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("windowObject");
        this._fileOpenPickerUI = fileOpenPickerUI;
        this._fileOpenPickerUI.title = WinJS.Resources.getString("file_picker_default_web_title").value;
        this._fileOpenPickerUI.addEventListener("fileremoved",BingApp.Utilities.createClosure(this,handleFileRemovedFromBasket));
        this._initializeBindingData(fileOpenPickerUI,direction);
        this._config = options && options.config || BingApp.Classes.Config.createInstance();
        var lastQuery = window.localStorage.getItem(localStorageLastSearchKey);
        if(!lastQuery)
            lastQuery = WinJS.Resources.getString("file_picker_default_query").value;
        this.text = lastQuery;
        this.performSearch();
        this._initializeBackground(windowObject)
    }
    function _initializeBackground(windowObject)
    {
        var landscapeImage = noBackgroundImage;
        var portraitImage = noBackgroundImage;
        var that = this;
        function setBackground()
        {
            if(landscapeImage === noBackgroundImage || portraitImage === noBackgroundImage)
            {
                landscapeImage = "url('" + BingApp.HomePageData.defaultImage.landscape + "')";
                portraitImage = "url('" + BingApp.HomePageData.defaultImage.portrait + "')"
            }
            if(windowObject.innerWidth > windowObject.innerHeight)
                that.bindings.setProperty("backgroundImage",landscapeImage);
            else
                that.bindings.setProperty("backgroundImage",portraitImage)
        }
        windowObject.addEventListener("resize",setBackground);
        BingApp.HPDataProvider.loadSettings().then(function settingsLoaded(cachedSettings)
        {
            if(cachedSettings)
            {
                var currentSettings;
                try
                {
                    currentSettings = JSON.parse(cachedSettings)
                }
                catch(error)
                {
                    BingApp.traceError("FilePickerViewModel.constructor: could not load cached settings.")
                }
                if(currentSettings && BingApp.HPDataProvider.validateSettings(currentSettings,backgroundImageCount))
                    return BingApp.HPDataProvider.doHPFilesExist(currentSettings).then(function handleHPFilesExist(exists)
                        {
                            if(exists)
                            {
                                var collection = BingApp.HPDataProvider.getSettingsLangAndRegion();
                                if(collection)
                                {
                                    var cc = collection["cc"];
                                    var setLang = collection["setlang"];
                                    if(!cc || cc === BingApp.HPDataProvider.getUserCC() || !setLang || setLang === BingApp.HPDataProvider.getUserLang())
                                    {
                                        var index = currentSettings[0].index;
                                        landscapeImage = "url('ms-appdata:///local/landscape" + index + ".jpg')";
                                        portraitImage = "url('ms-appdata:///local/portrait" + index + ".jpg')"
                                    }
                                }
                            }
                        })
            }
            return WinJS.Promise.as()
        }).then(setBackground,setBackground)
    }
    function _initializeBindingData(fileOpenPickerUI, direction)
    {
        if(this._fileOpenPickerUI.selectionMode === Windows.Storage.Pickers.Provider.FileSelectionMode.single)
            this.bindings.updateProperty("selectionMode","single");
        else
            this.bindings.updateProperty("selectionMode","multi");
        this.bindings.updateProperty("dataSource",null);
        this.bindings.updateProperty("selectionErrorIndex",-1);
        this.bindings.updateProperty("selectionErrorMessage","");
        if(direction === BingApp.Classes.FilePickerViewModel.rightToLeftDirection)
            this.bindings.updateProperty("selectionErrorPlacement","left");
        else
            this.bindings.updateProperty("selectionErrorPlacement","right");
        this.bindings.updateProperty("selectionIndices",[]);
        this.bindings.updateProperty("listViewDisplay","none");
        this.bindings.updateProperty("errorDisplay","none");
        this.bindings.updateProperty("errorMessage","");
        this.bindings.updateProperty("searchBoxClassName","");
        this.bindings.updateProperty("backgroundImage","");
        this._selectCreativeCommonsFilter();
        var markSupportedForProcessing = WinJS.Utilities.markSupportedForProcessing;
        var createClosure = BingApp.Utilities.createClosure;
        this.performSearch = markSupportedForProcessing(createClosure(this,performSearch));
        this.handleEnterKeyDown = markSupportedForProcessing(createClosure(this,handleEnterKeyDown));
        this.handleTextValueChange = markSupportedForProcessing(createClosure(this,handleTextValueChange));
        this.handleSelectionChanged = markSupportedForProcessing(createClosure(this,handleSelectionChanged));
        this.handleSearchBoxFocus = markSupportedForProcessing(createClosure(this,handleSearchBoxFocus));
        this.handleSearchBoxBlur = markSupportedForProcessing(createClosure(this,handleSearchBoxBlur));
        this.handleCreativeCommonsClicked = markSupportedForProcessing(createClosure(this,handleCreativeCommonsClicked));
        this.handleAllWebImagesClicked = markSupportedForProcessing(createClosure(this,handleAllWebImagesClicked));
        this.handleCreativeCommonsKeyDown = markSupportedForProcessing(createClosure(this,handleCreativeCommonsKeyDown));
        this.handleAllWebImagesKeyDown = markSupportedForProcessing(createClosure(this,handleAllWebImagesKeyDown));
        this.handleLoadingStateChanged = markSupportedForProcessing(createClosure(this,handleLoadingStateChanged))
    }
    function _notifyError(message)
    {
        this.bindings.setProperty("listViewDisplay","none");
        this.bindings.setProperty("errorDisplay","");
        this.bindings.setProperty("errorMessage",message)
    }
    function _createNoNetworkErrorMessage()
    {
        return WinJS.Resources.getString("error_no_network").value
    }
    function _createNoResultsErrorMessage(query)
    {
        var message = BingApp.Utilities.format("{0}<br/><br/>{1}<ul><li>{2}</li><li>{3}</li><li>{4}</li></ul>",BingApp.Utilities.format(WinJS.Resources.getString("error_file_picker_no_results").value,query),WinJS.Resources.getString("error_file_picker_no_results_try_tips").value,WinJS.Resources.getString("error_file_picker_no_results_try_tips_spelling").value,WinJS.Resources.getString("error_file_picker_no_results_try_tips_general_terms").value,WinJS.Resources.getString("error_file_picker_no_results_try_tips_shorter_phrase").value);
        return message
    }
    function _createGenericErrorMessage()
    {
        return WinJS.Resources.getString("error_file_picker_error_occurred").value
    }
    function _createSafeSearchErrorMessage()
    {
        return BingApp.Utilities.format(WinJS.Resources.getString("error_file_picker_safe_search").value,showSafeSearchUri)
    }
    function _getFileName(uri, mediaFileFormat)
    {
        var fileName;
        var extension = uri.extension;
        var lowerCaseExtension = extension.toLowerCase();
        var overridingExtension;
        if(mediaFileFormat)
            overridingExtension = "." + mediaFileFormat.toLowerCase();
        if(supportedExtensions[lowerCaseExtension])
        {
            var path = uri.path;
            var extensionDotIndex = path.length - extension.length;
            var fileNameStart = path.lastIndexOf("/",extensionDotIndex - 1) + 1;
            fileName = path.substring(fileNameStart,extensionDotIndex);
            if(supportedExtensions[overridingExtension] && overridingExtension !== lowerCaseExtension && (overridingExtension !== ".jpg" || lowerCaseExtension !== ".jpeg") && (overridingExtension !== ".jpeg" || lowerCaseExtension !== ".jpg") && (overridingExtension !== ".tif" || lowerCaseExtension !== ".tiff") && (overridingExtension !== ".tiff" || lowerCaseExtension !== ".tif"))
                extension = overridingExtension;
            if(fileName.length + extension.length > fileNameMaxLength)
                fileName = fileName.substring(0,fileNameMaxLength - extension.length);
            fileName = fileName + extension
        }
        else if(supportedExtensions[overridingExtension])
            fileName = defaultFileName + overridingExtension;
        else
        {
            BingApp.traceError("FilePickerViewModel._getFileName: could not get file name from {0}.",uri.absoluteUri);
            fileName = null
        }
        return fileName
    }
    function handleFileRemovedFromBasket(eventObject)
    {
        if(!eventObject)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("eventObject");
        var that = this;
        delete that._selection[eventObject.id];
        var selectionIndices = [];
        Object.keys(that._selection).forEach(function addIndex(key)
        {
            selectionIndices.push(that._selection[key])
        });
        return that.bindings.setProperty("selectionIndices",selectionIndices)
    }
    function performSearch()
    {
        var query = this.text.trim();
        this._selection = {};
        this.bindings.selectionIndices = [];
        window.localStorage.setItem(localStorageLastSearchKey,query);
        this.bindings.setProperty("listViewDisplay","");
        this.bindings.setProperty("errorDisplay","none");
        if(query)
        {
            var that = this;
            this._isFirstLoadingComplete = false;
            this._config.reload(false);
            var config = this._config.getConfig();
            if(this.imageDataSource)
                this.imageDataSource.bindings.unbind("error");
            this.imageDataSource = new BingApp.Classes.ImageDataSource({
                url: config.filePicker.general.url,
                query: query,
                appId: config.filePicker.general.appId,
                market: Windows.System.UserProfile.GlobalizationPreferences.languages[0],
                adult: config.search.safeSearch.safeSearch.toLowerCase(),
                creativeCommons: this._isCreativeCommonsSelected,
                apiVersion: config.filePicker.general.apiVersion
            });
            this.imageDataSource.bindings.bind("error",function handleError(error)
            {
                var errorEnumeration = BingApp.Classes.ImageDataSource.errors;
                switch(error)
                {
                    case errorEnumeration.noNetwork:
                        that._notifyError(that._createNoNetworkErrorMessage());
                        break;
                    case errorEnumeration.noResults:
                        that._notifyError(that._createNoResultsErrorMessage(query));
                        break;
                    case errorEnumeration.safeSearch:
                        that._notifyError(that._createSafeSearchErrorMessage());
                        break;
                    case errorEnumeration.generic:
                        that._notifyError(that._createGenericErrorMessage());
                        break;
                    default:
                        break
                }
            });
            this.bindings.setProperty("dataSource",this.imageDataSource)
        }
        else
        {
            this.bindings.setProperty("dataSource",null);
            this.bindings.updateProperty("listViewDisplay","none")
        }
    }
    function handleTextValueChange(eventObject)
    {
        if(!eventObject)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("eventObject");
        if(eventObject.propertyName === "value" && eventObject.srcElement)
            this.text = eventObject.srcElement.value
    }
    function handleSearchBoxFocus()
    {
        this.bindings.setProperty("searchBoxClassName","semiTransparentBackground")
    }
    function handleSearchBoxBlur()
    {
        if(!this.text)
            this.bindings.setProperty("searchBoxClassName","")
    }
    function handleEnterKeyDown(eventObject)
    {
        if(!eventObject)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("eventObject");
        if(eventObject.which === WinJS.Utilities.Key.enter)
        {
            eventObject.preventDefault();
            this.performSearch();
            eventObject.target.blur()
        }
    }
    function _selectCreativeCommonsFilter()
    {
        this.bindings.setProperty("creativeCommonsClassName",selectedFilterStyle);
        this.bindings.setProperty("allWebImagesClassName",unselectedFilterStyle);
        this.bindings.setProperty("creativeCommonsTabIndex",unfocusableTabIndex);
        this.bindings.setProperty("allWebImagesTabIndex",focusableTabIndex);
        this.bindings.setProperty("filterInformation",WinJS.Resources.getString("file_picker_creative_commons_notice").value)
    }
    function handleCreativeCommonsClicked()
    {
        if(this.bindings.creativeCommonsClassName !== selectedFilterStyle)
        {
            this._isCreativeCommonsSelected = true;
            this._selectCreativeCommonsFilter();
            this.performSearch()
        }
    }
    function _selectAllWebImagesFilter()
    {
        this.bindings.setProperty("creativeCommonsClassName",unselectedFilterStyle);
        this.bindings.setProperty("allWebImagesClassName",selectedFilterStyle);
        this.bindings.setProperty("creativeCommonsTabIndex",focusableTabIndex);
        this.bindings.setProperty("allWebImagesTabIndex",unfocusableTabIndex);
        this.bindings.setProperty("filterInformation",WinJS.Resources.getString("file_picker_creative_commons_removal_notice").value)
    }
    function handleAllWebImagesClicked()
    {
        if(this.bindings.allWebImagesClassName !== selectedFilterStyle)
        {
            this._isCreativeCommonsSelected = false;
            this._selectAllWebImagesFilter();
            this.performSearch()
        }
    }
    function handleCreativeCommonsKeyDown(eventObject)
    {
        if(!eventObject)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("eventObject");
        if(eventObject.which === WinJS.Utilities.Key.enter || eventObject.which === WinJS.Utilities.Key.space)
        {
            eventObject.preventDefault();
            this.handleCreativeCommonsClicked();
            eventObject.target.blur()
        }
    }
    function handleAllWebImagesKeyDown(eventObject)
    {
        if(!eventObject)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("eventObject");
        if(eventObject.which === WinJS.Utilities.Key.enter || eventObject.which === WinJS.Utilities.Key.space)
        {
            eventObject.preventDefault();
            this.handleAllWebImagesClicked();
            eventObject.target.blur()
        }
    }
    function handleLoadingStateChanged(eventObject)
    {
        if(!this._isFirstLoadingComplete)
        {
            if(!(eventObject && eventObject.target && eventObject.target.winControl instanceof WinJS.UI.ListView))
                throw new BingApp.Classes.ErrorArgument("eventObject");
            var listView = eventObject.target.winControl;
            if(listView.loadingState === "complete")
            {
                BingApp.traceInfo("FilePickerViewModel.handleLoadingStateChanged: first loading complete.");
                this._isFirstLoadingComplete = true
            }
        }
    }
    function handleSelectionChanged(eventObject)
    {
        if(!(eventObject && eventObject.target && eventObject.target.winControl instanceof WinJS.UI.ListView))
            throw new BingApp.Classes.ErrorArgument("eventObject");
        var that = this;
        var listView = eventObject.target.winControl;
        var indices = listView.selection.getIndices();
        function handleSelectionChangedClosure()
        {
            var newSelection = {};
            var newSelectionIndices = [];
            var indicesToUnselect = [];
            var newFilesToAdd = [];
            indices.forEach(function handleSelectedListItemIndex(index)
            {
                index = parseInt(index);
                that.bindings.dataSource.itemFromIndex(index).done(function handleSelectedListItem(item)
                {
                    var itemData = item.data;
                    var key = itemData.thumbnail;
                    if(that._selection[key] === index)
                    {
                        newSelection[key] = index;
                        delete that._selection[key]
                    }
                    else
                        newFilesToAdd.push({
                            key: key,
                            index: index,
                            itemData: itemData
                        })
                })
            });
            Object.keys(that._selection).forEach(function removeFile(key)
            {
                try
                {
                    that._fileOpenPickerUI.removeFile(key)
                }
                catch(error)
                {
                    BingApp.traceError("FilePickerViewModel.handleSelectionChanged: error while removing file from FileOpenPickerUI: {0}; error: {1}.",key,error.message)
                }
            });
            var promises = [];
            newFilesToAdd.forEach(function handleItem(item)
            {
                var key = item.key;
                var index = item.index;
                var itemData = item.itemData;
                var uri;
                var thumbnail;
                var fileName;
                try
                {
                    uri = new Windows.Foundation.Uri(itemData.mediaUrl);
                    fileName = that._getFileName(uri,itemData.mediaFileFormat)
                }
                catch(error)
                {
                    BingApp.traceError("FilePickerViewModel.handleSelectionChanged: failed to create URI from {0}; message: {1}",itemData.mediaUrl,error.message)
                }
                if(that._fileOpenPickerUI.selectionMode === Windows.Storage.Pickers.Provider.FileSelectionMode.single)
                    thumbnail = null;
                else
                    try
                    {
                        var thumbnailUri = new Windows.Foundation.Uri(itemData.thumbnail);
                        thumbnail = Windows.Storage.Streams.RandomAccessStreamReference.createFromUri(thumbnailUri)
                    }
                    catch(error)
                    {
                        BingApp.traceError("FilePickerViewModel.handleSelectionChanged: failed to create thumbnail URI from {0}; message: {1}",itemData.thumbnail,error.message)
                    }
                if(!(fileName && uri))
                {
                    that.bindings.setProperty("selectionErrorMessage",WinJS.Resources.getString("error_file_picker_selection_error_occurred").value);
                    that.bindings.selectionErrorIndex = index;
                    that.bindings.notify("selectionErrorIndex",that.bindings.selectionErrorIndex);
                    return
                }
                var streamedFileCreationPromise;
                try
                {
                    streamedFileCreationPromise = Windows.Storage.StorageFile.createStreamedFileFromUriAsync(fileName,uri,thumbnail)
                }
                catch(error)
                {
                    streamedFileCreationPromise = WinJS.Promise.wrapError(error)
                }
                promises.push(streamedFileCreationPromise.then(function addFile(fileToAdd)
                {
                    var addFileResult = that._fileOpenPickerUI.addFile(itemData.thumbnail,fileToAdd);
                    var addFileResultEnumeration = Windows.Storage.Pickers.Provider.AddFileResult;
                    var selectionErrorMessage;
                    switch(addFileResult)
                    {
                        case addFileResultEnumeration.added:
                            newSelection[key] = index;
                            newSelectionIndices.push(index);
                            break;
                        case addFileResultEnumeration.alreadyAdded:
                            selectionErrorMessage = WinJS.Resources.getString("error_file_picker_selection_file_already_added").value;
                            break;
                        case addFileResultEnumeration.notAllowed:
                            var extension = fileName.substring(fileName.lastIndexOf(".") + 1).toUpperCase();
                            selectionErrorMessage = BingApp.Utilities.format(WinJS.Resources.getString("error_file_picker_selection_cannot_open_file").value,extension);
                            break;
                        default:
                            selectionErrorMessage = WinJS.Resources.getString("error_file_picker_selection_error_occurred").value;
                            break
                    }
                    if(selectionErrorMessage)
                    {
                        indicesToUnselect.push(index);
                        BingApp.traceError("FilePickerViewModel.handleSelectionChanged: failed to select item; message: {0}.",selectionErrorMessage);
                        that._notifySelectionError(index,selectionErrorMessage)
                    }
                },function onAddFileError(error)
                {
                    BingApp.traceError("FilePickerViewModel.handleSelectionChanged: failed to create streamed file from Uri: {0}; error: {1}.",uri.absoluteUri,error && error.message);
                    that._notifySelectionError(index,WinJS.Resources.getString("error_file_picker_selection_error_occurred").value)
                }))
            });
            return WinJS.Promise.join(promises).then(function updateSelection()
                {
                    that._selection = newSelection;
                    listView.onselectionchanged = null;
                    function restoreSelectionChangedHandler()
                    {
                        listView.onselectionchanged = that.handleSelectionChanged
                    }
                    return WinJS.Promise.join([listView.selection.add(newSelectionIndices),listView.selection.remove(indicesToUnselect),]).then(restoreSelectionChangedHandler,restoreSelectionChangedHandler)
                })
        }
        setImmediate(function setImmediateHandleSelectionChanged()
        {
            handleSelectionChangedClosure()
        })
    }
    function _notifySelectionError(index, selectionErrorMessage)
    {
        this.bindings.setProperty("selectionErrorMessage",selectionErrorMessage);
        this.bindings.selectionErrorIndex = index;
        this.bindings.notify("selectionErrorIndex",this.bindings.selectionErrorIndex)
    }
    var instanceMembers = {
            bindings: WinJS.Binding.as({
                selectionMode: "single",
                dataSource: null,
                selectionErrorIndex: -1,
                selectionErrorMessage: "",
                selectionErrorPlacement: "right",
                selectionIndices: [],
                listViewDisplay: "none",
                errorDisplay: "none",
                errorMessage: "",
                searchBoxClassName: "",
                backgroundImage: "",
                creativeCommonsClassName: "fontMedium fontWeightRegular disabledFilter",
                allWebImagesClassName: "fontMedium fontWeightRegular disabledFilter",
                creativeCommonsTabIndex: unfocusableTabIndex,
                allWebImagesTabIndex: unfocusableTabIndex,
                filterInformation: ""
            }),
            performSearch: null,
            handleEnterKeyDown: null,
            handleTextValueChange: null,
            handleSelectionChanged: null,
            handleSearchBoxFocus: null,
            handleSearchBoxBlur: null,
            handleCreativeCommonsClicked: null,
            handleAllWebImagesClicked: null,
            handleCreativeCommonsKeyDown: null,
            handleAllWebImagesKeyDown: null,
            handleLoadingStateChanged: null,
            text: "",
            _notifyError: _notifyError,
            _notifySelectionError: _notifySelectionError,
            _initializeBindingData: _initializeBindingData,
            _getFileName: _getFileName,
            _initializeBackground: _initializeBackground,
            _createNoNetworkErrorMessage: _createNoNetworkErrorMessage,
            _createNoResultsErrorMessage: _createNoResultsErrorMessage,
            _createSafeSearchErrorMessage: _createSafeSearchErrorMessage,
            _createGenericErrorMessage: _createGenericErrorMessage,
            _selectCreativeCommonsFilter: _selectCreativeCommonsFilter,
            _selectAllWebImagesFilter: _selectAllWebImagesFilter,
            _selection: {},
            _fileOpenPickerUI: null,
            _config: null,
            _isFirstLoadingComplete: false,
            _isCreativeCommonsSelected: true,
            imageDataSource: null
        };
    var staticMembers = {rightToLeftDirection: "rtl"};
    function createThumbnailElement(itemPromise)
    {
        return itemPromise.then(function createElement(currentItem)
            {
                var div = document.createElement("div");
                div.className = "thumbnail";
                var image = new Image;
                image.onload = function()
                {
                    div.style.backgroundImage = "url('" + currentItem.data.thumbnail + "')";
                    WinJS.UI.Animation.fadeIn(div);
                    image = null
                };
                image.src = currentItem.data.thumbnail;
                var mediaUrl = currentItem.data.mediaUrl;
                var attributedLinkContainer = document.createElement("div");
                attributedLinkContainer.className = "attributedLinkContainer win-interactive";
                function invokeUri()
                {
                    BingApp.Utilities.invokeURI(mediaUrl)
                }
                attributedLinkContainer.onclick = invokeUri;
                attributedLinkContainer.onkeydown = function handleLinkEnterKeyDown(eventObject)
                {
                    if(eventObject.which === WinJS.Utilities.Key.enter || eventObject.which === WinJS.Utilities.Key.space)
                    {
                        eventObject.preventDefault();
                        invokeUri()
                    }
                };
                var attributedLink = document.createElement("div");
                attributedLink.tabIndex = "";
                attributedLink.className = "fontSmall fontWeightRegular attributedLink";
                attributedLink.innerText = mediaUrl;
                attributedLinkContainer.appendChild(attributedLink);
                div.appendChild(attributedLinkContainer);
                return div
            })
    }
    WinJS.Namespace.define("BingApp.Templates",{FilePickerThumbnail: WinJS.Utilities.markSupportedForProcessing(createThumbnailElement)});
    WinJS.Namespace.define("BingApp.Classes",{FilePickerViewModel: WinJS.Class.define(constructor,instanceMembers,staticMembers)})
})()