﻿(function imageDataSourceModule()
{
    "use strict";
    var errors = Object.freeze({
            noNetwork: 0,
            noResults: 1,
            safeSearch: 2,
            generic: 3,
            none: 4
        });
    var formCode = "QFPIW8";
    var instrumentationId = "imagexml";
    var maxCount = 1e3;
    var maxPageSize = 35;
    var minPageSize = 10;
    var overlapLimit = 10;
    var namespaces = new Windows.Foundation.Collections.PropertySet;
    namespaces["bdi"] = "http://schemas.microsoft.com/bing/bdi/generic/1";
    namespaces["img1"] = "http://schemas.microsoft.com/bing/bdi/generic/image/1";
    namespaces["gen1"] = "http://schemas.microsoft.com/bing/bdi/generic/1";
    var impressionGuidXPath = "/bdi:BingResponse/bdi:Responses/img1:Image/img1:ImpressionGuid";
    var estimateMatchesXPath = "/bdi:BingResponse/bdi:Responses/img1:Image/img1:EstimatedMatches";
    var thumbnailBaseUrlXPath = "/bdi:BingResponse/bdi:Responses/img1:Image/img1:ThumbnailBaseUrl";
    var imageResultXPath = "/bdi:BingResponse/bdi:Responses/img1:Image/img1:Results/img1:ImageResult";
    var isAdultQueryXPath = "/bdi:BingResponse/bdi:Responses/img1:Image/img1:IsAdultQuery";
    var thumbnailIdXPath = "./img1:StaticThumbnail/gen1:ThumbnailID";
    var mediaUrlXPath = "./img1:MediaUrl";
    var mediaFileFormatXPath = "./img1:MediaFileFormat";
    function listDataAdapterConstructor(options, bindings)
    {
        if(!options || !options.appId || !options.query || !options.url || !options.adult || !options.market || typeof options.creativeCommons !== "boolean" || !options.apiVersion)
            throw new BingApp.Classes.ErrorArgument("options");
        this.bindings = bindings;
        this._fragmentedPages = {};
        var adultOption = options.adult.toLowerCase();
        if(adultOption === "off")
            this._isSafeSearchOn = false;
        var collection = {
                q: options.query,
                appid: options.appId,
                mkt: options.market,
                adlt: options.adult,
                v: options.apiVersion,
                iid: instrumentationId,
                form: formCode
            };
        if(options.creativeCommons)
            collection.qft = "+filterui:licensetype-any";
        this._url = options.url + BingApp.Utilities.QueryString.serialize(collection);
        this._initializeInternetAccessHandler()
    }
    function _initializeInternetAccessHandler()
    {
        var that = this;
        var networkInformation = Windows.Networking.Connectivity.NetworkInformation;
        var networkConnectivityLevel = Windows.Networking.Connectivity.NetworkConnectivityLevel;
        function updateInternetAccess(eventData)
        {
            var connectionProfile = networkInformation.getInternetConnectionProfile();
            that._hasInternetAccess = connectionProfile !== null && connectionProfile.getNetworkConnectivityLevel() === networkConnectivityLevel.internetAccess
        }
        networkInformation.addEventListener("networkstatuschanged",updateInternetAccess);
        updateInternetAccess()
    }
    function _createRequestUrl(first, count)
    {
        var requestUrl = this._url + "&first=" + first + "&count=" + count + "&sfx=" + this._requestCount;
        if(this._impressionGuid)
            requestUrl += "&ig=" + this._impressionGuid;
        return requestUrl
    }
    function _notifyGenericError()
    {
        this.bindings.setProperty("error",BingApp.Classes.ImageDataSource.errors.generic)
    }
    function _notifyNoResultsError()
    {
        this.bindings.setProperty("error",BingApp.Classes.ImageDataSource.errors.noResults)
    }
    function _notifyNetworkError()
    {
        if(this._hasInternetAccess)
            this.bindings.setProperty("error",BingApp.Classes.ImageDataSource.errors.generic);
        else
            this.bindings.setProperty("error",BingApp.Classes.ImageDataSource.errors.noNetwork)
    }
    function _notifySafeSearchError()
    {
        this.bindings.setProperty("error",BingApp.Classes.ImageDataSource.errors.safeSearch)
    }
    function getCount()
    {
        if(this._totalCount !== undefined)
            return WinJS.Promise.as(this._totalCount);
        this._requestCount++;
        var that = this;
        var url = this._createRequestUrl(0,maxPageSize);
        return WinJS.xhr({url: url}).then(function onGetCountSuccess(response)
            {
                var xml = new Windows.Data.Xml.Dom.XmlDocument;
                xml.loadXml(response.responseText);
                if(that._isSafeSearchOn)
                {
                    var isAdultQueryNode = xml.selectSingleNodeNS(isAdultQueryXPath,namespaces);
                    if(isAdultQueryNode && isAdultQueryNode.innerText.toLowerCase() === "true")
                    {
                        BingApp.traceError("ImageListDataAdapter.getCount: SafeSearch is on, and the query was identified as an adult query.");
                        that._notifySafeSearchError();
                        return WinJS.Promise.wrapError(new WinJS.ErrorFromName("safeSearchError","Query is an adult query and SafeSearch is on."))
                    }
                }
                var impressionGuidNode = xml.selectSingleNodeNS(impressionGuidXPath,namespaces);
                if(impressionGuidNode)
                    that._impressionGuid = impressionGuidNode.innerText;
                else
                {
                    BingApp.traceError("ImageListDataAdapter.getCount: ImpressionGuid not defined in response.");
                    that._notifyGenericError();
                    return WinJS.Promise.wrapError(new WinJS.ErrorFromName("missingImpressionGuidError","Missing ImpressionGuid node"))
                }
                var thumbnailBaseUrlNode = xml.selectSingleNodeNS(thumbnailBaseUrlXPath,namespaces);
                if(thumbnailBaseUrlNode)
                    that._thumbnailBaseUrl = thumbnailBaseUrlNode.innerText;
                else
                {
                    BingApp.traceError("ImageListDataAdapter.getCount: ThumbnailBaseUrl not defined in response.");
                    that._notifyGenericError();
                    return WinJS.Promise.wrapError(new WinJS.ErrorFromName("missingThumbnailBaseUrlError","Missing ThumbnailBaseUrl node"))
                }
                var estimateMatchesNode = xml.selectSingleNodeNS(estimateMatchesXPath,namespaces);
                if(estimateMatchesNode)
                {
                    that._totalCount = Math.min(parseInt(estimateMatchesNode.innerText),maxCount);
                    if(that._totalCount === 0)
                        that._notifyNoResultsError();
                    return that._totalCount
                }
                else
                {
                    BingApp.traceError("ImageListDataAdapter.getCount: EstimateMatches not defined in response.");
                    that._notifyGenericError();
                    return WinJS.Promise.wrapError(new WinJS.ErrorFromName("missingEstimateMatchesError","Missing EstimateMatches node"))
                }
            },function onGetCountError(error)
            {
                BingApp.traceError("ImageListDataAdapter.getCount: error while trying to get result count; status: {0} {1}.",error && error.status,error && error.statusText);
                that._notifyNetworkError();
                return WinJS.UI.FetchError.noResponse
            })
    }
    function itemsFromIndex(index, countBefore, countAfter)
    {
        if(index >= maxCount)
        {
            BingApp.traceError("ImageListDataAdapter.itemsFromIndex: the request index is greater than the allowed maximum count.");
            this._notifyGenericError();
            return WinJS.Promise.wrapError(new WinJS.ErrorFromName(WinJS.UI.FetchError.doesNotExist))
        }
        if(!this._thumbnailBaseUrl)
        {
            BingApp.traceError("ImageListDataAdapter.itemsFromIndex: _thumbnailBaseUrl not initialized yet.");
            this._notifyGenericError();
            return WinJS.Promise.wrapError(new WinJS.ErrorFromName(WinJS.UI.FetchError.doesNotExist))
        }
        if(!this._totalCount)
        {
            BingApp.traceError("ImageListDataAdapter.itemsFromIndex: _totalCount not initialized yet.");
            this._notifyGenericError();
            return WinJS.Promise.wrapError(new WinJS.ErrorFromName(WinJS.UI.FetchError.doesNotExist))
        }
        if(!this._impressionGuid)
        {
            BingApp.traceError("ImageListDataAdapter.itemsFromIndex: _impressionGuid not initialized yet.");
            this._notifyGenericError();
            return WinJS.Promise.wrapError(new WinJS.ErrorFromName(WinJS.UI.FetchError.doesNotExist))
        }
        this._requestCount++;
        var that = this;
        var pageIndex = Math.floor(index / maxPageSize);
        var fragmentedPage = this._fragmentedPages[pageIndex];
        var fragmentStartIndex = 0;
        var first;
        var count;
        if(fragmentedPage)
        {
            var fragmentedPageLength = fragmentedPage.length;
            for(var iterator = 0; iterator + 1 < fragmentedPageLength; iterator++)
                if(index >= fragmentedPage[iterator] && index < fragmentedPage[iterator + 1])
                {
                    fragmentStartIndex = iterator;
                    break
                }
            first = fragmentedPage[fragmentStartIndex];
            count = fragmentedPage[fragmentStartIndex + 1] - fragmentedPage[fragmentStartIndex]
        }
        else
        {
            first = pageIndex * maxPageSize;
            count = maxPageSize
        }
        var url = this._createRequestUrl(first,count);
        return WinJS.xhr({url: url}).then(function getResults(response)
            {
                var xml = new Windows.Data.Xml.Dom.XmlDocument;
                xml.loadXml(response.responseText);
                var imageResultNodes = xml.selectNodesNS(imageResultXPath,namespaces);
                if(imageResultNodes)
                {
                    var results = [];
                    var length = imageResultNodes.length;
                    if(length < count)
                        if(fragmentedPage)
                            fragmentedPage.splice(fragmentStartIndex + 1,0,fragmentedPage[fragmentStartIndex] + length);
                        else
                        {
                            var pageStartIndex = pageIndex * maxPageSize;
                            fragmentedPage = [0,length,maxPageSize];
                            that._fragmentedPages[pageIndex] = fragmentedPage
                        }
                    for(var i = 0; i < length; i++)
                    {
                        var imageResultNode = imageResultNodes[i];
                        var thumbnailIdNode = imageResultNode.selectSingleNodeNS(thumbnailIdXPath,namespaces);
                        if(!thumbnailIdNode)
                        {
                            BingApp.traceError("ImageListDataAdapter.itemsFromIndex: ThumbnailID not defined in response.");
                            that._notifyGenericError();
                            return WinJS.Promise.wrapError(new WinJS.ErrorFromName("missingThumbnailIdError","Missing ThumbnailID node"))
                        }
                        var mediaUrlNode = imageResultNode.selectSingleNodeNS(mediaUrlXPath,namespaces);
                        if(!mediaUrlNode)
                        {
                            BingApp.traceError("ImageListDataAdapter.itemsFromIndex: MediaUrl not defined in response.");
                            that._notifyGenericError();
                            return WinJS.Promise.wrapError(new WinJS.ErrorFromName("missingMediaUrlError","Missing MediaUrl node"))
                        }
                        var mediaFileFormatNode = imageResultNode.selectSingleNodeNS(mediaFileFormatXPath,namespaces);
                        if(!mediaFileFormatNode)
                        {
                            BingApp.traceError("ImageListDataAdapter.itemsFromIndex: MediaFileFormat not defined in response.");
                            that._notifyGenericError();
                            return WinJS.Promise.wrapError(new WinJS.ErrorFromName("missingMediaFileFormatError","Missing MediaFileFormat node"))
                        }
                        results.push({
                            key: String(first + i),
                            data: {
                                thumbnail: that._thumbnailBaseUrl + thumbnailIdNode.innerText,
                                mediaUrl: mediaUrlNode.innerText,
                                mediaFileFormat: mediaFileFormatNode.innerText
                            }
                        })
                    }
                    return{
                            items: results,
                            offset: index - first,
                            totalCount: that._totalCount
                        }
                }
                else
                {
                    BingApp.traceError("ImageListDataAdapter.itemsFromIndex: MediaUrl not defined in response.");
                    that._notifyGenericError();
                    return WinJS.Promise.wrapError(new WinJS.ErrorFromName("missingMediaUrlError","Missing MediaUrl"))
                }
            },function onGetItemsError(error)
            {
                BingApp.traceError("ImageListDataAdapter.itemsFromIndex: error while retrieving image results; status: {0} {1}.",error && error.status,error && error.statusText);
                return WinJS.UI.FetchError.noResponse
            })
    }
    var listDataAdapterInstanceMembers = {
            bindings: null,
            _url: null,
            _totalCount: undefined,
            _thumbnailBaseUrl: undefined,
            _hasInternetAccess: true,
            _isSafeSearchOn: true,
            _impressionGuid: null,
            _requestCount: 0,
            _fragmentedPages: null,
            getCount: getCount,
            itemsFromIndex: itemsFromIndex,
            _createRequestUrl: _createRequestUrl,
            _notifyGenericError: _notifyGenericError,
            _notifyNoResultsError: _notifyNoResultsError,
            _notifyNetworkError: _notifyNetworkError,
            _notifySafeSearchError: _notifySafeSearchError,
            _initializeInternetAccessHandler: _initializeInternetAccessHandler
        };
    WinJS.Namespace.define("BingApp.Classes",{ImageListDataAdapter: WinJS.Class.define(listDataAdapterConstructor,listDataAdapterInstanceMembers)});
    function dataSourceConstructor(options)
    {
        if(!options)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("options");
        this._initializeBindingData();
        this._baseDataSourceConstructor(new BingApp.Classes.ImageListDataAdapter(options,this.bindings))
    }
    function _initializeBindingData()
    {
        this.bindings.updateProperty("error",BingApp.Classes.ImageDataSource.errors.none)
    }
    var dataSourceStaticMembers = {errors: errors};
    var dataSourceInstanceMembers = {
            bindings: WinJS.Binding.as({error: dataSourceStaticMembers.errors.none}),
            _initializeBindingData: _initializeBindingData
        };
    WinJS.Namespace.define("BingApp.Classes",{ImageDataSource: WinJS.Class.derive(WinJS.UI.VirtualizedDataSource,dataSourceConstructor,dataSourceInstanceMembers,dataSourceStaticMembers)})
})()