﻿(function accountsModule()
{
    "use strict";
    var getWindowsLiveSignInButton = getById("windowsLiveSignInButton");
    var getWindowsLiveSignOutButton = getById("windowsLiveSignOutButton");
    var getWindowsLiveSignInStateLabel = getById("windowsLiveSignInStateLabel");
    var getFacebookSignInButton = getById("facebookSignInButton");
    var getFacebookSignOutButton = getById("facebookSignOutButton");
    var getFacebookSignInStateLabel = getById("facebookSignInStateLabel");
    var getFacebook = getById("facebook");
    var getRewards = getById("rewards");
    var liveIdAuthenticator;
    var facebookAuthenticator;
    var configuration;
    var networkDetectionService;
    var instrumentation;
    function onReady(options)
    {
        facebookAuthenticator = options && options.facebookAuthenticator || BingApp.locator.facebookAuthenticator;
        liveIdAuthenticator = options && options.liveIdAuthenticator || BingApp.locator.liveIdAuthenticator;
        configuration = options && options.config || BingApp.locator.config;
        networkDetectionService = options && options.networkDetectionService || BingApp.locator.networkDetectionService;
        instrumentation = options && options.instrumentation || BingApp.Instrumentation;
        var signInLiveId = getWindowsLiveSignInButton();
        var signOutLiveId = getWindowsLiveSignOutButton();
        var signInFacebook = getFacebookSignInButton();
        var signOutFacebook = getFacebookSignOutButton();
        var rewards = getRewards();
        var facebook = getFacebook();
        WinJS.Resources.processAll().done(function()
        {
            signInLiveId && signInLiveId.addEventListener("click",_signInLiveId);
            signOutLiveId && signOutLiveId.addEventListener("click",_signOutLiveId);
            if(facebook && configuration.getConfig().authentication.facebook.enabled)
            {
                signInFacebook && signInFacebook.addEventListener("click",_signInFacebook);
                signOutFacebook && signOutFacebook.addEventListener("click",_signOutFacebook);
                WinJS.Utilities.removeClass(facebook,"hidden")
            }
            if(rewards && configuration.getConfig().rewards.general.enabled)
            {
                rewards.addEventListener("click",_navigateToExploreRewards);
                rewards.onkeydown = _onKeyDownExploreRewards;
                WinJS.Utilities.removeClass(rewards,"hidden")
            }
            _refreshLiveIdState();
            _refreshFacebookState();
            instrumentation.log({
                name: "accountshown",
                type: "settings"
            },instrumentation.appEvent);
            BingApp.tracePerf("settings:accounts:ready")
        })
    }
    function _signInLiveId()
    {
        instrumentation.log({name: "signinliveid"},instrumentation.appEvent);
        if(!networkDetectionService.isConnected())
        {
            _showErrorMessage("_signInLiveId: Failed to signInLiveId due to no network connectivity");
            return
        }
        liveIdAuthenticator.signIn().done(_refreshLiveIdState,_refreshLiveIdState)
    }
    function _signOutLiveId()
    {
        instrumentation.log({name: "signoutliveid"},instrumentation.appEvent);
        if(!networkDetectionService.isConnected())
        {
            _showErrorMessage("_signOutLiveId: Failed to signOutLiveId due to no network connectivity");
            return
        }
        liveIdAuthenticator.signOut().done(_refreshLiveIdState,_refreshLiveIdState)
    }
    function _signInFacebook()
    {
        instrumentation.log({name: "signinfb"},instrumentation.appEvent);
        if(!networkDetectionService.isConnected())
        {
            _showErrorMessage("_signInFacebook: Failed to signInFacebook due to no network connectivity");
            return
        }
        facebookAuthenticator.signIn().done(_refreshFacebookState,_refreshFacebookState)
    }
    function _signOutFacebook()
    {
        instrumentation.log({name: "signoutfb"},instrumentation.appEvent);
        if(!networkDetectionService.isConnected())
        {
            _showErrorMessage("_signOutFacebook: Failed to signOutFacebook due to no network connectivity");
            return
        }
        facebookAuthenticator.signOut().done(_refreshFacebookState,_refreshFacebookState)
    }
    function _onKeyDownExploreRewards(eventData)
    {
        if(BingApp.Utilities.isKeyDownAndSpaceOrEnterKey(eventData))
            _navigateToExploreRewards()
    }
    function _navigateToExploreRewards()
    {
        instrumentation.log({name: "explorerewards"},instrumentation.appEvent);
        var bingHost = configuration.getConfig().app.general.host;
        var exploreRewardsPath = configuration.getConfig().rewards.general.exploreUrl;
        BingApp.Utilities.invokeURI(bingHost + exploreRewardsPath)
    }
    function _showErrorMessage(traceTag)
    {
        BingApp.Utilities.showErrorDialog("error_no_network",null,traceTag)
    }
    function _refreshLiveIdState(error)
    {
        if(BingApp.Utilities.isNotNullOrUndefined(error) && !BingApp.Utilities.isPromiseCancellationError(error))
        {
            _showErrorMessage(BingApp.Utilities.format("_refreshLiveIdState: Failed to signIn or signOut - error name: {0}, error message: {1}",error.name,error.message));
            return
        }
        var isAuthenticated = liveIdAuthenticator.isAuthenticated();
        BingApp.traceInfo("_refreshLiveIdState: account page status, user is authenticated via Live ID: {0}",isAuthenticated);
        var stateLabel = getWindowsLiveSignInStateLabel();
        if(stateLabel)
        {
            var text;
            if(isAuthenticated)
                text = liveIdAuthenticator.getAuthenticationData().signInName || WinJS.Resources.getString("settings_accounts_windowslive_signedin_label").value;
            else
                text = WinJS.Resources.getString("settings_accounts_windowslive_signedout_label").value;
            stateLabel.innerText = text;
            stateLabel.title = text
        }
        var signIn = getWindowsLiveSignInButton();
        var signOut = getWindowsLiveSignOutButton();
        signIn && (isAuthenticated ? signIn.classList.add("hidden") : signIn.classList.remove("hidden"));
        signOut && (isAuthenticated ? signOut.classList.remove("hidden") : signOut.classList.add("hidden"))
    }
    function _refreshFacebookState(error)
    {
        if(BingApp.Utilities.isNotNullOrUndefined(error) && !BingApp.Utilities.isPromiseCancellationError(error))
        {
            _showErrorMessage(BingApp.Utilities.format("_refreshFacebookState: Failed to signIn or signOut - error name: {0}, error message: {1}",error.name,error.message));
            return
        }
        var isAuthenticated = facebookAuthenticator.isAuthenticated();
        BingApp.traceInfo("_refreshFacebookState: account page status, user is authenticated via Facebook: {0}",isAuthenticated);
        var stateLabel = getFacebookSignInStateLabel();
        if(stateLabel)
        {
            var text;
            if(isAuthenticated)
                text = facebookAuthenticator.getAuthenticationData().signInName || WinJS.Resources.getString("settings_accounts_facebook_signedin_label").value;
            else
                text = WinJS.Resources.getString("settings_accounts_facebook_signedout_label").value;
            stateLabel.innerText = text;
            stateLabel.title = text
        }
        var signIn = getFacebookSignInButton();
        var signOut = getFacebookSignOutButton();
        signIn && (isAuthenticated ? signIn.classList.add("hidden") : signIn.classList.remove("hidden"));
        signOut && (isAuthenticated ? signOut.classList.remove("hidden") : signOut.classList.add("hidden"))
    }
    function getById(id)
    {
        return function()
            {
                return document.getElementById(id)
            }
    }
    WinJS.UI.Pages.define("/shell/html/accounts.html",{ready: onReady})
})()