﻿(function baseAuthenticationRelayModule()
{
    "use strict";
    var BaseAuthenticationRelay = WinJS.Class.define(function constructor(options)
        {
            var that = this;
            that._authenticator = options.authenticator;
            that._relay = options.relay;
            that._events = options.events;
            that._networkDetectionService = options && options.networkDetectionService || BingApp.locator.networkDetectionService;
            that._config = options && options.config || BingApp.locator.config;
            function _refreshStatus()
            {
                that._relay.fireEvent(that._events.statusRequested,that._authenticator.getAuthenticationData())
            }
            function _onNetworkStatusChanged()
            {
                BingApp.traceInfo("BaseAuthenticationRelay._onNetworkStatusChanged, shouldRefresh: {0}, isConnected: {1}",that._shouldRefreshAuthentication,that._networkDetectionService.isConnected());
                if(that._shouldRefreshAuthentication && that._networkDetectionService.isConnected())
                    that.trySilentSignIn()
            }
            that._relay.addEventListener(that._events.statusRequested,_refreshStatus);
            that._relay.addEventListener(BingApp.Classes.NetworkDetectionService.networkStatusChangedEvent,_onNetworkStatusChanged)
        },{
            _shouldRefreshAuthentication: false,
            trySilentSignIn: function()
            {
                BingApp.traceInfo("BaseAuthenticationRelay.trySilentSignIn: started");
                var that = this;
                return this._authenticator.trySilentSignIn().then(function silentSignInCompleted(success)
                    {
                        if(success)
                        {
                            that._relay.fireEvent(that._events.signedIn,that._authenticator.getAuthenticationData());
                            that._scheduleAuthenticationRefresh()
                        }
                        else
                            that._fireSignedOutEvent()
                    },function silentSignInError(error)
                    {
                        BingApp.traceError("BaseAuthenticationRelay.trySilentSignIn: silent sign in failed, swallowing error; name: {0}, message: {1}.",error && error.name,error && error.message);
                        that._fireSignedOutEvent()
                    })
            },
            signIn: function()
            {
                BingApp.traceInfo("BaseAuthenticationRelay.signIn: started");
                var that = this;
                return this._authenticator.signIn().then(function signInCompleted()
                    {
                        that._relay.fireEvent(that._events.signedIn,that._authenticator.getAuthenticationData());
                        that._scheduleAuthenticationRefresh()
                    })
            },
            signOut: function()
            {
                BingApp.traceInfo("BaseAuthenticationRelay.signOut: started");
                var that = this;
                return this._authenticator.signOut().then(function signOutCompleted()
                    {
                        that._fireSignedOutEvent();
                        if(that._scheduleAuthenticationRefreshOperation)
                        {
                            clearTimeout(that._scheduleAuthenticationRefreshOperation);
                            that._scheduleAuthenticationRefreshOperation = null
                        }
                    })
            },
            isAuthenticated: function()
            {
                return this._authenticator.isAuthenticated()
            },
            getAuthenticationData: function()
            {
                return this._authenticator.getAuthenticationData()
            },
            _fireSignedOutEvent: function()
            {
                this._relay.fireEvent(this._events.signedOut,{})
            },
            _scheduleAuthenticationRefresh: function()
            {
                var that = this;
                var tokenExpirySeconds = that._authenticator.getExpiry();
                that._shouldRefreshAuthentication = false;
                if(that._scheduleAuthenticationRefreshOperation)
                {
                    BingApp.traceInfo("BaseAuthenticationRelay._scheduleAuthenticationRefresh: already scheduled refresh operation, ignoring");
                    return
                }
                BingApp.traceInfo("BaseAuthenticationRelay._scheduleAuthenticationRefresh: tokens expire in {0}",tokenExpirySeconds);
                if(tokenExpirySeconds >= 0)
                {
                    var reauthenticationDelayBeforeExpirySeconds = that._config.getConfig().authentication.general.reauthenticationDelayBeforeExpirySeconds;
                    var timeout = 1e3 * Math.max(reauthenticationDelayBeforeExpirySeconds,tokenExpirySeconds - reauthenticationDelayBeforeExpirySeconds);
                    var startTime = Date.now();
                    that._scheduleAuthenticationRefreshOperation = setTimeout(function trySilentReauthentication()
                    {
                        var elapsedTime = Date.now() - startTime;
                        BingApp.traceInfo("BaseAuthenticationRelay.trySilentReauthentication: started (tokens near expiry), expectedElapsed: {0}, actualElapsed: {1}",timeout,elapsedTime);
                        that._shouldRefreshAuthentication = true;
                        that._scheduleAuthenticationRefreshOperation = null;
                        if(that._networkDetectionService.isConnected())
                        {
                            BingApp.traceInfo("BaseAuthenticationRelay.trySilentReauthentication: connected");
                            that.trySilentSignIn()
                        }
                        else
                            BingApp.traceInfo("BaseAuthenticationRelay.trySilentReauthentication: not connected")
                    },timeout)
                }
            }
        });
    WinJS.Namespace.define("BingApp.Classes",{BaseAuthenticationRelay: BaseAuthenticationRelay})
})()