﻿(function facebookAuthenticationModule()
{
    "use strict";
    var WebAuthentication = Windows.Security.Authentication.Web;
    var FacebookAuthenticator = WinJS.Class.derive(BingApp.Classes.BaseAuthenticator,function constructor(options)
        {
            this._config = options && options.config || BingApp.locator.config;
            this._authenticator = options && options.authenticator || WebAuthentication.WebAuthenticationBroker;
            this._env = options && options.env || BingApp.locator.env;
            this._createCsrf = options && options.createCsrf || this._generateCsrf;
            this._xhr = options && options.xhr || WinJS.xhr;
            options = options || {};
            options.settingsKey = "authentication.facebook";
            options.silentSignInPromptType = WebAuthentication.WebAuthenticationOptions.silentMode;
            options.name = "FacebookAuthenticator";
            options.frontDoorProviderName = "facebook";
            BingApp.Classes.BaseAuthenticator.call(this,options)
        },{
            signIn: function()
            {
                BingApp.traceInfo("FacebookAuthenticator.signIn: sign in started");
                return this._authenticate(WebAuthentication.WebAuthenticationOptions.none)
            },
            signOut: function()
            {
                var that = this;
                return this._signOutOfIFrames().then(function signOutViaProxyWithSIDComplete()
                    {
                        var configuration = that._config.getConfig();
                        var redirectUri = configuration.app.general.secureHost + configuration.authentication.facebook.ssoSignOutPath + BingApp.Utilities.QueryString.serialize(that._env.getIFrameParams()) + "#patch";
                        var startUri = configuration.authentication.facebook.facebookLogOutUrl + BingApp.Utilities.QueryString.serialize({
                                next: redirectUri,
                                access_token: that.getAuthenticationData().token
                            });
                        return that._authenticator.authenticateAsync(WebAuthentication.WebAuthenticationOptions.silentMode,new Windows.Foundation.Uri(startUri))
                    }).then(function authenticatorSignOutUserAsyncCompleted()
                    {
                        that._signOutCompleted()
                    },function authenticatorSignOutUserAsyncError(error)
                    {
                        BingApp.traceError("FacebookAuthenticator.signOut: sign out failed, name: {0}, message: {1}.",error && error.name,error && error.message);
                        return WinJS.Promise.wrapError(error)
                    })
            },
            _generateCsrf: function()
            {
                var cryptography = Windows.Security.Cryptography.CryptographicBuffer;
                var length = this._config.getConfig().authentication.general.csrfLength;
                var buffer = cryptography.generateRandom(length);
                var token = cryptography.encodeToBase64String(buffer).replace(/=/g,"").replace(/\//g,"_").replace(/\+/g,"-");
                BingApp.traceInfo("FacebookAuthenticator._generateCsrf: csrf token: {0}",token);
                return token
            },
            _authenticate: function(promptType)
            {
                BingApp.traceInfo("FacebookAuthenticator._authenticate: sign in started with promptype: {0}",promptType);
                var that = this;
                var configuration = this._config.getConfig();
                var csrf = this._createCsrf();
                var startUri = configuration.app.general.secureHost + configuration.authentication.facebook.ssoSignInPath + BingApp.Utilities.QueryString.serialize(this._env.getIFrameParams({csrf: csrf}));
                var graphUrl = configuration.authentication.facebook.facebookGraphUrl;
                return this._authenticator.authenticateAsync(promptType,new Windows.Foundation.Uri(startUri)).then(function authenticateUserAsyncCompleted(result)
                    {
                        var data = result && result.responseData;
                        if(data)
                        {
                            BingApp.traceInfo("FacebookAuthenticator._authenticate: facebook.com sign in completed");
                            var facebookTokenURI = new Windows.Foundation.Uri(data);
                            var facebookAuthentication = BingApp.Utilities.QueryString.parse(facebookTokenURI.fragment.substr(1));
                            if(facebookAuthentication && facebookAuthentication.error_reason === "user_denied")
                                return WinJS.Promise.wrapError(new WinJS.ErrorFromName("Canceled","Canceled"));
                            if(csrf !== facebookAuthentication.csrf)
                            {
                                BingApp.traceError("FacebookAuthenticator._authenticate: csrf mismatch: expected:'{0}', actual:'{1}'",csrf,facebookAuthentication.csrf);
                                return WinJS.Promise.wrapError(new WinJS.ErrorFromName("Facebook csrf mismatch"))
                            }
                            var token = facebookAuthentication.access_token;
                            if(token)
                            {
                                BingApp.traceInfo("FacebookAuthenticator._initialExpirySeconds: expires_in:{0}, config:{1}",facebookAuthentication.expires_in,configuration.authentication.facebook.tokenExpirySeconds);
                                that._initialExpirySeconds = configuration.authentication.facebook.tokenExpirySeconds;
                                var facebookExpiresInSeconds = parseInt(facebookAuthentication.expires_in,10);
                                if(facebookExpiresInSeconds < that._initialExpirySeconds)
                                    that._initialExpirySeconds = facebookExpiresInSeconds;
                                return WinJS.Promise.wrap({token: token})
                            }
                            else
                            {
                                BingApp.traceError("FacebookAuthenticator._authenticate: ticket not found: ticket: {0}.",data);
                                return WinJS.Promise.wrapError(new WinJS.ErrorFromName("Facebook ticket not found"))
                            }
                        }
                        else if(data === "")
                            return WinJS.Promise.wrapError(new WinJS.ErrorFromName("Canceled","Canceled"));
                        else
                        {
                            BingApp.traceError("FacebookAuthenticator._authenticate: facebook.com sign in failed, no responseData: {0}.",JSON.stringify(result));
                            return WinJS.Promise.wrapError(new WinJS.ErrorFromName("Facebook response format unexpected"))
                        }
                    },function authenticateUserAsyncFailed(error)
                    {
                        if(BingApp.Utilities.isPromiseCancellationError(error))
                            BingApp.traceInfo("FacebookAuthenticator._authenticate: sign in canceled");
                        else
                            BingApp.traceError("FacebookAuthenticator._authenticate: sign in failed unexpectedly, name: {0}, message: {1}",error && error.name,error && error.message);
                        return WinJS.Promise.wrapError(error)
                    }).then(function getFacebookProfile(authenticationData)
                    {
                        var query = BingApp.Utilities.QueryString.serialize({access_token: authenticationData.token});
                        return that._getProfile(graphUrl + query,authenticationData)
                    }).then(function getFacebookPermissions(authenticationData)
                    {
                        var query = BingApp.Utilities.QueryString.serialize({access_token: authenticationData.token});
                        return that._getPermissions(graphUrl + "/permissions" + query,authenticationData)
                    }).then(function onValidAuthResult(authenticationData)
                    {
                        var parameters = {
                                action: "token",
                                provider: "facebook",
                                completion_mode: "json",
                                token: authenticationData.token
                            };
                        return that._signInToIFrames(parameters,authenticationData)
                    })
            },
            _getPermissions: function(url, authenticationData)
            {
                BingApp.traceInfo("FacebookAuthenticator._getPermissions: requesting granted permissions");
                var permissions = this._xhr({url: url}).then(function(response)
                    {
                        if(!authenticationData)
                            return WinJS.Promise.wrapError(new WinJS.ErrorFromName("authenticationData null or undefined","FacebookAuthenticator._getPermissions could not set properties on null or undefined"));
                        if(response && response.status === 200)
                            try
                            {
                                authenticationData.permissions = JSON.parse(response.responseText).data;
                                BingApp.traceInfo("FacebookAuthenticator._getPermissions: received granted permissions: {0}",JSON.stringify(authenticationData.permissions));
                                if(authenticationData.permissions)
                                    return WinJS.Promise.wrap(authenticationData)
                            }
                            catch(ex)
                            {
                                BingApp.traceError("FacebookAuthenticator._getPermissions: bad response from permissions graph, name: {0}, message: {1}, response: {2}",ex.name,ex.message,response.responseText)
                            }
                        return WinJS.Promise.wrapError(new WinJS.ErrorFromName("facebook graph error: permissions"))
                    });
                return permissions
            },
            _getProfile: function(url, authenticationData)
            {
                BingApp.traceInfo("FacebookAuthenticator._getProfile: requesting profile");
                var profile = this._xhr({url: url}).then(function(response)
                    {
                        if(!authenticationData)
                            return WinJS.Promise.wrapError(new WinJS.ErrorFromName("authenticationData null or undefined","FacebookAuthenticator._getProfile could not set properties on null or undefined"));
                        if(response && response.status === 200)
                            try
                            {
                                var graph = JSON.parse(response.responseText);
                                authenticationData.profile = graph;
                                authenticationData.firstName = graph.first_name || authenticationData.firstName;
                                authenticationData.signInName = graph.email || authenticationData.signInName;
                                BingApp.traceInfo("FacebookAuthenticator._getProfile: received profile: {0}",JSON.stringify(authenticationData.profile));
                                if(authenticationData.profile)
                                    return WinJS.Promise.wrap(authenticationData)
                            }
                            catch(ex)
                            {
                                BingApp.traceError("FacebookAuthenticator._getProfile: bad response from profile graph, name: {0}, message: {1}, response: {2}",ex.name,ex.message,response.responseText)
                            }
                        return WinJS.Promise.wrapError(new WinJS.ErrorFromName("facebook graph error: profile"))
                    });
                return profile
            }
        });
    WinJS.Namespace.define("BingApp.Classes",{FacebookAuthenticator: FacebookAuthenticator})
})()