﻿(function liveIdAuthenticationModule()
{
    "use strict";
    function extractPolicyDomain(secureHost)
    {
        BingApp.traceInfo("liveIdAuthenticationModule.extractPolicyDomain: {0}",secureHost);
        try
        {
            var uri = new Windows.Foundation.Uri(secureHost);
            if(uri.schemeName === "https" && /\bbing\b/.test(uri.domain))
                return uri.domain
        }
        catch(error)
        {
            BingApp.traceError("liveIdAuthenticationModule.extractPolicyDomain: Could not extract domain from {0}: name: {1}, message: {2}.",secureHost,error.name,error.message)
        }
        return"bing.com"
    }
    var OnlineId = Windows.Security.Authentication.OnlineId;
    var LiveIdAuthenticator = WinJS.Class.derive(BingApp.Classes.BaseAuthenticator,function constructor(options)
        {
            this._config = options && options.config || BingApp.locator.config;
            this._authenticator = options && options.authenticator || new OnlineId.OnlineIdAuthenticator;
            Object.defineProperty(this,"policyDomain",{value: extractPolicyDomain(this._config.getConfig().app.general.secureHost)});
            this._ticketRequest = options && options.ticketRequest || new OnlineId.OnlineIdServiceTicketRequest(this.policyDomain,"MBI_SSL");
            options = options || {};
            options.settingsKey = "authentication.liveid";
            options.silentSignInPromptType = OnlineId.CredentialPromptType.doNotPrompt;
            options.name = "LiveIdAuthenticator";
            options.frontDoorProviderName = "windows_live_id";
            BingApp.Classes.BaseAuthenticator.call(this,options)
        },{
            signIn: function()
            {
                BingApp.traceInfo("LiveIdAuthenticator.signIn: sign in started");
                return this._authenticate(OnlineId.CredentialPromptType.promptIfNeeded)
            },
            signOut: function()
            {
                BingApp.traceInfo("LiveIdAuthenticator.signOut: sign out started");
                var that = this;
                return this._signOutOfIFrames().then(function signOutViaProxyWithSIDComplete()
                    {
                        if(that._authenticator.canSignOut)
                            return that._authenticator.signOutUserAsync().then(function authenticatorSignOutUserAsyncComplete()
                                {
                                    BingApp.traceInfo("LiveIdAuthenticator.signOut: signOutUserAsync succeeded.")
                                },function authenticatorSignOutUserAsyncError(error)
                                {
                                    BingApp.traceError("LiveIdAuthenticator.signOut: signOutUserAsync failed, ignoring failure, priorIsAuthenticated: {0}, name: {1}, message: {2}.",that._isAuthenticated,error && error.name,error && error.message)
                                });
                        else
                            BingApp.traceInfo("LiveIdAuthenticator.signOut: user signed in to OS, skipping OnlineIdAuthenticator sign out")
                    }).then(function authenticatorSignOutUserAsyncCompleted()
                    {
                        that._signOutCompleted()
                    },function authenticatorSignOutUserAsyncError(error)
                    {
                        BingApp.traceError("LiveIdAuthenticator.signOut: sign out failed, priorIsAuthenticated: {0}, name: {1}, message: {2}.",that._isAuthenticated,error && error.name,error && error.message);
                        return WinJS.Promise.wrapError(error)
                    })
            },
            _authenticate: function(promptType)
            {
                BingApp.traceInfo("LiveIdAuthenticator._authenticate: sign in started with promptype: {0}",promptType);
                var that = this;
                return this._authenticator.authenticateUserAsync([this._ticketRequest],promptType).then(function authenticateUserAsyncCompleted(authResult)
                    {
                        if(authResult.tickets.size === 1 && authResult.tickets[0].errorCode === 0)
                        {
                            BingApp.traceInfo("LiveIdAuthenticator._authenticate: sign in completed");
                            var ticket = authResult.tickets[0].value;
                            if(ticket)
                            {
                                that._initialExpirySeconds = that._config.getConfig().authentication.liveId.tokenExpirySeconds;
                                return WinJS.Promise.wrap({
                                        firstName: authResult.firstName,
                                        safeCustomerId: authResult.safeCustomerId,
                                        signInName: authResult.signInName,
                                        token: ticket
                                    })
                            }
                            else
                            {
                                BingApp.traceError("LiveIdAuthenticator._authenticate: ticket not found: canSignOut: {0}, ticket: {1}.",that._authenticator.canSignOut,JSON.stringify(ticket));
                                return WinJS.Promise.wrapError(new WinJS.ErrorFromName("Live ID ticket not found"))
                            }
                        }
                        else
                        {
                            BingApp.traceError("LiveIdAuthenticator._authenticate: sign in failed, unexpected number of tickets: canSignOut: {0}, tickets: {1}.",that._authenticator.canSignOut,JSON.stringify(authResult.tickets));
                            return WinJS.Promise.wrapError(new WinJS.ErrorFromName("Live ID ticket number unexpected"))
                        }
                    },function authenticateUserAsyncFailed(error)
                    {
                        if(BingApp.Utilities.isPromiseCancellationError(error))
                            BingApp.traceInfo("LiveIdAuthenticator._authenticate: sign in canceled");
                        else
                            BingApp.traceError("LiveIdAuthenticator._authenticate: sign in failed unexpectedly, name: {0}, message: {1}",error && error.name,error && error.message);
                        return WinJS.Promise.wrapError(error)
                    }).then(function onValidAuthResult(authenticationData)
                    {
                        var parameters = {
                                action: "token",
                                provider: "windows_live_id",
                                token: authenticationData.token,
                                save_token: 0
                            };
                        return that._signInToIFrames(parameters,authenticationData)
                    })
            }
        });
    WinJS.Namespace.define("BingApp.Classes",{LiveIdAuthenticator: LiveIdAuthenticator})
})()