﻿"use strict";
(function backgroundImageLoaderModule()
{
    var currentSettings,
        currentSettingsSavedToDisk,
        ccToSaveToDisk,
        setLangToSaveToDisk,
        numImages,
        host,
        hpArchiveUrl,
        refreshInterval,
        maxHotspots = 4,
        tagImage = "image",
        tagUrlBase = "urlBase",
        tagCopyright = "copyright",
        tagCopyrightSource = "copyrightsource",
        tagDate = "enddate",
        tagHotspots = "hotspots",
        tagHotspot = "hotspot",
        tagDescription = "desc",
        tagLink = "link",
        tagQuery = "query",
        tagLocX = "LocX",
        tagLocY = "LocY",
        textContent = "textContent",
        tallImageSuffix = "_768x1366.jpg",
        wideImageSuffix = "_1366x768.jpg";
    var events = Object.freeze({
            downloadComplete: "hpdownloader:downloadcomplete",
            settingsLoaded: "hpdownloader:settingsLoaded"
        });
    var isRetryingDownload = false;
    var initialInterval = 1e3;
    var retryCountLimit = 6;
    var retryCount = 0;
    var onNetworkStatusChangedWrapper = null;
    function handleDownloadError(error)
    {
        function removeNetworkStatusChangedEventHandler()
        {
            isRetryingDownload = false;
            if(onNetworkStatusChangedWrapper)
            {
                BingApp.locator.eventRelay.removeEventListener(BingApp.Classes.NetworkDetectionService.networkStatusChangedEvent,onNetworkStatusChangedWrapper);
                onNetworkStatusChangedWrapper = null
            }
        }
        function retryDownload()
        {
            var nextTryInterval = initialInterval * Math.pow(2,retryCount);
            WinJS.Promise.timeout(nextTryInterval).then(function onDelayTimeoutComplete()
            {
                BingApp.traceVerbose("BingApp.BackgroundImageLoader.start: {0} try to download homepage data after {1} ms.",retryCount + 1,nextTryInterval);
                return startHelper(false)
            }).done(function onDownloadRetrySuccess()
            {
                removeNetworkStatusChangedEventHandler()
            },function onDownloadRetryError()
            {
                retryCount++;
                if(retryCount < retryCountLimit)
                    retryDownload();
                else
                    removeNetworkStatusChangedEventHandler()
            })
        }
        function onNetworkStatusChanged(eventData)
        {
            if(eventData.connectionState === BingApp.Classes.NetworkDetectionService.connectionStates.internetAccess)
                if(!isRetryingDownload)
                {
                    isRetryingDownload = true;
                    retryCount = 0;
                    retryDownload()
                }
        }
        if(!onNetworkStatusChangedWrapper)
        {
            onNetworkStatusChangedWrapper = onNetworkStatusChanged;
            BingApp.locator.eventRelay.addEventListener(BingApp.Classes.NetworkDetectionService.networkStatusChangedEvent,onNetworkStatusChangedWrapper)
        }
    }
    function downloadPoll()
    {
        if(BingApp.Utilities.isNullOrUndefined(refreshInterval))
            refreshInterval = 216e5;
        WinJS.Promise.timeout(refreshInterval).then(function onComplete()
        {
            startHelper(false).done(downloadPoll,handleDownloadError)
        })
    }
    function start()
    {
        var promise = startHelper(true);
        promise.done(downloadPoll,handleDownloadError);
        return promise
    }
    function logError(trace, errorToLog, onError)
    {
        var instObject = errorToLog;
        if(!instObject)
            instObject = {ei: trace};
        BingApp.Instrumentation.logError(instObject);
        BingApp.traceError(trace);
        if(typeof onError === "function")
            onError(new WinJS.ErrorFromName("BingApp.BackgroundImageLoader.Error",trace))
    }
    function startHelper(firstRun)
    {
        var imagesToFetch = null;
        var currentIndex = 0;
        return new WinJS.Promise(function init(complete, error)
            {
                function singleImageDownloadComplete()
                {
                    currentIndex++;
                    if(currentIndex < imagesToFetch.length)
                    {
                        var imageDownloadPromise = singleDayImageDownload(imagesToFetch[currentIndex]);
                        imageDownloadPromise.done(singleImageDownloadComplete,error)
                    }
                    else
                    {
                        var collection = {};
                        collection["cc"] = ccToSaveToDisk;
                        collection["setlang"] = setLangToSaveToDisk;
                        BingApp.HPDataProvider.setSettingsLangAndRegion(collection);
                        currentSettingsSavedToDisk = currentSettings;
                        BingApp.HPDataProvider.updateSettings(JSON.stringify(currentSettings)).done(function settingWrite()
                        {
                            BingApp.locator.eventRelay.fireEvent(events.downloadComplete);
                            complete()
                        },function settingWriteError()
                        {
                            logError("Write settings to disk failed",null,error)
                        })
                    }
                }
                var environment = BingApp.locator.env;
                environment.configurationReady().done(function onComplete()
                {
                    loadConfiguration(environment);
                    getCachedSettings(firstRun).done(function readComplete()
                    {
                        var serverMetaDataPromise = BingApp.HPDataProvider.getCurrentServerHPMetaData(hpArchiveUrl);
                        serverMetaDataPromise.done(function metaDataComplete(result)
                        {
                            imagesToFetch = parseMetaDataResponse(result);
                            if(imagesToFetch && imagesToFetch.length > 0)
                                singleDayImageDownload(imagesToFetch[currentIndex],currentSettings).done(singleImageDownloadComplete,error);
                            else if(BingApp.Utilities.isNullOrUndefined(imagesToFetch))
                            {
                                var hpErrorMessage = BingApp.Utilities.format("Bad metadata format for url: {0}",hpArchiveUrl);
                                var instObject = {
                                        url: hpArchiveUrl,
                                        ei: hpErrorMessage
                                    };
                                logError(hpErrorMessage,instObject,error)
                            }
                            else
                                complete()
                        },function metaDataError()
                        {
                            var hpErrorMessage = BingApp.Utilities.format("getCurrentServerHPMetaData failed for url: {0}",hpArchiveUrl);
                            var instObject = {
                                    url: hpArchiveUrl,
                                    ei: hpErrorMessage
                                };
                            logError(hpErrorMessage,instObject,error)
                        })
                    },function readCompleteError()
                    {
                        logError("getCachedSettings failed",null,error)
                    })
                },function configurationError()
                {
                    logError("configurationReady failed",null,error)
                })
            })
    }
    function loadConfiguration(environment)
    {
        host = BingApp.locator.config.getConfig().app.general.host;
        numImages = environment.configuration["numImages"];
        refreshInterval = environment.configuration["refreshInterval"];
        var collection = {};
        collection["format"] = "xml";
        collection["idx"] = "0";
        collection["n"] = numImages;
        collection["h"] = "1";
        ccToSaveToDisk = BingApp.HPDataProvider.getUserCC();
        setLangToSaveToDisk = BingApp.HPDataProvider.getUserLang();
        collection["cc"] = ccToSaveToDisk;
        collection["setlang"] = setLangToSaveToDisk;
        try
        {
            hpArchiveUrl = host + environment.configuration["HPArchiveUrl"] + BingApp.Utilities.QueryString.serialize(collection)
        }
        catch(error)
        {
            var traceMessage = BingApp.Utilities.format("BackgroundImageLoader.getArchiveUrl: query string serialization error for text: {0}; error name: {1}; error message: {2}",error.queryText,error.name,error.message);
            hpArchiveUrl = host + environment.configuration["DefaultHPArchiveUrl"];
            var instObject = {
                    url: hpArchiveUrl,
                    ei: traceMessage
                };
            logError(traceMessage,instObject,null)
        }
    }
    function getCachedSettings(firstRun)
    {
        return new WinJS.Promise(function init(complete, error)
            {
                function finishPromise()
                {
                    currentSettingsSavedToDisk = currentSettings;
                    BingApp.locator.eventRelay.fireEvent(events.settingsLoaded);
                    complete(currentSettings)
                }
                if(!firstRun)
                {
                    currentSettings = currentSettingsSavedToDisk;
                    complete(currentSettings)
                }
                else
                    BingApp.HPDataProvider.loadSettings().done(function settingsLoaded(cachedSettings)
                    {
                        try
                        {
                            if(cachedSettings)
                            {
                                currentSettings = JSON.parse(cachedSettings);
                                if(!BingApp.HPDataProvider.validateSettings(currentSettings,numImages))
                                    currentSettings = [];
                                else
                                {
                                    BingApp.HPDataProvider.doHPFilesExist(currentSettings).done(function HPFilesExist(isExist)
                                    {
                                        if(!isExist)
                                            currentSettings = [];
                                        else
                                        {
                                            var collection = BingApp.HPDataProvider.getSettingsLangAndRegion();
                                            if(collection)
                                            {
                                                var cc = collection["cc"];
                                                var setLang = collection["setlang"]
                                            }
                                            if(cc && cc !== BingApp.HPDataProvider.getUserCC() || setLang && setLang !== BingApp.HPDataProvider.getUserLang())
                                                currentSettings = []
                                        }
                                        finishPromise()
                                    },function doesFileExistError()
                                    {
                                        error(new WinJS.ErrorFromName("BingApp.BackgroundImageLoader.Error","BingApp.HPDataProvider.doHPFilesExist() failed"))
                                    });
                                    return
                                }
                            }
                            else
                                currentSettings = []
                        }
                        catch(error)
                        {
                            BingApp.traceError("BackgroundImageLoader.getCachedSettings: JSON parsing failed on string: {0}",error.message);
                            currentSettings = []
                        }
                        finishPromise()
                    },function settingsLoadedError()
                    {
                        error(new WinJS.ErrorFromName("BingApp.BackgroundImageLoader.Error","BingApp.HPDataProvider.loadSettings() failed"))
                    })
            })
    }
    function parseMetaDataResponse(result)
    {
        if(BingApp.Utilities.isNullOrUndefined(result) || BingApp.Utilities.isNullOrUndefined(result.responseXML))
            return;
        var xmlDoc = result.responseXML;
        if(BingApp.Utilities.isNullOrUndefined(xmlDoc))
            return;
        var imgNodes,
            hotspotsNodes,
            imageData = [],
            needToDownload = true;
        try
        {
            imgNodes = getNodes(xmlDoc,tagImage);
            hotspotsNodes = getNodes(xmlDoc,tagHotspots)
        }
        catch(error)
        {
            BingApp.traceError("BackgroundImageLoader.parseBgResponse: error in xml: {0}",error.message);
            return
        }
        var countImages = Math.min(numImages,imgNodes.length,hotspotsNodes.length);
        var imagesToDownload = [];
        for(var i = 0; i < countImages; i++)
        {
            needToDownload = true;
            try
            {
                imageData[i] = parseImageData(xmlDoc,i,imgNodes[i],hotspotsNodes[i])
            }
            catch(error)
            {
                BingApp.traceError("BackgroundImageLoader.parseBgResponse: error in parseImageData: {0} for image {1}",error.message,i);
                continue
            }
            for(var j = 0; !BingApp.Utilities.isNullOrUndefined(currentSettings) && j < currentSettings.length; j++)
                if(imageData[i].imgDate === currentSettings[j].imgDate)
                {
                    needToDownload = false;
                    break
                }
            if(needToDownload)
                imagesToDownload.push(imageData[i])
        }
        return imagesToDownload
    }
    function parseImageData(xmlDoc, imageIndex, imgNode, hotspotsNodes)
    {
        var imgUrl = getNodeText(imgNode,tagUrlBase),
            imgDate = getNodeText(imgNode,tagDate),
            attrib = getNodeText(imgNode,tagCopyrightSource);
        if(!attrib)
            attrib = getNodeText(imgNode,tagCopyright);
        var collection = BingApp.Utilities.QueryString.parse(imgUrl);
        var fileName = "";
        if(collection["p"])
            fileName = collection["p"].replace("rb%2f","");
        var backgroundObject = {
                hs: [],
                tallImgUrl: host + imgUrl + tallImageSuffix,
                wideImgUrl: host + imgUrl + wideImageSuffix,
                imgDate: imgDate,
                info: attrib,
                name: fileName
            };
        var hotspotNodes = getNodes(hotspotsNodes,tagHotspot);
        for(var i = 0; i < hotspotsNodes.childElementCount && i < maxHotspots; i++)
        {
            var hotspot = hotspotNodes[i];
            var hsObject = {};
            hsObject[tagLocX] = getNodeText(hotspot,tagLocX);
            hsObject[tagLocY] = getNodeText(hotspot,tagLocY);
            hsObject[tagDescription] = getNodeText(hotspot,tagDescription);
            hsObject[tagLink] = getNodeText(hotspot,tagLink);
            hsObject[tagQuery] = getNodeText(hotspot,tagQuery);
            backgroundObject.hs.push(hsObject)
        }
        return backgroundObject
    }
    function getNodes(doc, nodeName)
    {
        return doc.querySelectorAll(nodeName)
    }
    function getNodeText(nodes, nodeName)
    {
        var node = nodes.querySelectorAll(nodeName);
        if(node.length > 0 && node[0][textContent])
            return node[0][textContent];
        return null
    }
    function singleDayImageDownload(imageToFetch)
    {
        return new WinJS.Promise(function init(complete, error)
            {
                var downloadPromise = singleDayImageDownloadHelper(imageToFetch);
                downloadPromise.done(function downloadComplete(byteArrayCollection)
                {
                    if(BingApp.Utilities.isNullOrUndefined(currentSettings) || BingApp.Utilities.isNullOrUndefined(byteArrayCollection) || byteArrayCollection.length !== 2 || BingApp.Utilities.isNullOrUndefined(byteArrayCollection[0]) || BingApp.Utilities.isNullOrUndefined(byteArrayCollection[1]))
                    {
                        logError("invalid params in downloadComplete",null,error);
                        return
                    }
                    var index = getDiskIndex();
                    var portraitFileCreatePromise = BingApp.HPDataProvider.createHPFile("portrait" + index + ".jpg");
                    var landscapeFileCreatePromise = BingApp.HPDataProvider.createHPFile("landscape" + index + ".jpg");
                    var dataPromises = [];
                    dataPromises.push(portraitFileCreatePromise);
                    dataPromises.push(landscapeFileCreatePromise);
                    WinJS.Promise.join(dataPromises).done(function fileCreateComplete(fileHandle)
                    {
                        if(BingApp.Utilities.isNullOrUndefined(fileHandle) || fileHandle.length !== 2 || BingApp.Utilities.isNullOrUndefined(fileHandle[0]) || BingApp.Utilities.isNullOrUndefined(fileHandle[1]))
                        {
                            logError("invalid params for filehandle",null,error);
                            return
                        }
                        var dataPromisesAsyncWrite = [];
                        dataPromisesAsyncWrite.push(BingApp.HPDataProvider.writeHPFile(fileHandle[0],byteArrayCollection[0]));
                        dataPromisesAsyncWrite.push(BingApp.HPDataProvider.writeHPFile(fileHandle[1],byteArrayCollection[1]));
                        WinJS.Promise.join(dataPromisesAsyncWrite).done(function fileWriteComplete()
                        {
                            var metaDataOfDayAdded = {
                                    tallImgUrl: imageToFetch.tallImgUrl + tallImageSuffix,
                                    wideImgUrl: imageToFetch.wideImgUrl + wideImageSuffix,
                                    imgDate: imageToFetch.imgDate,
                                    info: imageToFetch.info,
                                    index: index,
                                    hs: imageToFetch.hs,
                                    name: imageToFetch.name
                                };
                            var insertIndex = -1;
                            var newImageDate = parseInt(imageToFetch.imgDate,10);
                            for(var i = 0; i < currentSettings.length; i++)
                                if(newImageDate > parseInt(currentSettings[i].imgDate,10))
                                {
                                    insertIndex = i;
                                    currentSettings.splice(insertIndex,0,metaDataOfDayAdded);
                                    break
                                }
                            if(insertIndex === -1)
                            {
                                currentSettings.push(metaDataOfDayAdded);
                                insertIndex = currentSettings.length - 1
                            }
                            complete()
                        },function joinErrorAsyncWrite()
                        {
                            logError("join error on async write",null,error)
                        })
                    },function joinErrorAsyncCreate()
                    {
                        logError("join error on async create",null,error)
                    })
                },function downloadCompleteError(errorObject)
                {
                    logError("download image failed",null,error)
                })
            })
    }
    function getDiskIndex()
    {
        var index = -1;
        var emptyElementsArray = new Array(numImages * 2);
        var i;
        for(i = 0; i < currentSettings.length; i++)
        {
            var currentIndex = currentSettings[i].index;
            if(currentIndex < numImages * 2)
                emptyElementsArray[currentIndex] = 1
        }
        for(i = 0; i < emptyElementsArray.length; i++)
            if(BingApp.Utilities.isNullOrUndefined(emptyElementsArray[i]))
            {
                index = i;
                break
            }
        for(; index === -1; )
            if(currentSettings.length > 0)
            {
                index = currentSettings[currentSettings.length - 1].index;
                currentSettings.pop();
                if(index >= numImages * 2)
                    index = -1
            }
            else
                index = 0;
        return index
    }
    function fetchSingleImage(url)
    {
        return BingApp.HPDataProvider.singleImageDownload(url).then(function ParseSuccessResponse(response)
            {
                var byteArray = new Uint8Array(response.response);
                return byteArray
            },function ParseErrorResponse(response)
            {
                var errorMessage = BingApp.Utilities.format("fetchSingleImage failed for url:{0}",url);
                var instObject = {
                        url: url,
                        ei: errorMessage
                    };
                logError(errorMessage,instObject)
            })
    }
    function singleDayImageDownloadHelper(imageToFetch)
    {
        var dataPromises = [];
        var portraitImagePromise = fetchSingleImage(imageToFetch.tallImgUrl);
        var landscapeImagePromise = fetchSingleImage(imageToFetch.wideImgUrl);
        dataPromises.push(portraitImagePromise);
        dataPromises.push(landscapeImagePromise);
        return WinJS.Promise.join(dataPromises)
    }
    function getSettings()
    {
        return currentSettingsSavedToDisk
    }
    WinJS.Namespace.define("BingApp.BackgroundImageLoader",{
        start: start,
        getSettings: getSettings,
        events: events
    })
})();
(function homePageSettingsModule()
{
    var settingsFilename = "hpsettings.js";
    function singleImageDownload(url)
    {
        return WinJS.xhr({
                url: url,
                responseType: "arraybuffer"
            })
    }
    function getCurrentServerHPMetaData(url)
    {
        return WinJS.xhr({url: url})
    }
    function createHPFile(filename)
    {
        var localFolder = Windows.Storage.ApplicationData.current.localFolder;
        return localFolder.createFileAsync(filename,Windows.Storage.CreationCollisionOption.replaceExisting)
    }
    function writeHPFile(hpFileHandle, byteArray)
    {
        return Windows.Storage.FileIO.writeBytesAsync(hpFileHandle,byteArray)
    }
    function updateSettings(settings)
    {
        return new WinJS.Promise(function init(complete, error)
            {
                getSettingsFileHandle().done(function fileHandleComplete(settingsFileHandle)
                {
                    Windows.Storage.FileIO.writeTextAsync(settingsFileHandle,settings).done(function writeTextComplete()
                    {
                        complete()
                    },function writeTextError()
                    {
                        error(new WinJS.ErrorFromName("BingApp.BackgroundImageLoader.Error","writeTextAsync() failed"))
                    })
                },function fileHandleError()
                {
                    error(new WinJS.ErrorFromName("BingApp.BackgroundImageLoader.Error","getSettingsFileHandle() failed"))
                })
            })
    }
    function loadSettings()
    {
        return new WinJS.Promise(function init(complete, error)
            {
                getSettingsFileHandle().done(function fileHandleComplete(settingsFileHandle)
                {
                    Windows.Storage.FileIO.readTextAsync(settingsFileHandle).done(function readTextComplete(text)
                    {
                        complete(text)
                    },function readTextError()
                    {
                        error(new WinJS.ErrorFromName("BingApp.BackgroundImageLoader.Error","readTextAsync() failed"))
                    })
                },function fileHandleError()
                {
                    error(new WinJS.ErrorFromName("BingApp.BackgroundImageLoader.Error","getSettingsFileHandle() failed"))
                })
            })
    }
    function getSettingsFileHandle()
    {
        var localFolder = Windows.Storage.ApplicationData.current.localFolder;
        return localFolder.createFileAsync(settingsFilename,Windows.Storage.CreationCollisionOption.openIfExists)
    }
    function getSettingsLangAndRegion()
    {
        var collection = {};
        var marketSettings = Windows.Storage.ApplicationData.current.localSettings.values["backgroundimageloader.marketsettings"];
        if(marketSettings)
            try
            {
                collection = BingApp.Utilities.QueryString.parse(marketSettings,{decode: true})
            }
            catch(e)
            {
                BingApp.traceError("getSettingsLangAndRegion failed on string parsing for: {0}",marketSettings)
            }
        return collection
    }
    function setSettingsLangAndRegion(collection)
    {
        Windows.Storage.ApplicationData.current.localSettings.values["backgroundimageloader.marketsettings"] = BingApp.Utilities.QueryString.serialize(collection)
    }
    function getUserLang()
    {
        return Windows.System.UserProfile.GlobalizationPreferences.languages[0]
    }
    function getUserCC()
    {
        return Windows.System.UserProfile.GlobalizationPreferences.homeGeographicRegion
    }
    function doHPFilesExist(settings)
    {
        var localFolder = Windows.Storage.ApplicationData.current.localFolder;
        var fileQueryResults = localFolder.createFileQuery();
        return fileQueryResults.getFilesAsync().then(function filesAsyncComplete(files)
            {
                var collection = {};
                files.forEach(function addFilenameToCollection(file)
                {
                    var filename = file.name.toLowerCase();
                    if(filename.indexOf(".jpg",filename.length - 4) !== -1)
                        collection[filename] = "1"
                });
                var isExist = true;
                for(var i = 0; i < settings.length; i++)
                {
                    var curObject = settings[i];
                    if(!collection["portrait" + curObject.index + ".jpg"] || !collection["landscape" + curObject.index + ".jpg"])
                    {
                        BingApp.traceError("BackgroundImageLoader.doHPFilesExist: file at index: {0} missing",i);
                        isExist = false;
                        break
                    }
                }
                return isExist
            },function getFilesAsyncError()
            {
                throw new WinJS.ErrorFromName("BingApp.BackgroundImageLoader.Error","BingApp.HPDataProvider.getFilesAsync() failed");
            })
    }
    function validateSettings(settings, imageCount)
    {
        var settingsValid = false;
        if(settings.length <= 2 * imageCount)
        {
            var invalidObjectPresent = false;
            for(var i = 0; i < settings.length; i++)
            {
                var curObject = settings[i];
                var isValid = curObject && curObject.hasOwnProperty("index") && curObject.hs && curObject.imgDate && curObject.info && curObject.tallImgUrl && curObject.wideImgUrl;
                if(!isValid)
                {
                    invalidObjectPresent = true;
                    break
                }
            }
            if(!invalidObjectPresent)
                settingsValid = true
        }
        return settingsValid
    }
    WinJS.Namespace.define("BingApp.HPDataProvider",{
        singleImageDownload: singleImageDownload,
        getCurrentServerHPMetaData: getCurrentServerHPMetaData,
        updateSettings: updateSettings,
        loadSettings: loadSettings,
        createHPFile: createHPFile,
        writeHPFile: writeHPFile,
        doHPFilesExist: doHPFilesExist,
        getSettingsLangAndRegion: getSettingsLangAndRegion,
        setSettingsLangAndRegion: setSettingsLangAndRegion,
        getUserCC: getUserCC,
        getUserLang: getUserLang,
        validateSettings: validateSettings
    })
})();
(function homePageDataModule()
{
    var fileNamePrefix = "Bing";
    var fileNameExtension = ".jpg";
    var defaultImage = Object.freeze({
            landscape: "ms-appx:///shell/images/defaultBgWide.jpg",
            portrait: "ms-appx:///shell/images/defaultBgTall.jpg"
        });
    function getDefaultImageUri(isLandscapeMode)
    {
        var imagePath;
        if(isLandscapeMode)
            imagePath = defaultImage.landscape;
        else
            imagePath = defaultImage.portrait;
        return imagePath
    }
    function getImageUri(isLandscapeMode, imageIndex)
    {
        if(BingApp.Utilities.isNullOrUndefined(imageIndex))
            imageIndex = BingApp.HomePageData.currentImageIndex;
        var cachedSettings = BingApp.BackgroundImageLoader.getSettings();
        var imagePath = "ms-appdata:///local/";
        if(isLandscapeMode)
            imagePath += "landscape";
        else
            imagePath += "portrait";
        if(cachedSettings && cachedSettings.length > 0)
        {
            imagePath += cachedSettings[imageIndex].index + ".jpg";
            return imagePath
        }
        else
            return getDefaultImageUri(isLandscapeMode)
    }
    function getCurrentImageInfo()
    {
        var cachedSettings = BingApp.BackgroundImageLoader.getSettings();
        if(cachedSettings && cachedSettings.hasOwnProperty(BingApp.HomePageData.currentImageIndex))
            return cachedSettings[BingApp.HomePageData.currentImageIndex].info;
        else
            return""
    }
    function getCurrentImageFileName()
    {
        var cachedSettings = BingApp.BackgroundImageLoader.getSettings();
        if(cachedSettings && cachedSettings.hasOwnProperty(BingApp.HomePageData.currentImageIndex))
        {
            var filename = fileNamePrefix;
            if(cachedSettings[BingApp.HomePageData.currentImageIndex].name)
                filename += "-" + cachedSettings[BingApp.HomePageData.currentImageIndex].name;
            return filename
        }
        else
            return fileNamePrefix + fileNameExtension
    }
    function getCurrentHotspotsData()
    {
        var cachedSettings = BingApp.BackgroundImageLoader.getSettings();
        if(cachedSettings && cachedSettings.hasOwnProperty(BingApp.HomePageData.currentImageIndex))
            return cachedSettings[BingApp.HomePageData.currentImageIndex].hs;
        else
            return null
    }
    WinJS.Namespace.define("BingApp.HomePageData",{
        getDefaultImageUri: getDefaultImageUri,
        getImageUri: getImageUri,
        getCurrentImageInfo: getCurrentImageInfo,
        getCurrentImageFileName: getCurrentImageFileName,
        getCurrentHotspotsData: getCurrentHotspotsData,
        currentImageIndex: 0,
        defaultImage: defaultImage
    })
})()