﻿(function configurationModule()
{
    "use strict";
    var commandHRef = "/shell/html/configuration.html";
    var configurationDiv = "configSettingsDiv";
    var configurationDivSelector = "#" + configurationDiv;
    var environmentButtonSelector = configurationDivSelector + " .env button";
    var queryStringButtonSelector = configurationDivSelector + " .querystring button";
    var optionsButtonSelector = configurationDivSelector + " .options button";
    var environmentInUseSelector = "#envInUse";
    var parametersInUseSelector = "#paramsInUse";
    var localSettings = Windows.Storage.ApplicationData.current.localSettings;
    function configurationSettingsHandler()
    {
        var i;
        var length;
        var environmentButtons = document.querySelectorAll(environmentButtonSelector);
        length = environmentButtons.length;
        for(i = 0; i < length; i++)
            environmentButtons[i].addEventListener("MSPointerDown",environmentButtonHandler,false);
        var queryStringButtons = document.querySelectorAll(queryStringButtonSelector);
        length = queryStringButtons.length;
        for(i = 0; i < length; i++)
            queryStringButtons[i].addEventListener("MSPointerDown",queryStringButtonHandler,false);
        var optionsButtons = document.querySelectorAll(optionsButtonSelector);
        length = optionsButtons.length;
        for(i = 0; i < length; i++)
            optionsButtons[i].addEventListener("MSPointerDown",optionsButtonHandler,false);
        showEnvironment();
        showRequestParameters()
    }
    function showEnvironment()
    {
        var environmentLabel = document.querySelector(environmentInUseSelector);
        if(environmentLabel)
            environmentLabel.innerText = BingApp.locator.config.getConfig().app.general.host
    }
    function showRequestParameters()
    {
        var queryParametersLabel = document.querySelector(parametersInUseSelector);
        if(queryParametersLabel)
        {
            var queryParametersAsText;
            try
            {
                queryParametersAsText = JSON.stringify(BingApp.locator.env.requestParameters)
            }
            catch(e)
            {
                BingApp.traceError("ConfigurationSettings.showRequestParameters: failed to convert request parameters to string for visualization purposes: '{0}'.",e.message);
                queryParametersAsText = "Failed to stringify"
            }
            queryParametersLabel.innerText = queryParametersAsText
        }
    }
    function refreshView()
    {
        BingApp.locator.navigationManager.refresh();
        var mainView = BingApp.locator.viewManager.findLoadedView("mainview");
        if(mainView)
            mainView.getController().getSearchBar().refresh()
    }
    function environmentButtonHandler(eventData)
    {
        var environmentButton = eventData.target;
        var environment = environmentButton.getAttribute("data-env");
        var environmentRef = environmentButton.getAttribute("data-envref");
        if(environmentRef)
        {
            environment = document.querySelector(environmentRef);
            environment = environment && environment.value
        }
        if(environment && environment.indexOf("http://") !== 0)
            environment = "http://" + environment;
        try
        {
            var uri = new Windows.Foundation.Uri(environment);
            BingApp.locator.config.setOverrideSetting("app","general","host",environment);
            var secureEnvironment = environment.replace("www." + uri.domain,"ssl." + uri.domain).replace("http://","https://");
            BingApp.locator.config.setOverrideSetting("app","general","secureHost",secureEnvironment)
        }
        catch(ex)
        {
            BingApp.traceError("ConfigurationSettings.environmentButtonHandler: invalid environment passed: {0}.",environment)
        }
        showEnvironment();
        refreshView()
    }
    function queryStringButtonHandler(eventData)
    {
        var parameterButton = eventData.target;
        var parameterName = parameterButton.getAttribute("data-paramname");
        var parameterValue = parameterButton.getAttribute("data-paramvalue");
        var parameterNameRef = parameterButton.getAttribute("data-paramnameref");
        var parameterValueRef = parameterButton.getAttribute("data-paramvalueref");
        if(parameterNameRef)
        {
            parameterName = document.querySelector(parameterNameRef);
            parameterName = parameterName && parameterName.value
        }
        if(parameterValueRef)
        {
            parameterValue = document.querySelector(parameterValueRef);
            parameterValue = parameterValue && parameterValue.value
        }
        BingApp.locator.env.setRequestParameter(parameterName,parameterValue);
        showRequestParameters();
        refreshView()
    }
    function optionsButtonHandler(eventData)
    {
        var parameterButton = eventData.target;
        var parameterName = parameterButton.getAttribute("data-paramname");
        var parameterValue = parameterButton.getAttribute("data-paramvalue");
        switch(parameterName)
        {
            case"perf":
                BingApp.locator.env.setRequestParameter(parameterName,parameterValue);
                break;
            case"uncrunched":
                BingApp.locator.env.setRequestParameter(parameterName,parameterValue);
                break;
            case"reset":
                BingApp.locator.env.reset();
                BingApp.locator.config.clearOverrideSetting("app","general","host");
                BingApp.locator.config.clearOverrideSetting("app","general","secureHost");
                break;
            case"addloginsource":
                BingApp.locator.env.setRequestParameter(parameterName,parameterValue);
                break
        }
        showEnvironment();
        showRequestParameters();
        refreshView()
    }
    WinJS.UI.Pages.define(commandHRef,{ready: configurationSettingsHandler})
})()