﻿(function envModule()
{
    "use strict";
    var storageKey = "BingApp.Classes.Env";
    var cachedCCKey = "cc";
    var oemPartnerCode = null;
    var touchCapabilities = null;
    function constructor(options)
    {
        var that = this;
        touchCapabilities = options && options.touchCapabilities || new Windows.Devices.Input.TouchCapabilities;
        this._config = options && options.config || BingApp.locator.config;
        var cc = Windows.System.UserProfile.GlobalizationPreferences.homeGeographicRegion;
        var localSettings = Windows.Storage.ApplicationData.current.localSettings;
        var cachedCC = localSettings.values[cachedCCKey];
        this.requestParameters = {};
        if(cachedCC && cachedCC === cc)
        {
            var storageValue = null;
            try
            {
                storageValue = window.localStorage[storageKey]
            }
            catch(error)
            {
                BingApp.traceError("Env.ctor: failed to parse env data stored locally with error: '{0}'. Revert to using default values.",error.message)
            }
            if(storageValue)
            {
                try
                {
                    storageValue = JSON.parse(storageValue)
                }
                catch(error)
                {
                    BingApp.traceError("Env.ctor: failed to parse env data stored locally with error: '{0}'. Revert to using default values.",error.message);
                    storageValue = null
                }
                if(storageValue)
                    if(storageValue.requestParameters)
                        this.requestParameters = storageValue.requestParameters
            }
        }
        this.requestParameters.cc = cc;
        this.requestParameters.setlang = Windows.System.UserProfile.GlobalizationPreferences.languages[0];
        localSettings.values[cachedCCKey] = cc;
        this.configuration = {};
        var loadTestSettings = new BingApp.Classes.LoadJsonOperation;
        this._loadTestSettingsPromise = loadTestSettings.run({fileLocation: "/shell/configuration/env.configuration.json"}).then(function onEnvironmentConfigurationLoadComplete(configuration)
        {
            that.configuration = configuration
        });
        getOEMPartnerCode().done(function onGetOEMPartnerCodeComplete(code)
        {
            oemPartnerCode = code
        },function onGetOEMPartnerCodeError(err)
        {
            var message = err && err.message;
            BingApp.traceError("Env.ctor: failed to get OEM Partner code. Message: {0}.",message)
        })
    }
    function save(e)
    {
        if(e)
            try
            {
                window.localStorage[storageKey] = JSON.stringify({requestParameters: e.requestParameters})
            }
            catch(error)
            {
                BingApp.traceError("Env.save: failed to save data stored locally with error: '{0}'.",error.message)
            }
    }
    function reset()
    {
        this.requestParameters = {};
        BingApp.traceInfo("Reset the env, requestParamaters cleared");
        this.save(this)
    }
    function setRequestParameter(key, value)
    {
        if(key)
        {
            if(value)
            {
                this.requestParameters[key] = value;
                BingApp.traceInfo("Env.requestParameters['" + key + "'] set to " + value)
            }
            else
            {
                delete this.requestParameters[key];
                BingApp.traceInfo("Env.requestParameters['" + key + "'] was deleted")
            }
            this.save(this)
        }
    }
    function getQueryString()
    {
        var parameters = [];
        var that = this;
        Object.keys(this.requestParameters).forEach(function addParameter(key)
        {
            parameters.push(key + "=" + that.requestParameters[key])
        });
        return parameters.join("&")
    }
    function getIFrameParams(defaults, windowObject)
    {
        windowObject = windowObject || window;
        var collection = defaults || {};
        var height = 0;
        var width = 0;
        var deviceDPI;
        var body = windowObject && windowObject.document && windowObject.document.body;
        var screenInfo = windowObject && windowObject.screen;
        var addLoginSource = this._config.getConfig().instrumentation.general.addLoginSource;
        if(body)
        {
            height = body.offsetHeight;
            width = body.offsetWidth
        }
        if(screenInfo && screenInfo.deviceXDPI && screenInfo.deviceYDPI)
            deviceDPI = screenInfo.deviceXDPI + "," + screenInfo.deviceYDPI;
        collection["w8-height"] = height;
        collection["w8-width"] = width;
        collection["w8-version"] = BingApp.Classes.Shell.fullVersion;
        collection["bingapp-iv"] = BingApp.Instrumentation.Version.versionInfo;
        collection["bingapp-t"] = touchCapabilities.touchPresent ? 1 : 0;
        if(deviceDPI)
            collection["w8-devicedpi"] = deviceDPI;
        if(oemPartnerCode)
            collection["PC"] = oemPartnerCode;
        if(addLoginSource)
            collection["addloginsource"] = 1;
        if(this._config.getConfig().settingsPane.testConfiguration.enabled)
            if(collection["form"])
                collection["form"] = "MONITR";
            else
                collection["FORM"] = "MONITR";
        return collection
    }
    function getQueryStringWithIFrameParams(query, windowObject)
    {
        if(query)
            query = query + "&";
        else
            query = "?";
        query = query + this.getQueryString();
        var collection = BingApp.Utilities.QueryString.parse(query,{decode: true});
        var overrides = this.getIFrameParams(collection,windowObject);
        return BingApp.Utilities.QueryString.serialize(overrides)
    }
    function sanitizeQueryString(query)
    {
        var collection;
        try
        {
            collection = BingApp.Utilities.QueryString.parse(query,{decode: true})
        }
        catch(e)
        {
            BingApp.traceError("Env.sanitizeQueryString: failed to parse query '{0}' with error '{1}'. Returning empty query back.",query,e.message);
            return BingApp.Utilities.QueryString.serialize({})
        }
        Object.keys(this.requestParameters).forEach(function deleteParameter(key)
        {
            delete collection[key]
        });
        ["w8-height","w8-width","w8-version","w8-devicedpi","bingapp-iv","bingapp-t"].forEach(function deleteParameter(key)
        {
            delete collection[key]
        });
        return BingApp.Utilities.QueryString.serialize(collection)
    }
    function configurationReady()
    {
        return this._loadTestSettingsPromise
    }
    function getOEMPartnerCode()
    {
        if(oemPartnerCode)
            return WinJS.Promise.as(oemPartnerCode);
        return new WinJS.Promise(function init(complete, error)
            {
                Windows.ApplicationModel.Package.current.installedLocation.getFileAsync("custom.data").done(function onGetFileComplete(file)
                {
                    file.openAsync(Windows.Storage.FileAccessMode.read).then(function onFileOpenComplete(readStream)
                    {
                        try
                        {
                            var reader = new Windows.Storage.Streams.DataReader(readStream);
                            reader.loadAsync(readStream.size).done(function onLoadAsyncComplete(numBytesLoaded)
                            {
                                var fileContent = reader.readString(numBytesLoaded);
                                fileContent = BingApp.Utilities.trim(fileContent);
                                var validCode = /[A-Z]{1}[A-Z0-9]{1,10}$/m.test(fileContent);
                                if(validCode && fileContent.length <= 10)
                                    complete(fileContent);
                                else
                                {
                                    BingApp.traceError("env.getOEMPartnerCode invalid partner Code : {0} return {1} ",fileContent,validCode);
                                    error(new WinJS.ErrorFromName("BingApp.locator.Env","Invalid OEM code"))
                                }
                            },function onLoadError(err)
                            {
                                var message = err && err.message;
                                BingApp.traceError("env.getOEMPartnerCode failed to load : {0}",message);
                                error(new WinJS.ErrorFromName("BingApp.locator.Env","Invalid OEM code"))
                            })
                        }
                        catch(e)
                        {
                            BingApp.traceError("env.getOEMPartnerCode failed to load : {0}",e.message);
                            error(new WinJS.ErrorFromName("BingApp.locator.Env","Invalid OEM code"))
                        }
                    })
                },function onGetFileError(err)
                {
                    var message = err && err.message;
                    BingApp.traceError("env.getOEMPartnerCode failure : err {0}",message);
                    error(err)
                })
            })
    }
    var instanceMembers = {
            requestParameters: null,
            getIFrameParams: getIFrameParams,
            getQueryString: getQueryString,
            getQueryStringWithIFrameParams: getQueryStringWithIFrameParams,
            sanitizeQueryString: sanitizeQueryString,
            setRequestParameter: setRequestParameter,
            save: save,
            reset: reset,
            configuration: null,
            configurationReady: configurationReady,
            getOEMPartnerCode: getOEMPartnerCode,
            _loadTestSettingsPromise: null
        };
    var staticMembers = {};
    WinJS.Namespace.define("BingApp.Classes",{Env: WinJS.Class.define(constructor,instanceMembers,staticMembers)})
})()