﻿(function globalEventRelayModule()
{
    "use strict";
    var events = Object.freeze({
            subscribe: "eventrelay:subscribe",
            unsubscribe: "eventrelay:unsubscribe"
        });
    function handleIframeSubscribe(args, source)
    {
        if(!args)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("args");
        var eventName = args.eventName;
        if(BingApp.Utilities.isNullOrUndefined(eventName))
            throw new BingApp.Classes.ErrorArgument("args","Event arguments object must have 'eventName' property.");
        if(BingApp.Utilities.isNullOrUndefined(source))
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("source");
        this.addEventListener(eventName,{
            iframeId: source,
            source: args.source
        })
    }
    function handleIframeUnsubscribe(args, source)
    {
        if(!args)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("args");
        var eventName = args.eventName;
        if(BingApp.Utilities.isNullOrUndefined(eventName))
            throw new BingApp.Classes.ErrorArgument("args","Event arguments object must have 'eventName' property.");
        if(BingApp.Utilities.isNullOrUndefined(source))
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("source");
        this.removeEventListener(eventName,{
            iframeId: source,
            source: args.source
        })
    }
    var GlobalEventRelay = WinJS.Class.derive(BingApp.Classes.EventRelay,function constructor(windowObject)
        {
            if(!(this instanceof BingApp.Classes.GlobalEventRelay))
            {
                BingApp.traceWarning("GlobalEventRelay.ctor: Attempted using GlobalEventRelay ctor as function; redirecting to use 'new GlobalEventRelay()'.");
                return new BingApp.Classes.GlobalEventRelay(windowObject)
            }
            if(!windowObject)
                throw new BingApp.Classes.ErrorArgumentNullOrUndefined("windowObject");
            BingApp.Classes.EventRelay.call(this);
            var that = this;
            windowObject.addEventListener("message",function onWindowMessage(eventData)
            {
                var message;
                try
                {
                    message = JSON.parse(eventData.data)
                }
                catch(error)
                {
                    BingApp.traceError("GlobalEventRelay.onwindowmessage: error while parsing message data: '{0}'. Error message: '{1}'.",eventData.data,error.message);
                    return
                }
                if(!message)
                    BingApp.traceError("GlobalEventRelay.onwindowmessage: message was parsed into null object.");
                else if(BingApp.Utilities.isNullOrUndefined(message.name))
                    BingApp.traceError("GlobalEventRelay.onwindowmessage: message does not have 'name' member: '{0}'.",eventData.data);
                else
                {
                    BingApp.traceInfo("GlobalEventRelay.onwindowmessage: relayed from web to app: '{0}'.",eventData.data);
                    that.fireEvent(message.name,message.value,message.from)
                }
            },false);
            this.addEventListener(events.subscribe,function onIframeSubscribe(args, source)
            {
                handleIframeSubscribe.call(that,args,source)
            });
            this.addEventListener(events.unsubscribe,function onIframeUnsubscribe(args, source)
            {
                handleIframeUnsubscribe.call(that,args,source)
            })
        },{},{events: events});
    WinJS.Namespace.define("BingApp.Classes",{GlobalEventRelay: GlobalEventRelay})
})()