﻿(function messagebarModule()
{
    WinJS.Namespace.define("CommonJS",{
        _messageBar: null,
        _elementIds: 0,
        MessageBarLevelError: "MessageBarLevelError",
        MessageBarLevelWarning: "MessageBarLevelWarning",
        MessageBar: WinJS.Class.define(function constructor(message, options)
        {
            if(CommonJS._messageBar)
                CommonJS._messageBar._destroy();
            CommonJS._messageBar = this;
            this._domElement = document.createElement("div");
            this._domElement.setAttribute("aria-hidden",true);
            this._message = message;
            this._buttons = [];
            this._autoHide = false;
            this._autoHideTimeout = 3e3;
            this._level = CommonJS.MessageBarLevelWarning;
            this.isSticky = false;
            if(options)
                this._setOptions(options);
            this._init(this._domElement)
        },{
            _domElement: null,
            isSticky: false,
            message: {
                get: function()
                {
                    return this._message
                },
                set: function(value)
                {
                    this._message = value;
                    this._init(this._domElement)
                }
            },
            show: function()
            {
                var that = this;
                if(this._autoHide)
                    setTimeout(function messageBar_autoHideTimeout()
                    {
                        that.hide()
                    },this._autoHideTimeout);
                this._domElement.setAttribute("role","alert");
                this._domElement.setAttribute("aria-hidden",false);
                if(this.hideOnSnap)
                    WinJS.Utilities.addClass(this._domElement,"platformHideOnSnap");
                msSetImmediate(function messageBar_setVisibleClass()
                {
                    WinJS.Utilities.addClass(that._domElement,"platformMessageBarVisible")
                })
            },
            hide: function()
            {
                this._domElement.addEventListener("MSTransitionEnd",this._destroy);
                WinJS.Utilities.removeClass(this._domElement,"platformMessageBarVisible");
                this._domElement.setAttribute("aria-hidden",true)
            },
            addButton: function(label, callback)
            {
                if(this._buttons.length < 2)
                {
                    var button = document.createElement("button");
                    button.innerText = label;
                    button.onclick = callback;
                    WinJS.Utilities.addClass(button,"fontMedium fontWeightSemiBold");
                    this._buttons.push(button);
                    this._domElement.appendChild(button);
                    this._setAutomationId(button,this._domElement,label)
                }
                else
                    BingApp.traceError("The MessageBar can only contain up to 2 buttons")
            },
            _init: function()
            {
                var messageDiv = document.createElement("div");
                messageDiv.innerText = this.message;
                WinJS.Utilities.addClass(this._domElement,"platformMessageBar");
                WinJS.Utilities.addClass(messageDiv,"platformMessageBarText fontMedium fontWeightSemiBold");
                if(this._level === CommonJS.MessageBarLevelError)
                    WinJS.Utilities.addClass(this._domElement,"platformMessageBarError");
                this._domElement.appendChild(messageDiv);
                document.body.appendChild(this._domElement);
                this._setAutomationId(this._domElement);
                if(!this.isSticky)
                    WinJS.Navigation.addEventListener("beforenavigate",this._destroy)
            },
            _setOptions: function(options)
            {
                if(options["autoHide"])
                    this._autoHide = options["autoHide"];
                if(options["level"])
                    this._level = options["level"];
                if(options["isSticky"])
                    this.isSticky = options["isSticky"]
            },
            _destroy: function()
            {
                if(this._domElement)
                {
                    this._domElement.removeEventListener("MSTransitionEnd",this._destroy);
                    WinJS.Navigation.removeEventListener("beforenavigate",this._destroy);
                    this._domElement.outerHTML = ""
                }
                CommonJS._messageBar = null
            },
            _setAutomationId: function(domElement, parentElement, suffix, id)
            {
                if(id && id !== "")
                    domElement.id = id;
                else if(!domElement.id && !parentElement && !suffix)
                {
                    CommonJS._elementIds++;
                    domElement.id = "platformUAId" + CommonJS._elementIds
                }
                else if(parentElement && suffix)
                    domElement.id = parentElement.id + "_" + suffix
            }
        })
    })
})()