﻿(function nativeInstrumentationModule()
{
    "use strict";
    var environment = BingApp.locator.env;
    var muid = null;
    var cookieName = "MUID";
    var proxy = BingApp.locator.proxy;
    var lastInstrumentationEventTS = null;
    var activationFormCodeSent = false;
    var oemId = null;
    var checkedOemId = false;
    var errorQueue = [];
    var flushPending = false;
    var errorQueueLoaded = false;
    var errorQueueStorageKey = "BingApp.Instrumentation.errorQueue";
    var appNS = "BingApp";
    var pageLayoutHandlers = [];
    var navigationManager = BingApp.locator.navigationManager;
    var pageIGs = [];
    var patternsPage = [{
                pattern: "//app/home",
                pageName: "start"
            },{
                pattern: "//app/start",
                pageName: "trends"
            },{
                pattern: "//app/search",
                pageName: "serp"
            },{
                pattern: "//app/images/grid",
                pageName: "images"
            },{
                pattern: "//app/rewards",
                pageName: "rewards"
            },{
                pattern: "//app/videos/grid",
                pageName: "videos"
            }];
    var state = Object.freeze({
            ready: "ready",
            requestPending: "requestPending",
            error: "error",
            expired: "expired",
            requireRefresh: "requireRefresh",
            timeout: "timeout"
        });
    var kValueConstants = Object.freeze({
            kValueLogo: 7e3,
            kValueBack: 7005,
            kValueSearch: 7010,
            kValueVoice: 7015
        });
    var regexps = patternsPage.map(function patternsPageMap(p)
        {
            return new RegExp(p.pattern,"g")
        });
    var ts = BingApp.Utilities.getTimeStamp;
    function logClick(data)
    {
        log(data,BingApp.Instrumentation.clickEvent)
    }
    function logError(data, hint)
    {
        if(BingApp.locator.networkDetectionService.isConnected() && !hint)
            log(data,BingApp.Instrumentation.errorEvent);
        else
        {
            var xlsPayLoad;
            var igInfo = _getCurrentIGInfo();
            if(igInfo)
                xlsPayLoad = _createErrorXMLPayload(data,igInfo.ig,igInfo.pageName);
            else
                xlsPayLoad = _createErrorXMLPayload(data,null,_getPageName());
            _queueError(xlsPayLoad)
        }
    }
    function log(data, type, callback)
    {
        if(!data)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("data");
        if(!type)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("type");
        if(_excludeEvent(type))
            return;
        _initializeMuid().done(function onInitializeMuidComplete(muidArg)
        {
            _onReady().done(function onImpressionGUIDComplete(igInfo)
            {
                var xlsPayLoad = null;
                if(type === BingApp.Instrumentation.errorEvent)
                    xlsPayLoad = _createErrorXMLPayload(data,igInfo.ig,igInfo.pageName);
                else if(data.layout && data.name && data.pageLayout && data.pageDs)
                {
                    xlsPayLoad = _getLayoutXML(type,igInfo.ig,data);
                    var igCurrent = _getCurrentIGInfo();
                    if(igCurrent && (igCurrent.layoutSent || igCurrent.layoutPending))
                    {
                        BingApp.traceInfo("BingApp.Instrumentation.log : layout sent");
                        return
                    }
                    igCurrent.layoutPending = true
                }
                else
                    xlsPayLoad = _createXMLPayLoad(data,type,igInfo.ig,igInfo.pageName);
                if(!xlsPayLoad)
                {
                    BingApp.traceError("BingApp.Instrumentation.log failed: _createXMLPayLoad");
                    return
                }
                if(type === BingApp.Instrumentation.errorEvent)
                {
                    _queueError(xlsPayLoad);
                    flushErrorQueue()
                }
                else
                    _sendXlsPayload(xlsPayLoad,callback)
            },function onImpressionGUIDError(err)
            {
                if(type === BingApp.Instrumentation.errorEvent)
                {
                    var xlsPayLoad = _createErrorXMLPayload(data,null,_getPageName());
                    _queueError(xlsPayLoad)
                }
                else
                    BingApp.traceError("BingApp.Instrumentation.log failed: {0}.",err && err.message)
            })
        },function onInitializeMuidError(error)
        {
            BingApp.traceError("BingApp.Instrumentation.log failed: {0}.",error && error.message)
        })
    }
    function resumeLogging()
    {
        var currentTS = ts();
        var refreshInterval = environment.configuration["resumeRefreshInterval"];
        if(!lastInstrumentationEventTS || currentTS - lastInstrumentationEventTS > refreshInterval)
            pageIGs = []
    }
    function flushErrorQueue()
    {
        if(!errorQueueLoaded)
            _loadErrorQueue();
        if(flushPending)
            return;
        _sendErrorQueueAndClear()
    }
    function _sendErrorQueueAndClear()
    {
        flushPending = true;
        var xlsUrl = environment.configuration["xlsLoggingUrl"];
        if(!xlsUrl)
        {
            flushPending = false;
            BingApp.traceError("BingApp.Instrumentation failure, xlsUrl");
            return
        }
        xlsUrl += "&bingapp-error=1";
        if(errorQueue.length === 0)
        {
            flushPending = false;
            return
        }
        var xlsPayload = errorQueue[0];
        var detail = {
                url: xlsUrl,
                type: "POST",
                data: xlsPayload
            };
        proxy.xhr(detail).done(function onComplete(response)
        {
            if(response)
            {
                BingApp.traceInfo("instrumentation.log  success, status:{0} response: {1}.",response.status,response.responseText);
                lastInstrumentationEventTS = ts();
                if(xlsPayload === errorQueue[0])
                {
                    errorQueue.shift();
                    _saveErrorQueue()
                }
                _sendErrorQueueAndClear()
            }
        },function onError(error)
        {
            if(error)
            {
                flushPending = false;
                BingApp.traceError("BingApp.Instrumentation.log failed status:{0}, response:{1}.",error.status,error.responseText)
            }
        })
    }
    function logPartialImpression(pageName, layout, ds)
    {
        if(!pageName)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("pageName");
        if(!layout)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("layout");
        if(!ds)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("ds");
        var igInfo = _getCurrentIGInfo();
        if(igInfo && (igInfo.layoutSent || igInfo.layoutPending))
            return;
        var callback = function()
            {
                igInfo = _getCurrentIGInfo();
                if(igInfo.pageName === pageName)
                {
                    igInfo.layoutSent = true;
                    igInfo.layoutPending = false
                }
            };
        var pageLayout = {
                layout: true,
                name: pageName,
                pageLayout: layout,
                pageDs: ds
            };
        log(pageLayout,BingApp.Instrumentation.partialImpression,callback)
    }
    function _getLayoutXML(type, currentIG, data)
    {
        if(!type)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("type");
        if(!currentIG)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("currentIG");
        if(!data)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("data");
        var baseXML = "<ClientInstRequest>" + "<CID>{0}</CID>" + "<AppNS>BingApp</AppNS>" + "<STS>123456789000</STS>" + "<Events>" + "<E>" + "<T>{1}</T>" + "<IG>{2}</IG>" + "<TS>{3}</TS>" + "<DS><![CDATA[{4}]\]></DS>" + "<Page>" + "<Name>Page.BingWin8App.{5}</Name>" + "<IID>iid1</IID>" + "<SFX>sfx1</SFX>" + "<L><![CDATA[{6}]\]></L>" + "</Page>" + "</E>" + "</Events>" + "</ClientInstRequest>";
        var pageLayout = JSON.stringify(data.pageLayout);
        var pageDs = JSON.stringify(data.pageDs);
        var xmlPageImpression = BingApp.Utilities.format(baseXML,muid,type,currentIG,ts(),pageDs,data.name,pageLayout);
        return xmlPageImpression
    }
    function refresh(complete, error)
    {
        var pageName = _getPageName();
        if(!pageName)
        {
            error(new WinJS.ErrorFromName("BingApp.Instrumentation.RefreshError","Current page is invalid."));
            return
        }
        _refreshImpression(pageName,complete,error)
    }
    function updatePageIG(ig)
    {
        var pageName = _getPageName();
        if(pageName)
        {
            if(ig)
                pageIGs[pageName] = _makeIgInfo(pageName,ig,ts(),BingApp.Instrumentation.State.ready);
            else
            {
                var igInfo = getCurrentIGInfoForPage(pageName);
                if(igInfo && igInfo.state !== BingApp.Instrumentation.State.requestPending)
                {
                    igInfo.state = BingApp.Instrumentation.State.requireRefresh;
                    igInfo.layoutSent = false
                }
            }
            _fireUpdateIG()
        }
    }
    function clear()
    {
        muid = null;
        lastInstrumentationEventTS = null;
        pageIGs = [];
        activationFormCodeSent = false
    }
    function getCurrentIGInfoForPage(pageName)
    {
        if(pageName)
            return pageIGs[pageName];
        return null
    }
    function getIGForPage(pageName)
    {
        return _getCurrentIG(pageName)
    }
    function getIGForCurrentPage()
    {
        return _getCurrentIG()
    }
    function _sendXlsPayload(xlsPayLoad, callback)
    {
        var xlsUrl = environment.configuration["xlsLoggingUrl"];
        if(!xlsUrl)
        {
            BingApp.traceError("BingApp.Instrumentation failure, xlsUrl");
            return false
        }
        var addLoginSource = BingApp.locator.config.getConfig().instrumentation.general.addLoginSource;
        if(addLoginSource)
            xlsUrl += "&addloginsource=1";
        var detail = {
                url: xlsUrl,
                type: "POST",
                data: xlsPayLoad
            };
        proxy.xhr(detail).done(function onComplete(response)
        {
            if(response)
            {
                BingApp.traceInfo("instrumentation.log  success, status:{0} response: {1}.",response.status,response.responseText);
                lastInstrumentationEventTS = ts();
                if(callback)
                    callback()
            }
        },function onError(error)
        {
            if(callback)
                _getCurrentIGInfo().layoutPending = false;
            if(error)
                BingApp.traceError("BingApp.Instrumentation.log failed status:{0}, response:{1}.",error.status,error.responseText)
        })
    }
    function _queueError(xlsPayload)
    {
        if(!errorQueueLoaded)
            _loadErrorQueue();
        if(!xlsPayload)
        {
            BingApp.traceError("BingApp.Instrumentation._queueError: Attempt made to queue null error/payload");
            return
        }
        var targetLength = BingApp.locator.config.getConfig().errorLogging.general.maxErrorsInQueue;
        for(var i = errorQueue.length; i >= targetLength && i > 0; i--)
            errorQueue.shift();
        errorQueue.push(xlsPayload);
        _saveErrorQueue()
    }
    function _getCurrentIG(page)
    {
        var pageName = page || _getPageName();
        var igInfo = getCurrentIGInfoForPage(pageName);
        if(igInfo && igInfo.state === BingApp.Instrumentation.State.ready)
            return WinJS.Promise.as(igInfo.ig);
        return new WinJS.Promise(function init(complete, error)
            {
                if(!_isPageValidForIGRefresh(pageName))
                {
                    BingApp.traceError("BingApp.Instrumentation.getCurrentIG failed: Invalid pageName {0}.",pageName);
                    error(new WinJS.ErrorFromName("BingApp.Instrumentation.Error","Invalid pageName."))
                }
                else
                    _onReady().done(function onCurrentIGComplete(igInfoArg)
                    {
                        complete(igInfoArg.ig)
                    },function onCurrentIGError(err)
                    {
                        BingApp.traceError("BingApp.Instrumentation.getCurrentIG failed: {0}.",err && err.message);
                        error(err)
                    })
            })
    }
    function _refreshImpression(pageName, complete, error)
    {
        if(!pageName)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("pageName");
        if(!complete)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("complete");
        if(!error)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("error");
        if(!_isPageValidForIGRefresh(pageName))
        {
            error(new WinJS.ErrorFromName("BingApp.Instrumentation.Error","Invalid pageName."));
            return
        }
        environment.configurationReady().done(function onEnvironmentConfigurationComplete()
        {
            var igUrl = environment.configuration["igUrl"];
            igUrl = BingApp.Utilities.format(igUrl,pageName,BingApp.Classes.Shell.fullVersion);
            igUrl = appendQueryString(igUrl);
            var detail = {url: igUrl};
            proxy.xhr(detail).done(function onComplete(response)
            {
                _initializeMuid(true).done(function success(newMuid)
                {
                    BingApp.traceInfo("BingApp.Instrumentation._refreshImpression initializeMuid success")
                },function failure(err)
                {
                    BingApp.traceError("BingApp.Instrumentation._refreshImpression initializeMuid failed: {0}.",err.message)
                });
                if(response.status === 200)
                {
                    var data = null;
                    try
                    {
                        var instStudioAttached = response.responseText.indexOf("<!--");
                        if(instStudioAttached === -1)
                            data = JSON.parse(response.responseText);
                        else
                        {
                            var responseString = response.responseText.substring(0,instStudioAttached);
                            data = JSON.parse(responseString)
                        }
                    }
                    catch(e)
                    {
                        BingApp.traceError("nativeinstrumentation._refreshImpression: failed, status:{0}, response:{1}, error:{2}",response.status,response.responseText,e.message);
                        error(e);
                        _notifyWithErrorClearAndReset(pageName,e);
                        return
                    }
                    var completeHandlers = pageIGs[pageName] && pageIGs[pageName].complete;
                    pageIGs[pageName] = _makeIgInfo(pageName,data.ig,ts(),BingApp.Instrumentation.State.ready);
                    var layoutHandler = pageLayoutHandlers[pageName];
                    if(layoutHandler)
                        layoutHandler();
                    complete(pageIGs[pageName]);
                    if(completeHandlers)
                        completeHandlers.forEach(function notifyAdditionalListener(callback)
                        {
                            callback(pageIGs[pageName])
                        })
                }
            },function onRefreshImpressionError(err)
            {
                if(!BingApp.Utilities.isPromiseCancellationError(err))
                    BingApp.traceError("BingApp.Instrumentation._refreshImpression failed, name:{0}, message:{1}",err.name,err.message);
                error(err);
                _notifyWithErrorClearAndReset(pageName,err)
            })
        })
    }
    function _notifyWithErrorClearAndReset(page, err)
    {
        var igInfo = pageIGs[page];
        if(igInfo)
        {
            _notifyWithError(igInfo,err);
            _resetCallbacks(igInfo);
            igInfo.state = BingApp.Instrumentation.State.requireRefresh
        }
    }
    function _notifyWithError(igInfo, err)
    {
        if(igInfo && igInfo.error)
            igInfo.error.forEach(function notifyAdditionalListener(callback)
            {
                callback(err)
            })
    }
    function _resetCallbacks(igInfo)
    {
        if(igInfo)
        {
            igInfo.error = [];
            igInfo.complete = []
        }
    }
    function appendQueryString(igUrl)
    {
        if(!activationFormCodeSent && BingApp.Classes.Shell.lastActivationFormCode)
        {
            activationFormCodeSent = true;
            if(BingApp.locator.config.getConfig().settingsPane.testConfiguration.enabled)
                igUrl += "&form=MONITR";
            else
                igUrl += "&form=" + BingApp.Classes.Shell.lastActivationFormCode
        }
        else
        {
            var currentUri = navigationManager.getCurrentUri();
            if(currentUri && currentUri.query)
                igUrl += "&" + currentUri.query.substring(1)
        }
        var params = environment.getIFrameParams(null,window);
        var allowList = Object.keys(params).filter(function(element)
            {
                var ret;
                switch(element)
                {
                    case"w8-height":
                    case"w8-width":
                    case"bingapp-iv":
                    case"bingapp-t":
                    case"w8-devicedpi":
                    case"PC":
                        ret = true;
                        break;
                    default:
                        ret = false
                }
                return ret
            });
        var collection = [];
        var allowedParams = allowList.reduce(function(previous, allowedKey)
            {
                collection[allowedKey] = params[allowedKey];
                return collection
            },{});
        var queryString = BingApp.Utilities.QueryString.serialize(allowedParams);
        queryString = queryString.replace("?","&");
        return igUrl += queryString
    }
    function _getPageName()
    {
        var uri = navigationManager.getCurrentUri();
        if(uri)
            for(var i = 0; i < patternsPage.length; i++)
                if(regexps[i].test(uri.absoluteUri))
                {
                    regexps[i].lastIndex = 0;
                    return patternsPage[i].pageName
                }
        return uri
    }
    function _getCurrentIGInfo()
    {
        var pageName = _getPageName();
        if(!pageName)
        {
            BingApp.traceError("_getCurrentIG failure: pageName");
            return null
        }
        var expiration = environment.configuration["expirationInterval"];
        var currentTS = ts();
        var pageData = pageIGs[pageName];
        if(pageData)
        {
            var returnData = false;
            if(pageData.state === BingApp.Instrumentation.State.requestPending)
                returnData = true;
            if(currentTS - pageData.ts < expiration)
                returnData = true;
            if(returnData)
                return pageData
        }
        pageIGs[pageName] = _makeIgInfo(pageName,null,null,BingApp.Instrumentation.State.requireRefresh);
        return pageIGs[pageName]
    }
    function _loadErrorQueue()
    {
        if(errorQueue.length === 0)
            try
            {
                errorQueue = JSON.parse(window.localStorage.getItem(errorQueueStorageKey));
                if(errorQueue === null)
                    errorQueue = []
            }
            catch(e)
            {
                BingApp.traceError("BingApp.Instrumentation._loadErrorQueue failed, message:{0}",e.message);
                errorQueue = []
            }
        errorQueueLoaded = true
    }
    function _saveErrorQueue()
    {
        try
        {
            window.localStorage.setItem(errorQueueStorageKey,JSON.stringify(errorQueue))
        }
        catch(e)
        {
            BingApp.traceError("BingApp.Instrumentation._saveErrorQueue failed, message:{0}",e.message)
        }
    }
    function _createErrorXMLPayload(data, currentIG, pageName)
    {
        if(!muid)
        {
            BingApp.traceError("BingApp.Instrumentation._createXMLPayLoad failure: muid");
            return null
        }
        if(!data)
        {
            BingApp.traceError("BingApp.Instrumentation._createXMLPayLoad failure: data");
            return null
        }
        if(!pageName)
        {
            BingApp.traceError("BingApp.Instrumentation._createXMLPayLoad failure: pageName");
            return null
        }
        data.FID = _getFIDFromPageName(pageName);
        data.T = BingApp.Instrumentation.errorEvent;
        data.TS = ts();
        data.AppNS = appNS;
        data.NS = BingApp.locator.networkDetectionService.isConnected();
        data.CT = BingApp.locator.networkDetectionService.getConnectionType();
        var baseXML = "<ClientInstRequest>" + "<CID>{0}</CID>" + "<Events>" + "<E>" + "<T>Event.ClientInst</T>" + "{1}" + "<TS>{2}</TS>" + "<D><![CDATA[{3}]\]></D>" + "</E>" + "</Events>" + "</ClientInstRequest>";
        var igValue = currentIG ? "<IG>" + currentIG + "</IG>" : "";
        var xml = BingApp.Utilities.format(baseXML,muid,igValue,ts(),JSON.stringify(data));
        return xml
    }
    function _createXMLPayLoad(data, type, currentIG, pageName)
    {
        if(!muid)
        {
            BingApp.traceError("BingApp.Instrumentation._createXMLPayLoad failure: muid");
            return null
        }
        if(!currentIG)
        {
            BingApp.traceError("BingApp.Instrumentation._createXMLPayLoad failure: currentIG");
            return null
        }
        if(!data)
        {
            BingApp.traceError("BingApp.Instrumentation._createXMLPayLoad failure: data");
            return null
        }
        if(!type)
        {
            BingApp.traceError("BingApp.Instrumentation._createXMLPayLoad failure: type");
            return null
        }
        if(!pageName)
        {
            BingApp.traceError("BingApp.Instrumentation._createXMLPayLoad failure: pageName");
            return null
        }
        data.TS = ts();
        data.ViewMode = BingApp.Utilities.getViewMode();
        if(type === BingApp.Instrumentation.clickEvent)
        {
            var kValue = data.K;
            delete data.K;
            var clickData = {
                    T: type,
                    FID: _getFIDFromPageName(pageName),
                    K: kValue,
                    AppNS: appNS,
                    Properties: data
                };
            data = clickData
        }
        else
        {
            data.FID = _getFIDFromPageName(pageName);
            data.T = type;
            data.AppNS = appNS
        }
        var baseXML = "<ClientInstRequest>" + "<CID>{0}</CID>" + "<Events>" + "<E>" + "<T>Event.ClientInst</T>" + "<IG>{1}</IG>" + "<TS>{2}</TS>" + "<D><![CDATA[{3}]\]></D>" + "</E>" + "</Events>" + "</ClientInstRequest>";
        var xml = BingApp.Utilities.format(baseXML,muid,currentIG,ts(),JSON.stringify(data));
        return xml
    }
    function _makeIgInfo(pageName, ig, timestamp, stateArg)
    {
        var complete = [];
        var error = [];
        return{
                pageName: pageName,
                ig: ig,
                ts: timestamp,
                state: stateArg,
                layoutSent: false,
                layoutPending: false,
                complete: complete,
                error: error
            }
    }
    function _initializeMuid(forceRefresh)
    {
        if(muid && !forceRefresh)
            return new WinJS.Promise.as(muid);
        return new WinJS.Promise(function init(complete, error)
            {
                if(!BingApp.locator.networkDetectionService.isConnected())
                {
                    error(new WinJS.ErrorFromName("BingApp.Instrumentation.Error","Failure retrieving MUID."));
                    return
                }
                proxy.getCookie(cookieName).done(function onComplete(cookie)
                {
                    if(cookie)
                    {
                        muid = cookie.substring(cookieName.length + 1);
                        if(muid)
                            complete(muid);
                        else
                            error(new WinJS.ErrorFromName("BingApp.Instrumentation.Error","Failure retrieving MUID."))
                    }
                    else
                    {
                        BingApp.traceError("BingApp.Instrumentation._initializeMuid: failed to retrieve MUID");
                        error(new WinJS.ErrorFromName("BingApp.Instrumentation.Error","Failure retrieving MUID."))
                    }
                },function onError(err)
                {
                    error(new WinJS.ErrorFromName("BingApp.Instrumentation.Error","Failure retrieving MUID."))
                })
            })
    }
    function _onReady()
    {
        var igInfo = _getCurrentIGInfo();
        if(igInfo && igInfo.state === BingApp.Instrumentation.State.ready)
            return new WinJS.Promise.as(igInfo);
        return new WinJS.Promise(function init(complete, error)
            {
                if(igInfo && igInfo.state === BingApp.Instrumentation.State.requireRefresh)
                {
                    igInfo.state = BingApp.Instrumentation.State.requestPending;
                    refresh(complete,error)
                }
                else if(igInfo)
                {
                    igInfo.complete.push(complete);
                    igInfo.error.push(error)
                }
                else
                {
                    BingApp.traceError("BingApp.Instrumentation._onready: Unexpected IGInfo object.");
                    error(new WinJS.ErrorFromName("BingApp.Instrumentation.Error","Unexpected IGInfo object"))
                }
            })
    }
    function _isPageValidForIGRefresh(pageName)
    {
        return pageName === "start" || pageName === "trends"
    }
    function _fireUpdateIG()
    {
        BingApp.traceInfo("BingApp.Instrumentation._fireUpdateIG: received.");
        BingApp.Instrumentation.getIGForCurrentPage().done(function fireUpdateIG(ig)
        {
            BingApp.locator.eventRelay.fireEvent(BingApp.Instrumentation.updateIG,{currentIG: ig})
        },function fireUpdateIGError(err)
        {
            BingApp.traceError("BingApp.Instrumentation._fireUpdateIG: failure: Unable to retrieve IG for current view.")
        })
    }
    function _excludeEvent(eventType)
    {
        var pageName = _getPageName();
        return pageName === "serp" && eventType === BingApp.Instrumentation.modeChange
    }
    function _getFIDFromPageName(pageName)
    {
        var fid = "";
        switch(pageName)
        {
            case"start":
                fid = "Win8AppHP";
                break;
            case"trends":
                fid = "Win8AppTrends";
                break;
            case"serp":
                fid = "Win8AppSERP";
                break;
            case"images":
                fid = "Win8AppImages";
                break;
            case"rewards":
                fid = "Win8AppRewards";
                break;
            case"videos":
                fid = "Win8AppVideos";
                break;
            default:
                fid = pageName;
                break
        }
        return fid
    }
    function logFromHandlerEvent(event, suffix, type)
    {
        if(!event && !suffix)
            return;
        var entryName = "";
        if(event && event.srcElement && event.srcElement.id)
            entryName += event.srcElement.id;
        if(suffix)
            entryName += suffix;
        if(!type)
            type = "CI.AppEvent";
        var instrumentationObject = {name: entryName};
        log(instrumentationObject,type)
    }
    function registerLayoutHandler(pageName, layoutHandler)
    {
        if(!pageName)
        {
            BingApp.traceError("BingApp.Instrumentation.registerLayoutHandler failure: pageName");
            return null
        }
        if(!layoutHandler)
        {
            BingApp.traceError("BingApp.Instrumentation.registerLayoutHandler failure: layoutHandler");
            return null
        }
        pageLayoutHandlers[pageName] = layoutHandler
    }
    WinJS.Namespace.define("BingApp.Instrumentation",{
        modeChange: "CI.ModeChange",
        activation: "CI.Activation",
        clickEvent: "CI.Click",
        errorEvent: "CI.Error",
        appEvent: "CI.AppEvent",
        appResume: "CI.AppResume",
        partialImpression: "Event.PartialImpression",
        updateIG: "updateIg",
        log: log,
        logClick: logClick,
        logError: logError,
        logFromHandlerEvent: logFromHandlerEvent,
        logPartialImpression: logPartialImpression,
        refresh: refresh,
        flushErrorQueue: flushErrorQueue,
        resumeLogging: resumeLogging,
        updatePageIG: updatePageIG,
        clear: clear,
        registerLayoutHandler: registerLayoutHandler,
        State: state,
        KValues: kValueConstants,
        getCurrentIGInfoForPage: getCurrentIGInfoForPage,
        getIGForPage: getIGForPage,
        getIGForCurrentPage: getIGForCurrentPage
    });
    WinJS.Namespace.define("BingApp.Instrumentation.String",{
        stateChange: "stateChange",
        "native": "native",
        initialState: "none"
    })
})()