﻿(function navigationModule()
{
    "use strict";
    var NavigationOperation = WinJS.Class.derive(BingApp.Classes.AsyncOperation,function constructor()
        {
            BingApp.Classes.AsyncOperation.call(this,"NavigationOperation")
        },{
            matchOptions: function(options1, options2)
            {
                var uri1 = this._getUriFromOptions(options1);
                var uri2 = this._getUriFromOptions(options2);
                return uri1.equals(uri2)
            },
            handleStart: function(options)
            {
                var uri = this._getUriFromOptions(options);
                BingApp.traceInfo("NavigationOperation.handleStart: started navigation to {0}.",uri.absoluteUri)
            },
            handleExecute: function(options)
            {
                var uri = this._getUriFromOptions(options);
                var navigationManager = this._getNavigationManagerFromOptions(options);
                var viewManager = this._getViewManagerFromOptions(options);
                var that = this;
                return new WinJS.Promise(function init(complete, error)
                    {
                        viewManager.reset();
                        var promise;
                        var fullPath = (uri.host ? uri.host : "") + (uri.path ? uri.path : "");
                        var elements = fullPath.split("/");
                        var elementsCount = elements.length;
                        var analyzedPath;
                        BingApp.traceInfo("NavigationOperation.handleExecute: analyzing combined path '{0}' (broken down in {1} elements).",fullPath,elements.length);
                        for(var indexElement = 0; indexElement < elementsCount; indexElement++)
                        {
                            var element = elements[indexElement];
                            analyzedPath = analyzedPath ? analyzedPath + "/" + element : element;
                            var mapping = navigationManager.findMapping(analyzedPath);
                            if(mapping)
                            {
                                var viewId = mapping.viewId;
                                BingApp.traceInfo("NavigationOperation.handleExecute: found match for '{0}' subpath. '{1}' view will be loaded.",analyzedPath,viewId);
                                promise = promise ? promise.then(function onComplete()
                                {
                                    var viewIdInClosure = viewId;
                                    return function loadViewManager()
                                        {
                                            return viewManager.load(viewIdInClosure)
                                        }
                                }(),error) : viewManager.load(viewId)
                            }
                            else
                                BingApp.traceInfo("NavigationOperation.handleExecute: no match for '{0}' subpath.",analyzedPath)
                        }
                        if(promise)
                            promise.done(complete,error);
                        else
                        {
                            BingApp.traceError("NavigationOperation.handleExecute: failed to find view match for '{0}'.",fullPath);
                            error(new BingApp.Classes.ErrorNavigationPath(fullPath))
                        }
                    })
            },
            handleComplete: function(options, result, canceled, error)
            {
                var uri = this._getUriFromOptions(options);
                if(error)
                    BingApp.traceInfo("NavigationOperation.handleComplete: navigation to {0} failed with error '{1}'.",uri.absoluteUri,error.message);
                else if(canceled)
                    BingApp.traceInfo("NavigationOperation.handleComplete: navigation to {0} was cancelled.",uri.absoluteUri);
                else
                    BingApp.traceInfo("NavigationOperation.handleComplete: navigation to {0} completed successfully.",uri.absoluteUri)
            },
            _getUriFromOptions: function(options)
            {
                if(BingApp.Utilities.isNullOrUndefined(options))
                    throw new BingApp.Classes.ErrorArgumentNullOrUndefined("options");
                var uri = options.uri;
                if(BingApp.Utilities.isNullOrUndefined(uri))
                    throw new BingApp.Classes.ErrorArgument("options");
                return uri
            },
            _getNavigationManagerFromOptions: function(options)
            {
                if(BingApp.Utilities.isNullOrUndefined(options))
                    throw new BingApp.Classes.ErrorArgumentNullOrUndefined("options");
                var manager = options.navigationManager;
                if(!manager)
                    throw new BingApp.Classes.ErrorArgument("options");
                return manager
            },
            _getViewManagerFromOptions: function(options)
            {
                if(BingApp.Utilities.isNullOrUndefined(options))
                    throw new BingApp.Classes.ErrorArgumentNullOrUndefined("options");
                var manager = options.viewManager;
                if(!manager)
                    throw new BingApp.Classes.ErrorArgument("options");
                return manager
            }
        });
    var navigationStatus = Object.freeze({
            idle: 0,
            navigating: 1
        });
    var contentStatus = Object.freeze({
            empty: 0,
            loading: 1,
            loaded: 2,
            failed: 3,
            canceled: 4
        });
    var events = Object.freeze({
            navigationStarted: "navigation:started",
            navigationCompleted: "navigation:completed",
            contentLoaded: "navigation:contentLoaded",
            historyCleared: "navigation:historyCleared"
        });
    var maxHistoryLength = 50;
    var NavigationManager = WinJS.Class.derive(BingApp.Classes.EventSource,function constructor(mappingsSourceLocation)
        {
            if(!(this instanceof BingApp.Classes.NavigationManager))
            {
                BingApp.traceWarning("NavigationManager.ctor: Attempted using NavigationManager ctor as function; redirecting to use 'new NavigationManager()'.");
                return new BingApp.Classes.NavigationManager(mappingsSourceLocation)
            }
            BingApp.Classes.EventSource.call(this);
            Object.defineProperties(this,{
                navigationHistory: {
                    value: [],
                    writable: false,
                    enumerable: false,
                    configurable: false
                },
                navigationOperation: {
                    value: new NavigationOperation,
                    writable: false,
                    enumerable: false,
                    configurable: false
                },
                currentUri: {
                    writable: true,
                    enumerable: false,
                    configurable: true
                },
                currentNavigation: {
                    writable: true,
                    enumerable: false,
                    configurable: true
                },
                currentOptions: {
                    writable: true,
                    enumerable: false,
                    configurable: true
                },
                currentNavigationStatus: {
                    value: navigationStatus.idle,
                    writable: true,
                    enumerable: false,
                    configurable: false
                },
                currentContentStatus: {
                    value: contentStatus.empty,
                    writable: true,
                    enumerable: false,
                    configurable: false
                },
                rollbackToLastLoadedNavigationUriOnCancel: {
                    value: true,
                    writable: true,
                    enumerable: false,
                    configurable: false
                },
                loadMappingsOperation: {
                    value: new BingApp.Classes.LoadJsonOperation,
                    writable: false,
                    enumerable: false,
                    configurable: false
                },
                mappings: {
                    writable: true,
                    enumerable: false,
                    configurable: false
                }
            });
            if(mappingsSourceLocation)
                this._loadMappings(mappingsSourceLocation)
        },{
            navigateTo: function(uri, options)
            {
                if(typeof uri === "string")
                    uri = new Windows.Foundation.Uri(uri);
                if(!uri)
                    throw new BingApp.Classes.ErrorArgumentNullOrUndefined("uri");
                if(!this.currentUri || !this.currentUri.equals(uri) || !this._areSameOptions(this.currentOptions,options) || this.currentContentStatus === contentStatus.failed || this.currentContentStatus === contentStatus.canceled)
                {
                    this.cancelNavigation(false);
                    this.currentUri = uri;
                    this.currentOptions = options;
                    var that = this;
                    this.currentNavigation = new WinJS.Promise(function init(complete, error)
                    {
                        that._ready().done(function onReadyComplete()
                        {
                            that.notifyNavigationStarted(uri);
                            that.navigationOperation.run({
                                uri: uri,
                                navigationManager: that,
                                viewManager: BingApp.locator.viewManager
                            }).done(function onComplete()
                            {
                                if(that.currentNavigationStatus === navigationStatus.navigating)
                                    that.notifyNavigationCompleted(uri,null,false);
                                complete()
                            },function onError(err)
                            {
                                if(that.currentNavigationStatus === navigationStatus.navigating)
                                    if(BingApp.Utilities.isPromiseCancellationError(err))
                                        that.notifyNavigationCompleted(uri,null,true);
                                    else
                                        that.notifyNavigationCompleted(uri,err,false);
                                error(err)
                            })
                        },error)
                    },function cancel()
                    {
                        that.navigationOperation.cancel()
                    });
                    return this.currentNavigation
                }
                else if(this.navigationOperation.isRunning())
                {
                    BingApp.traceInfo("NavigationManager.navigateTo: app is already in the process of navigating to {0}. Returning reference to active navigation operation.",uri.absoluteUri);
                    return this.currentNavigation
                }
                else
                {
                    BingApp.traceInfo("NavigationManager.navigateTo: app is already in the UI state corresponding to {0}.",uri.absoluteUri);
                    return WinJS.Promise.as()
                }
            },
            cancelNavigation: function(rollback)
            {
                this.rollbackToLastLoadedNavigationUriOnCancel = rollback;
                if(this.currentNavigationStatus === navigationStatus.navigating)
                    if(this.navigationOperation.isRunning())
                        this.navigationOperation.cancel();
                    else
                        this.notifyNavigationCompleted(this.currentUri,null,true);
                else if(this.currentContentStatus === contentStatus.loading)
                    this.notifyContentLoaded(null,true);
                this.rollbackToLastLoadedNavigationUriOnCancel = true
            },
            goBack: function(options)
            {
                var historyLength = this.navigationHistory.length;
                if(historyLength > 1)
                {
                    BingApp.traceInfo("NavigationManager.goBack: removing '{0}' and '{1}' from navigation history stack. There are {2} remaining entries in navigation history.",this.navigationHistory[historyLength - 1].absoluteUri,this.navigationHistory[historyLength - 2].absoluteUri,historyLength - 2);
                    var uri = this.navigationHistory.splice(historyLength - 2,2)[0];
                    BingApp.traceInfo("NavigationManager.goBack: will navigate to '{0}'.",uri.absoluteUri);
                    return this.navigateTo(uri,options)
                }
                else
                {
                    BingApp.traceError("NavigationManager.goBack: cannot proceed because navigation history {0}.",historyLength === 0 ? "has no entries" : "has only one entry");
                    return WinJS.Promise.as()
                }
            },
            clearHistory: function()
            {
                if(this.navigationHistory.length > 0)
                {
                    this.navigationHistory.length = 0;
                    this.fireEvent(events.historyCleared)
                }
            },
            canGoBack: function()
            {
                return this.navigationHistory.length > 1
            },
            getMaxHistoryLength: function()
            {
                return maxHistoryLength
            },
            refresh: function(options)
            {
                if(this.navigationOperation.isRunning())
                {
                    this.currentOptions = options || this.getCurrentOptions();
                    return this.currentNavigation
                }
                else
                {
                    var uri = this.currentUri;
                    if(uri)
                    {
                        this.currentUri = null;
                        this.currentOptions = null;
                        return this.navigateTo(uri,options)
                    }
                    else
                        return WinJS.Promise.as()
                }
            },
            notifyNavigationStarted: function(uri)
            {
                if(typeof uri === "string")
                    uri = new Windows.Foundation.Uri(uri);
                if(!uri)
                    throw new BingApp.Classes.ErrorArgumentNullOrUndefined("uri");
                if(this.currentNavigationStatus !== navigationStatus.idle || this.currentContentStatus === contentStatus.loading)
                    throw new BingApp.Classes.ErrorInvalidOperation("notifyNavigationStarted",WinJS.Resources.getString("error_navigation_started_called_from_invalid_state").value);
                this.currentUri = uri;
                this.currentNavigationStatus = navigationStatus.navigating;
                this.currentContentStatus = contentStatus.loading;
                BingApp.tracePerf("Framework:Navigation:Start");
                BingApp.traceInfo("NavigationManager.notifyNavigationStarted: setting navigation manager into 'navigating' state and raising '{0}' event.",events.navigationStarted);
                this.fireEvent(events.navigationStarted,{uri: uri})
            },
            notifyNavigationCompleted: function(uri, error, canceled)
            {
                if(this.currentNavigationStatus !== navigationStatus.navigating)
                    throw new BingApp.Classes.ErrorInvalidOperation("notifyNavigationCompleted",WinJS.Resources.getString("error_navigation_completed_called_before_started").value);
                if(typeof uri === "string")
                    uri = new Windows.Foundation.Uri(uri);
                if(!uri)
                    throw new BingApp.Classes.ErrorArgumentNullOrUndefined("uri");
                this.currentNavigationStatus = navigationStatus.idle;
                var historyLength = this.navigationHistory.length;
                if(!error && !canceled)
                {
                    BingApp.tracePerf("Framework:Navigation:End:Succeeded");
                    this.currentUri = uri;
                    if(!BingApp.Utilities.isNullOrUndefined(this.currentUri))
                    {
                        var lastUri = historyLength > 0 ? this.navigationHistory[historyLength - 1] : null;
                        if(lastUri === null || !this.currentUri.equals(lastUri))
                        {
                            BingApp.traceInfo("NavigationManager.notifyNavigationCompleted: storing {0} in navigation history. Current number of entries in history is {1}.",this.currentUri.absoluteUri,historyLength + 1);
                            if(historyLength === this.getMaxHistoryLength())
                            {
                                var removedEntry = this.navigationHistory.shift();
                                BingApp.traceInfo("NavigationManager.notifyNavigationCompleted: maximum number of '{0}' entries in navigation history has been reached. Removing entry '{1}'.",historyLength,removedEntry.absoluteUri)
                            }
                            this.navigationHistory.push(this.currentUri)
                        }
                    }
                }
                else
                {
                    this.currentUri = null;
                    this.currentOptions = null;
                    this.currentContentStatus = contentStatus.empty;
                    var previousUri = null;
                    if(historyLength > 0)
                        if(canceled && this.rollbackToLastLoadedNavigationUriOnCancel)
                            previousUri = this.navigationHistory.pop();
                        else
                            previousUri = this.navigationHistory[historyLength - 1];
                    if(canceled)
                    {
                        BingApp.tracePerf("Framework:Navigation:End:Cancelled");
                        if(this.rollbackToLastLoadedNavigationUriOnCancel && previousUri)
                        {
                            BingApp.traceInfo("NavigationManager.notifyNavigationCompleted: operation cancelled. Application will rollback to last known navigation Uri: {0}.",previousUri.absoluteUri);
                            this.navigateTo(previousUri)
                        }
                        else
                        {
                            BingApp.traceInfo("NavigationManager.notifyNavigationCompleted: operation cancelled.");
                            this.currentUri = previousUri
                        }
                    }
                    else
                    {
                        BingApp.tracePerf("Framework:Navigation:End:Failed");
                        this.currentUri = previousUri
                    }
                }
                BingApp.traceInfo("NavigationManager.notifyNavigationCompleted: setting navigation manager into 'idle' state and raising '{0}' event.",events.navigationCompleted);
                this.fireEvent(events.navigationCompleted,{
                    uri: uri,
                    error: error,
                    canceled: canceled
                })
            },
            notifyContentLoaded: function(error, canceled)
            {
                if(this.currentContentStatus !== contentStatus.loading)
                    throw new BingApp.Classes.ErrorInvalidOperation("notifyContentLoaded",WinJS.Resources.getString("error_content_loaded_called_from_invalid_state").value);
                if(error)
                {
                    BingApp.traceError("NavigationManager.notifyContentLoaded: content loading completed with error: '{0}'",error.message);
                    this.currentContentStatus = contentStatus.failed
                }
                else if(canceled)
                {
                    BingApp.traceInfo("NavigationManager.notifyContentLoaded: content loading completed with cancellation.");
                    this.currentContentStatus = contentStatus.canceled
                }
                else
                {
                    BingApp.traceInfo("NavigationManager.notifyContentLoaded: content loading completed successfully.");
                    this.currentContentStatus = contentStatus.loaded
                }
                BingApp.traceInfo("NavigationManager.notifyContentLoaded: setting navigation manager into 'idle' state and raising '{0}' event.",events.contentLoaded);
                this.fireEvent(events.contentLoaded,{
                    error: error,
                    canceled: canceled
                })
            },
            getCurrentUri: function()
            {
                return this.currentUri
            },
            getCurrentOptions: function()
            {
                return this.currentOptions
            },
            getCurrentNavigationStatus: function()
            {
                return this.currentNavigationStatus
            },
            getCurrentContentStatus: function()
            {
                return this.currentContentStatus
            },
            setMappings: function(mappings)
            {
                BingApp.traceInfo("NavigationManager.setMappings: navigation mappings are set explicitly.");
                this.mappings = mappings;
                if(this.loadMappingsOperation.isRunning())
                {
                    BingApp.traceWarning("NavigationManager.setMappings: called while app is in the middle of loading navigation mappings from external source. This loading operation will be cancelled.");
                    this.loadMappingsOperation.cancel()
                }
            },
            findMapping: function(path)
            {
                if(BingApp.Utilities.isNullOrUndefined(path))
                    throw new BingApp.Classes.ErrorArgumentNullOrUndefined("path");
                if(this.mappings)
                {
                    var mappingsCount = this.mappings.length;
                    for(var indexMapping = 0; indexMapping < mappingsCount; indexMapping++)
                    {
                        var mapping = this.mappings[indexMapping];
                        if(BingApp.Utilities.areEqualIgnoreCase(mapping.path,path))
                            return mapping
                    }
                }
                return null
            },
            _ready: function(callback, async)
            {
                var that = this;
                return new WinJS.Promise(function init(complete, error)
                    {
                        function onMappingsLoaded()
                        {
                            if(callback)
                                try
                                {
                                    callback()
                                }
                                catch(err)
                                {
                                    error(err);
                                    return
                                }
                            complete()
                        }
                        function onLoadingError(loadingErr)
                        {
                            if(that.mappings && BingApp.Utilities.isPromiseCancellationError(loadingErr))
                                onMappingsLoaded();
                            else
                                error(loadingErr)
                        }
                        if(that.loadMappingsOperation.isRunning())
                            that.loadMappingsOperation.done(onMappingsLoaded,onLoadingError);
                        else if(async)
                            WinJS.Promise.timeout().done(onMappingsLoaded,onLoadingError);
                        else
                            onMappingsLoaded()
                    })
            },
            _loadMappings: function(fileLocation)
            {
                var that = this;
                return this.loadMappingsOperation.run({fileLocation: fileLocation}).then(function onMappingLoaded(result)
                    {
                        that.mappings = result
                    })
            },
            _areSameOptions: function(options1, options2)
            {
                if(options1 === options2)
                    return true;
                if(!options1)
                    return!options2;
                else if(!options2)
                    return false;
                var optionsAsString1;
                var optionsAsString2;
                try
                {
                    optionsAsString1 = JSON.stringify(options1);
                    optionsAsString2 = JSON.stringify(options2)
                }
                catch(e)
                {
                    BingApp.traceError("NavigationManager._areSameOptions: failed to compare options because of JSON.stringify failure: '{0}'. Assuming that options are different.",e.message);
                    return false
                }
                return optionsAsString1 === optionsAsString2
            }
        },{
            navigationStatus: navigationStatus,
            contentStatus: contentStatus,
            events: events
        });
    WinJS.Namespace.define("BingApp.Classes",{NavigationManager: NavigationManager})
})()