﻿(function navigationProgressModule()
{
    "use strict";
    var currentNavigationManager;
    var currentProgressIndicatorService;
    var currentNavigationJob;
    var jobCount = 0;
    function startNavigationJob()
    {
        jobCount++;
        BingApp.traceInfo("NavigationProgressTracking.startNavigationJob: incrementing job counter for new navigation operation. Current number is at {0}.",jobCount);
        if(!currentNavigationJob)
        {
            currentNavigationJob = currentProgressIndicatorService.startJob();
            BingApp.traceInfo("NavigationProgressTracking.startNavigationJob: creating progress tracking job for navigation.")
        }
    }
    function finishNavigationJob()
    {
        if(currentNavigationJob)
        {
            jobCount--;
            BingApp.traceInfo("NavigationProgressTracking.finishNavigationJob: decrementing job counter for new navigation operation. Current number is at {0}.",jobCount);
            if(jobCount === 0)
            {
                currentProgressIndicatorService.finishJob(currentNavigationJob);
                currentNavigationJob = null;
                BingApp.traceInfo("NavigationProgressTracking.finishNavigationJob: finishing progress tracking job for navigation.")
            }
        }
        else
            BingApp.traceError("NavigationProgressTracking.finishNavigationJob: handling navigation operation completion but there is no active progress tracking job for it.")
    }
    function navigationStartedHandler(data)
    {
        startNavigationJob()
    }
    function navigationCompletedHandler(data)
    {
        if(data.error || data.canceled)
            finishNavigationJob()
    }
    function contentLoadedHandler(data)
    {
        finishNavigationJob()
    }
    function initNavigationTracking(navigationManager, progressIndicatorService)
    {
        if(!navigationManager)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("navigationManager");
        if(!progressIndicatorService)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("progressIndicatorService");
        if(currentNavigationManager || currentProgressIndicatorService || currentNavigationJob)
            resetNavigationTracking();
        currentProgressIndicatorService = progressIndicatorService;
        currentNavigationManager = navigationManager;
        var navigationEvents = BingApp.Classes.NavigationManager.events;
        currentNavigationManager.addEventListener(navigationEvents.navigationStarted,navigationStartedHandler);
        currentNavigationManager.addEventListener(navigationEvents.navigationCompleted,navigationCompletedHandler);
        currentNavigationManager.addEventListener(navigationEvents.contentLoaded,contentLoadedHandler)
    }
    function resetNavigationTracking()
    {
        if(currentNavigationManager)
        {
            var navigationEvents = BingApp.Classes.NavigationManager.events;
            currentNavigationManager.removeEventListener(navigationEvents.navigationStarted,navigationStartedHandler);
            currentNavigationManager.removeEventListener(navigationEvents.navigationCompleted,navigationCompletedHandler);
            currentNavigationManager.removeEventListener(navigationEvents.contentLoaded,contentLoadedHandler)
        }
        else
            BingApp.traceWarning("NavigationProgressTracking.resetNavigationTracking: resetTracking was called before initTracking.");
        if(jobCount > 0 || currentNavigationJob)
        {
            BingApp.traceInfo("NavigationProgressTracking.resetNavigationTracking: job counter was at {0}; resetting it to zero.",jobCount);
            jobCount = 0;
            if(currentNavigationJob)
            {
                if(currentProgressIndicatorService)
                    currentProgressIndicatorService.finishJob(currentNavigationJob);
                currentNavigationJob = null;
                BingApp.traceInfo("NavigationProgressTracking.resetNavigationTracking: finishing progress tracking job for navigation.")
            }
            else
                BingApp.traceError("NavigationProgressTracking.resetNavigationTracking: job counter was non-zero but progress tracking job was not found.")
        }
        currentNavigationManager = null;
        currentProgressIndicatorService = null
    }
    WinJS.Namespace.define("BingApp.Classes.NavigationManager",{
        initTracking: initNavigationTracking,
        resetTracking: resetNavigationTracking
    })
})()