﻿(function networkDetectionModule()
{
    "use strict";
    var states = Object.freeze({
            none: 0,
            internetAccess: 1
        });
    var networkInfo = Windows.Networking.Connectivity.NetworkInformation;
    var networkConnectivityInfo = Windows.Networking.Connectivity.NetworkConnectivityLevel;
    var networkStatusChangedEvent = "networkstatuschanged";
    var connectionState = states.none;
    var unknownConnectionType = 1;
    var NetworkDetectionService = WinJS.Class.define(function constructor()
        {
            if(!(this instanceof BingApp.Classes.NetworkDetectionService))
            {
                BingApp.traceWarning("NetworkDetectionService.ctor: Attempted using NetworkDetectionService ctor as function; redirecting to use 'new NetworkDetectionService()'.");
                return new BingApp.Classes.NetworkDetectionService
            }
            this._setNetworkState();
            var that = this;
            this._onNetworkStatusChangeWrapper = function onNetworkStatusChange(eventData)
            {
                that._onNetworkStatusChange(eventData)
            };
            networkInfo.addEventListener(networkStatusChangedEvent,this._onNetworkStatusChangeWrapper)
        },{
            isConnected: function()
            {
                return connectionState === states.internetAccess
            },
            getConnectionType: function()
            {
                var connectionProfile = Windows.Networking.Connectivity.NetworkInformation.getInternetConnectionProfile();
                if(connectionProfile && connectionProfile.networkAdapter)
                    return connectionProfile.networkAdapter.ianaInterfaceType;
                return unknownConnectionType
            },
            unregisterForNetworkStatusChange: function()
            {
                networkInfo.removeEventListener(networkStatusChangedEvent,this._onNetworkStatusChangeWrapper)
            },
            _setNetworkState: function()
            {
                try
                {
                    var connectionProfile = networkInfo.getInternetConnectionProfile();
                    if(!connectionProfile)
                    {
                        connectionState = states.none;
                        return
                    }
                    switch(connectionProfile.getNetworkConnectivityLevel())
                    {
                        case networkConnectivityInfo.internetAccess:
                            connectionState = states.internetAccess;
                            BingApp.traceInfo("NetworkDetectionService._setNetworkState: internet connection available.");
                            break;
                        default:
                            connectionState = states.none;
                            BingApp.traceInfo("NetworkDetectionService._setNetworkState: internet connection not available.");
                            break
                    }
                }
                catch(error)
                {
                    connectionState = states.none;
                    BingApp.traceError("NetworkDetectionService._setNetworkState: unexpected error: {0}",error.message)
                }
            },
            _onNetworkStatusChange: function(eventData)
            {
                this._setNetworkState();
                BingApp.locator.eventRelay.fireEvent(networkStatusChangedEvent,{connectionState: connectionState})
            },
            _onNetworkStatusChangeWrapper: null
        },{
            connectionStates: states,
            networkStatusChangedEvent: networkStatusChangedEvent
        });
    WinJS.Namespace.define("BingApp.Classes",{NetworkDetectionService: NetworkDetectionService})
})()