﻿(function progressModule()
{
    "use strict";
    var anonymousJob = "anonymous";
    var events = Object.freeze({
            start: "progress:start",
            finish: "progress:finish"
        });
    var ProgressIndicatorService = WinJS.Class.define(function constructor(eventRelay)
        {
            if(!(this instanceof BingApp.Classes.ProgressIndicatorService))
            {
                BingApp.traceWarning("ProgressIndicatorService.ctor: Attempted using ProgressIndicatorService ctor as function; redirecting to use 'new ProgressIndicatorService()'.");
                return new BingApp.Classes.ProgressIndicatorService(eventRelay)
            }
            Object.defineProperties(this,{
                jobs: {
                    value: [],
                    writable: false,
                    enumerable: false,
                    configurable: false
                },
                namedJobs: {
                    value: {},
                    writable: false,
                    enumerable: false,
                    configurable: false
                },
                showingIndicator: {
                    value: false,
                    writable: true,
                    enumerable: false,
                    configurable: false
                }
            });
            var that = this;
            if(eventRelay)
            {
                eventRelay.addEventListener(events.start,{callback: function onMessageStartJob(data)
                    {
                        that._onMessageStartJob(data)
                    }});
                eventRelay.addEventListener(events.finish,{callback: function onMessageFinishJob(data)
                    {
                        that._onMessageFinishJob(data)
                    }})
            }
        },{
            startJob: function()
            {
                var startTime = Date.now();
                var job = {start: startTime};
                this.jobs.push(job);
                if(!this.showingIndicator)
                {
                    this.showingIndicator = true;
                    var shell = BingApp.locator.shell;
                    if(shell)
                        shell.showProgressIndicator()
                }
                return job
            },
            finishJob: function(job)
            {
                var index = this.jobs.indexOf(job);
                if(index >= 0)
                {
                    this.jobs.splice(index,1);
                    if(this.showingIndicator && this.jobs.length === 0)
                    {
                        this.showingIndicator = false;
                        var shell = BingApp.locator.shell;
                        if(shell)
                            shell.hideProgressIndicator()
                    }
                }
            },
            _onMessageStartJob: function(data)
            {
                var jobName = anonymousJob;
                if(data && data.name)
                    jobName = data.name;
                else
                    BingApp.traceWarning("ProgressIndicatorService._onMessageStartJob: job name has not been specified; using '{0}' name.",jobName);
                var runningJobs = this.namedJobs[jobName];
                if(!runningJobs)
                {
                    runningJobs = [];
                    this.namedJobs[jobName] = runningJobs
                }
                var job = this.startJob();
                runningJobs.push(job);
                BingApp.traceInfo("ProgressIndicatorService._onMessageStartJob: started running '{0}' job (job counter={1}, overall counter={2}).",jobName,runningJobs.length,this.jobs.length)
            },
            _onMessageFinishJob: function(data)
            {
                var jobName = anonymousJob;
                if(data && data.name)
                    jobName = data.name;
                else
                    BingApp.traceWarning("ProgressIndicatorService._onMessageFinishJob: job name has not been specified; using '{0}' name.",jobName);
                var runningJobs = this.namedJobs[jobName];
                if(!runningJobs)
                {
                    BingApp.traceError("ProgressIndicatorService._onMessageFinishJob: there is no record of running job with '{0}' name.",jobName);
                    return
                }
                var job = runningJobs.pop();
                this.finishJob(job);
                BingApp.traceInfo("ProgressIndicatorService._onMessageFinishJob: finished running '{0}' job (job counter={1}, overall counter={2}).",jobName,runningJobs.length,this.jobs.length)
            }
        },{events: events});
    WinJS.Namespace.define("BingApp.Classes",{ProgressIndicatorService: ProgressIndicatorService})
})()