﻿(function proxyModule()
{
    "use strict";
    var adltCookieName = "SRCHHPGUSR";
    var adltCookieCrumbName = "ADLT";
    var proxyFrameEvents = Object.freeze({contentLoaded: "proxy:contentLoaded"});
    var ProxyFrameContainer = WinJS.Class.define(function constructor(host, proxyName, options)
        {
            this._host = host;
            this._name = proxyName;
            this._eventRelay = options && options.eventRelay || BingApp.locator.eventRelay;
            this._networkDetectionService = options && options.networkDetectionService || BingApp.locator.networkDetectionService;
            this._setupEventListeners();
            this._setupProxyFrame()
        },{
            _setupEventListeners: function()
            {
                var that = this;
                this._onProxyFrameLoaded = function(eventData, from)
                {
                    that._proxyFrameLoaded = true;
                    if(that._complete && that._name === from)
                    {
                        that._complete(that._proxyFrame);
                        that._complete = null
                    }
                };
                this._onNetworkStatusChanged = function()
                {
                    if(!that._proxyFrameLoaded && that._networkDetectionService.isConnected())
                        that._proxyFrame.src = that._getProxyFrameUrl()
                };
                this._eventRelay.addEventListener(proxyFrameEvents.contentLoaded,this._onProxyFrameLoaded);
                this._eventRelay.addEventListener(BingApp.Classes.NetworkDetectionService.networkStatusChangedEvent,this._onNetworkStatusChanged)
            },
            _tearDownEventListeners: function()
            {
                this._eventRelay.removeEventListener(proxyFrameEvents.contentLoaded,this._onProxyFrameLoaded);
                this._eventRelay.removeEventListener(BingApp.Classes.NetworkDetectionService.networkStatusChangedEvent,this._onNetworkStatusChanged)
            },
            _getProxyFrameUrl: function()
            {
                return this._host + "/win8app/proxy" + "#" + this._name
            },
            _setupProxyFrame: function()
            {
                this._proxyFrame = document.createElement("iframe");
                this._setupProxyFramePromise();
                var that = this;
                WinJS.Utilities.ready(function onReady()
                {
                    var container = document.getElementById("proxy-container");
                    if(container)
                        container.appendChild(that._proxyFrame)
                })
            },
            _setupProxyFramePromise: function()
            {
                var that = this;
                this._proxyFrame.src = this._getProxyFrameUrl();
                this._proxyFrameLoaded = false;
                this._proxyFramePromise = new WinJS.Promise(function initProxyFramePromise(complete)
                {
                    that._complete = complete
                })
            },
            getProxyFrameAsync: function()
            {
                return this._proxyFramePromise
            },
            tearDown: function()
            {
                this._tearDownEventListeners()
            },
            setHost: function(newHost)
            {
                if(newHost !== this._host)
                {
                    this._host = newHost;
                    this._setupProxyFramePromise()
                }
            },
            getHost: function()
            {
                return this._host
            },
            getName: function()
            {
                return this._name
            }
        },{events: proxyFrameEvents});
    var MessageChannelFactory = WinJS.Class.define(null,{createInstance: function()
            {
                return new MessageChannel
            }});
    var Proxy = WinJS.Class.define(function constructor(proxyFrameContainer, options)
        {
            this._proxyFrameContainer = proxyFrameContainer;
            this._config = options && options.config || BingApp.locator.config;
            this._messageChannelFactory = options && options.messageChannelFactory || new MessageChannelFactory;
            var that = this;
            this.getCookie(adltCookieName,adltCookieCrumbName).done(function safesearchSettingCompleted(adlt)
            {
                if(adlt)
                    that._config.setOverrideSetting("search","safeSearch","safeSearch",adlt)
            },function safesearchSettingError(error)
            {
                if(error)
                    BingApp.traceError("proxy.store_safesearch_setting: failed to store safesearch setting; err.message: {0}.",error.message);
                else
                    BingApp.traceError("proxy.store_safesearch_setting: failed to store safesearch setting; err: {0}.",error)
            })
        },{
            _sendRequest: function(action, detail)
            {
                var that = this;
                var timeout = this._config.getConfig().app.general.proxyTimeoutInterval;
                return WinJS.Promise.timeout(timeout,new WinJS.Promise(function sendRequestToProxyFrame(complete, error)
                    {
                        that._proxyFrameContainer.getProxyFrameAsync().done(function getProxyFrameComplete(proxyFrame)
                        {
                            var messageChannel = that._messageChannelFactory.createInstance();
                            messageChannel.port1.onmessage = function onMessageChannelPort1Message(eventData)
                            {
                                messageChannel.port1.close();
                                if(eventData && eventData.data && eventData.data.result !== undefined)
                                    complete(eventData.data.result);
                                else
                                    error(eventData && eventData.data && eventData.data.error || "unknown error occurred")
                            };
                            BingApp.traceInfo("Proxy._sendRequest: Posting a message to the proxy service with action={0}.",action);
                            try
                            {
                                proxyFrame.contentWindow.postMessage(JSON.stringify({
                                    name: "proxy",
                                    action: action,
                                    detail: detail
                                }),"*",[messageChannel.port2])
                            }
                            catch(err)
                            {
                                BingApp.traceError("Proxy._sendRequest: Failed to send a postMessage across, Error message={0} and statusText={1}.",err.message,err.statusText);
                                error(err)
                            }
                        },error)
                    },function onPromiseCancel()
                    {
                        BingApp.traceError("Proxy._sendRequest: The request with action={0} has been cancelled. Timeout is set at {1} ms.",action,timeout);
                        var cancelErrorMessage = BingApp.Utilities.format("Proxy._sendRequest: The promise returned by _sendRequest has been cancelled after timeout of {0} milliseconds.",timeout);
                        var errorObject = {
                                action: action,
                                detail: detail,
                                ei: cancelErrorMessage
                            };
                        BingApp.Instrumentation.logError(errorObject,BingApp.Instrumentation.State.timeout)
                    }))
            },
            xhr: function(options)
            {
                if(options && options.uri && options.uri instanceof Windows.Foundation.Uri)
                {
                    options.url = options.uri.absoluteUri;
                    delete options["uri"]
                }
                if(options)
                    BingApp.traceInfo("Proxy.xhr: Calling xhr with options.url={0}, options.type={1}",options.url,options.type);
                else
                    BingApp.traceWarning("Proxy.xhr: Calling xhr with options={0}",options);
                return this._sendRequest("xhr",options)
            },
            getCookie: function(cookie, crumb)
            {
                BingApp.traceInfo("Proxy.getCookie: Calling getCookie with cookie={0}, crumb={1}",cookie,crumb);
                return this._sendRequest("get_cookie",{
                        cookie: cookie,
                        crumb: crumb
                    })
            },
            setLocation: function(currentLocation)
            {
                BingApp.traceInfo("Proxy.setLocation: Calling set_Location");
                return this._sendRequest("set_location",currentLocation)
            },
            clearLocation: function()
            {
                BingApp.traceInfo("Proxy.clearLocation: Calling clear_Location ");
                return this._sendRequest("clear_location")
            },
            getMUIDAsync: function()
            {
                var cookieName = "MUID";
                return this.getCookie(cookieName).then(function getCookieCompleted(response)
                    {
                        return response ? response.substr(cookieName.length + 1) : null
                    })
            },
            tearDown: function()
            {
                this._proxyFrameContainer.tearDown()
            },
            getProxyFrameContainer: function()
            {
                return this._proxyFrameContainer
            }
        });
    WinJS.Namespace.define("BingApp.Classes",{
        ProxyFrameContainer: ProxyFrameContainer,
        Proxy: Proxy
    })
})()