﻿(function safeSearchModule()
{
    "use strict";
    var proxy = null;
    var config = null;
    var navigationManager = null;
    var muidCookieName = "MUID";
    function onReady(options)
    {
        proxy = options && options.proxy || BingApp.locator.proxy;
        config = options && options.config || BingApp.locator.config;
        navigationManager = options && options.navigationManager || BingApp.locator.navigationManager;
        WinJS.Resources.processAll();
        var currentSetting = getSafeSearch();
        var restrictedMarket = config.getConfig().search.safeSearch.restricted;
        var element = document.querySelector("#" + currentSetting);
        element.checked = true;
        var safeSearchSettingsDivElement = document.querySelector("#safeSearchSettingsDiv");
        var safesearchRadioElements = safeSearchSettingsDivElement.querySelectorAll("input[name='safesearch']");
        for(var i = 0; i < safesearchRadioElements.length; i++)
        {
            if(restrictedMarket && safesearchRadioElements[i].id !== "STRICT")
            {
                safesearchRadioElements[i].parentElement.style.visibility = "hidden";
                continue
            }
            safesearchRadioElements[i].onclick = safeSearchChanged
        }
        var backButtonElement = document.querySelector("#BACKSETTINGS");
        if(backButtonElement)
            backButtonElement.addEventListener("click",clickHandler,false);
        safeSearchSettingsDivElement.addEventListener("aftershow",function safeSearch_onAfterShow(event)
        {
            BingApp.traceVerbose("SafeSearch opened")
        });
        BingApp.traceInfo("safeSearch.onReady: safeSearch opened")
    }
    function safeSearchChanged(e)
    {
        var setting = e.currentTarget.id;
        var feedbackMessage = e.currentTarget.parentElement.querySelector(".safesearchFeedback");
        var feedbackMessages = e.currentTarget.parentElement.parentElement.querySelectorAll(".safesearchFeedback");
        var instrumentationObject;
        instrumentationObject = {
            name: "Settings",
            type: "SafeSearchClick",
            state: {setting: setting}
        };
        BingApp.Instrumentation.logClick(instrumentationObject);
        for(var i = 0; i < feedbackMessages.length; i++)
            feedbackMessages[i].innerText = "";
        var safeSearchSettings = document.querySelector("#safeSearchSettingsDiv");
        proxy.getCookie(muidCookieName).done(function proxyMuidCompleted(muid)
        {
            muid = muid.substring(muidCookieName.length + 1);
            var safeSearchUrl = _getSafeSearchUrl(setting,muid);
            var detail = {url: safeSearchUrl};
            proxy.xhr(detail).done(function setSafeSearch(response)
            {
                if(response && response.status === 200)
                {
                    saveSafeSearchSetting(setting);
                    navigationManager.refresh();
                    safeSearchSettings.winControl.hide()
                }
                else
                {
                    var searchErrorMessage;
                    if(response)
                        searchErrorMessage = BingApp.Utilities.format("safeSearch.safeSearchChanged failed, response.status:{0}, response.responseText:{1}",response.status,response.responseText);
                    else
                        searchErrorMessage = BingApp.Utilities.format("safeSearch.safeSearchChanged failed, response:{0}",response);
                    BingApp.traceError(searchErrorMessage);
                    var instObject = {
                            url: safeSearchUrl,
                            ei: searchErrorMessage
                        };
                    BingApp.Instrumentation.logError(instObject);
                    feedbackMessage.innerText = WinJS.Resources.getString("settings_safesearch_update_failed_message").value
                }
            },function safeSearchChangedFailed(error)
            {
                var searchErrorMessage;
                if(error)
                    searchErrorMessage = BingApp.Utilities.format("safeSearch.safeSearchChanged failed, error.status:{0}, error.responseText:{1}",error.status,error.responseText);
                else
                    searchErrorMessage = BingApp.Utilities.format("safeSearch.safeSearchChanged failed, error:{0}",error);
                var instObject = {
                        url: safeSearchUrl,
                        ei: searchErrorMessage
                    };
                BingApp.Instrumentation.logError(instObject);
                BingApp.traceError(searchErrorMessage);
                feedbackMessage.innerText = WinJS.Resources.getString("settings_safesearch_update_failed_message").value
            })
        },function proxyMuidFailed(err)
        {
            if(err)
                BingApp.traceError("safeSearch.safeSearchChanged: failed to get MUID; err.message: {0}.",err.message);
            else
                BingApp.traceError("safeSearch.safeSearchChanged: failed to get MUID; err: {0}.",err);
            feedbackMessage.innerText = WinJS.Resources.getString("settings_safesearch_update_failed_message").value
        })
    }
    function _getSafeSearchUrl(setting, muid)
    {
        if(!setting)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("setting");
        var url = config.getConfig().search.safeSearch.settingUrl;
        url = BingApp.Utilities.format(url,setting,muid);
        return url
    }
    function clickHandler(e)
    {
        var currentState = Windows.UI.ViewManagement.ApplicationView.value;
        if(Windows.UI.ViewManagement.ApplicationViewState.snapped === currentState)
            Windows.UI.ViewManagement.ApplicationView.tryUnsnap();
        e.currentTarget.removeEventListener("click",clickHandler);
        try
        {
            WinJS.UI.SettingsFlyout.show()
        }
        catch(err)
        {
            BingApp.traceError("safeSearch.clickhandler: failed to show the SettingsFlyout; Error: {0}.",err.message)
        }
        BingApp.traceInfo("safeSearch.onReady: safeSearch closed")
    }
    function getSafeSearch()
    {
        var safeSearchSection = config.getConfig().search.safeSearch;
        return safeSearchSection.restricted ? "STRICT" : safeSearchSection.safeSearch
    }
    function saveSafeSearchSetting(val)
    {
        config.setOverrideSetting("search","safeSearch","safeSearch",val)
    }
    WinJS.UI.Pages.define("/shell/html/safesearch.html",{ready: onReady})
})()