﻿(function searchHistoryModule()
{
    "use strict";
    var environment = null;
    var proxy = null;
    var relay = null;
    var navigationManager = null;
    var liveIdAuthenticator = null;
    var facebookAuthenticator = null;
    var networkDetectionService = null;
    var sid = null;
    var historyToggleSetting = "HISTORY_TOGGLE_PREF";
    var isEnableHistoryToggled = false;
    var events = Object.freeze({clearVisitedLinks: "clearVisitedLinks"});
    var historyResponsePatterns = [{
                pattern: "1s*,s*1",
                state: true
            },{
                pattern: "1s*,s*0",
                state: false
            }];
    function onReady(options)
    {
        proxy = options && options.proxy || BingApp.locator.proxy;
        environment = options && options.environment || BingApp.locator.env;
        relay = options && options.relay || BingApp.locator.eventRelay;
        navigationManager = options && options.navigationManager || BingApp.locator.navigationManager;
        liveIdAuthenticator = options && options.liveIdAuthenticator || BingApp.locator.liveIdAuthenticator;
        facebookAuthenticator = options && options.facebookAuthenticator || BingApp.locator.facebookAuthenticator;
        networkDetectionService = options && options.networkDetectionService || BingApp.locator.networkDetectionService;
        WinJS.Resources.processAll();
        var searchHistorySettingsDivElement = document.querySelector("#searchHistorySettingsDiv");
        isEnableHistoryToggled = false;
        var historyEnabled = searchHistorySettingsDivElement.querySelector("#historyEnabled");
        historyEnabled.winControl.checked = isHistoryEnabled();
        historyEnabled.onchange = enableHistoryToggled;
        if(liveIdAuthenticator.isAuthenticated() || facebookAuthenticator.isAuthenticated())
            _currentHistorySetting().done(function currentHistorySettingComplete(enabled)
            {
                if(!isEnableHistoryToggled)
                    historyEnabled.winControl.checked = enabled
            });
        var clearHistoryButtonElement = searchHistorySettingsDivElement.querySelector("#clearHistoryButton");
        clearHistoryButtonElement.onclick = clearHistory;
        searchHistorySettingsDivElement.addEventListener("aftershow",function searchHistory_onAfterShow(event)
        {
            BingApp.traceVerbose("SearchHistory opened")
        })
    }
    function enableHistoryToggled(e)
    {
        var historyEnabledElement = document.querySelector("#historyEnabled");
        var enabled = historyEnabledElement.winControl.checked;
        var setting = enabled ? "toggle_on" : "toggle_off";
        isEnableHistoryToggled = true;
        var historyEnabledFeedback = document.querySelector("#historyEnabledFeedback");
        historyEnabledFeedback.innerText = "";
        var instrumentationObject;
        instrumentationObject = {
            name: "Settings",
            type: "SearchHistoryToggled",
            state: {enabled: enabled}
        };
        BingApp.Instrumentation.logClick(instrumentationObject);
        if(networkDetectionService.isConnected())
            _onSidAvailable().done(function onSidAvailableComplete(sidArg)
            {
                var historyToggleUrl = environment.configuration["omaHistoryUrl"];
                historyToggleUrl = BingApp.Utilities.format(historyToggleUrl,setting,sidArg,"abc");
                var detail = {url: historyToggleUrl};
                proxy.xhr(detail).done(function onProxyRequestComplete(response)
                {
                    if(response && response.status === 200)
                        _setHistoryToggleSetting(enabled);
                    else
                    {
                        if(response)
                            BingApp.traceWarning("searchHistory.enableHistoryToggled failed, status:{0} responseText: {1}",response.status,response.responseText);
                        else
                            BingApp.traceWarning("searchHistory.enableHistoryToggled failed, response:{0}",response);
                        historyEnabledFeedback.innerText = WinJS.Resources.getString("settings_searchhistory_failed_message").value
                    }
                },function onEnableHistoryToggledFailure(error)
                {
                    historyEnabledFeedback.innerText = WinJS.Resources.getString("settings_searchhistory_failed_message").value;
                    var xhrErrorMessage;
                    if(error)
                        xhrErrorMessage = BingApp.Utilities.format("safeSearch.enableHistoryToggled failed, error.status:{0}, error.responseText:{1}",error.status,error.responseText);
                    else
                        xhrErrorMessage = BingApp.Utilities.format("safeSearch.enableHistoryToggled failed, error:{0}",error);
                    var instObject = {
                            url: historyToggleUrl,
                            ei: xhrErrorMessage
                        };
                    BingApp.Instrumentation.logError(instObject);
                    BingApp.traceError(xhrErrorMessage)
                })
            },function onSidAvailableError(err)
            {
                historyEnabledFeedback.innerText = WinJS.Resources.getString("settings_searchhistory_failed_message").value;
                var searchErrorMessage;
                if(err)
                    searchErrorMessage = BingApp.Utilities.format("searchHistory.enableHistoryToggled failed err.message:{0}",err.message);
                else
                    searchErrorMessage = BingApp.Utilities.format("searchHistory.enableHistoryToggled failed err:{0}",err);
                var instObject = {
                        url: environment.configuration["omaHistoryUrl"],
                        ei: searchErrorMessage
                    };
                BingApp.Instrumentation.logError(instObject);
                BingApp.traceError(searchErrorMessage)
            });
        else
        {
            historyEnabledFeedback.innerText = WinJS.Resources.getString("settings_searchhistory_failed_message").value;
            BingApp.traceWarning("searchHistory.enableHistoryToggled failed, because user is not connected to the internet.")
        }
    }
    function _currentHistorySetting()
    {
        return new WinJS.Promise(function init(complete, error)
            {
                _onSidAvailable().done(function sidAvailableCompleted(sidArg)
                {
                    var omaHistoryQueryUrl = environment.configuration["omaHistoryQueryUrl"];
                    omaHistoryQueryUrl = BingApp.Utilities.format(omaHistoryQueryUrl,sidArg,"abc");
                    var detail = {url: omaHistoryQueryUrl};
                    proxy.xhr(detail).done(function onComplete(response)
                    {
                        if(response && response.status === 200)
                        {
                            var enabled = _onlineHistoryState(response.responseText);
                            _setHistoryToggleSetting(enabled);
                            complete(enabled)
                        }
                    },function currentHistorySettingFailed(err)
                    {
                        var xhrErrorMessage;
                        if(err)
                            xhrErrorMessage = BingApp.Utilities.format("safeSearch.currentHistorySetting failed, err.status:{0}, err.responseText:{1}",err.status,err.responseText);
                        else
                            xhrErrorMessage = BingApp.Utilities.format("safeSearch.currentHistorySetting failed, err:{0}",err);
                        var instObject = {
                                url: omaHistoryQueryUrl,
                                ei: xhrErrorMessage
                            };
                        BingApp.Instrumentation.logError(instObject);
                        BingApp.traceError(xhrErrorMessage)
                    })
                },function onSidAvailableError(err)
                {
                    var searchErrorMessage;
                    if(err)
                        searchErrorMessage = BingApp.Utilities.format("searchHistory.currentHistorySetting failed error.message:{0}",err.message);
                    else
                        searchErrorMessage = BingApp.Utilities.format("searchHistory.currentHistorySetting failed error:{0}",err);
                    var instObject = {
                            url: environment.configuration["omaHistoryUrl"],
                            ei: searchErrorMessage
                        };
                    BingApp.Instrumentation.logError(instObject);
                    BingApp.traceError(searchErrorMessage)
                })
            })
    }
    function _onlineHistoryState(data)
    {
        for(var i = 0; i < historyResponsePatterns.length; i++)
        {
            var regexp = new RegExp(historyResponsePatterns[i].pattern,"g");
            if(regexp.test(data))
                return historyResponsePatterns[i].state
        }
        return true
    }
    function isHistoryEnabled()
    {
        return _getHistoryToggleSetting()
    }
    function clearHistory()
    {
        var instrumentationObject;
        instrumentationObject = {
            name: "Settings",
            type: "ClearAllClick"
        };
        BingApp.Instrumentation.logClick(instrumentationObject);
        var clearHistoryFeedback = document.querySelector("#clearHistoryFeedback");
        var searchHistorySettings = document.querySelector("#searchHistorySettingsDiv");
        if(networkDetectionService.isConnected())
        {
            navigationManager.clearHistory();
            var enableLightDismiss = function()
                {
                    BingApp.Utilities.setLightDismiss(searchHistorySettings,true)
                };
            BingApp.Utilities.setLightDismiss(searchHistorySettings,false);
            _onSidAvailable().then(function onSidAvailableComplete(sidArg)
            {
                var clearHistoryUrl = environment.configuration["omaHistoryUrl"];
                clearHistoryUrl = BingApp.Utilities.format(clearHistoryUrl,"clear",sidArg,"abc");
                var detail = {url: clearHistoryUrl};
                proxy.xhr(detail).done(function onProxyRequestComplete(response)
                {
                    if(response && response.status === 200)
                    {
                        BingApp.traceInfo("searchHistory.clearHistory success, status:{0} response: {1}",response.status,response.responseText);
                        clearHistoryFeedback.innerText = WinJS.Resources.getString("settings_searchhistory_clear_succeeded_message").value;
                        WinJS.Utilities.removeClass(clearHistoryFeedback,"clearHistoryFeedbackError")
                    }
                    else
                    {
                        if(response)
                            BingApp.traceWarning("searchHistory.clearHistory failed, status:{0} responseText: {1}",response.status,response.responseText);
                        else
                            BingApp.traceWarning("searchHistory.clearHistory failed, response:{0}",response);
                        clearHistoryFeedback.innerText = WinJS.Resources.getString("settings_searchhistory_failed_message").value;
                        WinJS.Utilities.addClass(clearHistoryFeedback,"clearHistoryFeedbackError")
                    }
                },function onProxyRequestError(error)
                {
                    if(error)
                        if(BingApp.Utilities.isPromiseCancellationError(error))
                            BingApp.traceError("searchHistory.clearHistory failed. The error indicates that proxy.xhr request may have been cancelled due to timeout");
                        else
                            BingApp.traceError("searchHistory.clearHistory failed error.status:{0}, error.responseText:{1}",error.status,error.responseText);
                    else
                        BingApp.traceError("searchHistory.clearHistory failed error:{0}",error);
                    clearHistoryFeedback.innerText = WinJS.Resources.getString("settings_searchhistory_failed_message").value;
                    WinJS.Utilities.addClass(clearHistoryFeedback,"clearHistoryFeedbackError")
                });
                relay.fireEvent(events.clearVisitedLinks)
            },function onSidAvailableError(err)
            {
                var searchErrorMessage = BingApp.Utilities.format("searchHistory.clearHistory failed Error:{0}",err);
                var instObject = {
                        url: environment.configuration["omaHistoryUrl"],
                        ei: searchErrorMessage
                    };
                BingApp.Instrumentation.logError(instObject);
                BingApp.traceError(searchErrorMessage);
                clearHistoryFeedback.innerText = WinJS.Resources.getString("settings_searchhistory_failed_message").value;
                WinJS.Utilities.addClass(clearHistoryFeedback,"clearHistoryFeedbackError")
            }).done(enableLightDismiss)
        }
        else
        {
            clearHistoryFeedback.innerText = WinJS.Resources.getString("settings_searchhistory_failed_message").value;
            WinJS.Utilities.addClass(clearHistoryFeedback,"clearHistoryFeedbackError");
            BingApp.traceWarning("searchHistory.clearHistory failed, because user is not connected to the internet.")
        }
    }
    function _setHistoryToggleSetting(val)
    {
        if(typeof val !== "boolean")
            throw new BingApp.Classes.ErrorArgument("val","Invalid historytoggle setting");
        var localSettings = Windows.Storage.ApplicationData.current.localSettings;
        localSettings.values[historyToggleSetting] = val
    }
    function _getHistoryToggleSetting()
    {
        var localSettings = Windows.Storage.ApplicationData.current.localSettings;
        var val = localSettings.values[historyToggleSetting];
        if(typeof val === "boolean")
            return val;
        return true
    }
    function _onSidAvailable()
    {
        if(sid)
            return new WinJS.Promise.as(sid);
        return new WinJS.Promise(function init(complete, error)
            {
                proxy.getCookie("_SS","SID").done(function onGetCookieComplete(cookie)
                {
                    if(cookie)
                    {
                        sid = cookie;
                        complete(sid)
                    }
                    else
                        error(new WinJS.ErrorFromName("Unknown","proxy.getCookie completed without a cookie value"))
                },function onGetCookieFailed(err)
                {
                    if(!err)
                        err = new WinJS.ErrorFromName("Unknown","proxy.getCookie failed with unknown error.");
                    error(err);
                    BingApp.traceError("_onSidAvailable failed :getCookie Error:{0}",err)
                })
            })
    }
    function resetState()
    {
        sid = null
    }
    WinJS.UI.Pages.define("/shell/html/searchhistory.html",{
        ready: onReady,
        resetState: resetState
    });
    WinJS.Namespace.define("BingApp.SearchHistory",{events: events})
})()