﻿(function searchPaneAutoSuggestModule()
{
    "use strict";
    var config = null;
    var eventRelay = null;
    var events = Object.freeze({
            querySubmitted: "querysubmitted",
            suggestionsRequested: "suggestionsrequested",
            resultSuggestionChosen: "resultsuggestionchosen",
            searchBarSuggestionsResponded: "searchBar:suggestionsResponded",
            searchBarQuerySubmitted: "searchBar:querySubmitted",
            searchBarResultSuggestionChosen: "searchBar:resultSuggestionChosen",
            searchBarSuggestionsRequested: "searchBar:suggestionsRequested",
            searchBarUpdatedQuery: "searchbar:updatedQuery"
        });
    var localSettings = new Windows.ApplicationModel.Search.LocalContentSuggestionSettings;
    localSettings.enabled = false;
    var searchPane = Windows.ApplicationModel.Search.SearchPane.getForCurrentView(),
        registered = false,
        onSuggestionsRequested,
        onResultSuggestionChosen,
        onQuerySubmitted,
        callbackCount = 0;
    searchPane.setLocalContentSuggestionSettings(localSettings);
    searchPane.searchHistoryEnabled = true;
    function register(options)
    {
        if(!registered)
        {
            config = options && options.config || BingApp.locator.config;
            eventRelay = options && options.eventRelay || BingApp.locator.eventRelay;
            onSuggestionsRequested = createSuggestionsRequested();
            onResultSuggestionChosen = createOnResultSuggestionChosen();
            onQuerySubmitted = createOnQuerySubmitted();
            searchPane.addEventListener(events.querySubmitted,onQuerySubmitted,false);
            searchPane.addEventListener(events.suggestionsRequested,onSuggestionsRequested,false);
            searchPane.addEventListener(events.resultSuggestionChosen,onResultSuggestionChosen,false);
            eventRelay.addEventListener(events.searchBarSuggestionsResponded,{callback: searchBarSuggestionRespondedHandler});
            if(searchPane.trySetQueryText)
                eventRelay.addEventListener(events.searchBarUpdatedQuery,searchBarUpdatedQueryHandler);
            registered = true
        }
    }
    function unregister()
    {
        if(registered)
        {
            searchPane.removeEventListener(events.querySubmitted,onQuerySubmitted);
            searchPane.removeEventListener(events.suggestionsRequested,onSuggestionsRequested,false);
            searchPane.removeEventListener(events.resultSuggestionChosen,onResultSuggestionChosen,false);
            eventRelay.removeEventListener(events.searchBarSuggestionsResponded,{callback: searchBarSuggestionRespondedHandler});
            eventRelay.removeEventListener(events.searchBarUpdatedQuery,{callback: searchBarUpdatedQueryHandler});
            registered = false
        }
    }
    function createOnQuerySubmitted()
    {
        return function onQuerySubmittedHandler(e)
            {
                var query = e.queryText.trim();
                BingApp.traceInfo("BingApp.SearchPaneHandler.onQuerySubmitted: handled querysubmitted '{0}'.",query);
                eventRelay.fireEvent(events.searchBarQuerySubmitted,{query: query})
            }
    }
    function createOnResultSuggestionChosen()
    {
        return function onResultSuggestionChosenHandler(e)
            {
                if(config.getConfig().autosuggest.searchPane.enabled)
                {
                    BingApp.traceInfo("BingApp.SearchPaneHandler.onResultSuggestionChosen: handled resultsuggestionchosen. Tag: '{0}'.",e.tag);
                    var tagData;
                    try
                    {
                        tagData = e.tag ? JSON.parse(e.tag) : null
                    }
                    catch(err)
                    {
                        BingApp.traceError("BingApp.SearchPaneHandler.onResultSuggestionChosen: error parsing Tag Data Tag: '{0}' Error: '{1}'.",e.tag,err.toString());
                        tagData = null
                    }
                    eventRelay.fireEvent(events.searchBarResultSuggestionChosen,{tagData: tagData})
                }
            }
    }
    function searchBarSuggestionRespondedHandler(payload)
    {
        if(!payload || !payload.callbackId)
        {
            BingApp.traceError("BingApp.SearchPaneHandler.searchBarSuggestionsResponded: bad arguments.");
            return
        }
        var callbackData = BingApp.SearchPaneHandler.callbackData;
        while(callbackData.length > 0)
        {
            var item = callbackData.shift();
            if(!item || !item.id || !item.cb)
            {
                var errorMessage = BingApp.Utilities.format("BingApp.SearchPaneHandler.searchBarSuggestionsResponded: queue ended up with invalid item. Query: '{0}'. id: '{1}'.",payload.query,payload.callbackId);
                var instObject = {ei: errorMessage};
                BingApp.Instrumentation.logError(instObject);
                BingApp.traceError(errorMessage);
                BingApp.SearchPaneHandler.callbackData = [];
                return
            }
            if(item.id > payload.callbackId)
            {
                callbackData.unshift(item);
                return
            }
            if(item.id === payload.callbackId)
            {
                item.cb(payload);
                return
            }
        }
    }
    function searchBarUpdatedQueryHandler(payload)
    {
        if(payload)
        {
            BingApp.traceInfo("SearchPaneHandler.searchBarUpdatedQueryHandler: received '{0}' message with query '{1}' which is relayed from web compartment. 'Trying' to set the QueryText for searchPane.",events.searchBarUpdatedQuery,payload.queryText);
            searchPane.trySetQueryText(payload.queryText)
        }
    }
    function fireAndHandleEvents(e)
    {
        callbackCount++;
        var promise = new WinJS.Promise(function init(complete, error, progress)
            {
                var deferral = e.request.getDeferral();
                eventRelay.fireEvent(events.searchBarSuggestionsRequested,{
                    query: e.queryText,
                    callbackId: callbackCount
                });
                BingApp.SearchPaneHandler.callbackData.push({
                    id: callbackCount,
                    cb: function onComplete(data)
                    {
                        complete({
                            suggestions: data.suggestions,
                            searchSuggestionCollection: e.request.searchSuggestionCollection
                        });
                        deferral.complete()
                    }
                })
            });
        return promise
    }
    function createSuggestionsRequested()
    {
        return function onSuggestionsRequestedHandler(e)
            {
                if(config.getConfig().autosuggest.searchPane.enabled)
                {
                    BingApp.traceInfo("BingApp.SearchPaneHandler.onSuggestionsRequested for query: '{0}'.",e.queryText);
                    fireAndHandleEvents(e).then(function fulfilled(data)
                    {
                        for(var i in data.suggestions)
                        {
                            var suggestion = data.suggestions[i];
                            if(suggestion.type === "query")
                            {
                                if(suggestion.query)
                                    data.searchSuggestionCollection.appendQuerySuggestion(suggestion.query)
                            }
                            else if(suggestion.type === "result")
                            {
                                if(suggestion.text && suggestion.detailText && suggestion.imageUri && suggestion.tagData)
                                    data.searchSuggestionCollection.appendResultSuggestion(suggestion.text,suggestion.detailText,JSON.stringify(suggestion.tagData),Windows.Storage.Streams.RandomAccessStreamReference.createFromUri(new Windows.Foundation.Uri(suggestion.imageUri)),suggestion.imageAlternateText)
                            }
                            else if(suggestion.type === "line")
                                if(suggestion.label)
                                    data.searchSuggestionCollection.appendSearchSeparator(suggestion.label)
                        }
                    })
                }
            }
    }
    WinJS.Namespace.define("BingApp.SearchPaneHandler",{
        register: register,
        unregister: unregister,
        callbackData: [],
        isRegistered: function()
        {
            return registered
        },
        events: events
    })
})()