﻿(function serviceLocatorModule()
{
    "use strict";
    var configInstance = BingApp.Classes.Config.createInstance();
    var environmentInstance = new BingApp.Classes.Env({config: configInstance});
    var eventRelayInstance = new BingApp.Classes.GlobalEventRelay(window);
    var navigationManagerInstance = new BingApp.Classes.NavigationManager("/shell/configuration/navigationmappings.json");
    var viewManagerInstance = new BingApp.Classes.ViewManager("/shell/configuration/viewregistry.json");
    var shellInstance = new BingApp.Classes.Shell(WinJS.Application);
    var networkDetectionService = new BingApp.Classes.NetworkDetectionService;
    var progressIndicatorServiceInstance = new BingApp.Classes.ProgressIndicatorService(eventRelayInstance);
    var proxyInstance = new BingApp.Classes.Proxy(new BingApp.Classes.ProxyFrameContainer(configInstance.getConfig().app.general.host,"BingProxy",{
            eventRelay: eventRelayInstance,
            networkDetectionService: networkDetectionService
        }),{config: configInstance});
    var secureProxyInstance = new BingApp.Classes.Proxy(new BingApp.Classes.ProxyFrameContainer(configInstance.getConfig().app.general.secureHost,"SecureBingProxy",{
            eventRelay: eventRelayInstance,
            networkDetectionService: networkDetectionService
        }),{config: configInstance});
    var liveIdAuthenticatorInstance = new BingApp.Classes.LiveIdAuthenticator({
            config: configInstance,
            secureProxy: secureProxyInstance
        });
    var liveIdAuthenticationRelayInstance = new BingApp.Classes.LiveIdAuthenticationRelay({
            authenticator: liveIdAuthenticatorInstance,
            config: configInstance,
            networkDetectionService: networkDetectionService,
            relay: eventRelayInstance
        });
    var facebookAuthenticatorInstance;
    if(configInstance.getConfig().authentication.facebook.enabled)
        facebookAuthenticatorInstance = new BingApp.Classes.FacebookAuthenticator({
            config: configInstance,
            secureProxy: secureProxyInstance,
            env: environmentInstance
        });
    else
        facebookAuthenticatorInstance = new BingApp.Classes.DisabledAuthenticator;
    var facebookAuthenticationRelayInstance = new BingApp.Classes.FacebookAuthenticationRelay({
            authenticator: facebookAuthenticatorInstance,
            config: configInstance,
            networkDetectionService: networkDetectionService,
            relay: eventRelayInstance
        });
    var geoLocationServiceInstance = new BingApp.Classes.GeoLocationService({
            proxy: proxyInstance,
            config: configInstance
        });
    var locator = {};
    Object.defineProperties(locator,{
        env: {
            value: environmentInstance,
            writable: false,
            enumerable: false,
            configurable: false
        },
        eventRelay: {
            value: eventRelayInstance,
            writable: false,
            enumerable: false,
            configurable: false
        },
        shell: {
            value: shellInstance,
            writable: false,
            enumerable: false,
            configurable: false
        },
        navigationManager: {
            value: navigationManagerInstance,
            writable: false,
            enumerable: false,
            configurable: false
        },
        viewManager: {
            value: viewManagerInstance,
            writable: false,
            enumerable: false,
            configurable: false
        },
        progressIndicatorService: {
            value: progressIndicatorServiceInstance,
            writable: false,
            enumerable: false,
            configurable: false
        },
        networkDetectionService: {
            value: networkDetectionService,
            writable: false,
            enumerable: false,
            configurable: false
        },
        proxy: {
            value: proxyInstance,
            writable: false,
            enumerable: false,
            configurable: false
        },
        secureProxy: {
            value: secureProxyInstance,
            writable: false,
            enumerable: false,
            configurable: false
        },
        config: {
            value: configInstance,
            writeable: false,
            enumerable: false,
            configurable: false
        },
        liveIdAuthenticator: {
            value: liveIdAuthenticationRelayInstance,
            writeable: false,
            enumerable: false,
            configurable: false
        },
        facebookAuthenticator: {
            value: facebookAuthenticationRelayInstance,
            writeable: false,
            enumerable: false,
            configurable: false
        },
        geoLocationService: {
            value: geoLocationServiceInstance,
            writable: false,
            enumerable: false,
            configurable: false
        }
    });
    WinJS.Namespace.define("BingApp",{locator: locator});
    BingApp.Tracing.init();
    setImmediate(function attemptSilentSignInLiveId()
    {
        BingApp.locator.liveIdAuthenticator.trySilentSignIn()
    });
    setImmediate(function attemptSilentSignInFacebook()
    {
        BingApp.locator.facebookAuthenticator.trySilentSignIn()
    });
    setImmediate(function initializeGeoLocationService()
    {
        BingApp.locator.geoLocationService.init()
    })
})()