﻿(function settingsModule()
{
    "use strict";
    var safeSearchDiv = "safeSearchSettingsDiv";
    var safeSearchPath = "/shell/html/safesearch.html";
    var accountsDiv = "accountsSettingsDiv";
    var accountsPath = "/shell/html/accounts.html";
    var events = Object.freeze({
            showSafeSearch: "showSafeSearchSettings",
            showAccounts: "showAccountsSettings"
        });
    var SettingsPaneHandler = WinJS.Class.define(function settingsPaneHandlerConstructor(options)
        {
            this._application = options && options.application || WinJS.Application;
            this._config = options && options.config || BingApp.locator.config;
            this._eventRelay = options && options.eventRelay || BingApp.locator.eventRelay;
            this._settingsFlyout = options && options.settingsFlyout || WinJS.UI.SettingsFlyout;
            this._settingsPane = options && options.settingsPane || Windows.UI.ApplicationSettings.SettingsPane.getForCurrentView();
            this._registerEventHandlers()
        },{
            _registerEventHandlers: function()
            {
                this._onSettingsClosure = BingApp.Utilities.createClosure(this,this._onSettingsHandler);
                this._application.addEventListener("settings",this._onSettingsClosure,false);
                this._onShowSafeSearchClosure = BingApp.Utilities.createClosure(this,this._onShowSafeSearchHandler);
                this._eventRelay.addEventListener(events.showSafeSearch,this._onShowSafeSearchClosure);
                this._onShowAccountsClosure = BingApp.Utilities.createClosure(this,this._onShowAccountsHandler);
                this._eventRelay.addEventListener(events.showAccounts,this._onShowAccountsClosure);
                this._onCommandsRequestedClosure = BingApp.Utilities.createClosure(this,this._onCommandsRequestedHandler);
                this._settingsPane.addEventListener("commandsrequested",this._onCommandsRequestedClosure)
            },
            _unregisterEventHandlers: function()
            {
                this._application.removeEventListener("settings",this._onSettingsClosure);
                this._eventRelay.removeEventListener(events.showSafeSearch,this._onShowSafeSearchClosure);
                this._eventRelay.removeEventListener(events.showAccounts,this._onShowAccountsClosure);
                this._settingsPane.removeEventListener("commandsrequested",this._onCommandsRequestedClosure)
            },
            _onSettingsHandler: function(e)
            {
                var commands = {};
                var settingsPaneSection = this._config.getConfig().settingsPane;
                commands["accountsSettingsDiv"] = {
                    title: WinJS.Resources.getString("settings_accounts_title").value,
                    href: "/shell/html/accounts.html"
                };
                commands[safeSearchDiv] = {
                    title: WinJS.Resources.getString("settings_safesearch_title").value,
                    href: safeSearchPath
                };
                if(this._config.getConfig().searchHistory.general.enabled)
                    commands["searchHistorySettingsDiv"] = {
                        title: WinJS.Resources.getString("settings_searchhistory_title").value,
                        href: "/shell/html/searchhistory.html"
                    };
                if(!settingsPaneSection.internetContentProvider.showMessage && this._config.getConfig().settingsPane.testConfiguration.enabled && !settingsPaneSection.impressum.enabled)
                    commands["configSettingsDiv"] = {
                        title: WinJS.Resources.getString("settings_configuration_title").value,
                        href: "/shell/html/configuration.html"
                    };
                e.detail.applicationcommands = commands;
                this._settingsFlyout.populateSettings(e)
            },
            _onShowSafeSearchHandler: function()
            {
                this._settingsFlyout.showSettings(safeSearchDiv,safeSearchPath)
            },
            _onShowAccountsHandler: function()
            {
                this._settingsFlyout.showSettings(accountsDiv,accountsPath)
            },
            _onCommandsRequestedHandler: function(e)
            {
                var settingsPaneSection = this._config.getConfig().settingsPane;
                var appSettings = Windows.UI.ApplicationSettings;
                var commands = e.request.applicationCommands;
                var privacy = new appSettings.SettingsCommand("privacy",WinJS.Resources.getString("settings_link_privacy").value,function openTOUPrivacyLink()
                    {
                        BingApp.Utilities.invokeURI(settingsPaneSection.termsOfUse.privacyStatementUrl)
                    });
                commands.append(privacy);
                var terms = new appSettings.SettingsCommand("terms",WinJS.Resources.getString("settings_link_tou").value,function openTOUServiceLink()
                    {
                        var marketStrings = new Windows.ApplicationModel.Resources.ResourceLoader("market");
                        BingApp.Utilities.invokeURI(marketStrings.getString("TOU_SERVICE_AGREEMENT_URL"))
                    });
                commands.append(terms);
                var help = new appSettings.SettingsCommand("help",WinJS.Resources.getString("settings_link_help").value,BingApp.Help.onHelpCmd);
                commands.append(help);
                var feedback = new appSettings.SettingsCommand("feedback",WinJS.Resources.getString("settings_link_feedback").value,CommonJS.Feedback.onFeedbackCmd);
                commands.append(feedback);
                if(settingsPaneSection.impressum.enabled)
                {
                    var impressum = new appSettings.SettingsCommand("impressum",WinJS.Resources.getString("settings_link_impressum").value,function openImpressumLink()
                        {
                            BingApp.Utilities.invokeURI(settingsPaneSection.impressum.url)
                        });
                    commands.append(impressum)
                }
                if(settingsPaneSection.internetContentProvider.showMessage)
                {
                    var icp = new appSettings.SettingsCommand("icp",WinJS.Resources.getString("internet_content_provider").value,function icpNoImplementation(){});
                    commands.append(icp)
                }
            },
            unload: function()
            {
                this._unregisterEventHandlers()
            }
        });
    WinJS.Namespace.define("BingApp.Classes",{SettingsPaneHandler: SettingsPaneHandler});
    WinJS.Namespace.define("BingApp.Settings",{
        init: function(options)
        {
            return new BingApp.Classes.SettingsPaneHandler(options)
        },
        events: events
    })
})()