﻿(function shareSourceModule()
{
    "use strict";
    var ShareData = WinJS.Class.define(function constructor(options)
        {
            this._config = options && options.config || BingApp.locator.config;
            Object.defineProperties(this,{
                title: {
                    writable: true,
                    enumerable: true,
                    configurable: false
                },
                description: {
                    writable: true,
                    enumerable: true,
                    configurable: false
                },
                uri: {
                    writable: true,
                    enumerable: true,
                    configurable: false
                },
                text: {
                    writable: true,
                    enumerable: true,
                    configurable: false
                },
                thumbnail: {
                    writable: true,
                    enumerable: true,
                    configurable: false
                },
                bitmap: {
                    writable: true,
                    enumerable: true,
                    configurable: false
                },
                storageItems: {
                    value: [],
                    writable: true,
                    enumerable: true,
                    configurable: false
                },
                markup: {
                    writable: true,
                    enumerable: true,
                    configurable: false
                },
                bitmapDataProvider: {
                    writable: true,
                    enumerable: true,
                    configurable: false
                }
            });
            this.setDefaults()
        },{
            setDefaults: function()
            {
                var defaultHost = this._config.getConfig().app.general.host;
                var collection = {};
                collection["form"] = BingApp.Classes.Shell.formCodes.fromShareCharmDefault;
                this.uri = new Windows.Foundation.Uri(defaultHost + BingApp.Utilities.QueryString.serialize(collection));
                this.text = null;
                this.thumbnail = null;
                this.bitmap = null;
                this.storageItems = [];
                this.markup = null;
                this.bitmapDataProvider = null;
                this.title = WinJS.Resources.getString("app_name").value;
                this.description = WinJS.Resources.getString("app_description_short").value
            },
            isValid: function()
            {
                return this.title && this.description && (this.uri instanceof Windows.Foundation.Uri || this.text || this.thumbnail || this.bitmap || Array.isArray(this.storageItems) && this.storageItems.length > 0 || this.markup || typeof this.bitmapDataProvider === "function")
            }
        });
    WinJS.Namespace.define("BingApp.Classes",{ShareData: ShareData});
    var _shareData;
    var _dataTransferManager;
    function init()
    {
        checkDataTransferManager()
    }
    function checkDataTransferManager()
    {
        if(!_dataTransferManager)
        {
            _dataTransferManager = Windows.ApplicationModel.DataTransfer.DataTransferManager.getForCurrentView();
            if(_dataTransferManager)
                _dataTransferManager.ondatarequested = dataRequestedHandler
        }
    }
    function validateShare()
    {
        if(!_shareData)
            _shareData = new ShareData;
        if(!_dataTransferManager)
            checkDataTransferManager();
        if(!_shareData.isValid())
            _shareData.setDefaults()
    }
    function addShareSources(shareData)
    {
        if(!(shareData instanceof BingApp.Classes.ShareData))
            throw new BingApp.Classes.ErrorArgument("shareData");
        if(shareData.isValid())
        {
            checkDataTransferManager();
            _shareData = shareData
        }
        else
            _shareData.setDefaults()
    }
    function showShareUI()
    {
        validateShare();
        Windows.ApplicationModel.DataTransfer.DataTransferManager.showShareUI()
    }
    function dataRequestedHandler(eventArgs)
    {
        var dataRequestPromise = null;
        if(typeof BingApp.ShareSource.ondatarequested === "function")
            dataRequestPromise = BingApp.ShareSource.ondatarequested();
        var request = eventArgs.request;
        var deferral = request.getDeferral();
        dataRequestPromise = WinJS.Promise.as(dataRequestPromise);
        dataRequestPromise.done(function onCompleted()
        {
            validateShare();
            if(request && request.data && request.data.properties)
            {
                request.data.properties.title = _shareData.title;
                request.data.properties.description = _shareData.description;
                if(BingApp.Utilities.isNotNullOrUndefined(_shareData.uri))
                    request.data.setUri(_shareData.uri);
                if(_shareData.text)
                    request.data.setText(_shareData.text);
                if(_shareData.thumbnail)
                    request.data.properties.thumbnail = _shareData.thumbnail;
                if(_shareData.bitmap)
                    request.data.setBitmap(_shareData.bitmap);
                if(Array.isArray(_shareData.storageItems) && _shareData.storageItems.length > 0)
                    request.data.setStorageItems(_shareData.storageItems);
                if(_shareData.markup)
                {
                    var htmlFormat = Windows.ApplicationModel.DataTransfer.HtmlFormatHelper.createHtmlFormat(window.toStaticHTML(_shareData.markup));
                    request.data.setHtmlFormat(htmlFormat)
                }
                if(typeof _shareData.bitmapDataProvider === "function")
                    request.data.setDataProvider(Windows.ApplicationModel.DataTransfer.StandardDataFormats.bitmap,_shareData.bitmapDataProvider)
            }
            deferral.complete()
        })
    }
    WinJS.Namespace.define("BingApp.ShareSource",{
        init: init,
        ondatarequested: null,
        addShareSources: addShareSources,
        showShareUI: showShareUI
    });
    BingApp.ShareSource.init()
})()