﻿(function shareSourceImageDataCacheModule()
{
    "use strict";
    var _cacheFromIframe;
    var _deferredImageLoadData = {};
    var eventRelay = BingApp.locator.eventRelay;
    var events = Object.freeze({shareSourceData: "ImageShareSourceData"});
    function consume()
    {
        if(!_cacheFromIframe)
            return;
        var data = _cacheFromIframe;
        if(!data.title || !data.description)
        {
            _cacheFromIframe = null;
            return
        }
        var url = data.url && prepareShareUrl(data.url,data.form) || data.imgurl;
        var uri = null;
        if(url)
            try
            {
                uri = new Windows.Foundation.Uri(url)
            }
            catch(err)
            {
                BingApp.traceError("ShareSourceImageDataCache.consume: Error creating a URI from '{0}'; fails with message = {1}.",url,err.message)
            }
        if(data.thurl)
            _deferredImageLoadData.id = data.thurl;
        var shareData = new BingApp.Classes.ShareData;
        shareData.title = data.title;
        shareData.description = data.description;
        shareData.uri = uri;
        shareData.text = data.text;
        shareData.bitmap = data.bitmap;
        shareData.storageitems = data.storageitems;
        shareData.markup = data.markup;
        shareData.bitmapDataProvider = _deferredImageLoadData.id ? onDeferredImageRequested : null;
        BingApp.ShareSource.addShareSources(shareData);
        _cacheFromIframe = null
    }
    function prepareShareUrl(url, formCode)
    {
        if(!url)
            return null;
        var uri = new Windows.Foundation.Uri(url);
        var queryParsed = BingApp.Utilities.QueryString.parse(url);
        var query = queryParsed ? queryParsed["q"] || queryParsed["Q"] : null;
        var updatedUrl = uri.schemeName + "://" + uri.host + (uri.port === 80 ? "" : ":" + uri.port) + uri.path;
        var queryParameters = [];
        if(query)
            queryParameters.push("q=" + query);
        if(formCode)
            queryParameters.push("form=" + formCode);
        var queryString = queryParameters.join("&");
        return updatedUrl + (queryString ? "?" + queryString : "")
    }
    function save(data)
    {
        _cacheFromIframe = data;
        if(_deferredImageLoadData.id)
        {
            delete _deferredImageLoadData.id;
            deferralComplete("Discarding existing deferral as new data is being shared")
        }
        BingApp.ShareSource.ondatarequested = consume
    }
    function onDeferredImageRequested(request)
    {
        _deferredImageLoadData.deferral = request.getDeferral();
        if(!_deferredImageLoadData.id)
        {
            deferralComplete();
            return
        }
        try
        {
            var thUri = new Windows.Foundation.Uri(_deferredImageLoadData.id);
            var thStream = Windows.Storage.Streams.RandomAccessStreamReference.createFromUri(thUri);
            if(thStream)
                request.setData(thStream);
            deferralComplete()
        }
        catch(ex)
        {
            deferralComplete(ex)
        }
    }
    function deferralComplete(errorMessage)
    {
        if(_deferredImageLoadData.deferral && _deferredImageLoadData.deferral.complete)
        {
            _deferredImageLoadData.deferral.complete();
            delete _deferredImageLoadData.deferral
        }
        if(errorMessage)
            if(errorMessage.message)
                BingApp.traceError("ShareSourceImageDataCache.deferralComplete {0}",errorMessage.message);
            else
                BingApp.traceError("ShareSourceImageDataCache.deferralComplete Message: {0}",errorMessage)
    }
    eventRelay.addEventListener(events.shareSourceData,{callback: function onShareSourceData(payload)
        {
            save(payload)
        }})
})()