﻿(function shellModule()
{
    "use strict";
    WinJS.Binding.optimizeBindingReferences = true;
    var navigationRoot = "bingsearch://app";
    var showSafeSearchPath = "settings/safesearch";
    var uriPaths = Object.freeze({
            home: "/home",
            start: "/start",
            search: "/search",
            imagesgrid: "/images/grid",
            rewards: "/rewards",
            videosgrid: "/videos/grid"
        });
    var uris = Object.freeze({
            home: navigationRoot + uriPaths.home,
            start: navigationRoot + uriPaths.start,
            search: navigationRoot + uriPaths.search,
            imagesgrid: navigationRoot + uriPaths.imagesgrid,
            rewards: navigationRoot + uriPaths.rewards,
            videosgrid: navigationRoot + uriPaths.videosgrid
        });
    var webPaths = Object.freeze({
            search: "/search",
            imagesgrid: "/images/search",
            videosgrid: "/videos/search"
        });
    var webPathsMap = Object.keys(webPaths).reduce(function webPathsMapper(memo, key)
        {
            memo[webPaths[key]] = key;
            return memo
        },{});
    var ieArgumentPrefix = "META:";
    var iePaths = Object.freeze({
            home: "/",
            search: "/search",
            imagesgrid: "/images/search",
            videosgrid: "/videos/search"
        });
    var iePathsMap = Object.keys(iePaths).reduce(function iePathsMapper(memo, key)
        {
            memo[iePaths[key]] = key;
            return memo
        },{});
    var formCodes = Object.freeze({
            fromLiveTile: "ENTLTL",
            fromIE: "ENIEW8",
            fromProtocol: "ENPCW8",
            fromBackButton: "ENBBW8",
            fromSearchCharmActivation: "QSBW8P",
            fromSearchCharmQuery: "QSBW8A",
            fromSearchCharmSuggestion: "ASAPIB",
            fromShareCharmDefault: "QRDSW8",
            fromShareCharmStart: "QRSTW8",
            fromShareCharmTrending: "QRTSW8",
            fromShareCharmRecent: "QRRSW8",
            fromShareCharmSerp: "QRSSW8",
            fromShareCharmImages: "QRISW8",
            fromShareCharmImagesSubviews: "QRIDW8",
            fromTrendingSearch: "W8SPTR",
            fromHomePopularNow: "QRPNW8",
            fromHotspot: "SPHOT",
            fromVoiceOnHomePage: "QVCHW8",
            fromVoiceOnTrendingPage: "QVCTW8",
            fromVoiceOnSERPPage: "QVCSW8",
            fromVoiceOnImageGridPage: "QVCIW8",
            fromVoiceOnVideoGridPage: "QVCVW8"
        });
    var testFormCodes = {};
    Object.keys(formCodes).forEach(function replaceKey(key)
    {
        testFormCodes[key] = "MONITR"
    });
    testFormCodes = Object.freeze(testFormCodes);
    var activationKind = Object.freeze({
            launch: "launch",
            ieLaunch: "ielaunch",
            search: "search",
            protocol: "protocol"
        });
    var events = Object.freeze({
            invokeApp: "invokeApp",
            unsnap: "viewUnsnap",
            appResuming: "app:resuming",
            appVisible: "app:visible"
        });
    var version = Windows.ApplicationModel.Package.current.id.version;
    var fullVersion = [version.major,version.minor,version.build,version.revision].join(".");
    var lookupSupportedUris = Object.keys(uris).reduce(function supportedUrisMapper(memo, key)
        {
            memo[uris[key]] = key;
            return memo
        },{});
    var lookupNeedSearchQueryUris = Object.keys(uris).reduce(function needSearchQueryUrisMapper(memo, key)
        {
            var uri = uris[key];
            if(uri === uris.search || uri === uris.imagesgrid || uri === uris.videosgrid)
                memo[uri] = key;
            return memo
        },{});
    var Shell = WinJS.Class.define(function constructor(app, windowObject)
        {
            if(!(this instanceof BingApp.Classes.Shell))
            {
                BingApp.traceWarning("Shell.ctor: attempted using Shell ctor as function; redirecting to use 'new Shell()'.");
                return new BingApp.Classes.Shell(app,windowObject)
            }
            windowObject = windowObject || window;
            Object.defineProperty(this,"app",{
                value: app,
                writable: false,
                enumerable: false,
                configurable: false
            });
            Object.defineProperty(this,"appWindow",{
                value: windowObject,
                writable: false,
                enumerable: false,
                configurable: false
            });
            if(app)
            {
                var that = this;
                document.addEventListener("visibilitychange",function visibilityChanged(eventData)
                {
                    var hidden = document.hidden;
                    if(!hidden)
                    {
                        BingApp.Instrumentation.log({Name: "AppResume"},BingApp.Instrumentation.appResume);
                        BingApp.locator.eventRelay.fireEvent(events.appVisible)
                    }
                    BingApp.traceInfo("Shell._visibilitychange. State: " + !hidden)
                });
                Windows.UI.WebUI.WebUIApplication.addEventListener("resuming",function _resumeHandler(eventData)
                {
                    BingApp.traceInfo("Shell._resumeHandler: Resuming the app.");
                    BingApp.Instrumentation.resumeLogging()
                });
                app.onactivated = function onActivated(eventData)
                {
                    that._lastActivationData = eventData;
                    var previousState = eventData.detail.previousExecutionState;
                    var coldStart = BingApp.Utilities.isColdStart(previousState);
                    if(coldStart)
                        BingApp.traceInfo("Shell.onactivated: Bing App is activated via 'cold' start (previous execution state is '{0}').",previousState);
                    else
                        BingApp.traceInfo("Shell.onactivated: Bing App is activated without 'cold' start (previous execution state is '{0}').",previousState);
                    if(coldStart)
                    {
                        that._coldStartActivationData = eventData;
                        var systemSplashScreen = eventData.detail.splashScreen;
                        if(systemSplashScreen)
                        {
                            that._updateSplashScreenDisplay = function updateSplashScreenDisplay()
                            {
                                var splashScreenImageElement = document.querySelector("#splashscreenimage");
                                if(splashScreenImageElement)
                                {
                                    splashScreenImageElement.style.left = systemSplashScreen.imageLocation.x + "px";
                                    splashScreenImageElement.style.top = systemSplashScreen.imageLocation.y + "px";
                                    splashScreenImageElement.style.width = systemSplashScreen.imageLocation.width + "px";
                                    splashScreenImageElement.style.height = systemSplashScreen.imageLocation.height + "px"
                                }
                            };
                            that._updateSplashScreenDisplay();
                            that.appWindow.addEventListener("resize",that._updateSplashScreenDisplay,false);
                            var callbackSplashScreenDismissed = function onSystemSplashScreenDismissed()
                                {
                                    BingApp.traceInfo("Shell.onSystemSplashScreenDismissed: system splash screen has been dismissed; started showing {0} now.",that.showingSecondarySplashScreen ? "secondary splash screen" : "content");
                                    systemSplashScreen.ondismissed = null
                                };
                            systemSplashScreen.ondismissed = callbackSplashScreenDismissed
                        }
                    }
                    var deferredActivation = eventData.detail.activatedOperation.getDeferral();
                    eventData.setPromise(that._initShell().done(function onInitShellDone()
                    {
                        if(deferredActivation)
                        {
                            deferredActivation.complete();
                            deferredActivation = null
                        }
                        BingApp.traceInfo("Shell.onactivated: deferred activation of Bing App has been completed.")
                    }));
                    that._readyForHandlingActivation().done(function onComplete()
                    {
                        if(coldStart)
                            that._initGlobalEventSources();
                        that._handleActivation(that._lastActivationData)
                    })
                }
            }
        },{
            run: function()
            {
                if(this.app)
                {
                    BingApp.traceInfo("Shell.run: starting Bing App.");
                    this.app.start()
                }
            },
            getHost: function()
            {
                return this.host
            },
            showProgressIndicator: function()
            {
                this.showingBlockingProgressIndicator = this.showingSecondarySplashScreen;
                var progressIndicator = this.showingBlockingProgressIndicator ? this.blockingProgressIndicator : this.nonBlockingProgressIndicator;
                WinJS.Utilities.removeClass(progressIndicator,"hidden");
                WinJS.UI.Animation.fadeIn(progressIndicator);
                BingApp.traceInfo("Shell.showProgressIndicator: started showing {0} progress indicator.",this.showingBlockingProgressIndicator ? "blocking" : "non-blocking")
            },
            hideProgressIndicator: function()
            {
                var progressIndicator = this.showingBlockingProgressIndicator ? this.blockingProgressIndicator : this.nonBlockingProgressIndicator;
                var that = this;
                WinJS.UI.Animation.fadeOut(progressIndicator).done(function onComplete()
                {
                    WinJS.Utilities.addClass(progressIndicator,"hidden");
                    BingApp.traceInfo("Shell.hideProgressIndicator: stopped showing {0} progress indicator.",that.showingBlockingProgressIndicator ? "blocking" : "non-blocking");
                    if(that.showingSecondarySplashScreen)
                        that._hideSecondarySplashScreen()
                })
            },
            getLastActivationData: function()
            {
                return this._lastActivationData
            },
            getColdStartActivationData: function()
            {
                return this._coldStartActivationData
            },
            isAppLaunchedFromLiveTile: function()
            {
                var activationData = this.getLastActivationData();
                if(activationData.detail.kind === Windows.ApplicationModel.Activation.ActivationKind.launch)
                {
                    var args = activationData.detail.arguments;
                    if(BingApp.Utilities.isNullOrUndefined(args) || args.indexOf(ieArgumentPrefix) === -1)
                        return true
                }
                return false
            },
            _initShell: function()
            {
                if(this.initOperation)
                {
                    var operationName = "BingApp.Classes.Shell._initShell";
                    BingApp.traceWarning(operationName + " has already been called: this method should be called only once during cold start. Skipping Shell initialization.")
                }
                else
                {
                    BingApp.traceInfo("Shell._initShell: Started Shell initialization; must happen once during 'cold' start.");
                    var that = this;
                    var promise = WinJS.UI.processAll().then(function onComplete()
                        {
                            return WinJS.Resources.processAll()
                        }).then(function initializeShell()
                        {
                            that._initElements();
                            that._initSettings();
                            that._initNavigationProgressTracking();
                            that._initSecondarySplashScreen();
                            that._initGlobalEventHandlers();
                            that._initConfig();
                            that._portSettings();
                            typeof MSApp !== "undefined" && that._whiteListURIs();
                            BingApp.traceInfo("Shell._initShell: Finished Shell initialization; now it is ready to host app UI content.")
                        });
                    Object.defineProperty(this,"initOperation",{
                        value: promise,
                        writable: false,
                        enumerable: false,
                        configurable: false
                    })
                }
                return this.initOperation
            },
            _readyForHandlingActivation: function()
            {
                var that = this;
                return WinJS.Utilities.ready().then(function onComplete()
                    {
                        return that.initOperation || WinJS.Promise.as()
                    })
            },
            _handleActivation: function(activationData)
            {
                BingApp.Classes.Shell.lastActivationFormCode = null;
                var coldStart = BingApp.Utilities.isColdStart(activationData.detail.previousExecutionState);
                if(coldStart)
                    this._initSearchPaneHandler();
                BingApp.SearchPane && BingApp.SearchPane.handleActivation(activationData);
                var coldStartShowSafeSearch = false;
                var navigationUri;
                var collection;
                if(activationData.detail.kind === Windows.ApplicationModel.Activation.ActivationKind.launch)
                {
                    var args = activationData.detail.arguments;
                    if(args && args.indexOf(ieArgumentPrefix) === 0)
                    {
                        var pathStart = args.indexOf("@");
                        if(pathStart !== -1)
                        {
                            pathStart++;
                            var queryStart = args.indexOf("?",pathStart);
                            var fragmentStart = args.indexOf("#",queryStart !== -1 ? queryStart : pathStart);
                            var pathEnd = queryStart !== -1 ? queryStart : fragmentStart !== -1 ? fragmentStart : args.length;
                            var iePath = args.substring(pathStart,pathEnd);
                            var appPath = iePathsMap[iePath];
                            if(appPath)
                            {
                                var queryString;
                                if(queryStart !== -1)
                                {
                                    queryString = args.substring(queryStart,fragmentStart !== -1 ? fragmentStart : args.length);
                                    queryString = BingApp.locator.env.sanitizeQueryString(queryString);
                                    collection = BingApp.Utilities.QueryString.parse(queryString,{decode: true});
                                    Object.keys(collection).forEach(function removeQueryParameterExceptForQParameter(parameter)
                                    {
                                        if(!BingApp.Utilities.areEqualIgnoreCase(parameter,"q"))
                                            delete collection[parameter]
                                    });
                                    collection["form"] = Shell.formCodes.fromIE;
                                    queryString = BingApp.Utilities.QueryString.serialize(collection)
                                }
                                else
                                    queryString = "?form=" + Shell.formCodes.fromIE;
                                var fragmentString = fragmentStart !== -1 ? args.substring(fragmentStart) : "";
                                navigationUri = uris[appPath] + queryString + fragmentString
                            }
                        }
                        if(!BingApp.Classes.Shell.isSupportedAppUri(navigationUri))
                        {
                            BingApp.traceWarning("Shell._handleActivation: Failed to convert metadata '{0}' for launching from IE into valid navigation Uri. Defaulting navigation to Home page.",args);
                            navigationUri = Shell.uris.home + "?form=" + Shell.formCodes.fromIE
                        }
                        BingApp.traceInfo("Shell._handleActivation: Activated from IE with navigation Uri: {0}.",navigationUri)
                    }
                    else
                    {
                        navigationUri = Shell.uris.home + "?form=" + Shell.formCodes.fromLiveTile;
                        BingApp.traceInfo("Shell._handleActivation: Activated from live tile with navigation Uri: {0}.",navigationUri)
                    }
                }
                else if(activationData.detail.kind === Windows.ApplicationModel.Activation.ActivationKind.search)
                {
                    var query = activationData.detail.queryText;
                    if(query && query.trim())
                        navigationUri = Shell.uris.search + "?q=" + encodeURIComponent(query) + "&form=" + Shell.formCodes.fromSearchCharmActivation;
                    else
                        navigationUri = Shell.uris.home + "?form=" + Shell.formCodes.fromSearchCharmActivation;
                    BingApp.traceInfo("Shell._handleActivation: Activated from Search charm with navigation Uri: {0}.",navigationUri)
                }
                else if(activationData.detail.kind === Windows.ApplicationModel.Activation.ActivationKind.protocol)
                {
                    var protocolUri = activationData.detail.uri;
                    if(BingApp.Utilities.areEqualIgnoreCase(protocolUri.host + protocolUri.path,showSafeSearchPath))
                    {
                        var currentUri = BingApp.locator.navigationManager.getCurrentUri();
                        if(currentUri)
                        {
                            BingApp.traceInfo("Shell._handleActivation: Activation from SafeSearch protocol: {0}; app was previously suspended.",protocolUri.absoluteUri);
                            BingApp.locator.eventRelay.fireEvent(BingApp.Settings.events.showSafeSearch);
                            return
                        }
                        else
                        {
                            BingApp.traceInfo("Shell._handleActivation: Cold start activation from SafeSearch protocol: {0}.",protocolUri.absoluteUri);
                            coldStartShowSafeSearch = true;
                            protocolUri = new Windows.Foundation.Uri(uris.home)
                        }
                    }
                    navigationUri = protocolUri.absoluteUri;
                    var protocolQuery = protocolUri.query;
                    if(protocolQuery)
                    {
                        var sanitizedQuery = BingApp.locator.env.sanitizeQueryString(protocolQuery);
                        navigationUri = navigationUri.replace(protocolQuery,sanitizedQuery)
                    }
                    if(!BingApp.Classes.Shell.isSupportedAppUri(navigationUri))
                    {
                        var errorMessage = BingApp.Utilities.format("Shell._handleActivation: Protocol Uri '{0}' is not supported by app; defaulting to home page Uri: {1}.",protocolUri.absoluteUri,Shell.uris.home);
                        var instObject = {
                                url: protocolUri.absoluteUri,
                                ei: errorMessage
                            };
                        BingApp.Instrumentation.logError(instObject);
                        BingApp.traceError(errorMessage);
                        navigationUri = Shell.uris.home
                    }
                    navigationUri = BingApp.Utilities.upsertFormCode(navigationUri,Shell.formCodes.fromProtocol);
                    BingApp.traceInfo("Shell._handleActivation: Activated from protocol with navigation Uri: {0}.",navigationUri)
                }
                else
                    BingApp.traceError("Shell._handleActivation: unexpected activation kind <{0}>.",activationData.detail.kind);
                if(navigationUri)
                {
                    this._extractActivationFormCode(navigationUri);
                    var that = this;
                    BingApp.locator.navigationManager.navigateTo(navigationUri).done(function onComplete()
                    {
                        if(coldStart)
                        {
                            that._initLiveTile();
                            that._logColdStartActivation()
                        }
                        if(coldStartShowSafeSearch)
                            BingApp.locator.eventRelay.fireEvent(BingApp.Settings.events.showSafeSearch)
                    })
                }
            },
            _updateSplashScreenDisplay: null,
            _hideSecondarySplashScreen: function()
            {
                var outgoing = this.secondarySplashScreen;
                var incoming = this.host;
                var that = this;
                this.appWindow.removeEventListener("resize",this._updateSplashScreenDisplay,false);
                this._updateSplashScreenDisplay = null;
                if(outgoing && incoming)
                    WinJS.UI.Animation.crossFade(incoming,outgoing).done(function onCrossFadeComplete()
                    {
                        that.showingSecondarySplashScreen = false;
                        if(that.secondarySplashScreen && that.secondarySplashScreen.parentNode)
                        {
                            var splashScreenContainer = that.secondarySplashScreen.parentNode;
                            if(splashScreenContainer)
                                splashScreenContainer.removeChild(that.secondarySplashScreen);
                            else
                                BingApp.traceError("Shell._hideSecondarySplashScreen: secondary splash screen is orphaned (does not have parent); no action to take but this is abnormal situation.");
                            that.secondarySplashScreen = null;
                            BingApp.traceInfo("Shell._hideSecondarySplashScreen: secondary splash screen is hidden; application content must be shown now.")
                        }
                        that.host.style.opacity = "";
                        WinJS.Utilities.removeClass(that.host,"translucent")
                    })
            },
            _initGlobalEventSources: function()
            {
                BingApp.traceInfo("Shell._initGlobalEventSources: initializing sources for global events exposed by the app: must happen once during 'cold' start.");
                var relay = BingApp.locator.eventRelay;
                BingApp.SoftKeyboard.init(relay);
                BingApp.SearchPane.init(relay);
                BingApp.ViewState.init(window,document,relay)
            },
            _initGlobalEventHandlers: function()
            {
                BingApp.traceInfo("Shell._initGlobalEventHandlers: initializing handlers for global events supported by the app: must happen once during 'cold' start.");
                var relay = BingApp.locator.eventRelay;
                relay.addEventListener(events.invokeApp,function onInvokeApp(args)
                {
                    BingApp.Utilities.invokeURI(args).then(function invokeUriComplete(){},function invokeUriFailed(err)
                    {
                        if(err)
                            BingApp.traceError("shell.invokeURI: failed to invoke {0} uri; Error: {1}.",args,err.message);
                        else
                            BingApp.traceError("shell.invokeURI: failed to invoke {0} uri; Error: {1}.",args,err)
                    })
                });
                relay.addEventListener(events.unsnap,function onUnsnap()
                {
                    Windows.UI.ViewManagement.ApplicationView.tryUnsnap()
                });
                relay.addEventListener(BingApp.Classes.ConfigStore.events.updated,function onConfigUpdated()
                {
                    if(BingApp.locator.config.getConfig().settingsPane.testConfiguration.enabled)
                        Shell.formCodes = testFormCodes;
                    else
                        Shell.formCodes = formCodes
                })
            },
            _initElements: function()
            {
                BingApp.traceInfo("Shell._initElements: initializing Shell UI elements: must happen once during 'cold' start.");
                if(!this.host)
                {
                    var hostElement = document.getElementById("mainviewhost");
                    Object.defineProperty(this,"host",{
                        value: hostElement,
                        writable: false,
                        enumerable: false,
                        configurable: false
                    })
                }
                if(!this.blockingProgressIndicator)
                {
                    var blockingProgressIndicatorElement = document.getElementById("progressindicatorblocking");
                    Object.defineProperty(this,"blockingProgressIndicator",{
                        value: blockingProgressIndicatorElement,
                        writable: false,
                        enumerable: false,
                        configurable: false
                    })
                }
                if(!this.nonBlockingProgressIndicator)
                {
                    var nonBlockingProgressIndicatorElement = document.getElementById("progressindicatornonblocking");
                    Object.defineProperty(this,"nonBlockingProgressIndicator",{
                        value: nonBlockingProgressIndicatorElement,
                        writable: false,
                        enumerable: false,
                        configurable: false
                    })
                }
                if(!this.secondarySplashScreen)
                {
                    var secondarySplashScreenElement = document.getElementById("secondarysplashscreen");
                    Object.defineProperty(this,"secondarySplashScreen",{
                        value: secondarySplashScreenElement,
                        writable: true,
                        enumerable: false,
                        configurable: false
                    })
                }
            },
            _initSearchPaneHandler: function()
            {
                BingApp.traceInfo("Shell._initSearchPaneHandler: registering to handle Search Charm integration: must happen once during 'cold' start.");
                BingApp.SearchPaneHandler.register()
            },
            _initLiveTile: function()
            {
                BingApp.traceInfo("Shell._initLiveTile: registering to poll for live tile update: must happen once during 'cold' start (must happen after initial navigation is completed).");
                if(!this.liveTile)
                {
                    var liveTile = new BingApp.Classes.LiveTile(BingApp.locator.env,Windows.UI.Notifications);
                    Object.defineProperty(this,"liveTile",{
                        value: liveTile,
                        writable: false,
                        enumerable: false,
                        configurable: false
                    })
                }
                this.liveTile.registerPolling()
            },
            _initSettings: function()
            {
                BingApp.traceInfo("Shell._initSettings: initializing app settings: must happen once during 'cold' start.");
                BingApp.Settings.init(this.app,BingApp.locator.eventRelay)
            },
            _initNavigationProgressTracking: function()
            {
                BingApp.traceInfo("Shell._initNavigationProgressTracking: initializing service for navigation progress tracking: must happen once during 'cold' start.");
                var navigationManager = BingApp.locator.navigationManager;
                var progressIndicatorService = BingApp.locator.progressIndicatorService;
                BingApp.Classes.NavigationManager.initTracking(navigationManager,progressIndicatorService)
            },
            _initSecondarySplashScreen: function()
            {
                BingApp.traceInfo("Shell._initSecondarySplashScreen: initializing secondary splash screen; must happen once during 'cold' start.");
                this.showingSecondarySplashScreen = true
            },
            _initConfig: function()
            {
                BingApp.locator.eventRelay.addEventListener(BingApp.Classes.ConfigStore.events.updated,function startPeriodicRemoteUpdateOnConfigUpdated()
                {
                    BingApp.locator.config.startPeriodicRemoteUpdate(BingApp.locator.config.getConfig().app.general.configUpdateInterval)
                });
                var config = BingApp.locator.config;
                config.forceRemoteUpdate();
                config.startPeriodicRemoteUpdate(config.getConfig().app.general.configUpdateInterval);
                config.loadTestOverrideSettings();
                this._upgradeCheck = new BingApp.Classes.UpgradeCheck;
                if(config.getConfig().settingsPane.testConfiguration.enabled)
                    Shell.formCodes = testFormCodes
            },
            _portSettings: function()
            {
                try
                {
                    var localSettings = Windows.Storage.ApplicationData.current.localSettings
                }
                catch(e)
                {
                    BingApp.traceError("Shell._portSettings: error accessing localSettings: " + e.message);
                    return
                }
                var safeSearchSetting = localSettings.values["SAFE_SEARCH_PREF"];
                if(safeSearchSetting && BingApp.locator.config)
                {
                    localSettings.values.remove("SAFE_SEARCH_PREF");
                    BingApp.locator.config.setOverrideSetting("search","safeSearch","safeSearch",safeSearchSetting);
                    BingApp.traceInfo("Shell._portSettings: RTM safe search setting was found and used as override.")
                }
            },
            _whiteListURIs: function()
            {
                var uriList = BingApp.locator.config.getConfig().app.directNavigationAppUris;
                Object.keys(uriList).forEach(function whiteListURI(key)
                {
                    MSApp.addPublicLocalApplicationUri(uriList[key].value)
                })
            },
            _logColdStartActivation: function()
            {
                var kind;
                var args;
                switch(this._coldStartActivationData.detail.kind)
                {
                    case Windows.ApplicationModel.Activation.ActivationKind.launch:
                        args = this._coldStartActivationData.detail.arguments;
                        if(typeof args === "string" && args.indexOf(ieArgumentPrefix) === 0)
                            kind = activationKind.ieLaunch;
                        else
                            kind = activationKind.launch;
                        break;
                    case Windows.ApplicationModel.Activation.ActivationKind.search:
                        args = this._coldStartActivationData.detail.queryText;
                        kind = activationKind.search;
                        break;
                    case Windows.ApplicationModel.Activation.ActivationKind.protocol:
                        args = this._coldStartActivationData.detail.uri.rawUri;
                        kind = activationKind.protocol;
                        break;
                    default:
                        throw new BingApp.Classes.ErrorInvalidOperation("Shell._logColdStartActivation","Unexpected activation kind <" + this._coldStartActivationData.detail.kind + ">.");
                }
                BingApp.Instrumentation.log({
                    name: "AppLaunch",
                    kind: kind,
                    args: args
                },BingApp.Instrumentation.activation)
            },
            _extractActivationFormCode: function(navigationUri)
            {
                var match = /(\?|&)form=(\w+)/m.exec(navigationUri);
                if(match && match.length > 0)
                    BingApp.Classes.Shell.lastActivationFormCode = match[2]
            },
            _lastActivationData: null,
            _coldStartActivationData: null
        },{
            uriPaths: uriPaths,
            uris: uris,
            webPaths: webPaths,
            formCodes: formCodes,
            fullVersion: fullVersion,
            events: events,
            lastActivationFormCode: null,
            getAppPathFromWebPath: function(webUrl)
            {
                var uri;
                if(webUrl)
                {
                    try
                    {
                        uri = new Windows.Foundation.Uri(webUrl)
                    }
                    catch(e)
                    {
                        BingApp.traceError("BingApp.Classes.Shell.getAppPathFromWebPath: error creating uri from '{0}', {1}",webUrl,e.message);
                        return
                    }
                    var appPath = webPathsMap[uri.path];
                    if(appPath)
                    {
                        var updatedQuery = BingApp.locator.env.sanitizeQueryString(uri.query);
                        var appUri = uris[appPath] + updatedQuery + uri.absoluteUri.substring(uri.absoluteUri.length - uri.fragment.length);
                        if(BingApp.Classes.Shell.isSupportedAppUri(appUri))
                            return appUri
                    }
                }
            },
            isSupportedAppUri: function(uri)
            {
                if(typeof uri === "string")
                    try
                    {
                        uri = new Windows.Foundation.Uri(uri)
                    }
                    catch(e)
                    {
                        BingApp.traceError("BingApp.Classes.Shell.isSupportedAppUri: given Uri '{0}' is invalid; error message: '{1}'",uri,e.message);
                        return false
                    }
                if(!uri)
                    return false;
                var uriNoQueryNoFragment = uri.absoluteUri.substring(0,uri.absoluteUri.length - uri.query.length - uri.fragment.length);
                if(!(uriNoQueryNoFragment in lookupSupportedUris) || !BingApp.locator.config.getConfig().app.general.videoVerticalSupportedInApp && uriNoQueryNoFragment === uris.videosgrid)
                {
                    BingApp.traceInfo("BingApp.Classes.Shell.isSupportedAppUri: given Uri '{0}' is invalid because it does not map to any of the supported navigation uri.",uri.absoluteUri);
                    return false
                }
                if(uriNoQueryNoFragment in lookupNeedSearchQueryUris)
                {
                    var collection = BingApp.Utilities.QueryString.parse(uri.query,{decode: true});
                    var searchQuery = (collection["q"] || collection["Q"] || "").trim();
                    if(!searchQuery)
                    {
                        BingApp.traceInfo("BingApp.Classes.Shell.isSupportedAppUri: given Uri '{0}' is invalid because it does not have non-empty search query.",uri.absoluteUri);
                        return false
                    }
                }
                return true
            }
        });
    WinJS.Namespace.define("BingApp.Classes",{Shell: Shell})
})()