﻿(function homeViewModule()
{
    "use strict";
    var containedviewhostHeight = "65px";
    var containedviewhostHeightNoDisplay = "0px";
    var refreshStorageKey = "HomeViewPopularNowLastRefresh";
    var cacheProviderIdentity = "bingsearch://cache/homepopularnow";
    var maxDisplayedTrends = 4;
    var thumbnailLongEdgeSize = 256;
    var overlayMessageDuration = 1500;
    var setAsLockScreenConfirmationMessage;
    var copyLinkConfirmationMessage;
    var copyImageConfirmationMessage;
    var trendsRefreshed = false;
    var biValues = Object.freeze({
            clickSuffix: "Clicked",
            selectSuffix: "Selected",
            errorSuffix: "Error",
            errorType: "CI.Error"
        });
    var startPageLayout = BingApp.Instrumentation.ClientPages.StartPageInstrumention;
    function hideTrendsControl()
    {
        var trendsPeekElement = document.querySelector("#trendsPeek");
        var containedviewhostElement = document.querySelector("#containedviewhost");
        trendsPeekElement.style.display = "none";
        containedviewhostElement.style.height = containedviewhostHeightNoDisplay
    }
    function showTrendsControl()
    {
        var trendsPeekElement = document.querySelector("#trendsPeek");
        var containedviewhostElement = document.querySelector("#containedviewhost");
        trendsPeekElement.style.display = "";
        containedviewhostElement.style.height = containedviewhostHeight
    }
    function openStartView(eventData)
    {
        var src = eventData.currentTarget;
        var kValue = src && src.k ? src.k + .1 : 0;
        var srcOfEvent = src && src.evtSourceOrigin ? src.evtSourceOrigin : "Unknown";
        var instrumentationObject;
        instrumentationObject = {
            name: "Trends",
            trendType: "Text",
            K: kValue,
            Type: srcOfEvent
        };
        BingApp.Instrumentation.logClick(instrumentationObject);
        BingApp.locator.navigationManager.navigateTo(BingApp.Classes.Shell.uris.start)
    }
    function ondatarequested()
    {
        var defaultHost = this._config.getConfig().app.general.host;
        var collection = {};
        collection["form"] = BingApp.Classes.Shell.formCodes.fromShareCharmStart;
        var uri = null;
        try
        {
            uri = new Windows.Foundation.Uri(defaultHost + BingApp.Utilities.QueryString.serialize(collection))
        }
        catch(e)
        {
            BingApp.traceError("HomeViewController.ondatarequested: invalid uri. Setting to null.")
        }
        var imageUri = new Windows.Foundation.Uri(BingApp.HomePageData.getImageUri(true));
        var shareData = new BingApp.Classes.ShareData;
        shareData.title = WinJS.Resources.getString("app_name").value;
        shareData.description = WinJS.Resources.getString("app_description_short").value;
        shareData.uri = uri;
        return Windows.Storage.StorageFile.getFileFromApplicationUriAsync(imageUri).then(function onGetFileCompleted(file)
            {
                if(file)
                    return file.getThumbnailAsync(Windows.Storage.FileProperties.ThumbnailMode.singleItem,thumbnailLongEdgeSize).then(function onCreateThumbnailCompleted(thumbnail)
                        {
                            if(thumbnail)
                            {
                                var imageStream = Windows.Storage.Streams.RandomAccessStreamReference.createFromStream(thumbnail.cloneStream());
                                shareData.bitmap = imageStream;
                                shareData.thumbnail = imageStream;
                                thumbnail.close()
                            }
                            BingApp.ShareSource.addShareSources(shareData)
                        },function onCreateThumbnailError(error)
                        {
                            BingApp.ShareSource.addShareSources(shareData)
                        });
                else
                    BingApp.ShareSource.addShareSources(shareData)
            },function onGetFileError(error)
            {
                BingApp.ShareSource.addShareSources(shareData)
            })
    }
    function populateTrendsContainer(container, trends)
    {
        WinJS.Utilities.empty(container);
        if(trends.length === 0)
        {
            hideTrendsControl();
            return
        }
        var fragment = document.createDocumentFragment();
        var kStartValue = BingApp.Instrumentation.ClientPages.StartPageInstrumention.KValues.kFooterStartingValue;
        var tabStartIndex = 12;
        trends.forEach(function setItemTitle(item, index)
        {
            if(item)
            {
                var title = item.query;
                item.k = kStartValue++;
                var element = document.createElement("a");
                var elementId = "trendsArticle" + index;
                element.id = elementId;
                element.setAttribute("aria-label",title);
                element.setAttribute("role","link");
                element.tabIndex = tabStartIndex++;
                element.innerText = title;
                WinJS.Utilities.addClass(element,"trendsArticle");
                fragment.appendChild(element);
                element.onclick = function onTrendsArticleClick(eventData)
                {
                    handleTrendsArticleClick(item,index,eventData.currentTarget)
                };
                element.onkeydown = function onTrendsArticleKeyPressed(eventData)
                {
                    if(BingApp.Utilities.isKeyDownAndSpaceOrEnterKey(eventData))
                        handleTrendsArticleClick(item,index,eventData.currentTarget)
                };
                element.addEventListener("MSPointerDown",BingApp.Utilities.onMSPointerDown,true)
            }
        });
        var moreButtonArrowElement = document.querySelector("#moreButtonArrow");
        moreButtonArrowElement.tabIndex = tabStartIndex;
        if(fragment.childNodes.length > 0)
            showTrendsControl();
        else
            hideTrendsControl();
        container.appendChild(fragment);
        showHideTrendsArticles()
    }
    function showHideTrendsArticles()
    {
        var trendsArticleElements = document.querySelectorAll(".trendsArticle");
        var moreButtonElement = document.querySelector("#moreButton");
        for(var i = 1; i < trendsArticleElements.length; i++)
        {
            var articleElement = trendsArticleElements[i];
            if(articleElement.offsetLeft + articleElement.offsetWidth > moreButtonElement.offsetLeft)
                articleElement.style.display = "none";
            else
                articleElement.style.display = ""
        }
    }
    function handleConfigChanged()
    {
        var trendsSection = this._config.getConfig().trends;
        if(trendsSection.general.enabled && trendsSection.popNowTrends.enabled)
        {
            var refreshInterval = trendsSection.popNowTrends.ttl;
            if(refreshInterval !== this.refresher.interval)
                this.refresher.updateInterval(refreshInterval)
        }
        else
            hideTrendsControl()
    }
    function handleTrendsArticleClick(item, index, src)
    {
        var instrumentationObject;
        instrumentationObject = {
            name: "PopularClick",
            trendPos: index,
            trendType: "Text",
            K: item.k,
            Type: src ? src.evtSourceOrigin : null
        };
        BingApp.Instrumentation.logClick(instrumentationObject);
        var navigationQuery = BingApp.locator.env.sanitizeQueryString("?q=" + item.query);
        var navigationUri = BingApp.Classes.Shell.uris.search + navigationQuery;
        navigationUri = BingApp.Utilities.upsertFormCode(navigationUri,BingApp.Classes.Shell.formCodes.fromTrendingSearch);
        BingApp.locator.navigationManager.navigateTo(navigationUri)
    }
    function handleKeyDownForTrendsAndMore(eventData)
    {
        if(BingApp.Utilities.isKeyDownAndSpaceOrEnterKey(eventData))
            openStartView(eventData)
    }
    function appBarAfterShow()
    {
        BingApp.traceVerbose("AppBarShown");
        BingApp.Instrumentation.logFromHandlerEvent(null,"AppBarLaunch")
    }
    function appBarAfterHide()
    {
        BingApp.traceVerbose("AppBarHidden")
    }
    function getBackgroundImageUri()
    {
        return BingApp.HomePageData.getImageUri(true)
    }
    function imageSetLockScreen(event)
    {
        function reportError(error)
        {
            BingApp.Utilities.removeOverlayMessage();
            BingApp.Utilities.showErrorDialog("error_image_set_lockscreen",null,"homeview.imageSetLockScreen",error.message);
            BingApp.Instrumentation.logFromHandlerEvent(event,biValues.errorSuffix,biValues.errorType)
        }
        var homeViewAppBarElement = document.querySelector("#homeViewAppBar");
        homeViewAppBarElement.winControl.hide();
        BingApp.Instrumentation.logFromHandlerEvent(event,biValues.clickSuffix);
        try
        {
            var promises = {
                    timeout: WinJS.Promise.timeout(overlayMessageDuration),
                    animation: BingApp.Utilities.displayOverlayMessage(setAsLockScreenConfirmationMessage),
                    lockScreen: BingApp.ImageExportTool.setLockScreen(getBackgroundImageUri())
                };
            WinJS.Promise.join(promises).done(BingApp.Utilities.removeOverlayMessage,reportError)
        }
        catch(error)
        {
            reportError(error)
        }
    }
    function imageSaveAs(event)
    {
        function reportError(error)
        {
            if(!BingApp.Utilities.isPromiseCancellationError(error))
            {
                BingApp.Utilities.showErrorDialog("error_image_saveas","error_image_saveas_with_system_message","homeview.imageSaveAs",error.message);
                BingApp.Instrumentation.logFromHandlerEvent(event,biValues.errorSuffix,biValues.errorType)
            }
        }
        var homeViewAppBarElement = document.querySelector("#homeViewAppBar");
        homeViewAppBarElement.winControl.hide();
        BingApp.Instrumentation.logFromHandlerEvent(event,biValues.clickSuffix);
        try
        {
            var fileName = BingApp.HomePageData.getCurrentImageFileName();
            BingApp.ImageExportTool.saveImageAs(getBackgroundImageUri(),fileName).done(null,reportError)
        }
        catch(error)
        {
            reportError(error)
        }
    }
    function imageCopy(event)
    {
        var homeViewAppBarElement = document.querySelector("#homeViewAppBar");
        homeViewAppBarElement.winControl.hide();
        BingApp.Instrumentation.logFromHandlerEvent(event,biValues.clickSuffix);
        var promises = {
                timeout: WinJS.Promise.timeout(overlayMessageDuration),
                animation: BingApp.Utilities.displayOverlayMessage(copyImageConfirmationMessage)
            };
        WinJS.Promise.join(promises).then(function copyImage()
        {
            BingApp.ImageExportTool.copyImageToClipboard(getBackgroundImageUri())
        }).done(BingApp.Utilities.removeOverlayMessage,function imageCopyError(error)
        {
            BingApp.Utilities.removeOverlayMessage();
            BingApp.traceError("{0}: {1}","homeview.imageCopy",error && error.message);
            BingApp.Instrumentation.logFromHandlerEvent(event,biValues.errorSuffix,biValues.errorType)
        })
    }
    function imageLinkCopy(event)
    {
        var homeViewAppBarElement = document.querySelector("#homeViewAppBar");
        homeViewAppBarElement.winControl.hide();
        BingApp.Instrumentation.logFromHandlerEvent(event,biValues.clickSuffix);
        var promises = {
                timeout: WinJS.Promise.timeout(overlayMessageDuration),
                animation: BingApp.Utilities.displayOverlayMessage(copyLinkConfirmationMessage)
            };
        WinJS.Promise.join(promises).then(function copyLink()
        {
            BingApp.ImageExportTool.copyLinkToClipboard(getBackgroundImageUri())
        }).done(BingApp.Utilities.removeOverlayMessage,function imageLinkCopyError(error)
        {
            BingApp.Utilities.removeOverlayMessage();
            BingApp.traceError("{0}: {1}","homeview.imageLinkCopy",error && error.message);
            BingApp.Instrumentation.logFromHandlerEvent(event,biValues.errorSuffix,biValues.errorType)
        })
    }
    function registerAppBar()
    {
        var homeViewAppBarElement = document.querySelector("#homeViewAppBar");
        var commandSetLockScreenElement = document.querySelector("#commandSetLockScreen");
        var commandSaveAsElement = document.querySelector("#commandSaveAs");
        var commandCopyElement = document.querySelector("#commandCopy");
        var commandCopyLinkElement = document.querySelector("#commandCopyLink");
        homeViewAppBarElement.winControl.addEventListener("aftershow",appBarAfterShow);
        homeViewAppBarElement.winControl.addEventListener("afterhide",appBarAfterHide);
        commandSetLockScreenElement.addEventListener("click",imageSetLockScreen);
        commandSaveAsElement.addEventListener("click",imageSaveAs);
        commandCopyElement.addEventListener("click",imageCopy);
        commandCopyLinkElement.addEventListener("click",imageLinkCopy)
    }
    function deregisterAppBar()
    {
        var homeViewAppBarElement = document.querySelector("#homeViewAppBar");
        var commandSetLockScreenElement = document.querySelector("#commandSetLockScreen");
        var commandSaveAsElement = document.querySelector("#commandSaveAs");
        var commandCopyElement = document.querySelector("#commandCopy");
        var commandCopyLinkElement = document.querySelector("#commandCopyLink");
        homeViewAppBarElement.winControl.removeEventListener("aftershow",appBarAfterShow);
        homeViewAppBarElement.winControl.removeEventListener("afterhide",appBarAfterHide);
        commandSetLockScreenElement.removeEventListener("click",imageSetLockScreen);
        commandSaveAsElement.removeEventListener("click",imageSaveAs);
        commandCopyElement.removeEventListener("click",imageCopy);
        commandCopyLinkElement.removeEventListener("click",imageLinkCopy)
    }
    function initInstrumentation()
    {
        startPageLayout = BingApp.Instrumentation.ClientPages.StartPageInstrumention;
        startPageLayout.init();
        var trendsTitleElement = document.querySelector("#trendsTitle");
        var moreButtonElement = document.querySelector("#moreButton");
        trendsTitleElement.k = startPageLayout.KValues.kTrendTitle;
        moreButtonElement.k = startPageLayout.KValues.kMoreButton
    }
    var HomeViewController = WinJS.Class.derive(BingApp.Classes.ViewController,function constructor(options)
        {
            this._config = options && options.config || BingApp.locator.config;
            BingApp.Classes.ViewController.call(this)
        },{
            setNavigationUri: function(uri)
            {
                if(BingApp.locator.navigationManager.getCurrentContentStatus() === BingApp.Classes.NavigationManager.contentStatus.loading && this._viewIsInitialized)
                    BingApp.locator.navigationManager.notifyContentLoaded()
            },
            notifyViewReady: function()
            {
                WinJS.Resources.processAll();
                setAsLockScreenConfirmationMessage = WinJS.Resources.getString("appbar_setas_lockscreen_confirmation").value;
                copyLinkConfirmationMessage = WinJS.Resources.getString("appbar_copylink_confirmation").value;
                copyImageConfirmationMessage = WinJS.Resources.getString("appbar_copy_confirmation").value;
                initInstrumentation();
                var direction = document.defaultView.getComputedStyle(document.body,null).direction;
                var popNowProvider = null;
                var trendsSection = this._config.getConfig().trends;
                if(trendsSection.general.enabled && trendsSection.popNowTrends.enabled && direction === "ltr")
                    popNowProvider = new BingApp.Classes.PopNowDataProvider;
                if(popNowProvider)
                {
                    var listViewModel = new BingApp.Classes.ListViewModel(popNowProvider,new BingApp.Classes.LocalStorageCacheProvider(cacheProviderIdentity,"id"));
                    var currentDate = new Date;
                    var that = this;
                    this.refresher = new BingApp.Classes.ListViewModelRefresher(listViewModel,{
                        interval: trendsSection.popNowTrends.ttl,
                        lastRefreshDateStorageKey: refreshStorageKey,
                        refreshOptions: {count: maxDisplayedTrends},
                        onConfigChanged: function handleConfigChangedClosure()
                        {
                            that._handleConfigChanged()
                        }
                    });
                    this._trendsRefreshListener = function trendsRefreshListener()
                    {
                        var list = listViewModel.getList();
                        var trendsTitlesContainerElement = document.querySelector("#trendsTitlesContainer");
                        populateTrendsContainer(trendsTitlesContainerElement,list);
                        startPageLayout.populateFooterRegion(list)
                    };
                    listViewModel.addEventListener(BingApp.Classes.ListViewModel.events.refreshFinished,this._trendsRefreshListener);
                    this._trendsRefreshListener();
                    var trendsTitleElement = document.querySelector("#trendsTitle");
                    var moreButtonElement = document.querySelector("#moreButton");
                    trendsTitleElement.onclick = openStartView;
                    moreButtonElement.onclick = openStartView;
                    trendsTitleElement.addEventListener("MSPointerDown",BingApp.Utilities.onMSPointerDown,true);
                    moreButtonElement.addEventListener("MSPointerDown",BingApp.Utilities.onMSPointerDown,true);
                    trendsTitleElement.onkeydown = handleKeyDownForTrendsAndMore;
                    moreButtonElement.onkeydown = handleKeyDownForTrendsAndMore;
                    if(!this._onNetworkStatusChangeWrapper)
                        this._onNetworkStatusChangeWrapper = function onNetworkStatusChangeWrapper(eventData)
                        {
                            if(eventData.connectionState === BingApp.Classes.NetworkDetectionService.connectionStates.internetAccess)
                            {
                                var trendsTitlesContainerElement = document.querySelector("#trendsTitlesContainer");
                                if(trendsTitlesContainerElement.childNodes.length > 0)
                                    showTrendsControl();
                                else
                                    that.refresher.forceRefresh()
                            }
                            else
                                hideTrendsControl()
                        };
                    if(this.refresher.lastRefreshDate >= currentDate)
                        trendsRefreshed = true;
                    if(BingApp.locator.networkDetectionService.isConnected())
                    {
                        if(!trendsRefreshed)
                        {
                            this.refresher.forceRefresh();
                            trendsRefreshed = true
                        }
                    }
                    else
                        hideTrendsControl();
                    BingApp.locator.eventRelay.addEventListener(BingApp.Classes.NetworkDetectionService.networkStatusChangedEvent,this._onNetworkStatusChangeWrapper)
                }
                else
                    hideTrendsControl();
                BingApp.ShareSource.ondatarequested = BingApp.Utilities.createClosure(this,this._onDataRequested);
                window.addEventListener("resize",showHideTrendsArticles,false);
                registerAppBar();
                var bgImagePanoramaElement = document.querySelector("#bgImagePanorama");
                bgImagePanoramaElement.focus();
                BingApp.Instrumentation.updatePageIG();
                startPageLayout.logLayout();
                BingApp.locator.navigationManager.notifyContentLoaded();
                this._viewIsInitialized = true
            },
            notifyUnloading: function()
            {
                var trendsTitleElement = document.querySelector("#trendsTitle");
                var moreButtonElement = document.querySelector("#moreButton");
                trendsTitleElement.onclick = null;
                moreButtonElement.onclick = null;
                trendsTitleElement.onkeydown = null;
                moreButtonElement.onkeydown = null;
                window.removeEventListener("resize",showHideTrendsArticles);
                trendsTitleElement.removeEventListener("MSPointerDown",BingApp.Utilities.onMSPointerDown);
                moreButtonElement.removeEventListener("MSPointerDown",BingApp.Utilities.onMSPointerDown);
                BingApp.ShareSource.ondatarequested = null;
                if(this.refresher)
                {
                    this.refresher.listViewModel.removeEventListener(BingApp.Classes.ListViewModel.events.refreshFinished,this._trendsRefreshListener);
                    this._trendsRefreshListener = null;
                    this.refresher.dispose();
                    this.refresher = null
                }
                if(this._onNetworkStatusChangeWrapper)
                {
                    BingApp.locator.eventRelay.removeEventListener(BingApp.Classes.NetworkDetectionService.networkStatusChangedEvent,this._onNetworkStatusChangeWrapper);
                    this._onNetworkStatusChangeWrapper = null
                }
                deregisterAppBar();
                this._viewIsInitialized = false;
                BingApp.Classes.ViewController.prototype.notifyUnloading.call(this)
            },
            _handleConfigChanged: handleConfigChanged,
            _onDataRequested: ondatarequested,
            _trendsRefreshListener: null,
            _viewIsInitialized: false
        });
    function getController(options)
    {
        if(!this._controller)
            this._controller = new HomeViewController(options);
        return this._controller
    }
    function ready()
    {
        this.getController().notifyViewReady()
    }
    WinJS.UI.Pages.define("/views/home/html/homeview.html",{
        ready: ready,
        getController: getController,
        _controller: null
    })
})()