﻿(function popNowDataProviderModule()
{
    "use strict";
    var popNowIdentity = "bingsearch://data/popularnow";
    var popNowCount = 5;
    function parsePopNowDocument(responseText)
    {
        if(BingApp.Utilities.isNullOrUndefined(responseText))
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("responseText");
        var doc = new Windows.Data.Xml.Dom.XmlDocument;
        try
        {
            doc.loadXml(responseText)
        }
        catch(err)
        {
            var xhrErrorMessage = BingApp.Utilities.format("PopNowDataProvider.parsePopNowDocument: Input is not valid xml - error name: {0}, error message: {1}",err.name || "",err.message || "");
            var instObject = {
                    url: this._getPopNowUrl().url,
                    ei: xhrErrorMessage
                };
            BingApp.Instrumentation.logError(instObject);
            BingApp.traceError(xhrErrorMessage);
            throw new BingApp.Classes.ErrorArgument("responseText","response from live tile end point is not a valid Xml");
        }
        var results = [];
        var queries = doc.selectNodes('/tile/visual/binding[@template="TileWidePeekImage02"]/text[position() > 1]/text()');
        if(!queries || queries.length === 0)
            return results;
        queries.forEach(function(queryNode)
        {
            results.push({query: queryNode.data})
        });
        return results
    }
    function getPopNowUrl()
    {
        var url = this._config.getConfig().app.general.host + this._config.getConfig().trends.popNowTrends.path + "?";
        url = url + this._env.getQueryString() + "&w8-version=" + BingApp.Classes.Shell.fullVersion;
        return{url: url}
    }
    WinJS.Namespace.define("BingApp.Classes",{PopNowDataProvider: WinJS.Class.derive(BingApp.Classes.XhrDataProvider,function constructor(options)
        {
            if(!(this instanceof BingApp.Classes.PopNowDataProvider))
            {
                BingApp.traceWarning("PopNowDataProvider.ctor: Attempted using PopNowDataProvider ctor as function; redirecting to use 'new PopNowDataProvider()'.");
                return new BingApp.Classes.PopNowDataProvider(options)
            }
            this._config = options && options.config || BingApp.locator.config;
            this._env = options && options.env || BingApp.locator.env;
            var that = this;
            BingApp.Classes.XhrDataProvider.call(this,popNowIdentity,popNowCount,{
                urlOptions: function getUrlOptionsClosure()
                {
                    return that._getPopNowUrl()
                },
                parser: function parseClosure(responseText)
                {
                    return that._parsePopNowDocument(responseText)
                }
            })
        },{
            _getPopNowUrl: getPopNowUrl,
            _parsePopNowDocument: parsePopNowDocument
        })})
})()