﻿(function iframeHostModule(navigationManager, environment, eventRelay)
{
    "use strict";
    var events = BingApp.Classes.IframeHostController.eventNames;
    var contentStates = Object.freeze({
            empty: 0,
            loading: 1,
            ready: 2,
            loaded: 3
        });
    var iframeRelayId = "BingSearchResults";
    var containedViewHostSelector = "#containedviewhost";
    var filterMapping = {
            menuCommandSmall: "imagesize-small",
            menuCommandMedium: "imagesize-medium",
            menuCommandLarge: "imagesize-large",
            menuCommandWallpaper: "imagesize-wallpaper",
            menuCommandColorOnly: "color2-color",
            menuCommandBlackAndWhite: "color2-bw",
            menuCommandRed: "color2-FGcls_RED",
            menuCommandOrange: "color2-FGcls_ORANGE",
            menuCommandYellow: "color2-FGcls_YELLOW",
            menuCommandGreen: "color2-FGcls_GREEN",
            menuCommandTeal: "color2-FGcls_TEAL",
            menuCommandBlue: "color2-FGcls_BLUE",
            menuCommandPurple: "color2-FGcls_PURPLE",
            menuCommandPink: "color2-FGcls_PINK",
            menuCommandBrown: "color2-FGcls_BROWN",
            menuCommandBlack: "color2-FGcls_BLACK",
            menuCommandGray: "color2-FGcls_GRAY",
            menuCommandWhite: "color2-FGcls_WHITE",
            menuCommandPhotograph: "photo-photo",
            menuCommandClipart: "photo-clipart",
            menuCommandLineDrawing: "photo-linedrawing",
            menuCommandSquare: "aspect-square",
            menuCommandWide: "aspect-wide",
            menuCommandTall: "aspect-tall",
            menuCommandJustFaces: "face-face",
            menuCommandHeadAndShoulders: "face-portrait"
        };
    var biValues = Object.freeze({
            clickSuffix: "Clicked",
            selectSuffix: "Selected",
            errorSuffix: "Error",
            errorType: "CI.Error"
        });
    var filterReverseMapping = {
            small: "menuCommandSmall",
            medium: "menuCommandMedium",
            large: "menuCommandLarge",
            wallpaper: "menuCommandWallpaper",
            color: "menuCommandColorOnly",
            bw: "menuCommandBlackAndWhite",
            FGcls_RED: "menuCommandRed",
            FGcls_ORANGE: "menuCommandOrange",
            FGcls_YELLOW: "menuCommandYellow",
            FGcls_GREEN: "menuCommandGreen",
            FGcls_TEAL: "menuCommandTeal",
            FGcls_BLUE: "menuCommandBlue",
            FGcls_PURPLE: "menuCommandPurple",
            FGcls_PINK: "menuCommandPink",
            FGcls_BROWN: "menuCommandBrown",
            FGcls_BLACK: "menuCommandBlack",
            FGcls_GRAY: "menuCommandGray",
            FGcls_WHITE: "menuCommandWhite",
            photo: "menuCommandPhotograph",
            clipart: "menuCommandClipart",
            linedrawing: "menuCommandLineDrawing",
            square: "menuCommandSquare",
            wide: "menuCommandWide",
            tall: "menuCommandTall",
            face: "menuCommandJustFaces",
            portrait: "menuCommandHeadAndShoulders"
        };
    var selectedFilterLabel = "SelectedFilterLabel";
    var overlayMessageDuration = 1500;
    var setAsLockScreenConfirmationMessage;
    var copyLinkConfirmationMessage;
    var copyImageConfirmationMessage;
    function getIframe()
    {
        return document.getElementById("externalcontenthost")
    }
    function handleNavigationStarted(args)
    {
        var navigationUri = IframeHostController.getNavigationUriFromEventArgs(args);
        BingApp.traceInfo("IframeHostController.handleNavigationStarted: received 'navigationStarted' message which is relayed from web compartment with Uri: {0}.",navigationUri);
        var currentNavigationStatus = navigationManager.getCurrentNavigationStatus();
        var currentContentStatus = navigationManager.getCurrentContentStatus();
        var currentUri = navigationManager.getCurrentUri();
        if(currentNavigationStatus === BingApp.Classes.NavigationManager.navigationStatus.navigating)
        {
            BingApp.traceWarning("IframeHostController.handleNavigationStarted: 'navigationStarted' message was received while in 'navigating' state. Will cancel current navigation to {0}.",currentUri.absoluteUri);
            navigationManager.cancelNavigation(false)
        }
        else if(currentContentStatus === BingApp.Classes.NavigationManager.contentStatus.loading)
        {
            BingApp.traceWarning("IframeHostController.handleNavigationStarted: 'navigationStarted' message was received while in waiting for completion of content loading. Will cancel current loading of {0}.",currentUri.absoluteUri);
            navigationManager.cancelNavigation(false)
        }
        navigationManager.notifyNavigationStarted(navigationUri)
    }
    function handleNavigationCompleted(args)
    {
        var navigationUri = IframeHostController.getNavigationUriFromEventArgs(args);
        BingApp.traceInfo("IframeHostController.handleNavigationCompleted: received 'navigationCompleted' message which is relayed from web compartment with Uri: {0}.",navigationUri);
        var currentNavigationStatus = navigationManager.getCurrentNavigationStatus();
        if(currentNavigationStatus === BingApp.Classes.NavigationManager.navigationStatus.idle)
        {
            var currentContentStatus = navigationManager.getCurrentContentStatus();
            if(currentContentStatus === BingApp.Classes.NavigationManager.contentStatus.loading)
            {
                var currentUri = navigationManager.getCurrentUri();
                if(BingApp.Utilities.areEqualIgnoreCase(navigationUri,currentUri.absoluteUri))
                    BingApp.traceInfo("IframeHostController.handleNavigationCompleted: navigation was initiated and completed inside app compartment; ignoring 'navigationCompleted' generated in web compartment.");
                else
                    BingApp.traceWarning("IframeHostController.handleNavigationCompleted: ignoring 'navigationCompleted' message that was received while waiting for completing content loading: {0}.",navigationUri)
            }
            else
                BingApp.traceWarning("IframeHostController.handleNavigationCompleted: ignoring 'navigationCompleted' message that was received while in 'idle' state: {0}.",navigationUri);
            return
        }
        navigationManager.notifyNavigationCompleted(navigationUri,args.error,args.canceled)
    }
    function handleContentReady(args)
    {
        BingApp.traceInfo("IframeHostController.handleContentReady: received 'contentReady' message which is relayed from web compartment.");
        var currentContentStatus = navigationManager.getCurrentContentStatus();
        if(currentContentStatus !== BingApp.Classes.NavigationManager.contentStatus.loading)
        {
            BingApp.traceWarning("IframeHostController.handleContentReady: ignoring 'contentReady' message that was received while content is not in 'loading' state.");
            return
        }
        if(args)
            BingApp.Instrumentation.updatePageIG(args.IG);
        var iframe = getIframe();
        WinJS.Utilities.removeClass(iframe,"hidden");
        if(args)
            eventRelay.registerIframe(iframeRelayId,iframe,args.targetOrigin);
        this._currentContentState = contentStates.ready;
        var lastPendingNavigationBack;
        for(var indexInHistory = this._iframeForwardNavigationHistory.length - 1; indexInHistory >= 0; indexInHistory--)
        {
            var historyRecord = this._iframeForwardNavigationHistory[indexInHistory];
            if(historyRecord.hasOwnProperty("pendingBackOptions"))
                lastPendingNavigationBack = historyRecord;
            else
                break
        }
        if(lastPendingNavigationBack)
            this.setNavigationUri(lastPendingNavigationBack.navigationUri,lastPendingNavigationBack.pendingBackOptions)
    }
    function handleContentLoaded(args)
    {
        var navigationUri = IframeHostController.getNavigationUriFromEventArgs(args);
        BingApp.traceInfo("IframeHostController.handleContentLoaded: received 'contentLoaded' message which is relayed from web compartment with Uri: {0}.",navigationUri);
        var currentContentStatus = navigationManager.getCurrentContentStatus();
        if(currentContentStatus !== BingApp.Classes.NavigationManager.contentStatus.loading)
        {
            BingApp.traceWarning("IframeHostController.handleContentLoaded: ignoring 'contentLoaded' message that was received while content is not in 'loading' state: {0}.",navigationUri);
            return
        }
        navigationManager.notifyContentLoaded(args.error,args.canceled);
        this._currentContentState = contentStates.loaded
    }
    function handleViewModeChanged(args)
    {
        BingApp.traceInfo("IframeHostController.handleViewModeChanged: received 'viewModeChanged' message which is relayed from web compartment.");
        var imageGridAppBarElement = document.querySelector("#imageGridAppBar");
        var imageDetailAppBarElement = document.querySelector("#imageDetailAppBar");
        if(!args || !args.mode)
        {
            imageGridAppBarElement.disabled = true;
            imageDetailAppBarElement.disabled = true;
            BingApp.traceError("IframeHostController.handleViewModeChanged: Message related data is null or empty.");
            return
        }
        var viewMode = args.mode.toLowerCase();
        if(viewMode === "imagedetail")
        {
            imageGridAppBarElement.disabled = true;
            imageDetailAppBarElement.disabled = false
        }
        else if(viewMode === "imagegrid")
        {
            imageGridAppBarElement.disabled = false;
            imageDetailAppBarElement.disabled = true
        }
        else
        {
            BingApp.traceInfo("IframeHostController.handleViewModeChanged: View mode is not supported");
            imageGridAppBarElement.disabled = true;
            imageDetailAppBarElement.disabled = true
        }
    }
    function AppBarHelper()
    {
        var filters = null;
        var imageDetailImageUrl;
        var imageDetailTitle;
        function handleImageShareSourceData(imageData)
        {
            imageDetailImageUrl = imageData.imgurl;
            imageDetailTitle = imageData.title
        }
        function imageSetLockScreen(event)
        {
            function reportError(error)
            {
                BingApp.Utilities.removeOverlayMessage();
                BingApp.Utilities.showErrorDialog("error_image_set_lockscreen",null,"iframehost.imageSetLockScreen",error.message);
                BingApp.Instrumentation.logFromHandlerEvent(event,biValues.errorSuffix,biValues.errorType)
            }
            var imageDetailAppBarElement = document.querySelector("#imageDetailAppBar");
            imageDetailAppBarElement.winControl.hide();
            BingApp.Instrumentation.logFromHandlerEvent(event,biValues.clickSuffix);
            try
            {
                var promises = {
                        timeout: WinJS.Promise.timeout(overlayMessageDuration),
                        animation: BingApp.Utilities.displayOverlayMessage(setAsLockScreenConfirmationMessage),
                        lockScreen: BingApp.ImageExportTool.setLockScreen(imageDetailImageUrl)
                    };
                WinJS.Promise.join(promises).done(BingApp.Utilities.removeOverlayMessage,reportError)
            }
            catch(error)
            {
                reportError(error)
            }
        }
        function imageSaveAs(event)
        {
            function reportError(error)
            {
                if(!BingApp.Utilities.isPromiseCancellationError(error))
                {
                    BingApp.Utilities.showErrorDialog("error_image_saveas","error_image_saveas_with_system_message","iframehost.imageSaveAs",error.message);
                    BingApp.Instrumentation.logFromHandlerEvent(event,biValues.errorSuffix,biValues.errorType)
                }
            }
            var imageDetailAppBarElement = document.querySelector("#imageDetailAppBar");
            imageDetailAppBarElement.winControl.hide();
            BingApp.Instrumentation.logFromHandlerEvent(event,biValues.clickSuffix);
            try
            {
                BingApp.ImageExportTool.saveImageAs(imageDetailImageUrl).done(null,reportError)
            }
            catch(error)
            {
                reportError(error)
            }
        }
        function imageCopy(event)
        {
            var imageDetailAppBarElement = document.querySelector("#imageDetailAppBar");
            imageDetailAppBarElement.winControl.hide();
            BingApp.Instrumentation.logFromHandlerEvent(event,biValues.clickSuffix);
            var promises = {
                    timeout: WinJS.Promise.timeout(overlayMessageDuration),
                    animation: BingApp.Utilities.displayOverlayMessage(copyImageConfirmationMessage)
                };
            WinJS.Promise.join(promises).then(function copyImage()
            {
                BingApp.ImageExportTool.copyImageToClipboard(imageDetailImageUrl,imageDetailTitle)
            }).done(BingApp.Utilities.removeOverlayMessage,function imageCopyError(error)
            {
                BingApp.Utilities.removeOverlayMessage();
                BingApp.traceError("{0}: {1}","iframehost.imageCopy",error && error.message);
                BingApp.Instrumentation.logFromHandlerEvent(event,biValues.errorSuffix,biValues.errorType)
            })
        }
        function imageLinkCopy(event)
        {
            var imageDetailAppBarElement = document.querySelector("#imageDetailAppBar");
            imageDetailAppBarElement.winControl.hide();
            BingApp.Instrumentation.logFromHandlerEvent(event,biValues.clickSuffix);
            var promises = {
                    timeout: WinJS.Promise.timeout(overlayMessageDuration),
                    animation: BingApp.Utilities.displayOverlayMessage(copyLinkConfirmationMessage)
                };
            WinJS.Promise.join(promises).then(function copyLink()
            {
                BingApp.ImageExportTool.copyLinkToClipboard(imageDetailImageUrl,imageDetailTitle)
            }).done(BingApp.Utilities.removeOverlayMessage,function imageLinkCopyError(error)
            {
                BingApp.Utilities.removeOverlayMessage();
                BingApp.traceError("{0}: {1}","iframehost.imageLinkCopy",error && error.message);
                BingApp.Instrumentation.logFromHandlerEvent(event,biValues.errorSuffix,biValues.errorType)
            })
        }
        function registerImageDetailAppBar()
        {
            var commandSetLockScreenElement = document.querySelector("#commandSetLockScreen");
            var commandSaveAsElement = document.querySelector("#commandSaveAs");
            var commandCopyElement = document.querySelector("#commandCopy");
            var commandCopyLinkElement = document.querySelector("#commandCopyLink");
            commandSetLockScreenElement.addEventListener("click",imageSetLockScreen);
            commandSaveAsElement.addEventListener("click",imageSaveAs);
            commandCopyElement.addEventListener("click",imageCopy);
            commandCopyLinkElement.addEventListener("click",imageLinkCopy)
        }
        function deregisterImageDetailAppBar()
        {
            var commandSetLockScreenElement = document.querySelector("#commandSetLockScreen");
            var commandSaveAsElement = document.querySelector("#commandSaveAs");
            var commandCopyElement = document.querySelector("#commandCopy");
            var commandCopyLinkElement = document.querySelector("#commandCopyLink");
            commandSetLockScreenElement.removeEventListener("click",imageSetLockScreen);
            commandSaveAsElement.removeEventListener("click",imageSaveAs);
            commandCopyElement.removeEventListener("click",imageCopy);
            commandCopyLinkElement.removeEventListener("click",imageLinkCopy);
            imageDetailImageUrl = null;
            imageDetailTitle = null
        }
        function showHideImageGridAppBar(uri)
        {
            if(uri && !uri.path)
            {
                BingApp.traceError("IframeHostController.showHideImageGridAppBar: uri is null or empty");
                return
            }
            var imageGridAppBarElement = document.querySelector("#imageGridAppBar");
            if(BingApp.Classes.Shell.uris.imagesgrid.indexOf(uri.path) === -1)
            {
                if(!imageGridAppBarElement.disabled)
                    clearFilters();
                imageGridAppBarElement.disabled = true
            }
            else
                imageGridAppBarElement.disabled = false;
            var imageDetailAppBarElement = document.querySelector("#imageDetailAppBar");
            imageDetailAppBarElement.disabled = true
        }
        function registerImageGridHandlers()
        {
            Array.prototype.forEach.call(document.querySelectorAll("#menuCommandSizeAll, #menuCommandSmall, #menuCommandMedium, #menuCommandLarge, #menuCommandWallpaper, #menuCommandLayoutAll, #menuCommandSquare," + "#menuCommandWide, #menuCommandTall, #menuCommandPeopleAll, #menuCommandJustFaces, #menuCommandHeadAndShoulders, #menuCommandTypeAll, #menuCommandPhotograph," + "#menuCommandClipart, #menuCommandLineDrawing, #menuCommandColorAll, #menuCommandColorOnly, #menuCommandBlackAndWhite, #menuCommandRed, #menuCommandOrange," + "#menuCommandYellow, #menuCommandGreen, #menuCommandTeal, #menuCommandBlue, #menuCommandPurple, #menuCommandPink, #menuCommandBrown, #menuCommandBlack," + "#menuCommandGray, #menuCommandWhite"),function addEventListenerForMenuCommand(menuCommand)
            {
                menuCommand.addEventListener("click",handleAppBarMenuCommandClick)
            });
            var commandResetAllFiltersElement = document.querySelector("#commandResetAllFilters");
            commandResetAllFiltersElement.addEventListener("click",handleImageGridResetFilters);
            Array.prototype.forEach.call(document.querySelectorAll("#imageGridAppBar, #imageDetailAppBar"),function addEventListenerForAppBarAfterShowAfterHide(appBar)
            {
                appBar.addEventListener("aftershow",handleAppBarShown);
                appBar.addEventListener("afterhide",handleAppBarHidden)
            });
            Array.prototype.forEach.call(document.querySelectorAll("#sizeFlyout, #colorFlyout, #typeFlyout, #layoutFlyout, #peopleFlyout"),function addEventListenerForMenuAfterShow(flyout)
            {
                flyout.addEventListener("aftershow",handleMenuShown)
            });
            createAppBarCommandSelectedLabelElements()
        }
        function removeImageGridHandlers()
        {
            Array.prototype.forEach.call(document.querySelectorAll("#menuCommandSizeAll, #menuCommandSmall, #menuCommandMedium, #menuCommandLarge, #menuCommandWallpaper, #menuCommandLayoutAll, #menuCommandSquare," + "#menuCommandWide, #menuCommandTall, #menuCommandPeopleAll, #menuCommandJustFaces, #menuCommandHeadAndShoulders, #menuCommandTypeAll, #menuCommandPhotograph," + "#menuCommandClipart, #menuCommandLineDrawing, #menuCommandColorAll, #menuCommandColorOnly, #menuCommandBlackAndWhite, #menuCommandRed, #menuCommandOrange," + "#menuCommandYellow, #menuCommandGreen, #menuCommandTeal, #menuCommandBlue, #menuCommandPurple, #menuCommandPink, #menuCommandBrown, #menuCommandBlack," + "#menuCommandGray, #menuCommandWhite"),function removeEventListenerForMenuCommand(menuCommand)
            {
                menuCommand.removeEventListener("click",handleAppBarMenuCommandClick)
            });
            var commandResetAllFiltersElement = document.querySelector("#commandResetAllFilters");
            commandResetAllFiltersElement.removeEventListener("click",handleImageGridResetFilters);
            Array.prototype.forEach.call(document.querySelectorAll("#imageGridAppBar, #imageDetailAppBar"),function removeEventListenerForAppBarAfterShowAfterHide(appBar)
            {
                appBar.removeEventListener("aftershow",handleAppBarShown);
                appBar.removeEventListener("afterhide",handleAppBarHidden)
            });
            Array.prototype.forEach.call(document.querySelectorAll("#sizeFlyout, #colorFlyout, #typeFlyout, #layoutFlyout, #peopleFlyout"),function removeEventListenerForMenuAfterShow(flyout)
            {
                flyout.removeEventListener("aftershow",handleMenuShown)
            })
        }
        function handleAppBarMenuCommandClick(args)
        {
            if(!args)
            {
                BingApp.traceError("IframeHostController.handleAppBarMenuCommandClick: event args is null");
                return
            }
            BingApp.Instrumentation.logFromHandlerEvent(args,biValues.clickSuffix);
            var menuCommandElement = args.srcElement;
            var appBarCommandSelected = false;
            selectMenuCommandItem(menuCommandElement,menuCommandElement.winControl.selected);
            for(var filter in filters)
                if(filters[filter])
                {
                    appBarCommandSelected = true;
                    break
                }
            var commandResetAllFiltersElement = document.querySelector("#commandResetAllFilters");
            commandResetAllFiltersElement.winControl.disabled = !appBarCommandSelected;
            var imageGridAppBarElement = document.querySelector("#imageGridAppBar");
            imageGridAppBarElement.winControl.hide();
            navigationManager.navigateTo(getImageGridNavigationUrl(navigationManager.getCurrentUri()))
        }
        function selectMenuCommandItem(menuCommandElement, isMenuCommandElementSelected)
        {
            var menuAllItem,
                isMenuAllItem = false;
            var menuItemsParentElement = menuCommandElement.parentElement;
            if(menuItemsParentElement && menuItemsParentElement.children)
            {
                var menuItems = menuItemsParentElement.children;
                for(var index = 0; index < menuItems.length; index++)
                {
                    var menuItem = menuItems[index];
                    if(menuItem.winControl)
                    {
                        if(menuItem && menuItem.className === "win-command" && menuItem.winControl.type !== "separator")
                            menuItem.winControl.selected = false;
                        if(menuItem.winControl.type === "toggle")
                            menuAllItem = menuItem
                    }
                }
            }
            if(!isMenuCommandElementSelected)
                menuCommandElement.winControl.selected = true;
            else
                menuAllItem.winControl.selected = true;
            if(!filters)
                filters = {};
            if(menuAllItem && menuAllItem.id === menuCommandElement.id)
                isMenuAllItem = true;
            var menuCommandSelectionState = !isMenuCommandElementSelected && !isMenuAllItem;
            switch(menuItemsParentElement.id)
            {
                case"sizeFlyout":
                    var commandSizeElement = document.querySelector("#commandSize");
                    commandSizeElement.winControl.selected = menuCommandSelectionState;
                    break;
                case"colorFlyout":
                    var commandColorElement = document.querySelector("#commandColor");
                    commandColorElement.winControl.selected = menuCommandSelectionState;
                    break;
                case"peopleFlyout":
                    var commandPeopleElement = document.querySelector("#commandPeople");
                    commandPeopleElement.winControl.selected = menuCommandSelectionState;
                    break;
                case"layoutFlyout":
                    var commandLayoutElement = document.querySelector("#commandLayout");
                    commandLayoutElement.winControl.selected = menuCommandSelectionState;
                    break;
                case"typeFlyout":
                    var commandTypeElement = document.querySelector("#commandType");
                    commandTypeElement.winControl.selected = menuCommandSelectionState;
                    break;
                default:
                    BingApp.traceError("IframeHostController.selectMenuCommandItem: AppBar menu command is not found");
                    break
            }
            updateFilterAndMenu(menuCommandElement,isMenuCommandElementSelected,menuItemsParentElement.id,isMenuAllItem)
        }
        function updateFilterAndMenu(menuCommandElement, isMenuCommandElementSelected, menuItemsParentElementId, isMenuAllItem)
        {
            var selectedFilterLabelControl = document.querySelector("#" + menuItemsParentElementId + selectedFilterLabel);
            var labelText;
            if(isMenuCommandElementSelected || isMenuAllItem)
                labelText = "";
            else
                labelText = menuCommandElement.innerText ? menuCommandElement.innerText : menuCommandElement.attributes["aria-label"].textContent;
            if(selectedFilterLabelControl)
                selectedFilterLabelControl.innerText = labelText;
            filters[menuItemsParentElementId] = !isMenuCommandElementSelected ? menuCommandElement.id : ""
        }
        function getImageGridNavigationUrl(uri)
        {
            if(!filters)
                return uri;
            var filterString = "";
            Object.keys(filters).forEach(function appendFilter(key)
            {
                var filter = filters[key];
                if(!filter)
                    BingApp.traceError("IframeHostController.getImageGridNavigationUrl: filter does not match for the key '{0}'",key);
                else
                    filterString += "+filterui:" + filterMapping[filter]
            });
            return getFilterAppliedUri(uri,filterString)
        }
        function handleImageGridResetFilters(event)
        {
            BingApp.Instrumentation.logFromHandlerEvent(event,biValues.clickSuffix);
            clearFilters();
            var imageGridAppBarElement = document.querySelector("#imageGridAppBar");
            imageGridAppBarElement.winControl.hide();
            BingApp.locator.navigationManager.navigateTo(getFilterAppliedUri(navigationManager.getCurrentUri()),"")
        }
        function clearFilters()
        {
            filters = null;
            var commandResetAllFiltersElement = document.querySelector("#commandResetAllFilters");
            commandResetAllFiltersElement.winControl.disabled = true;
            var commandSizeElement = document.querySelector("#commandSize");
            var commandColorElement = document.querySelector("#commandColor");
            var commandTypeElement = document.querySelector("#commandType");
            var commandLayoutElement = document.querySelector("#commandLayout");
            var commandPeopleElement = document.querySelector("#commandPeople");
            commandSizeElement.winControl.selected = false;
            commandColorElement.winControl.selected = false;
            commandTypeElement.winControl.selected = false;
            commandLayoutElement.winControl.selected = false;
            commandPeopleElement.winControl.selected = false;
            Array.prototype.forEach.call(document.querySelectorAll("#sizeFlyout, #colorFlyout, #typeFlyout, #layoutFlyout, #peopleFlyout"),function clearMenuItemSelection(flyout)
            {
                clearMenuItemsSelection(flyout);
                var selectedFilterLabelControl = document.querySelector("#" + flyout.id + selectedFilterLabel);
                if(selectedFilterLabelControl)
                    selectedFilterLabelControl.innerText = ""
            })
        }
        function restoreFiltersFromUri(uri)
        {
            clearFilters();
            var queryParameters = BingApp.Utilities.QueryString.parse(uri.query,{decode: true});
            var filterQueryStringKey = "qft";
            var filterQueryStringValue = queryParameters[filterQueryStringKey];
            if(!filterQueryStringValue)
            {
                BingApp.traceVerbose("IframeHostController.updatedFilters: no filters in the uri");
                return
            }
            var filterCollection = filterQueryStringValue.split("+filterui:");
            var mappedControl,
                filter,
                filterSelected;
            var enableResetFiltersControl = false;
            for(var index = 0; index < filterCollection.length; index++)
            {
                filter = filterCollection[index];
                if(filter)
                {
                    var filterPair = filter.split("-");
                    if(filterPair && filterPair[1])
                    {
                        filterSelected = filterPair[1];
                        if(filterSelected)
                        {
                            mappedControl = document.querySelector("#" + filterReverseMapping[filterSelected]);
                            if(mappedControl)
                            {
                                enableResetFiltersControl = true;
                                selectMenuCommandItem(mappedControl,false)
                            }
                        }
                    }
                }
            }
            if(enableResetFiltersControl)
            {
                var commandResetAllFiltersElement = document.querySelector("#commandResetAllFilters");
                commandResetAllFiltersElement.winControl.disabled = false
            }
        }
        this.handleImageShareSourceData = handleImageShareSourceData;
        this.registerImageDetailAppBar = registerImageDetailAppBar;
        this.deregisterImageDetailAppBar = deregisterImageDetailAppBar;
        this.showHideImageGridAppBar = showHideImageGridAppBar;
        this.registerImageGridHandlers = registerImageGridHandlers;
        this.removeImageGridHandlers = removeImageGridHandlers;
        this.getImageGridNavigationUrl = getImageGridNavigationUrl;
        this.handleImageGridResetFilters = handleImageGridResetFilters;
        this.restoreFiltersFromUri = restoreFiltersFromUri
    }
    function handleAppBarShown()
    {
        BingApp.traceVerbose("AppBarShown");
        BingApp.Instrumentation.logFromHandlerEvent(null,"AppBarLaunch")
    }
    function handleAppBarHidden()
    {
        BingApp.traceVerbose("AppBarHidden")
    }
    function handleMenuShown(event)
    {
        BingApp.Instrumentation.logFromHandlerEvent(event,biValues.selectSuffix)
    }
    function getFilterAppliedUri(uri, filterString)
    {
        var queryParameters = BingApp.Utilities.QueryString.parse(uri.query,{decode: true});
        var filterKey = "qft";
        if(!filterString)
        {
            if(queryParameters[filterKey])
                delete queryParameters[filterKey]
        }
        else
            queryParameters[filterKey] = filterString;
        var updatedQuery = BingApp.Utilities.QueryString.serialize(queryParameters);
        var originalUri = uri.absoluteUri;
        return new Windows.Foundation.Uri(originalUri.substring(0,originalUri.length - uri.query.length - uri.fragment.length) + updatedQuery + originalUri.substring(originalUri.length - uri.fragment.length))
    }
    function clearMenuItemsSelection(flyout)
    {
        var menuItem;
        for(var menuItemIndex = 0; menuItemIndex < flyout.children.length; menuItemIndex++)
        {
            menuItem = flyout.children[menuItemIndex];
            if(menuItem)
                if(menuItem && menuItem.className === "win-command" && menuItem.winControl.type !== "separator" && menuItem.winControl.type !== "toggle")
                    menuItem.winControl.selected = false;
                else if(menuItem.winControl.type === "toggle")
                    menuItem.winControl.selected = true
        }
    }
    function createAppBarCommandSelectedLabelElements()
    {
        Array.prototype.forEach.call(document.querySelectorAll("#commandSize, #commandColor, #commandType, #commandLayout, #commandPeople"),function createAppBarCommandSelectedLabelElement(command)
        {
            var spanElement = document.createElement("span");
            spanElement.id = command.winControl.flyout.element.id + selectedFilterLabel;
            spanElement.textContent = "";
            spanElement.className = "win-label";
            command.appendChild(spanElement)
        })
    }
    var IframeHostController = WinJS.Class.derive(BingApp.Classes.ViewController,function constructor()
        {
            BingApp.Classes.ViewController.call(this);
            Object.defineProperties(this,{
                _currentContentState: {
                    value: contentStates.empty,
                    writable: true,
                    enumerable: false,
                    configurable: false
                },
                _handlersAreRegistered: {
                    value: false,
                    writable: true,
                    enumerable: false,
                    configurable: false
                },
                _iframeForwardNavigationHistory: {
                    value: [],
                    writable: false,
                    enumerable: false,
                    configurable: false
                },
                _pendingNavigationOutUris: {
                    value: [],
                    writable: false,
                    enumerable: false,
                    configurable: false
                },
                _appBarHelper: {
                    value: new AppBarHelper,
                    writable: false,
                    enumerable: false,
                    configurable: false
                }
            })
        },{
            notifyUnloading: function()
            {
                this._unregisterHandlers();
                this._iframeForwardNavigationHistory.length = 0;
                this._pendingNavigationOutUris.length = 0;
                var iframe = getIframe();
                if(!WinJS.Utilities.hasClass(iframe,"hidden"))
                    WinJS.Utilities.addClass(iframe,"hidden");
                iframe.src = "about:blank";
                BingApp.Classes.ViewController.prototype.notifyUnloading.call(this)
            },
            setNavigationUri: function(uri, options)
            {
                this._registerHandlers();
                BingApp.traceInfo("IframeHostController.setNavigationUri: updating UI to reflect new navigation Uri: {0}.",uri.absoluteUri);
                this._appBarHelper.showHideImageGridAppBar(uri);
                var iframe = getIframe();
                var bingUri = this.translateNavigationUriToBingUri(uri,options);
                if(bingUri)
                {
                    uri = new Windows.Foundation.Uri(IframeHostController.translateBingUriToNavigationUri(new Windows.Foundation.Uri(bingUri)));
                    var historyLength = this._iframeForwardNavigationHistory.length;
                    var goBackInBrowserHistory = options && options.navigatingBackInHistory && historyLength > 0;
                    if(goBackInBrowserHistory)
                    {
                        var indexInHistory;
                        var historyRecord;
                        if(this._currentContentState === contentStates.ready || this._currentContentState === contentStates.loaded)
                        {
                            BingApp.traceInfo("IframeHostController.setNavigationUri: notifying iframe that it should use browser history to navigate back to: '{0}'.",bingUri);
                            this._currentContentState = contentStates.loading;
                            var navigationBackDistance = 1;
                            for(indexInHistory = historyLength - 2; indexInHistory >= 0; indexInHistory--)
                            {
                                historyRecord = this._iframeForwardNavigationHistory[indexInHistory];
                                if(historyRecord.hasOwnProperty("pendingBackOptions"))
                                    navigationBackDistance++;
                                else
                                    break
                            }
                            this._iframeForwardNavigationHistory.splice(historyLength - navigationBackDistance,navigationBackDistance);
                            if(!WinJS.Utilities.hasClass(iframe,"hidden"))
                                WinJS.Utilities.addClass(iframe,"hidden");
                            eventRelay.fireEvent(events.browserHistoryGoBack,{distance: navigationBackDistance})
                        }
                        else
                            for(indexInHistory = historyLength - 1; indexInHistory >= 0; indexInHistory--)
                            {
                                historyRecord = this._iframeForwardNavigationHistory[indexInHistory];
                                if(!historyRecord.hasOwnProperty("pendingBackOptions"))
                                {
                                    historyRecord.pendingBackOptions = options;
                                    break
                                }
                            }
                    }
                    else if(this._iframeForwardNavigationHistory.length === 0 || options && options.initiatedFromSerp || this._currentContentState === contentStates.empty || this._currentContentState === contentStates.loading)
                    {
                        BingApp.traceInfo("IframeHostController.setNavigationUri: setting iframe.src to: {0}.",bingUri);
                        this._currentContentState = contentStates.loading;
                        iframe.src = bingUri;
                        if(!(options && options.navigatingBackInHistory))
                            this._iframeForwardNavigationHistory.push({
                                navigationUri: uri,
                                bingUri: bingUri
                            })
                    }
                    else
                    {
                        BingApp.traceInfo("IframeHostController.setNavigationUri: search was initiated from outside of SERP. Sending '{0}' event and waiting for '{1}' event before updating iframe content.",events.navigateOut,events.navigateTo);
                        this._pendingNavigationOutUris.push(uri);
                        eventRelay.fireEvent(events.navigateOut,{navigationUri: bingUri});
                        navigationManager.cancelNavigation(false)
                    }
                }
            },
            _registerHandlers: function()
            {
                if(!this._handlersAreRegistered)
                {
                    var that = this;
                    var iframe = getIframe();
                    eventRelay.registerIframe(iframeRelayId,iframe);
                    BingApp.WebErrors.register(iframe);
                    eventRelay.addEventListener(BingApp.SoftKeyboard.events.visibilityChanged,iframeRelayId);
                    eventRelay.addEventListener(BingApp.Classes.NetworkDetectionService.networkStatusChangedEvent,iframeRelayId);
                    eventRelay.addEventListener(events.getImageData,iframeRelayId);
                    eventRelay.addEventListener(BingApp.Classes.FacebookAuthenticationRelay.events.signedIn,iframeRelayId);
                    eventRelay.addEventListener(BingApp.Classes.FacebookAuthenticationRelay.events.signedOut,iframeRelayId);
                    eventRelay.addEventListener(BingApp.Classes.LiveIdAuthenticationRelay.events.signedIn,iframeRelayId);
                    eventRelay.addEventListener(BingApp.Classes.LiveIdAuthenticationRelay.events.signedOut,iframeRelayId);
                    eventRelay.addEventListener(BingApp.Rewards.events.refreshRewardsStatus,iframeRelayId);
                    eventRelay.addEventListener(BingApp.SearchHistory.events.clearVisitedLinks,iframeRelayId);
                    eventRelay.addEventListener(events.browserHistoryGoBack,iframeRelayId);
                    eventRelay.addEventListener(events.navigateOut,iframeRelayId);
                    eventRelay.addEventListener(events.navigationStarted,handleNavigationStarted);
                    eventRelay.addEventListener(events.navigationCompleted,handleNavigationCompleted);
                    this._contentReadyListener = {callback: function callback(args)
                        {
                            handleContentReady.call(that,args)
                        }};
                    eventRelay.addEventListener(events.contentReady,this._contentReadyListener);
                    this._contentLoadedListener = {callback: function callback(args)
                        {
                            handleContentLoaded.call(that,args)
                        }};
                    eventRelay.addEventListener(events.contentLoaded,this._contentLoadedListener);
                    this._navigateToListener = {callback: function callback(args)
                        {
                            that._handleNavigateTo(args)
                        }};
                    eventRelay.addEventListener(events.navigateTo,this._navigateToListener);
                    eventRelay.addEventListener(events.viewModeChanged,handleViewModeChanged);
                    eventRelay.addEventListener(events.imageGridResetFilters,this._appBarHelper.handleImageGridResetFilters);
                    eventRelay.addEventListener(events.shareSourceData,this._appBarHelper.handleImageShareSourceData);
                    this._appBarHelper.registerImageDetailAppBar();
                    this._appBarHelper.registerImageGridHandlers();
                    this._handlersAreRegistered = true
                }
            },
            _unregisterHandlers: function()
            {
                if(this._handlersAreRegistered)
                {
                    var iframe = getIframe();
                    eventRelay.unregisterIframe(iframeRelayId);
                    BingApp.WebErrors.unregister(iframe);
                    eventRelay.removeEventListener(events.navigationStarted,handleNavigationStarted);
                    eventRelay.removeEventListener(events.navigationCompleted,handleNavigationCompleted);
                    eventRelay.removeEventListener(events.contentReady,this._contentReadyListener);
                    eventRelay.removeEventListener(events.contentLoaded,this._contentLoadedListener);
                    eventRelay.removeEventListener(events.navigateTo,this._navigateToListener);
                    eventRelay.removeEventListener(events.viewModeChanged,handleViewModeChanged);
                    eventRelay.removeEventListener(events.imageGridResetFilters,this._appBarHelper.handleImageGridResetFilters);
                    eventRelay.removeEventListener(events.shareSourceData,this._appBarHelper.handleImageShareSourceData);
                    this._appBarHelper.removeImageGridHandlers();
                    this._appBarHelper.deregisterImageDetailAppBar();
                    this._navigateToListener = null;
                    this._contentReadyListener = null;
                    this._contentLoadedListener = null;
                    this._handlersAreRegistered = false
                }
            },
            _handleNavigateTo: function(args)
            {
                var navigationUri = IframeHostController.getNavigationUriFromEventArgs(args);
                var options = IframeHostController.getNavigationOptionsFromEventArgs(args) || {};
                BingApp.traceInfo("IframeHostController.handleNavigateTo: received '{0}' message which is relayed from web compartment with Uri: {1}.",events.navigateTo,navigationUri);
                var pendingNavigationsCount = this._pendingNavigationOutUris.length;
                for(var indexPending = 0; indexPending < pendingNavigationsCount; indexPending++)
                {
                    var pendingUri = this._pendingNavigationOutUris[indexPending];
                    if(pendingUri.absoluteUri === navigationUri)
                        if(indexPending === pendingNavigationsCount - 1)
                        {
                            BingApp.traceInfo("IframeHostController.handleNavigateTo: reconciling most recent '{0}' event with '{1}' event.",events.navigateOut,events.navigateTo);
                            this._pendingNavigationOutUris.length = 0
                        }
                        else
                        {
                            BingApp.traceInfo("IframeHostController.handleNavigateTo: ignoring '{0}' event that was raised as a result of handling '{1}' that has been superceded by newer navigation.",events.navigateTo,events.navigateOut);
                            this._pendingNavigationOutUris.splice(indexPending,1);
                            return
                        }
                }
                var iframe = getIframe();
                if(!WinJS.Utilities.hasClass(iframe,"hidden"))
                    WinJS.Utilities.addClass(iframe,"hidden");
                options.initiatedFromSerp = true;
                var currentNavigationUri = navigationManager.getCurrentUri();
                if(currentNavigationUri && BingApp.Utilities.areEqualIgnoreCase(navigationUri,currentNavigationUri.absoluteUri))
                {
                    var currentOptions = navigationManager.getCurrentOptions();
                    if(currentOptions)
                        Object.keys(currentOptions).forEach(function copyNavigationOptionsProperties(propertyName)
                        {
                            if(!options.hasOwnProperty(propertyName))
                                options[propertyName] = currentOptions[propertyName]
                        });
                    navigationManager.refresh(options)
                }
                else
                    navigationManager.navigateTo(navigationUri,options)
            },
            translateNavigationUriToBingUri: function(uri, options)
            {
                var query = uri.query;
                var path = uri.path;
                var bingUrl = BingApp.locator.config.getConfig().app.general.host;
                var bingPath;
                var bingQuery;
                var bingFragment = uri.fragment;
                switch(path)
                {
                    case"/search":
                        bingPath = "/search";
                        bingQuery = query;
                        break;
                    case"/images/grid":
                        bingPath = "/images/search";
                        if(options && options.navigatingBackInHistory)
                        {
                            this._appBarHelper.restoreFiltersFromUri(uri);
                            bingQuery = uri.query
                        }
                        else
                        {
                            var imageUri = this._appBarHelper.getImageGridNavigationUrl(uri);
                            bingQuery = imageUri.query
                        }
                        break;
                    case"/rewards":
                        bingPath = "/rewards/dashboard";
                        bingQuery = query;
                        break;
                    case"/videos/grid":
                        bingPath = "/videos/search";
                        bingQuery = query;
                        break;
                    default:
                        BingApp.traceError("IframeHostController.translateNavigationUriToBingUri: cannot translate '{0}' to Bing Uri; unknown Uri path '{1}'.",uri.absoluteUri,path);
                        throw new BingApp.Classes.ErrorArgument("uri",BingApp.Utilities.format(WinJS.Resources.getString("error_unhandled_navigation_uri").value,path));
                }
                bingQuery = environment.getQueryStringWithIFrameParams(bingQuery,window);
                bingUrl = bingUrl + bingPath + bingQuery;
                if(bingFragment)
                    bingUrl = bingUrl + bingFragment;
                return bingUrl
            }
        },{
            translateBingUriToNavigationUri: function(uri)
            {
                var bingHost = BingApp.locator.config.getConfig().app.general.host;
                var indexInsideUri = uri.absoluteUri.indexOf(bingHost);
                if(indexInsideUri !== 0)
                    BingApp.traceError("IframeHostController.translateBingUriToNavigationUri: given Bing Url comes from unexpected host: {0}.",uri.absoluteUri);
                var navigationProtocolAndHost = "bingsearch://app";
                var navigationPath;
                var navigationQuery;
                var navigationFragment = uri.fragment;
                var path = uri.path;
                var query = uri.query;
                switch(path)
                {
                    case"/search":
                        navigationPath = "/search";
                        navigationQuery = query;
                        break;
                    case"/images/search":
                        navigationPath = "/images/grid";
                        navigationQuery = query;
                        break;
                    case"/rewards/dashboard":
                        navigationPath = "/rewards";
                        navigationQuery = query;
                        break;
                    case"/videos/search":
                        navigationPath = "/videos/grid";
                        navigationQuery = query;
                        break;
                    default:
                        BingApp.traceError("IframeHostController.translateBingUriToNavigationUri: cannot translate '{0}' to navigation Uri; unknown Uri path '{1}'.",uri.absoluteUri,path);
                        throw new BingApp.Classes.ErrorArgument("uri",BingApp.Utilities.format(WinJS.Resources.getString("error_unhandled_bing_uri").value,uri.absoluteUri));
                }
                var navigationUri = navigationProtocolAndHost + navigationPath;
                navigationQuery = environment.sanitizeQueryString(navigationQuery);
                if(navigationQuery && navigationQuery.length > 1)
                    navigationUri = navigationUri + navigationQuery;
                if(navigationFragment)
                    navigationUri = navigationUri + navigationFragment;
                return navigationUri
            },
            getNavigationUriFromEventArgs: function(args)
            {
                if(!args)
                    throw new BingApp.Classes.ErrorArgumentNullOrUndefined("args");
                if(!args.uri)
                    throw new BingApp.Classes.ErrorArgument("args",WinJS.Resources.getString("error_navigation_message_payload_must_contain_uri").value);
                var bingUri = args.uri;
                var uriList = BingApp.locator.config.getConfig().app.directNavigationAppUris;
                var isWhiteListedUri = false;
                Object.keys(uriList).forEach(function whiteListURI(key)
                {
                    isWhiteListedUri = isWhiteListedUri || uriList[key].value === bingUri
                });
                if(isWhiteListedUri)
                    return navigationManager.getCurrentUri();
                if(typeof bingUri === "string")
                    bingUri = new Windows.Foundation.Uri(bingUri);
                return IframeHostController.translateBingUriToNavigationUri(bingUri)
            },
            getNavigationOptionsFromEventArgs: function(args)
            {
                return args ? args.options : null
            },
            events: events
        });
    function getController()
    {
        if(!this._controller)
            this._controller = new IframeHostController;
        return this._controller
    }
    function ready()
    {
        var containedViewHost = document.querySelector(containedViewHostSelector);
        WinJS.Resources.processAll();
        setAsLockScreenConfirmationMessage = WinJS.Resources.getString("appbar_setas_lockscreen_confirmation").value;
        copyLinkConfirmationMessage = WinJS.Resources.getString("appbar_copylink_confirmation").value;
        copyImageConfirmationMessage = WinJS.Resources.getString("appbar_copy_confirmation").value;
        if(containedViewHost)
            containedViewHost.style.height = "100%"
    }
    WinJS.UI.Pages.define("/views/iframehost/html/iframehost.html",{
        ready: ready,
        getController: getController,
        _controller: null
    })
})(BingApp.locator.navigationManager,BingApp.locator.env,BingApp.locator.eventRelay)