﻿(function webErrorsModule()
{
    "use strict";
    var events = Object.freeze({
            error: "error:network",
            retry: "error:retry",
            navigateOut: "navigateOut"
        });
    var iframeRelayId = "ContentLoadingFailureHost";
    function handleError(errorInfo)
    {
        if(BingApp.locator.networkDetectionService.isConnected())
        {
            var queryString = null;
            if(errorInfo && errorInfo.error)
                queryString = BingApp.Utilities.QueryString.parse(errorInfo.error);
            var instObject = {
                    url: queryString && queryString.failureUrl || "unknown",
                    ei: queryString && queryString.failureName || "unknown"
                };
            BingApp.Instrumentation.logError(instObject)
        }
        var navigationManager = BingApp.locator.navigationManager;
        var relay = BingApp.locator.eventRelay;
        navigationManager.cancelNavigation(false)
    }
    function handleRetry()
    {
        BingApp.locator.navigationManager.refresh()
    }
    function register(iframe)
    {
        if(!iframe)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("iframe");
        var relay = BingApp.locator.eventRelay;
        var appTargetOrigin = window.location.protocol + "//" + window.location.host;
        relay.registerIframe(iframeRelayId,iframe,appTargetOrigin);
        relay.addEventListener(events.navigateOut,iframeRelayId);
        relay.addEventListener(events.error,handleError);
        relay.addEventListener(events.retry,handleRetry)
    }
    function unregister(iframe)
    {
        if(!iframe)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("iframe");
        var relay = BingApp.locator.eventRelay;
        relay.unregisterIframe(iframeRelayId);
        relay.removeEventListener(events.error,handleError);
        relay.removeEventListener(events.retry,handleRetry)
    }
    WinJS.Namespace.define("BingApp.WebErrors",{
        register: register,
        unregister: unregister,
        events: events
    })
})()