﻿(function hotspotsModule()
{
    "use strict";
    var displayBlock = "block",
        displayNone = "none",
        pixelUnit = "px";
    var hotspotIdleTimeoutMS = 3500,
        hotspotFadeTimeMS = 500,
        fadeInOutAnimationTimeMS = 300,
        movingAnimationTimeMS = 600,
        hotspotSelectedOpacity = "1.0",
        opaqueOpacity = "1",
        transparentOpacity = "0",
        defImageDimension = 100,
        hotspotAreaSize = 40,
        hotspotDescriptionWidthOffset = 486,
        hotspotDescriptionLeftOffset = "-" + hotspotDescriptionWidthOffset + pixelUnit,
        hotspotDescriptionDefaultOffset = "46px";
    var startPageLayout = BingApp.Instrumentation.ClientPages.StartPageInstrumention;
    var HotspotState = WinJS.Class.define(function constructor(stateName)
        {
            this.name = stateName
        },{
            name: null,
            enter: function(hotspot, context)
            {
                return WinJS.Promise.as()
            },
            leave: function(hotspot, context)
            {
                return WinJS.Promise.as()
            },
            canTransit: function(targetState)
            {
                return false
            }
        });
    var ShownState = WinJS.Class.derive(HotspotState,function constructor()
        {
            HotspotState.call(this,"shown")
        },{
            enter: function(hotspot, context)
            {
                hotspot._hotspotContainer.style.display = displayBlock;
                hotspot._hotspotContainer.style.opacity = opaqueOpacity;
                return WinJS.Promise.as()
            },
            canTransit: function(targetState)
            {
                if(targetState === hotspotStates.showing || targetState === hotspotStates.shown)
                    return false;
                return true
            }
        });
    var ShowingState = WinJS.Class.derive(HotspotState,function constructor()
        {
            HotspotState.call(this,"showing")
        },{
            enter: function(hotspot, context)
            {
                hotspot._hotspotContainer.style.display = displayBlock;
                return WinJS.UI.executeTransition(hotspot._hotspotContainer,{
                        property: "opacity",
                        delay: 0,
                        duration: hotspotFadeTimeMS,
                        timing: "linear",
                        to: 1
                    }).then(function transitFromShowingToShown()
                    {
                        return hotspot.transit(hotspotStates.shown)
                    })
            },
            leave: function(hotspot, context)
            {
                if(context && context.cancelUncompletedPromise && hotspot._currentPromise)
                    hotspot._currentPromise.cancel();
                return WinJS.Promise.as()
            },
            canTransit: function(targetState)
            {
                if(targetState === hotspotStates.shown)
                    return true;
                return false
            }
        });
    var MovingState = WinJS.Class.derive(HotspotState,function constructor()
        {
            HotspotState.call(this,"moving")
        },{
            enter: function(hotspot, context)
            {
                var transitionPromise = WinJS.Promise.as();
                if(context && context.targetTransform)
                {
                    var translate = context.targetTransform;
                    transitionPromise = WinJS.UI.executeTransition(hotspot._hotspotContainer,{
                        property: "transform",
                        delay: 0,
                        duration: movingAnimationTimeMS,
                        timing: "cubic-bezier(0.1, 0.9, 0.2, 1)",
                        to: translate
                    })
                }
                return transitionPromise.then(function transitFromMovingToShown()
                    {
                        return hotspot.transit(hotspotStates.shown)
                    })
            },
            canTransit: function(targetState)
            {
                if(targetState === hotspotStates.shown)
                    return true;
                return false
            }
        });
    var ExpandedState = WinJS.Class.derive(HotspotState,function constructor()
        {
            HotspotState.call(this,"expanded")
        },{
            enter: function(hotspot, context)
            {
                hotspot._hotspotAnchor.style.display = displayBlock;
                hotspot._hotspotTrigger.focus();
                var transitionPromises = [];
                transitionPromises.push(WinJS.UI.executeTransition(hotspot._hotspotContainer,{
                    property: "display",
                    delay: 0,
                    duration: 0,
                    timing: "linear",
                    to: displayBlock
                }));
                transitionPromises.push(WinJS.UI.executeTransition(hotspot._hotspotAnchor,{
                    property: "opacity",
                    delay: 0,
                    duration: fadeInOutAnimationTimeMS,
                    timing: "linear",
                    to: hotspotSelectedOpacity
                }));
                return WinJS.Promise.join(transitionPromises)
            },
            leave: function(hotspot, context)
            {
                if(context && context.hasTransition)
                    return WinJS.UI.executeTransition(hotspot._hotspotAnchor,{
                            property: "opacity",
                            delay: 0,
                            duration: fadeInOutAnimationTimeMS,
                            timing: "linear",
                            to: transparentOpacity
                        }).then(function()
                        {
                            hotspot._hotspotAnchor.style.display = displayNone;
                            return WinJS.Promise.as()
                        });
                else
                {
                    hotspot._hotspotAnchor.opacity = transparentOpacity;
                    hotspot._hotspotAnchor.style.display = displayNone;
                    return WinJS.Promise.as()
                }
            },
            canTransit: function(targetState)
            {
                if(targetState === hotspotStates.shown)
                    return true;
                return false
            }
        });
    var HidingState = WinJS.Class.derive(HotspotState,function constructor()
        {
            HotspotState.call(this,"hiding")
        },{
            enter: function(hotspot, context)
            {
                hotspot._hotspotContainer.style.display = displayBlock;
                return WinJS.UI.executeTransition(hotspot._hotspotContainer,{
                        property: "opacity",
                        delay: 0,
                        duration: hotspotFadeTimeMS,
                        timing: "linear",
                        from: hotspot._hotspotContainer.style.opacity,
                        to: 0
                    }).then(function transitFromHidingToHidden()
                    {
                        return hotspot.transit(hotspotStates.hidden)
                    })
            },
            leave: function(hotspot, context)
            {
                if(context && context.cancelUncompletedPromise && hotspot._currentPromise)
                    hotspot._currentPromise.cancel();
                return WinJS.Promise.as()
            },
            canTransit: function(targetState)
            {
                if(targetState === hotspotStates.hidden || targetState === hotspotStates.shown)
                    return true;
                return false
            }
        });
    var HiddenState = WinJS.Class.derive(HotspotState,function constructor()
        {
            HotspotState.call(this,"hidden")
        },{
            enter: function(hotspot, context)
            {
                hotspot._hotspotContainer.style.display = displayNone;
                hotspot._hotspotContainer.style.opacity = transparentOpacity;
                return WinJS.Promise.as()
            },
            canTransit: function(targetState)
            {
                if(targetState === hotspotStates.showing || targetState === hotspotStates.shown)
                    return true;
                return false
            }
        });
    var hotspotStates = Object.freeze({
            shown: new ShownState,
            showing: new ShowingState,
            moving: new MovingState,
            expanded: new ExpandedState,
            hiding: new HidingState,
            hidden: new HiddenState
        });
    var Hotspot = WinJS.Class.define(function constructor(hotspotDiv, id)
        {
            if(!hotspotDiv)
                throw new BingApp.Classes.ErrorArgumentNullOrUndefined("hotspotDiv");
            this._hotspotContainer = hotspotDiv;
            this._hotspotTrigger = hotspotDiv.querySelector(".hsTrigger");
            this._hotspotAnchor = hotspotDiv.querySelector(".hsAnchor");
            this._hotspotDescription = hotspotDiv.querySelector(".hsDescription");
            this._hotspotLinkText = hotspotDiv.querySelector(".hsLinkText");
            this._id = id;
            this.transit(hotspotStates.hidden)
        },{
            updateInfo: function(info, hasTransition)
            {
                if(!info || !info.link || !info.desc || !info.query)
                    throw new BingApp.Classes.ErrorArgument("info");
                var pageDimension = BingApp.Utilities.getPageDimensions();
                var pageWidth = pageDimension.width;
                var pageHeight = pageDimension.height;
                var locX = parseInt(info.LocX) * (pageWidth / defImageDimension);
                var locY = parseInt(info.LocY) * (pageHeight / defImageDimension);
                var transitionPromise = null;
                if(locX >= 0 && locX < pageWidth - hotspotAreaSize && locY >= 0 && locY < pageHeight - hotspotAreaSize)
                {
                    this._hotspotTrigger.style.display = displayBlock;
                    var newLink = updateHotspotLinkWithFormCode(info.link,this._id + 1);
                    this._hotspotAnchor.href = newLink;
                    this._hotspotAnchor.k = info.k;
                    this._hotspotDescription.innerHTML = info.desc;
                    this._hotspotLinkText.innerHTML = info.query;
                    if(locX + hotspotAreaSize + hotspotDescriptionWidthOffset >= pageWidth)
                        this._hotspotAnchor.style.left = hotspotDescriptionLeftOffset;
                    else
                        this._hotspotAnchor.style.left = hotspotDescriptionDefaultOffset;
                    var translate = "translateX(" + locX + pixelUnit + ") translateY(" + locY + pixelUnit + ")";
                    if(hasTransition)
                    {
                        var transitionContext = {targetTransform: translate};
                        var previousPromise = WinJS.Promise.as();
                        if(this._currentState === hotspotStates.showing)
                        {
                            transitionContext.cancelUncompletedPromise = true;
                            previousPromise = this.transit(hotspotStates.shown,transitionContext)
                        }
                        var hotspot = this;
                        transitionPromise = previousPromise.then(function onPreviousPromiseComplete()
                        {
                            return hotspot.transit(hotspotStates.moving,transitionContext)
                        })
                    }
                    else
                        this._hotspotContainer.style.transform = translate
                }
                else
                    this._hotspotTrigger.style.display = displayNone;
                return transitionPromise || WinJS.Promise.as()
            },
            transit: function(targetState, context)
            {
                if(!(targetState instanceof HotspotState))
                    throw new BingApp.Classes.ErrorArgument("targetState");
                if(targetState === this._currentState)
                    return WinJS.Promise.as();
                var leaveStatePromise = null;
                var hotspot = this;
                if(hotspot._currentState)
                {
                    if(!hotspot._currentState.canTransit(targetState))
                    {
                        BingApp.traceVerbose("Hotspot.transit: hotspot {0} can't transit from state {1} to state {2}.",hotspot._id,hotspot._currentState.name,targetState.name);
                        return WinJS.Promise.as()
                    }
                    BingApp.traceVerbose("Hotspot.transit: hotspot {0} leaves state {1}.",hotspot._id,hotspot._currentState.name);
                    leaveStatePromise = hotspot._currentState.leave(hotspot,context)
                }
                else
                    leaveStatePromise = WinJS.Promise.as();
                hotspot._currentPromise = leaveStatePromise.then(function enterTargetState()
                {
                    hotspot._currentState = targetState;
                    BingApp.traceVerbose("Hotspot.transit: hotspot {0} enters state {1}.",hotspot._id,targetState.name);
                    return targetState.enter(hotspot,context)
                });
                return hotspot._currentPromise
            },
            getState: function()
            {
                return this._currentState
            },
            _id: -1,
            _currentState: null,
            _currentPromise: null,
            _hotspotContainer: null,
            _hotspotTrigger: null,
            _hotspotDescription: null,
            _hotspotAnchor: null,
            _hotspotLinkText: null
        },{hotspotStates: hotspotStates});
    var events = Object.freeze({
            hotspotClick: "hotspotClick",
            hotspotsHiddenCompleted: "hotspotsHiddenCompleted"
        });
    var Hotspots = WinJS.Class.derive(BingApp.Classes.EventSource,function constructor(hotspotsDiv)
        {
            if(!hotspotsDiv)
                throw new BingApp.Classes.ErrorArgumentNullOrUndefined("hotspotsDiv");
            BingApp.Classes.EventSource.call(this);
            this._hotspotsContainer = hotspotsDiv;
            this._hotspotsCount = hotspotsDiv.childElementCount;
            var children = hotspotsDiv.querySelectorAll(".hsContainer");
            this._hotspotsList = [];
            for(var i = 0; i < this._hotspotsCount; i++)
                this._hotspotsList.push(new Hotspot(children[i],i));
            bindEvents(this)
        },{
            updateInfo: function(hotspotsInfo, hasTransition)
            {
                if(!this._isHotspotsEnabled)
                    return WinJS.Promise.as();
                if(!hotspotsInfo)
                    throw new BingApp.Classes.ErrorArgumentNullOrUndefined("hotspotsInfo");
                if(hotspotsInfo.length === 0)
                {
                    this.setContainerVisibility(false);
                    return WinJS.Promise.as()
                }
                this._cancelIdleTimer();
                var transitionPromises = [];
                var hotSpotKValues = startPageLayout.KValues.kCoreStartingValue;
                for(var i = 0; i < this._hotspotsCount; i++)
                {
                    hotspotsInfo[i].k = hotSpotKValues + i;
                    transitionPromises.push(this._hotspotsList[i].updateInfo(hotspotsInfo[i],hasTransition))
                }
                startPageLayout.populateCoreRegion(hotspotsInfo);
                var joinedPromises = WinJS.Promise.join(transitionPromises);
                var hotspots = this;
                joinedPromises.then(function onHotspotsUpdateInfoCompleted()
                {
                    hotspots._startIdleTimer();
                    hotspots.setContainerVisibility(true)
                });
                return joinedPromises
            },
            unload: function()
            {
                this._cancelIdleTimer()
            },
            setEnabled: function(isEnabled)
            {
                if(!isEnabled)
                    this.hideAll(false);
                this._isHotspotsEnabled = isEnabled
            },
            isEnabled: function()
            {
                return this._isHotspotsEnabled
            },
            setContainerVisibility: function(isVisible)
            {
                if(isVisible)
                    WinJS.Utilities.removeClass(this._hotspotsContainer,"hidden");
                else
                    WinJS.Utilities.addClass(this._hotspotsContainer,"hidden")
            },
            showAll: function(hasTransition, keepShown)
            {
                if(!this._isHotspotsEnabled)
                    return WinJS.Promise.as();
                var targetState = hasTransition ? hotspotStates.showing : hotspotStates.shown;
                var transitionPromises = [];
                var transitionContext = {hasTransition: hasTransition};
                this._hotspotsList.forEach(function showHotspot(hotspot)
                {
                    var state = hotspot.getState();
                    if(state === hotspotStates.expanded || state === hotspotStates.hiding)
                    {
                        transitionContext.cancelUncompletedPromise = true;
                        transitionPromises.push(hotspot.transit(hotspotStates.shown,transitionContext))
                    }
                    else
                        transitionPromises.push(hotspot.transit(targetState,transitionContext))
                });
                if(!keepShown)
                    this._startIdleTimer();
                return WinJS.Promise.join(transitionPromises)
            },
            hideAll: function(hasTransition)
            {
                if(!this._isHotspotsEnabled)
                    return WinJS.Promise.as();
                var targetState = hasTransition ? hotspotStates.hiding : hotspotStates.hidden;
                var transitionPromises = [];
                transitionPromises.push(this.closeExpandedHotspot(false));
                this._hotspotsList.forEach(function hideHotspot(hotspot)
                {
                    transitionPromises.push(hotspot.transit(targetState))
                });
                var joinedPromises = WinJS.Promise.join(transitionPromises);
                var hotspots = this;
                joinedPromises.then(function onHotspotsHiddenCompleted()
                {
                    hotspots.fireEvent(events.hotspotsHiddenCompleted)
                });
                return joinedPromises
            },
            clickHotspot: function(index)
            {
                if(!this._isHotspotsEnabled)
                    return WinJS.Promise.as();
                if(index < 0 || index >= this._hotspotsCount)
                    throw new BingApp.Classes.ErrorArgument("index","hotspot index is out of range.");
                var hotspots = this;
                var transitionPromises = [];
                this._cancelIdleTimer();
                this.fireEvent(events.hotspotClick);
                if(!this.hasExpandedHotspot())
                {
                    this._hotspotsList.forEach(function hideHotspot(hotspot, i)
                    {
                        if(i !== index)
                            transitionPromises.push(hotspots._hotspotsList[i].transit(hotspotStates.hiding));
                        else
                            transitionPromises.push(hotspots._hotspotsList[i].transit(hotspotStates.expanded))
                    });
                    return WinJS.Promise.join(transitionPromises)
                }
                else
                    return this.showAll(true,false)
            },
            closeExpandedHotspot: function(hasTransition)
            {
                if(!this._isHotspotsEnabled)
                    return WinJS.Promise.as();
                if(this.hasExpandedHotspot())
                    return this.showAll(hasTransition,false);
                else
                    return WinJS.Promise.as()
            },
            hasExpandedHotspot: function()
            {
                for(var i = 0; i < this._hotspotsList.length; i++)
                {
                    var hotspot = this._hotspotsList[i];
                    if(hotspot.getState() === hotspotStates.expanded)
                        return true
                }
                return false
            },
            _startIdleTimer: function()
            {
                clearTimeout(this._hotspotsIdleTimer);
                this._hotspotsIdleTimer = setTimeout(function hotspotsOnIdleTimerTimeout(hotspots)
                {
                    hotspots.hideAll(true)
                },hotspotIdleTimeoutMS,this)
            },
            _cancelIdleTimer: function()
            {
                if(this._hotspotsIdleTimer)
                {
                    clearTimeout(this._hotspotsIdleTimer);
                    this._hotspotsIdleTimer = null
                }
            },
            _hotspotsList: null,
            _hotspotsIdleTimer: null,
            _hotspotsContainer: null,
            _hotspotsCount: 0,
            _isHotspotsEnabled: true
        },{events: events});
    function navigateHotspotLink(link, k, eventObject)
    {
        startPageLayout.logCoreClick("HotSpot",k,eventObject.currentTarget.evtSourceOrigin);
        if(link)
        {
            var appLink = BingApp.Classes.Shell.getAppPathFromWebPath(link);
            if(appLink && BingApp.Classes.Shell.isSupportedAppUri(appLink))
                BingApp.locator.navigationManager.navigateTo(appLink);
            else
                BingApp.Utilities.invokeURI(link)
        }
    }
    function updateHotspotLinkWithFormCode(link, index)
    {
        var lowCaseLink = link.toLowerCase();
        var formCode = BingApp.Classes.Shell.formCodes.fromHotspot + index;
        var updatedLink = BingApp.Utilities.upsertFormCode(lowCaseLink,formCode);
        return updatedLink
    }
    function bindEvents(hotspots)
    {
        var hotspotsList = hotspots._hotspotsList;
        hotspotsList.forEach(function bindHotspotEvents(hotspot, index)
        {
            var hotspotTrigger = hotspot._hotspotTrigger;
            hotspotTrigger.addEventListener("click",function hotspotOnTriggerClick()
            {
                hotspots.clickHotspot(index)
            });
            hotspotTrigger.addEventListener("keydown",function hotspotOnTriggerKeyDown(eventObject)
            {
                if(BingApp.Utilities.isKeyDownAndSpaceOrEnterKey(eventObject))
                    hotspots.clickHotspot(index)
            });
            var hotspotLink = hotspot._hotspotLinkText;
            if(!hotspotLink.onblur)
                hotspotLink.onblur = function hotspotOnLinkBlur()
                {
                    hotspots.showAll(true)
                };
            var hotspotAnchor = hotspot._hotspotAnchor;
            hotspotAnchor.addEventListener("keydown",function hotspotOnAnchorKeyDown(eventData)
            {
                if(BingApp.Utilities.isKeyDownAndSpaceOrEnterKey(eventData))
                    navigateHotspotLink(this.href,this.k,eventData)
            });
            if(!hotspotAnchor.onclick)
                hotspotAnchor.onclick = function hotspotOnAnchorClick(eventData)
                {
                    hotspots.closeExpandedHotspot(false);
                    navigateHotspotLink(this.href,this.k,eventData)
                };
            hotspotAnchor.addEventListener("MSPointerDown",BingApp.Utilities.onMSPointerDown,false)
        })
    }
    WinJS.Namespace.define("BingApp.Classes",{
        Hotspot: Hotspot,
        Hotspots: Hotspots
    })
})()