﻿(function mainViewModule(environment)
{
    "use strict";
    var searchBarMessageContentLoaded = "searchBar:contentLoaded";
    var searchBarMessageResize = "searchBar:resize";
    var searchBarMessageResizeNormal = "normal";
    var searchBarNormalSize = "100px";
    var searchBarNormalSnappedSize = "60px";
    var searchBarHiddenSize = "0%";
    var isLandscapeMode = true,
        isScrollComplete = true,
        scrollTimer,
        scrollTimeoutMS = 100,
        attribUnselectedOpacity = "0.4",
        attribSelectedOpacity = "0.6",
        pageWidth = 0,
        pageHeight = 0,
        defaultHost,
        cachedSettings = [],
        styleProperty = "style",
        selectedClass = "selected",
        displayBlock = "block",
        displayNone = "none",
        maxNumImages = 8,
        isLayoutRightToLeft = false,
        newImageTimeoutMS = 2e3;
    var hotspots = null;
    var hotspotsTransitionPromise = null;
    var previousScrollLeft = null;
    var isInSnapTransition = false;
    var isGettingImagesAfterFirstRun = false;
    var viewManagement = Windows.UI.ViewManagement;
    var backgroundInitTimeout = 2e3;
    var MainViewController = WinJS.Class.derive(BingApp.Classes.ViewController,function constructor(mainViewPage)
        {
            BingApp.Classes.ViewController.call(this);
            var hotspotsDiv = document.querySelector("#hotspotsContainer");
            hotspots = new BingApp.Classes.Hotspots(hotspotsDiv);
            var searchBarIFrame = document.querySelector("#searchbariframe");
            this._searchBar = new BingApp.Classes.SearchBar(searchBarIFrame);
            this._mainViewPage = mainViewPage
        },{
            getHost: function()
            {
                return document.querySelector("#containedviewhost")
            },
            setNavigationUri: function(uri)
            {
                if(!this._currentNavigationUri || this._currentNavigationUri.absoluteUri !== uri.absoluteUri)
                {
                    this._currentNavigationUri = uri;
                    this._refreshContentDisplay()
                }
            },
            notifyUnloading: function()
            {
                var eventRelay = BingApp.locator.eventRelay;
                if(this._mainViewPage._onNetworkStatusChangeWrapper)
                    eventRelay.removeEventListener(BingApp.Classes.NetworkDetectionService.networkStatusChangedEvent,this._mainViewPage._onNetworkStatusChangeWrapper);
                if(this._searchBarResizeEventListener)
                    eventRelay.removeEventListener(searchBarMessageResize,this._searchBarResizeEventListener);
                if(this._searchBarContentLoadedEventListener)
                    eventRelay.removeEventListener(searchBarMessageContentLoaded,this._searchBarContentLoadedEventListener);
                if(this._settingsCompleteListener)
                    eventRelay.removeEventListener(BingApp.BackgroundImageLoader.events.settingsLoaded,this._settingsCompleteListener);
                if(this._downloadCompleteListener)
                    eventRelay.removeEventListener(BingApp.BackgroundImageLoader.events.downloadComplete,this._downloadCompleteListener);
                if(this._windowResizeEventListener)
                    window.removeEventListener("resize",this._windowResizeEventListener);
                this._searchBar.unregister();
                this._mainViewPage._onNetworkStatusChangeWrapper = null;
                this._searchBarResizeEventListener = null;
                this._searchBarContentLoadedEventListener = null;
                this._settingsCompleteListener = null;
                this._downloadCompleteListener = null;
                this._windowResizeEventListener = null;
                hotspots.unload();
                BingApp.Classes.ViewController.prototype.notifyUnloading.call(this)
            },
            getSearchBar: function()
            {
                return this._searchBar
            },
            notifyViewReady: function()
            {
                var that = this;
                this._isViewReady = true;
                var eventRelay = BingApp.locator.eventRelay;
                if(!this._searchBarResizeEventListener)
                {
                    this._searchBarResizeEventListener = {callback: function callback(data)
                        {
                            that._refreshContentDisplay(data)
                        }};
                    eventRelay.addEventListener(searchBarMessageResize,this._searchBarResizeEventListener)
                }
                if(!this._searchBarContentLoadedEventListener)
                {
                    this._searchBarContentLoadedEventListener = {callback: function callback(data)
                        {
                            BingApp.tracePerf("onPP:CT");
                            that._refreshContentDisplay(data)
                        }};
                    eventRelay.addEventListener(searchBarMessageContentLoaded,this._searchBarContentLoadedEventListener)
                }
                if(!this._windowResizeEventListener)
                {
                    this._windowResizeEventListener = function windowResizeEventListener()
                    {
                        if(that._currentSearchBarSize === searchBarMessageResizeNormal)
                        {
                            var headerElement = document.querySelector("#header");
                            if(viewManagement.ApplicationView.value === viewManagement.ApplicationViewState.snapped)
                                headerElement.style.height = searchBarNormalSnappedSize;
                            else
                                headerElement.style.height = searchBarNormalSize
                        }
                        if(viewManagement.ApplicationView.value === viewManagement.ApplicationViewState.snapped || viewManagement.ApplicationView.value === viewManagement.ApplicationViewState.fullScreenPortrait)
                        {
                            hotspots.setContainerVisibility(false);
                            hotspots.setEnabled(false)
                        }
                        else
                            hotspots.setEnabled(true);
                        var previousIsLandscapeMode = isLandscapeMode;
                        getPageDimensions();
                        if(hotspots.isEnabled())
                        {
                            var navigationUri = BingApp.locator.navigationManager.getCurrentUri();
                            var isUpdateHotspots = false;
                            if(navigationUri)
                            {
                                navigationUri = navigationUri.schemeName + "://" + navigationUri.host + navigationUri.path;
                                if(BingApp.Utilities.areEqualIgnoreCase(navigationUri,BingApp.Classes.Shell.uris.home))
                                    isUpdateHotspots = true
                            }
                            if(previousIsLandscapeMode !== isLandscapeMode)
                                isUpdateHotspots = true;
                            BingApp.Instrumentation.ClientPages.StartPageInstrumention.ignoreNextRequest();
                            updatePage(true,isUpdateHotspots)
                        }
                        if(viewManagement.ApplicationView.value === viewManagement.ApplicationViewState.snapped)
                            showAttribTextInSnap();
                        else
                            closeOpenAttribText();
                        var currentImageDiv = querySelector("#bgImage" + BingApp.HomePageData.currentImageIndex);
                        if(currentImageDiv)
                            currentImageDiv.scrollIntoView();
                        that._refreshContentDisplay()
                    };
                    window.addEventListener("resize",this._windowResizeEventListener)
                }
                var voiceOverlay = document.getElementById("voice-overlay");
                if(voiceOverlay && !voiceOverlay.hasChildNodes())
                    WinJS.UI.Pages.render("/voice/html/voice.html",voiceOverlay);
                this._refreshContentDisplay()
            },
            _refreshContentDisplay: function(resizeData)
            {
                var navigationUri = BingApp.locator.navigationManager.getCurrentUri();
                if(this._isViewReady && this._currentNavigationUri)
                {
                    navigationUri = this._currentNavigationUri.schemeName + "://" + this._currentNavigationUri.host + this._currentNavigationUri.path;
                    this._currentSearchBarSize = BingApp.MainViewTransitions.update(this._currentSearchBarSize,navigationUri,hotspots,resizeData)
                }
            },
            _mainViewPage: null,
            _searchBar: null,
            _searchBarResizeEventListener: null,
            _searchBarContentLoadedEventListener: null,
            _settingsCompleteListener: null,
            _downloadCompleteListener: null,
            _windowResizeEventListener: null,
            _currentSearchBarSize: null,
            _isViewReady: false,
            _currentNavigationUri: null
        });
    function getController()
    {
        if(!this._controller)
            this._controller = new MainViewController(this);
        return this._controller
    }
    function ready()
    {
        var eventRelay = BingApp.locator.eventRelay;
        eventRelay.fireEvent(BingApp.Classes.ProgressIndicatorService.events.start);
        var isBackgroundInitTimeoutHandled = false;
        function notifyBackgroundInitDone()
        {
            if(!isBackgroundInitTimeoutHandled)
            {
                isBackgroundInitTimeoutHandled = true;
                eventRelay.fireEvent(BingApp.Classes.ProgressIndicatorService.events.finish)
            }
        }
        WinJS.Promise.timeout(backgroundInitTimeout).then(notifyBackgroundInitDone,notifyBackgroundInitDone);
        WinJS.Resources.processAll();
        defaultHost = BingApp.locator.config.getConfig().app.general.host;
        bindEvents();
        getPageDimensions();
        var environmentObject = environment || BingApp.locator.env;
        environmentObject.configurationReady().done(function onComplete()
        {
            maxNumImages = environmentObject.configuration["numImages"]
        });
        var direction = document.body.currentStyle.direction;
        if(direction === "rtl")
            isLayoutRightToLeft = true;
        var settingLoadedFromDisk = false;
        var controller = this.getController();
        if(!controller._settingsCompleteListener)
        {
            controller._settingsCompleteListener = {callback: function settingsLoaded()
                {
                    BingApp.traceInfo("mainview.ready: Received settings loaded event");
                    if(!settingLoadedFromDisk)
                    {
                        getCachedSettings();
                        updatePage(true,true);
                        notifyBackgroundInitDone();
                        settingLoadedFromDisk = true
                    }
                }};
            eventRelay.addEventListener(BingApp.BackgroundImageLoader.events.settingsLoaded,controller._settingsCompleteListener)
        }
        if(!controller._downloadCompleteListener)
        {
            controller._downloadCompleteListener = {callback: function onBackgroundImageDownloadComplete()
                {
                    BingApp.traceInfo("mainview.ready: Received download complete event");
                    getCachedSettings();
                    closeOpenAttribText();
                    hotspots.closeExpandedHotspot();
                    var bgImages = [];
                    for(var i = 0; i < maxNumImages; i++)
                    {
                        var bgImage = querySelector("#bgImage" + i);
                        if(bgImage)
                            bgImages.push(bgImage)
                    }
                    WinJS.Promise.timeout(newImageTimeoutMS).then(function HPTimer()
                    {
                        return WinJS.UI.Animation.fadeOut(bgImages)
                    }).then(function onFadeOutComplete()
                    {
                        return updatePage(false,true)
                    }).then(function onUpdatePageComplete()
                    {
                        return WinJS.UI.Animation.fadeIn(bgImages)
                    }).then(function onFadeInComplete()
                    {
                        isGettingImagesAfterFirstRun = false
                    })
                }};
            eventRelay.addEventListener(BingApp.BackgroundImageLoader.events.downloadComplete,controller._downloadCompleteListener)
        }
        BingApp.BackgroundImageLoader.start();
        if(!BingApp.locator.networkDetectionService.isConnected())
            onNetworkStatusChange({connectionState: BingApp.Classes.NetworkDetectionService.connectionStates.none});
        else
            BingApp.Instrumentation.flushErrorQueue();
        this._onNetworkStatusChangeWrapper = function onNetworkStatusChangeWrapper(eventData)
        {
            onNetworkStatusChange(eventData)
        };
        eventRelay.addEventListener(BingApp.Classes.NetworkDetectionService.networkStatusChangedEvent,this._onNetworkStatusChangeWrapper);
        var headerElement = document.querySelector("#header");
        headerElement.style.height = searchBarHiddenSize;
        this.getController().notifyViewReady()
    }
    function updatePage(isCachedImage, isUpdateHotspots)
    {
        var promise = null;
        if(!cachedSettings || cachedSettings.length === 0)
        {
            isGettingImagesAfterFirstRun = true;
            showDefaultImage()
        }
        else
        {
            updateBackgroundImages();
            updateAttribInfo();
            if(isUpdateHotspots)
                promise = updateHotspotInfo(false)
        }
        if(isCachedImage)
            BingApp.tracePerf("Homepage:CachedImageLoaded");
        else
            BingApp.tracePerf("Homepage:DownloadedImageLoaded");
        return promise || WinJS.Promise.as()
    }
    function getCachedSettings()
    {
        cachedSettings = BingApp.BackgroundImageLoader.getSettings()
    }
    function bindEvents()
    {
        var bgImagePanoramaDiv = document.querySelector("#bgImagePanorama");
        if(bgImagePanoramaDiv && bgImagePanoramaDiv.addEventListener)
        {
            bgImagePanoramaDiv.addEventListener("wheel",wheelScrollHandler);
            bgImagePanoramaDiv.addEventListener("mousemove",onMouseMoveHandler);
            bgImagePanoramaDiv.addEventListener("scroll",onScrollHandler);
            bgImagePanoramaDiv.addEventListener("click",onPageClick);
            bgImagePanoramaDiv.addEventListener("keydown",keyDownHandler)
        }
        var attribButtonDiv = document.querySelector("#attribButton");
        if(attribButtonDiv && attribButtonDiv.addEventListener)
        {
            attribButtonDiv.addEventListener("click",onAttribClick);
            attribButtonDiv.addEventListener("keydown",onAttribClick)
        }
        if(hotspots)
            hotspots.addEventListener("hotspotClick",onHotspotClick)
    }
    function onHotspotClick()
    {
        closeOpenAttribText()
    }
    function onMouseMoveHandler()
    {
        if(!hotspots.hasExpandedHotspot() && !isGettingImagesAfterFirstRun)
            hotspots.showAll(true)
    }
    function onPageClick()
    {
        closeOpenAttribText();
        hotspots.showAll(true)
    }
    function keyDownHandler(e)
    {
        e.preventDefault();
        if(isInSnapTransition || !isScrollComplete || isGettingImagesAfterFirstRun)
            return;
        var inRangeIndex;
        var bgImagePanoramaElement;
        var key = WinJS.Utilities.Key;
        if(e.which === key.leftArrow && !isLayoutRightToLeft || e.which === key.rightArrow && isLayoutRightToLeft || e.which === key.upArrow || e.which === key.pageUp)
        {
            bgImagePanoramaElement = document.querySelector("#bgImagePanorama");
            bgImagePanoramaElement.scrollLeft += pageWidth;
            inRangeIndex = Math.min(maxNumImages - 1,Math.max(0,BingApp.HomePageData.currentImageIndex + 1));
            BingApp.HomePageData.currentImageIndex = inRangeIndex
        }
        else if(e.which === key.rightArrow && !isLayoutRightToLeft || e.which === key.leftArrow && isLayoutRightToLeft || e.which === key.downArrow || e.which === key.pageDown)
        {
            bgImagePanoramaElement = document.querySelector("#bgImagePanorama");
            bgImagePanoramaElement.scrollLeft -= pageWidth;
            inRangeIndex = Math.min(maxNumImages - 1,Math.max(0,BingApp.HomePageData.currentImageIndex - 1));
            BingApp.HomePageData.currentImageIndex = inRangeIndex
        }
        updateAttribInfo();
        updateHotspotInfo(true)
    }
    function wheelScrollHandler(e)
    {
        var move = 0;
        var currentImageIndex = BingApp.HomePageData.currentImageIndex;
        e.preventDefault();
        if(isInSnapTransition || !isScrollComplete || isGettingImagesAfterFirstRun)
            return;
        if(e.deltaY !== 0)
        {
            move = e.deltaY > 0 ? -1 : 1;
            currentImageIndex = Math.min(maxNumImages - 1,Math.max(0,currentImageIndex + move));
            BingApp.HomePageData.currentImageIndex = currentImageIndex;
            var bgImagePanoramaElement = document.querySelector("#bgImagePanorama");
            bgImagePanoramaElement.scrollLeft += move * pageWidth;
            if(bgImagePanoramaElement.scrollLeft < 0)
                bgImagePanoramaElement.scrollLeft = 0;
            updateAttribInfo();
            updateHotspotInfo(true)
        }
        else if(e.deltaX !== 0)
        {
            isInSnapTransition = true;
            move = e.deltaX > 0 ? -1 : 1;
            if(isLayoutRightToLeft)
                move = -move;
            currentImageIndex = Math.min(maxNumImages - 1,Math.max(0,currentImageIndex + move));
            BingApp.HomePageData.currentImageIndex = currentImageIndex;
            snapBackgroundImage(function updateHomepageData()
            {
                updateAttribInfo();
                updateHotspotInfo(true)
            })
        }
    }
    function snapBackgroundImage(completedCallback)
    {
        var bgImagePanoramaElement = document.querySelector("#bgImagePanorama");
        var targetScrollLeftOffset = pageWidth * BingApp.HomePageData.currentImageIndex - bgImagePanoramaElement.scrollLeft;
        if(isLayoutRightToLeft)
            targetScrollLeftOffset = -targetScrollLeftOffset;
        var handleImagePanoramaTransitioned = function()
            {
                bgImagePanoramaElement.removeEventListener("transitionend",handleImagePanoramaTransitioned);
                stopSnapTransition();
                completedCallback()
            };
        bgImagePanoramaElement.addEventListener("transitionend",handleImagePanoramaTransitioned);
        startSnapTransition(targetScrollLeftOffset)
    }
    function startSnapTransition(targetScrollLeftOffset)
    {
        var bgImagePanoramaElement = document.querySelector("#bgImagePanorama");
        previousScrollLeft = bgImagePanoramaElement.scrollLeft;
        Array.prototype.forEach.call(bgImagePanoramaElement.children,function startSnapAnimation(item)
        {
            WinJS.Utilities.addClass(item,"bgImagePanoramaSnapAnimation");
            var transformStyle = "translateX(" + targetScrollLeftOffset + "px)";
            item.style.msTransform = transformStyle
        })
    }
    function stopSnapTransition()
    {
        var bgImagePanoramaElement = document.querySelector("#bgImagePanorama");
        var currentStyle = window.getComputedStyle(bgImagePanoramaElement.children[0]);
        var movedOffset = new MSCSSMatrix(currentStyle.msTransform).m41;
        if(isLayoutRightToLeft)
            movedOffset = -movedOffset;
        Array.prototype.forEach.call(bgImagePanoramaElement.children,function stopSnapAnimation(item)
        {
            WinJS.Utilities.removeClass(item,"bgImagePanoramaSnapAnimation");
            item.style.msTransform = ""
        });
        if(movedOffset)
            bgImagePanoramaElement.scrollLeft = previousScrollLeft + movedOffset;
        isInSnapTransition = false
    }
    function onScrollHandler()
    {
        clearTimeout(scrollTimer);
        scrollTimer = setTimeout(onScrollComplete,scrollTimeoutMS);
        if(isScrollComplete)
        {
            isScrollComplete = false;
            closeOpenAttribText();
            hotspots.closeExpandedHotspot()
        }
    }
    function onScrollComplete()
    {
        var bgImagePanoramaElement = document.querySelector("#bgImagePanorama");
        var offsetLeft = bgImagePanoramaElement.scrollLeft;
        isScrollComplete = true;
        var thisImageIndex = Math.round(offsetLeft / pageWidth);
        var currentImageIndex = BingApp.HomePageData.currentImageIndex;
        if(thisImageIndex !== currentImageIndex)
        {
            var instrumentationObject = {
                    name: "Swipe",
                    state: currentImageIndex < thisImageIndex ? "Back" : "Front"
                };
            BingApp.Instrumentation.log(instrumentationObject,"Cl.Swipe");
            BingApp.HomePageData.currentImageIndex = thisImageIndex;
            updateAttribInfo();
            updateHotspotInfo(true)
        }
    }
    function updateBackgroundImages()
    {
        var imageDiv;
        for(var i = 0; i < cachedSettings.length && i < maxNumImages; i++)
        {
            imageDiv = querySelector("#bgImage" + i);
            var imagePath = "url('" + BingApp.HomePageData.getImageUri(isLandscapeMode,i) + "')";
            imageDiv[styleProperty].backgroundImage = imagePath
        }
        updateImageDisplay(cachedSettings.length)
    }
    function updateHotspotInfo(hasTransition)
    {
        var hotspotsData = BingApp.HomePageData.getCurrentHotspotsData();
        if(!hotspotsData)
        {
            BingApp.traceVerbose("mainViewModule.updateHotspotInfo: Fail to get hotspots data and skip updating hotspot.");
            return WinJS.Promise.as()
        }
        if(hotspotsTransitionPromise)
            hotspotsTransitionPromise.cancel();
        hotspotsTransitionPromise = hotspots.showAll(false).then(function onComplete()
        {
            return hotspots.updateInfo(hotspotsData,hasTransition)
        });
        return hotspotsTransitionPromise
    }
    function isKeyDownAndSpaceOrEnterKey(eventObject)
    {
        var key = WinJS.Utilities.Key;
        if(eventObject.type === "keydown" && eventObject.which !== key.space && eventObject.which !== key.enter)
            return false;
        return true
    }
    function onAttribClick(eventData)
    {
        if(!isKeyDownAndSpaceOrEnterKey(eventData))
            return;
        var attribTextElement = document.querySelector("#attribText");
        if(WinJS.Utilities.hasClass(attribTextElement,selectedClass))
            closeOpenAttribText();
        else
        {
            hotspots.closeExpandedHotspot(true);
            attribTextElement.style.display = displayBlock;
            BingApp.MainViewTransitions.fadeAttribButton(attribSelectedOpacity);
            WinJS.Utilities.addClass(attribTextElement,selectedClass)
        }
    }
    function closeOpenAttribText()
    {
        var attribTextElement = document.querySelector("#attribText");
        if(viewManagement.ApplicationView.value !== viewManagement.ApplicationViewState.snapped)
        {
            attribTextElement.style.display = displayNone;
            BingApp.MainViewTransitions.fadeAttribButton(attribUnselectedOpacity);
            WinJS.Utilities.removeClass(attribTextElement,selectedClass)
        }
        else
            showAttribTextInSnap()
    }
    function showAttribTextInSnap()
    {
        var accessibilitySettings = new viewManagement.AccessibilitySettings;
        if(!accessibilitySettings.highContrast)
        {
            var attribTextElement = document.querySelector("#attribText");
            attribTextElement.style.display = displayBlock;
            WinJS.Utilities.addClass(attribTextElement,selectedClass)
        }
    }
    function updateAttribInfo()
    {
        var attribTextElement = document.querySelector("#attribText");
        attribTextElement.innerHTML = BingApp.HomePageData.getCurrentImageInfo()
    }
    function getPageDimensions()
    {
        pageWidth = BingApp.Utilities.getPageDimensions().width;
        pageHeight = BingApp.Utilities.getPageDimensions().height;
        isLandscapeMode = pageWidth > pageHeight
    }
    function showDefaultImage()
    {
        var imagePath = "url('" + BingApp.HomePageData.getDefaultImageUri(isLandscapeMode) + "')";
        var bgImage0Element = document.querySelector("#bgImage0");
        bgImage0Element["style"].backgroundImage = imagePath;
        var attribTextElement = document.querySelector("#attribText");
        hotspots.setContainerVisibility(false);
        attribTextElement.innerHTML = WinJS.Resources.getString("default_background_image_text").value;
        updateImageDisplay(1)
    }
    function updateImageDisplay(numImages)
    {
        var index;
        var imageDiv;
        for(index = 0; index < numImages && index < maxNumImages; index++)
        {
            imageDiv = querySelector("#bgImage" + index);
            imageDiv["style"].display = "inline-block"
        }
        for(index = numImages; index < maxNumImages; index++)
        {
            imageDiv = querySelector("#bgImage" + index);
            imageDiv["style"].display = "none"
        }
    }
    function querySelector(selector)
    {
        return document.querySelector(selector)
    }
    function onNetworkStatusChange(eventData)
    {
        var networkStatusElement = document.querySelector("#networkStatus");
        if(eventData.connectionState === BingApp.Classes.NetworkDetectionService.connectionStates.internetAccess)
        {
            networkStatusElement.style.display = displayNone;
            BingApp.Instrumentation.flushErrorQueue()
        }
        else
        {
            var networkStatusTextElement = document.querySelector("#networkStatusText");
            networkStatusTextElement.innerText = WinJS.Resources.getString("main_internet_connection_not_available").value;
            networkStatusElement.style.display = "-ms-grid"
        }
    }
    WinJS.UI.Pages.define("/views/main/html/mainview.html",{
        _controller: null,
        _onNetworkStatusChangeWrapper: null,
        getController: getController,
        showDefaultImage: showDefaultImage,
        ready: ready
    })
})(BingApp.locator.env)