﻿(function MainViewTransitionsModule()
{
    "use strict";
    var darkOpacityComplement = "0.2",
        lightOpacityComplement = "0.4",
        transparentOpacityComplement = "1";
    var opaqueOpacity = "1",
        displayNone = "none",
        searchBarHiddenSize = "0%",
        searchBarFullSize = "100%",
        searchBarNormalSize = "100px",
        searchBarNormalSnappedSize = "60px",
        searchBarShieldDark = "dark",
        searchBarShieldLight = "light",
        searchBarMessageResizeFull = "full",
        searchBarMessageResizeNormal = "normal",
        searchBarMessageResizeHidden = "hidden",
        attribUnselectedOpacity = "0.4",
        attribSelectedOpacity = "0.6",
        headerSelector = "#header",
        bgImagePanoramaSelector = "#bgImagePanorama",
        containedViewHostSelector = "#containedviewhost",
        fadeInOutAnimationTimeMS = 300,
        searchBarOpacityChangeDuration = 250,
        mainViewMessageShowContent = "mainView:showContent";
    function fadeAttribButton(newOpacity)
    {
        var attribButtonElement = document.querySelector("#attribButton");
        var currentOpacity = attribButtonElement.style.opacity;
        if(currentOpacity !== newOpacity)
            WinJS.UI.executeTransition(attribButtonElement,{
                property: "opacity",
                delay: 0,
                duration: fadeInOutAnimationTimeMS,
                timing: "linear",
                from: currentOpacity,
                to: newOpacity
            })
    }
    function update(searchBarSize, navigationUri, hotspots, resizeData)
    {
        var newHeaderSize,
            newSearchBarSize = searchBarSize,
            searchBarSizeChanged = false,
            newBackgroundOpacity = getNewBackgroundOpacity(navigationUri,resizeData);
        if(resizeData && searchBarSize !== resizeData.height)
        {
            newHeaderSize = getNewHeaderSize(resizeData);
            if(newHeaderSize)
            {
                newSearchBarSize = resizeData.height;
                updateHeader(newBackgroundOpacity,newHeaderSize)
            }
            searchBarSizeChanged = true
        }
        updateContent(navigationUri,newBackgroundOpacity,newSearchBarSize,searchBarSizeChanged,hotspots);
        return newSearchBarSize
    }
    function updateHeader(newBackgroundOpacity, newHeaderSize)
    {
        var headerDiv = document.querySelector(headerSelector);
        var bgImagePanoramaDiv = document.querySelector(bgImagePanoramaSelector);
        headerDiv.style.display = "";
        if(newHeaderSize !== searchBarFullSize)
            updateHeaderBackground(headerDiv,bgImagePanoramaDiv,newHeaderSize,newBackgroundOpacity);
        headerDiv.style.height = newHeaderSize;
        if(newHeaderSize === searchBarFullSize)
            updateHeaderBackground(headerDiv,bgImagePanoramaDiv,newHeaderSize,newBackgroundOpacity)
    }
    function updateContent(navigationUri, newBackgroundOpacity, searchBarSize, searchBarSizeChanged, hotspots)
    {
        var containedViewHostDiv = document.querySelector(containedViewHostSelector),
            fadeOutPromise,
            hideHotspotsPromise;
        var attribButtonElement = document.querySelector("#attribButton");
        var attribTextElement = document.querySelector("#attribText");
        if(searchBarSize === searchBarMessageResizeFull)
        {
            if(searchBarSizeChanged)
            {
                fadeOutPromise = WinJS.UI.Animation.fadeOut([attribButtonElement,attribTextElement,containedViewHostDiv]);
                hideHotspotsPromise = hotspots.hideAll(true);
                WinJS.Promise.join(fadeOutPromise,hideHotspotsPromise).done(function hideElements()
                {
                    attribButtonElement.style.display = displayNone;
                    attribTextElement.style.display = displayNone;
                    containedViewHostDiv.style.display = displayNone
                })
            }
        }
        else
        {
            if(BingApp.Utilities.areEqualIgnoreCase(navigationUri,BingApp.Classes.Shell.uris.home))
            {
                attribButtonElement.style.display = "";
                attribTextElement.style.display = "";
                fadeAttribButton(attribUnselectedOpacity);
                WinJS.UI.Animation.fadeIn(attribTextElement);
                hotspots.showAll(true)
            }
            else
            {
                fadeOutPromise = WinJS.UI.Animation.fadeOut([attribButtonElement,attribTextElement]);
                hideHotspotsPromise = hotspots.hideAll(true);
                WinJS.Promise.join(fadeOutPromise,hideHotspotsPromise).done(function hideElements()
                {
                    attribButtonElement.style.display = displayNone;
                    attribTextElement.style.display = displayNone
                })
            }
            var bgImagePanoramaDiv = document.querySelector(bgImagePanoramaSelector);
            if(bgImagePanoramaDiv.style.opacity !== newBackgroundOpacity)
                WinJS.UI.executeTransition(bgImagePanoramaDiv,{
                    property: "opacity",
                    delay: 0,
                    duration: searchBarOpacityChangeDuration,
                    timing: "linear",
                    from: bgImagePanoramaDiv.style.opacity,
                    to: newBackgroundOpacity
                });
            var fireShowEvent = containedViewHostDiv.style.display === displayNone;
            containedViewHostDiv.style.display = "";
            if(fireShowEvent)
                BingApp.locator.eventRelay.fireEvent(mainViewMessageShowContent,null);
            var from = containedViewHostDiv.style.opacity;
            if(from !== "" && from !== opaqueOpacity)
                WinJS.UI.Animation.fadeIn(containedViewHostDiv)
        }
    }
    function getNewHeaderSize(resizeData)
    {
        var newHeaderSize;
        if(resizeData.height === searchBarMessageResizeFull)
            newHeaderSize = searchBarFullSize;
        else if(resizeData.height === searchBarMessageResizeNormal)
            if(Windows.UI.ViewManagement.ApplicationView.value === Windows.UI.ViewManagement.ApplicationViewState.snapped)
                newHeaderSize = searchBarNormalSnappedSize;
            else
                newHeaderSize = searchBarNormalSize;
        else if(resizeData.height === searchBarMessageResizeHidden)
            newHeaderSize = searchBarHiddenSize;
        return newHeaderSize
    }
    function updateHeaderBackground(headerDiv, bgImagePanoramaDiv, newSize, newBackgroundOpacity)
    {
        if(newSize === searchBarHiddenSize)
            headerDiv.style.display = displayNone;
        else
        {
            headerDiv.style.display = "";
            if(newSize === searchBarFullSize)
                bgImagePanoramaDiv.style.opacity = newBackgroundOpacity
        }
    }
    function getNewBackgroundOpacity(navigationUri, resizeData)
    {
        var newBackgroundOpacity;
        if(resizeData && resizeData.shield)
            if(resizeData.shield === searchBarShieldLight)
                newBackgroundOpacity = lightOpacityComplement;
            else if(resizeData.shield === searchBarShieldDark)
                newBackgroundOpacity = darkOpacityComplement;
        if(!newBackgroundOpacity)
        {
            var shell = BingApp.Classes.Shell,
                areTheSame = BingApp.Utilities.areEqualIgnoreCase;
            if(areTheSame(navigationUri,shell.uris.search) || areTheSame(navigationUri,shell.uris.start))
                newBackgroundOpacity = lightOpacityComplement;
            else if(areTheSame(navigationUri,shell.uris.imagesgrid) || areTheSame(navigationUri,shell.uris.videosgrid))
                newBackgroundOpacity = darkOpacityComplement;
            else
                newBackgroundOpacity = transparentOpacityComplement
        }
        return newBackgroundOpacity
    }
    WinJS.Namespace.define("BingApp.MainViewTransitions",{
        fadeAttribButton: fadeAttribButton,
        update: update
    })
})()