﻿(function searchBarModule()
{
    "use strict";
    var messageResizeSearchBar = "searchBar:resize";
    var messageNavigateCompletedSearchBar = "searchBar:navigationCompleted";
    var messageJustType = "searchBar:enableJustType";
    var linkCount = .1;
    var events = Object.freeze({
            refresh: "searchBar:refresh",
            errorRetry: "error:retry",
            contentLoaded: "searchBar:contentLoaded",
            navigateBack: "searchBar:navigateBack",
            navigateTo: "searchBar:navigateTo"
        });
    var iframeRelayId = "BingSearchBar";
    WinJS.Namespace.define("BingApp.Classes",{SearchBar: WinJS.Class.define(function constructor(searchBarIFrame, options)
        {
            if(BingApp.Utilities.isNullOrUndefined(searchBarIFrame))
                throw new BingApp.Classes.ErrorArgumentNullOrUndefined("searchBarIFrame");
            this._searchBarIFrame = searchBarIFrame;
            this._config = options && options.config || BingApp.locator.config;
            this._env = options && options.env || BingApp.locator.env;
            this._eventRelay = options && options.eventRelay || BingApp.locator.eventRelay;
            this._navigationManager = options && options.navigationManager || BingApp.locator.navigationManager;
            this._keyboardCapabilities = options && options.keyboardCapabilities || new Windows.Devices.Input.KeyboardCapabilities;
            this._registerEvents();
            var navigationUri = this._getNavigationUrl();
            BingApp.traceInfo("SearchBar.initialize: navigating searchBarIFrame to Uri: {0}.",navigationUri);
            searchBarIFrame.src = navigationUri
        },{
            refresh: function()
            {
                this._isLoaded = false;
                this._eventRelay.fireEvent(messageResizeSearchBar,{height: "hidden"});
                this._searchBarIFrame.src = this._getNavigationUrl()
            },
            unregister: function()
            {
                if(this._registered)
                {
                    this._eventRelay.unregisterIframe(iframeRelayId);
                    this._eventRelay.removeEventListener(events.refresh,this._refreshListener);
                    this._eventRelay.removeEventListener(events.errorRetry,this._refreshListener);
                    this._eventRelay.removeEventListener(events.contentLoaded,this._contentLoadedListener);
                    this._eventRelay.removeEventListener(events.navigateBack,this._handleNavigateBackClosure);
                    this._eventRelay.removeEventListener(events.navigateTo,this._handleNavigateToClosure);
                    this._eventRelay.removeEventListener(BingApp.Classes.NetworkDetectionService.networkStatusChangedEvent,this._networkStatusChangedListener);
                    this._navigationManager.removeEventListener(BingApp.Classes.NavigationManager.events.navigationCompleted,this._navigationCompletedListener);
                    this._navigationManager.removeEventListener(BingApp.Classes.NavigationManager.events.historyCleared,this._navigationHistoryClearedListener);
                    this._refreshListener = null;
                    this._contentLoadedListener = null;
                    this._networkStatusChangedListener = null;
                    this._navigationCompletedListener = null;
                    this._navigationHistoryClearedListener = null;
                    this._registered = false
                }
            },
            _enableJustType: function()
            {
                if(this._config.getConfig().justType.general.enabled && this._keyboardCapabilities.keyboardPresent)
                {
                    var settingsFlyoutElement = document.querySelector(".pagecontrol .win-settingsflyout");
                    if(settingsFlyoutElement)
                        BingApp.traceInfo("SearchBar._enableJustType: because a settings flyout is currently displayed, JustType won't be enabled.");
                    else
                    {
                        BingApp.traceInfo("SearchBar._enableJustType: enabling JustType.");
                        this._eventRelay.fireEvent(messageJustType,true)
                    }
                }
            },
            _registerEvents: function()
            {
                if(!this._registered)
                {
                    var that = this;
                    this._handleNavigateToClosure = BingApp.Utilities.createClosure(this,this._handleNavigateTo);
                    this._handleNavigateBackClosure = BingApp.Utilities.createClosure(this,this._handleNavigateBack);
                    this._eventRelay.registerIframe(iframeRelayId,this._searchBarIFrame);
                    this._eventRelay.addEventListener(BingApp.Classes.Shell.events.appVisible,iframeRelayId);
                    this._eventRelay.addEventListener(messageJustType,iframeRelayId);
                    this._eventRelay.addEventListener(messageNavigateCompletedSearchBar,iframeRelayId);
                    this._eventRelay.addEventListener(BingApp.SearchPaneHandler.events.searchBarQuerySubmitted,iframeRelayId);
                    this._eventRelay.addEventListener(BingApp.SearchPaneHandler.events.searchBarResultSuggestionChosen,iframeRelayId);
                    this._eventRelay.addEventListener(BingApp.SearchPaneHandler.events.searchBarSuggestionsRequested,iframeRelayId);
                    this._eventRelay.addEventListener(BingApp.SearchPane.events.visibilityChanged,iframeRelayId);
                    this._eventRelay.addEventListener(BingApp.SoftKeyboard.events.visibilityChanged,iframeRelayId);
                    this._eventRelay.addEventListener(BingApp.Instrumentation.updateIG,iframeRelayId);
                    this._eventRelay.addEventListener(BingApp.SearchHistory.events.clearVisitedLinks,iframeRelayId);
                    this._eventRelay.addEventListener(BingApp.Classes.VoiceEvents.events.update,iframeRelayId);
                    this._eventRelay.addEventListener(BingApp.Classes.FacebookAuthenticationRelay.events.signedIn,iframeRelayId);
                    this._eventRelay.addEventListener(BingApp.Classes.FacebookAuthenticationRelay.events.signedOut,iframeRelayId);
                    this._eventRelay.addEventListener(BingApp.Classes.LiveIdAuthenticationRelay.events.signedIn,iframeRelayId);
                    this._eventRelay.addEventListener(BingApp.Classes.LiveIdAuthenticationRelay.events.signedOut,iframeRelayId);
                    this._eventRelay.addEventListener(BingApp.Rewards.events.refreshRewardsStatus,iframeRelayId);
                    this._refreshListener = {callback: function refreshSearchBar()
                        {
                            that.refresh()
                        }};
                    this._contentLoadedListener = {callback: function onContentLoaded(args)
                        {
                            that._handleContentLoaded(args)
                        }};
                    this._networkStatusChangedListener = {callback: function onNetworkStatusChanged(args)
                        {
                            that._handleNetworkStatusChanged(args)
                        }};
                    this._navigationCompletedListener = {callback: function onNavigationCompleted(args)
                        {
                            that._handleNavigationCompleted(args)
                        }};
                    this._navigationHistoryClearedListener = {callback: function onNavigationHistoryCleared()
                        {
                            that._handleNavigationHistoryCleared()
                        }};
                    this._eventRelay.addEventListener(events.refresh,this._refreshListener);
                    this._eventRelay.addEventListener(events.errorRetry,this._refreshListener);
                    this._eventRelay.addEventListener(events.contentLoaded,this._contentLoadedListener);
                    this._eventRelay.addEventListener(events.navigateBack,this._handleNavigateBackClosure);
                    this._eventRelay.addEventListener(events.navigateTo,this._handleNavigateToClosure);
                    this._eventRelay.addEventListener(BingApp.Classes.NetworkDetectionService.networkStatusChangedEvent,this._networkStatusChangedListener);
                    this._navigationManager.addEventListener(BingApp.Classes.NavigationManager.events.navigationCompleted,this._navigationCompletedListener);
                    this._navigationManager.addEventListener(BingApp.Classes.NavigationManager.events.historyCleared,this._navigationHistoryClearedListener);
                    this._registered = true
                }
            },
            _getNavigationUrl: function()
            {
                return this._config.getConfig().app.general.host + this._env.configuration["searchBoxPage"] + this._env.getQueryStringWithIFrameParams(this._env.configuration["searchBoxPageParameters"],window)
            },
            _handleNetworkStatusChanged: function(eventArgs)
            {
                if(!this._isLoaded && eventArgs.connectionState === BingApp.Classes.NetworkDetectionService.connectionStates.internetAccess)
                    this.refresh()
            },
            _handleContentLoaded: function(args)
            {
                BingApp.traceInfo("SearchBar.handleContentLoaded: received 'contentLoaded' message which is relayed from search bar inside web compartment.");
                this._isLoaded = true;
                this._eventRelay.registerIframe(iframeRelayId,this._searchBarIFrame,args.targetOrigin);
                this._eventRelay.fireEvent(BingApp.Classes.VoiceEvents.events.update,{show: this._config.getConfig().voice.general.enabled});
                this._enableJustType();
                this._handleNavigationCompleted({uri: this._navigationManager.getCurrentUri()})
            },
            _handleNavigationCompleted: function(data)
            {
                BingApp.traceInfo("SearchBar.handleNavigationCompleted: received 'navigationCompleted' message from navigation manager.");
                if(data && data.uri && !data.error && !data.canceled)
                    this._eventRelay.fireEvent(messageNavigateCompletedSearchBar,{
                        canGoBack: this._navigationManager.canGoBack(),
                        searchPaneIsVisible: BingApp.SearchPane.isVisible(),
                        uri: {path: data.uri.path},
                        parsedParameters: BingApp.Utilities.QueryString.parse(data.uri.query,{decode: true})
                    })
            },
            _handleNavigateTo: function(args)
            {
                var navigationUri = args.uri;
                BingApp.traceInfo("SearchBar.handleNavigateTo: received 'navigateTo' message which is relayed from search bar inside web compartment with Uri: {0}.",navigationUri);
                if(args.ref === "Logo")
                    BingApp.Instrumentation.logClick({
                        name: args.ref,
                        K: BingApp.Instrumentation.KValues.kValueLogo + linkCount
                    });
                if(BingApp.Classes.Shell.isSupportedAppUri(args.uri))
                    this._navigationManager.navigateTo(args.uri);
                else
                    BingApp.traceError("SearchBar.handleNavigateTo: discarding '{0}' navigation Uri because it cannot be handled by application.",navigationUri)
            },
            _handleNavigateBack: function()
            {
                BingApp.traceInfo("SearchBar.handleNavigateBack: received 'navigateBack' message which is relayed from search bar inside web compartment.");
                BingApp.Instrumentation.log({name: "Back"},BingApp.Instrumentation.appEvent);
                this._navigationManager.goBack({navigatingBackInHistory: true})
            },
            _handleNavigationHistoryCleared: function()
            {
                BingApp.traceInfo("SearchBar.handleNavigationHistoryCleared: received 'historyCleared' message from navigation manager.");
                if(this._navigationManager.getCurrentNavigationStatus() === BingApp.Classes.NavigationManager.navigationStatus.idle)
                {
                    var currentUri = this._navigationManager.getCurrentUri();
                    this._eventRelay.fireEvent(messageNavigateCompletedSearchBar,{
                        canGoBack: this._navigationManager.canGoBack(),
                        searchPaneIsVisible: BingApp.SearchPane.isVisible(),
                        uri: {path: currentUri && currentUri.path},
                        parsedParameters: BingApp.Utilities.QueryString.parse(currentUri.query,{decode: true})
                    })
                }
            },
            _isLoaded: false,
            _registered: false,
            _searchBarIFrame: null,
            _refreshListener: null,
            _contentLoadedListener: null,
            _networkStatusChangedListener: null,
            _navigationCompletedListener: null
        },{events: events})})
})()