﻿(function startPageInstrumentationModule()
{
    "use strict";
    var startPageLayout = null;
    var startPageDs = null;
    var appNS = "BingApp";
    var footerReady = false;
    var coreReady = false;
    var ignoreLayoutRequest = false;
    var pageName = "start";
    var sendLayoutWhenAvailable = false;
    var kStartPage = Object.freeze({
            kTrendTitle: 6e3,
            kMoreButton: 6005,
            kCoreStartingValue: 6010,
            kCoreAttributeValue: 6100,
            kFooterStartingValue: 6510,
            kHotspotStartValue: 5e3
        });
    function createStartPageLayoutTemplate()
    {
        startPageLayout = [{
                Region: "Header",
                AppNS: appNS,
                L: [{
                        Region: "Searchbox",
                        T: "L.Box",
                        L: [{
                                T: "L.Form",
                                AppNS: appNS,
                                K: BingApp.Instrumentation.KValues.kValueSearch + .1
                            }]
                    }]
            },{
                Region: "Core",
                AppNS: appNS,
                L: [{
                        Region: "Hotspots",
                        T: "L.Box",
                        K: kStartPage.kHotspotStartValue + .1,
                        L: []
                    },{
                        T: "L.Url",
                        AppNS: appNS,
                        K: kStartPage.kCoreAttributeValue + .1
                    }]
            },{
                Region: "Footer",
                T: "L.Box",
                K: kStartPage.kFooterStartingValue + .1,
                AppNS: appNS,
                L: []
            }]
    }
    function createStartPageDsTemplate()
    {
        startPageDs = [{
                AppNS: appNS,
                T: "D.Form",
                K: BingApp.Instrumentation.KValues.kValueSearch,
                Url: "/search"
            },{
                AppNS: appNS,
                T: "D.Top.RichBackground",
                K: kStartPage.kHotspotStartValue,
                DS: []
            },{
                AppNS: appNS,
                T: "D.Attribution",
                K: kStartPage.kCoreAttributeValue
            },{DS: [{
                        AppNS: appNS,
                        T: "D.Top",
                        Scenario: "footer",
                        Service: "config",
                        DS: []
                    }]}]
    }
    function populateCoreRegion(hotspotsData)
    {
        coreReady = false;
        if(!startPageDs || !startPageLayout)
        {
            createStartPageDsTemplate();
            createStartPageLayoutTemplate()
        }
        for(var i = 0; i < hotspotsData.length; i++)
        {
            startPageLayout[1].L[0].L.push({
                T: "L.Url",
                K: hotspotsData[i].k + .1
            });
            startPageDs[1].DS.push({
                T: "D.Hotspot",
                AppNS: appNS,
                DS: [{
                        T: "D.Url",
                        K: hotspotsData[i].k,
                        Url: hotspotsData[i].link
                    }]
            })
        }
        coreReady = true;
        trySendLayout()
    }
    function populateHeaderRegion(startData){}
    function resetFooterLayout()
    {
        startPageLayout[2].L = [];
        startPageDs[3].DS[0].DS = [];
        footerReady = false
    }
    function populateFooterRegion(startData)
    {
        resetFooterLayout();
        if(!startData.getAt)
            return;
        var trendsTitleElement = document.querySelector("#trendsTitle");
        startPageLayout[2].L.push({
            T: "L.Url",
            K: trendsTitleElement.k + .1
        });
        startPageDs[3].DS[0].DS.push({
            T: "D.Url",
            Url: "bingsearch://app/start",
            K: trendsTitleElement.k,
            AppNS: appNS,
            N: "trends"
        });
        for(var i = 0; i < startData.length; i++)
        {
            var item = startData.getAt(i);
            startPageLayout[2].L.push({
                T: "L.Url",
                K: item.k + .1
            });
            startPageDs[3].DS[0].DS.push({
                T: "D.Url",
                Url: item.Url,
                K: item.k,
                AppNS: appNS,
                N: "PN" + i
            })
        }
        var moreButtonElement = document.querySelector("#moreButton");
        startPageLayout[2].L.push({
            T: "L.Url",
            K: moreButtonElement.k + .1
        });
        startPageDs[3].DS[0].DS.push({
            T: "D.Url",
            Url: "bingsearch://app/start",
            K: moreButtonElement.k,
            AppNS: appNS,
            N: "more"
        });
        footerReady = true;
        trySendLayout()
    }
    function trySendLayout()
    {
        if(sendLayoutWhenAvailable && footerReady && coreReady)
        {
            logLayout();
            sendLayoutWhenAvailable = false
        }
    }
    function logCoreClick(eventName, k, src)
    {
        var instrumentationObject = {
                name: eventName,
                K: k + .1,
                Type: src
            };
        BingApp.Instrumentation.logClick(instrumentationObject)
    }
    function logFooterClick(page, k, origin)
    {
        var instrumentationObject = {
                name: page,
                K: k + .1,
                Type: origin
            };
        BingApp.Instrumentation.logClick(instrumentationObject)
    }
    function logLayout()
    {
        if(footerReady && coreReady && !ignoreLayoutRequest)
            BingApp.Instrumentation.logPartialImpression(pageName,startPageLayout,startPageDs);
        else if(!ignoreLayoutRequest)
            sendLayoutWhenAvailable = true;
        ignoreLayoutRequest = false
    }
    function generateLayout(startData)
    {
        var hotspots = document.querySelector("#hotspotsContainer");
        populateCoreRegion(hotspots);
        populateFooterRegion(startData)
    }
    function init()
    {
        createStartPageLayoutTemplate();
        createStartPageDsTemplate();
        BingApp.Instrumentation.registerLayoutHandler(pageName,logLayout)
    }
    function ignoreNextRequest()
    {
        ignoreLayoutRequest = true
    }
    WinJS.Namespace.define("BingApp.Instrumentation.ClientPages.StartPageInstrumention",{
        populateCoreRegion: populateCoreRegion,
        populateHeaderRegion: populateHeaderRegion,
        populateFooterRegion: populateFooterRegion,
        logCoreClick: logCoreClick,
        logFooterClick: logFooterClick,
        logLayout: logLayout,
        generateLayout: generateLayout,
        ignoreNextRequest: ignoreNextRequest,
        init: init,
        KValues: kStartPage
    })
})()