﻿(function voiceEventsModule()
{
    "use strict";
    var events = {
            search: "voice:search",
            update: "voice:enabledChanged"
        };
    function addVoiceSearchEvents(voice)
    {
        function voiceSearch()
        {
            BingApp.Instrumentation.log({name: "voiceLaunch"},BingApp.Instrumentation.appEvent);
            voice.recognizeVoiceAsync().done(voiceSearchComplete,voiceSearchError)
        }
        function voiceSearchComplete(response)
        {
            if(!response || !response.recognitionText)
            {
                BingApp.traceWarning("VoiceEvents.addVoiceSearchEvents.voiceSearchComplete: Missing response or recognition text");
                return
            }
            var recognitionText = response.recognitionText.replace(/<profanity>/g,"").replace(/<\/profanity>/g,"");
            var shellClass = BingApp.Classes.Shell;
            var navigationManager = BingApp.locator.navigationManager;
            var searchUri,
                formCode;
            switch(navigationManager.getCurrentUri().path)
            {
                case shellClass.uriPaths.home:
                    searchUri = shellClass.uris.search;
                    formCode = shellClass.formCodes.fromVoiceOnHomePage;
                    break;
                case shellClass.uriPaths.start:
                    searchUri = shellClass.uris.search;
                    formCode = shellClass.formCodes.fromVoiceOnTrendingPage;
                    break;
                case shellClass.uriPaths.search:
                    searchUri = shellClass.uris.search;
                    formCode = shellClass.formCodes.fromVoiceOnSERPPage;
                    break;
                case shellClass.uriPaths.imagesgrid:
                    searchUri = shellClass.uris.imagesgrid;
                    formCode = shellClass.formCodes.fromVoiceOnImageGridPage;
                    break;
                case shellClass.uriPaths.videosgrid:
                    searchUri = shellClass.uris.videosgrid;
                    formCode = shellClass.formCodes.fromVoiceOnVideoGridPage;
                    break;
                default:
                    BingApp.traceWarning("VoiceEvents.addVoiceSearchEvents.voiceSearchComplete: Voice search initiated on unknown page");
                    searchUri = shellClass.uris.search;
                    break
            }
            var query = {q: recognitionText};
            if(formCode)
                query.form = formCode;
            var uri = searchUri + BingApp.Utilities.QueryString.serialize(query);
            if(!shellClass.isSupportedAppUri(uri))
            {
                BingApp.traceWarning("VoiceEvents.voiceSearchComplete: Tried to navigate to unsupported URI: " + uri);
                return
            }
            navigationManager.navigateTo(uri)
        }
        function voiceSearchError(error)
        {
            BingApp.traceWarning("VoiceEvents.voiceSearch: Failed to recognize voice: " + error)
        }
        BingApp.locator.eventRelay.addEventListener(events.search,voiceSearch)
    }
    function addVoiceEnabledEvents()
    {
        function configurationUpdated()
        {
            BingApp.locator.eventRelay.fireEvent(events.update,{show: BingApp.locator.config.getConfig().voice.general.enabled})
        }
        BingApp.locator.eventRelay.addEventListener(BingApp.Classes.ConfigStore.events.updated,configurationUpdated)
    }
    WinJS.UI.Pages.define("/voice/html/voice.html",{ready: function voiceReady()
        {
            var voice = new BingApp.Classes.Voice;
            WinJS.Resources.processAll().done(function onResourceProcessingComplete()
            {
                var tipsElement = document.querySelector("#tips");
                tipsElement.textContent = BingApp.Utilities.format(WinJS.Resources.getString("voice_search_tips").value,WinJS.Resources.getString("voice_search_tip1").value);
                addVoiceSearchEvents(voice);
                addVoiceEnabledEvents()
            })
        }});
    WinJS.Namespace.define("BingApp.Classes.VoiceEvents",{events: events})
})()