﻿(function bingNewsDataProviderModule()
{
    "use strict";
    var bingNewsIdentity = "bingsearch://data/bingnews";
    var bingNewsCount = 20;
    var RequiredProperties = Object.freeze({
            Category: false,
            Date: false,
            Snippet: true,
            Source: true,
            Title: true,
            Url: true,
            Image: true
        });
    function parseBingNewsDocument(responseText)
    {
        if(BingApp.Utilities.isNullOrUndefined(responseText))
        {
            var errorMessage = "BingNewsDataProvider.parseBingNewsDocument: Null or undefined responseText";
            var instObject = {
                    url: this._getBingNewsUrl().url,
                    ei: errorMessage
                };
            BingApp.Instrumentation.logError(instObject);
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("responseText");
        }
        var doc;
        try
        {
            doc = JSON.parse(responseText)
        }
        catch(err)
        {
            var errorMessage = BingApp.Utilities.format("BingNewsDataProvider.parseBingNewsDocument: Input is not valid json - error name: {0} and error message:{1}",err.name || "",err.message || "");
            var instObject = {
                    url: this._getBingNewsUrl().url,
                    ei: errorMessage
                };
            BingApp.Instrumentation.logError(instObject);
            BingApp.traceError(errorMessage);
            throw new BingApp.Classes.ErrorArgument("responseText",errorMessage);
        }
        if(doc && doc.SearchResponse && doc.SearchResponse.News && doc.SearchResponse.News.Results)
        {
            var newsItems = doc.SearchResponse.News.Results;
            var results = [];
            for(var i = 0; i < newsItems.length; i++)
                if(newsItems[i] && newsItems[i].Cluster)
                {
                    var cluster = newsItems[i].Cluster;
                    var currentNewsItem = null;
                    var key;
                    if(cluster.Articles)
                        for(key in cluster.Articles)
                        {
                            var currentItem = cluster.Articles[key];
                            if(currentItem)
                            {
                                var corruptDataElement = false;
                                for(var property in RequiredProperties)
                                {
                                    if(property === "Image")
                                        continue;
                                    if(RequiredProperties[property] && !currentItem[property])
                                    {
                                        corruptDataElement = true;
                                        break
                                    }
                                }
                                if(corruptDataElement)
                                    continue;
                                currentItem.Date = currentItem.Date ? new Date(currentItem.Date) : new Date("");
                                currentItem.SnapSnippet = "";
                                currentNewsItem = currentItem;
                                break
                            }
                        }
                    if(RequiredProperties.Image && currentNewsItem && cluster.Images)
                        for(key in cluster.Images)
                        {
                            var currentImageItem = cluster.Images[key];
                            if(currentImageItem && currentImageItem.Thumbnail && currentImageItem.Thumbnail.Url)
                            {
                                currentNewsItem.Image = currentImageItem.Thumbnail;
                                if(!currentNewsItem.Image.Url.match(/^http:\/\//i))
                                    currentNewsItem.Image.Url = this._config.getConfig().app.general.host + currentNewsItem.Image.Url;
                                currentNewsItem.type = "news";
                                results.push(currentNewsItem);
                                break
                            }
                        }
                }
            return results
        }
        errorMessage = "BingNewsDataProvider.parseBingNewsDocument: Input does not the have the expected format";
        instObject = {
            url: this._getBingNewsUrl().url,
            ei: errorMessage
        };
        BingApp.Instrumentation.logError(instObject);
        BingApp.traceError(errorMessage);
        throw new BingApp.Classes.ErrorArgument("responseText",errorMessage);
    }
    function getBingNewsUrl()
    {
        var trendsSection = this._config.getConfig().trends;
        return{url: trendsSection.bingNews.url + "&" + this._env.getQueryString()}
    }
    WinJS.Namespace.define("BingApp.Classes",{BingNewsDataProvider: WinJS.Class.derive(BingApp.Classes.XhrDataProvider,function constructor(options)
        {
            if(!(this instanceof BingApp.Classes.BingNewsDataProvider))
            {
                BingApp.traceWarning("BingNewsDataProvider.ctor: Attempted using BingNewsDataProvider ctor as function; redirecting to use 'new BingNewsDataProvider()'.");
                return new BingApp.Classes.BingNewsDataProvider(options)
            }
            this._config = options && options.config || BingApp.locator.config;
            this._env = options && options.env || BingApp.locator.env;
            var that = this;
            BingApp.Classes.XhrDataProvider.call(this,bingNewsIdentity,bingNewsCount,{
                urlOptions: function getUrlOptionsClosure()
                {
                    return that._getBingNewsUrl()
                },
                parser: function parseClosure(responseText)
                {
                    return that._parseBingNewsDocument(responseText)
                }
            })
        },{
            _getBingNewsUrl: getBingNewsUrl,
            _parseBingNewsDocument: parseBingNewsDocument
        })})
})()