﻿(function msnTrendingDataProviderModule()
{
    "use strict";
    var msnTrendsIdentity = "bingsearch://data/msntrending";
    var msnTrendsCount = 15;
    var RequiredProperties = Object.freeze({
            guid: false,
            link: true,
            pubDate: false,
            title: true
        });
    function parseMsnTrendDocument(responseText)
    {
        if(BingApp.Utilities.isNullOrUndefined(responseText))
        {
            var errorMessage = "MsnTrendingDataProvider.parseMsnTrendDocument: Empty responseText";
            var instObject = {
                    url: this._getMsnTrendUrl().url,
                    ei: errorMessage
                };
            BingApp.Instrumentation.logError(instObject);
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("responseText");
        }
        var doc;
        try
        {
            doc = new Windows.Data.Xml.Dom.XmlDocument;
            doc.loadXml(responseText)
        }
        catch(err)
        {
            var errorMessage = BingApp.Utilities.format("MsnTrendingDataProvider.parseMsnTrendDocument: Input is not valid xml - error name: {0}, error message: {1}",err.name || "",err.message || "");
            var instObject = {
                    url: this._getMsnTrendUrl().url,
                    ei: errorMessage
                };
            BingApp.Instrumentation.logError(instObject);
            BingApp.traceError(errorMessage);
            throw new BingApp.Classes.ErrorArgument("responseText",errorMessage);
        }
        var elements = doc.selectNodes("//item");
        var actualCount = elements.length;
        var results = [];
        for(var i = 0; i < actualCount; i++)
        {
            var data = elements[i].childNodes;
            var singleItem = {};
            var corruptDataElement = false;
            for(var j = 0; j < data.length; j++)
            {
                var value = data[j];
                if(value && !value.nodeName)
                {
                    BingApp.traceInfo("MsnTrendingDataProvider.parseMsnTrendDocument: element in document is defective: {0}",value);
                    corruptDataElement = true;
                    break
                }
                singleItem[value.nodeName] = value.nodeName === "pubDate" ? !value.innerText ? new Date("") : new Date(value.innerText) : value.innerText
            }
            if(!corruptDataElement)
                for(var property in RequiredProperties)
                    if(RequiredProperties[property] && !singleItem[property])
                    {
                        corruptDataElement = true;
                        break
                    }
            if(corruptDataElement)
                continue;
            singleItem.type = "text";
            singleItem.shortTitle = "";
            results.push(singleItem)
        }
        return results
    }
    function getMsnTrendUrl()
    {
        var trendsSection = this._config.getConfig().trends;
        return{url: trendsSection.msnTrends.url}
    }
    WinJS.Namespace.define("BingApp.Classes",{MsnTrendingDataProvider: WinJS.Class.derive(BingApp.Classes.XhrDataProvider,function constructor(options)
        {
            if(!(this instanceof BingApp.Classes.MsnTrendingDataProvider))
            {
                BingApp.traceWarning("MsnTrendingDataProvider.ctor: Attempted using MsnTrendingDataProvider ctor as function; redirecting to use 'new MsnTrendingDataProvider()'.");
                return new BingApp.Classes.MsnTrendingDataProvider(options)
            }
            this._config = options && options.config || BingApp.locator.config;
            var that = this;
            BingApp.Classes.XhrDataProvider.call(this,msnTrendsIdentity,msnTrendsCount,{
                urlOptions: function getUrlOptionsClosure()
                {
                    return that._getMsnTrendUrl()
                },
                parser: function parseClosure(responseText)
                {
                    return that._parseMsnTrendDocument(responseText)
                }
            })
        },{
            _getMsnTrendUrl: getMsnTrendUrl,
            _parseMsnTrendDocument: parseMsnTrendDocument
        })})
})()