﻿(function popTextDataProviderModule()
{
    "use strict";
    var popTextIdentity = "bingsearch://data/poptext";
    var popTextCount = 8;
    var elementTagForVisualAndTextPopTrends = "Element";
    var RequiredProperties = Object.freeze({
            searchtermactual: true,
            title: true,
            Url: true
        });
    function parsePopTextDocument(responseText)
    {
        if(BingApp.Utilities.isNullOrUndefined(responseText))
        {
            var errorMessage = "PopTextDataProvider.parsePopTextDocument: Empty responseText";
            var instObject = {
                    url: this._getPopTextUrl().url,
                    ei: errorMessage
                };
            BingApp.traceError(errorMessage);
            BingApp.Instrumentation.logError(instObject);
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("responseText");
        }
        var doc;
        try
        {
            doc = new Windows.Data.Xml.Dom.XmlDocument;
            doc.loadXml(responseText)
        }
        catch(err)
        {
            var errorMessage = BingApp.Utilities.format("PopTextDataProvider.parsePopTextDocument: Input is not valid xml - error name: {0}, error message: {1}",err.name || "",err.message || "");
            var instObject = {
                    url: this._getPopTextUrl().url,
                    ei: errorMessage
                };
            BingApp.Instrumentation.logError(instObject);
            BingApp.traceError(errorMessage);
            throw new BingApp.Classes.ErrorArgument("responseText",errorMessage);
        }
        var elements = doc.selectNodes("//" + elementTagForVisualAndTextPopTrends);
        var count = elements.length;
        var results = [];
        for(var i = 0; i < count; i++)
        {
            var element = elements[i];
            var data = element.childNodes;
            var singleItem = {};
            var corruptDataElement = false;
            for(var j = 0; j < data.length; j++)
            {
                var value = data[j];
                if(value && !value.nodeName)
                {
                    BingApp.traceInfo("PopTextDataProvider.parsePopTextDocument: element in document is defective: {0}",value);
                    corruptDataElement = true;
                    break
                }
                var nodeName = value.nodeName;
                if(nodeName === "SearchTermDisplay")
                    nodeName = "title";
                else if(nodeName === "SearchTermActual")
                    nodeName = "searchtermactual";
                singleItem[nodeName] = value.innerText
            }
            if(!corruptDataElement)
                for(var property in RequiredProperties)
                    if(RequiredProperties[property] && !singleItem[property])
                    {
                        corruptDataElement = true;
                        break
                    }
            if(corruptDataElement)
                continue;
            singleItem.type = "text";
            singleItem.shortTitle = "";
            results.push(singleItem)
        }
        return results
    }
    function getPopTextUrl()
    {
        var trendsSection = this._config.getConfig().trends;
        return{url: trendsSection.popularNowText.url}
    }
    WinJS.Namespace.define("BingApp.Classes",{PopTextDataProvider: WinJS.Class.derive(BingApp.Classes.XhrDataProvider,function constructor(options)
        {
            if(!(this instanceof BingApp.Classes.PopTextDataProvider))
            {
                BingApp.traceWarning("PopTextDataProvider.ctor: Attempted using PopTextDataProvider ctor as function; redirecting to use 'new PopTextDataProvider()'.");
                return new BingApp.Classes.PopTextDataProvider(options)
            }
            this._config = options && options.config || BingApp.locator.config;
            var that = this;
            BingApp.Classes.XhrDataProvider.call(this,popTextIdentity,popTextCount,{
                urlOptions: function getUrlOptionsClosure()
                {
                    return that._getPopTextUrl()
                },
                parser: function parseClosure(responseText)
                {
                    return that._parsePopTextDocument(responseText)
                }
            })
        },{
            _getPopTextUrl: getPopTextUrl,
            _parsePopTextDocument: parsePopTextDocument
        })})
})()