﻿(function popVisualDataProviderModule()
{
    "use strict";
    var popVisualIdentity = "bingsearch://data/popvisual";
    var popVisualCount = 10;
    var elementTagForVisualAndTextPopTrends = "Element";
    var RequiredProperties = Object.freeze({
            Image: true,
            ImageCredit: true,
            SearchTermActual: true,
            SearchTermDisplay: true,
            Text: false,
            ThumbnailURI: false,
            Url: true
        });
    function dateWithOffset(offset)
    {
        var current = new Date;
        var utc = current.getTime() + current.getTimezoneOffset() * 6e4;
        var newDate = new Date(utc + 36e5 * offset);
        return newDate
    }
    function createYYYYMMDDFormat(date)
    {
        if(!date || isNaN(date.getTime()))
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("date");
        var month = date.getMonth() + 1;
        if(month < 10)
            month = "0" + month;
        var day = date.getDate();
        if(day < 10)
            day = "0" + day;
        return"" + date.getFullYear() + month + day
    }
    function getPopVisualUrl()
    {
        var trendsSection = this._config.getConfig().trends;
        var url = trendsSection.popularNowVisual.url;
        var date = this._date || createYYYYMMDDFormat(dateWithOffset(-8));
        return{url: url.replace("{date}",date)}
    }
    function parsePopVisualDocument(responseText)
    {
        if(BingApp.Utilities.isNullOrUndefined(responseText))
        {
            var errorMessage = "PopVisualDataProvider.parsePopVisualDocument: Empty responseText";
            var instObject = {
                    url: this._getPopVisualUrl().url,
                    ei: errorMessage
                };
            BingApp.Instrumentation.logError(instObject);
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("responseText");
        }
        var doc;
        try
        {
            doc = new Windows.Data.Xml.Dom.XmlDocument;
            doc.loadXml(responseText)
        }
        catch(err)
        {
            var errorMessage = BingApp.Utilities.format("PopVisualDataProvider.parsePopVisualDocument: Input is not valid xml - error name: {0}, error message: {1}",err.name || "",err.message || "");
            var instObject = {
                    url: this._getPopVisualUrl().url,
                    ei: errorMessage
                };
            BingApp.Instrumentation.logError(instObject);
            BingApp.traceError(errorMessage);
            throw new BingApp.Classes.ErrorArgument("responseText",errorMessage);
        }
        var elements = doc.selectNodes("//" + elementTagForVisualAndTextPopTrends);
        var count = elements.length;
        var results = [];
        for(var i = 0; i < count; i++)
        {
            var element = elements[i];
            var data = element.childNodes;
            var singleItem = {};
            var corruptDataElement = false;
            for(var j = 0; j < data.length; j++)
            {
                var value = data[j];
                if(value && !value.nodeName)
                {
                    BingApp.traceInfo("PopVisualDataProvider.parsePopVisualDocument: element in document is defective: {0}",value);
                    corruptDataElement = true;
                    break
                }
                singleItem[value.nodeName] = value.innerText
            }
            if(!corruptDataElement)
                for(var property in RequiredProperties)
                    if(RequiredProperties[property] && !singleItem[property])
                    {
                        corruptDataElement = true;
                        break
                    }
            if(corruptDataElement)
                continue;
            singleItem.type = "image";
            results.push(singleItem)
        }
        return results
    }
    WinJS.Namespace.define("BingApp.Classes",{PopVisualDataProvider: WinJS.Class.derive(BingApp.Classes.XhrDataProvider,function constructor(options)
        {
            if(!(this instanceof BingApp.Classes.PopVisualDataProvider))
            {
                BingApp.traceWarning("PopVisualDataProvider.ctor: Attempted using PopVisualDataProvider ctor as function; redirecting to use 'new PopVisualDataProvider()'.");
                return new BingApp.Classes.PopVisualDataProvider(options)
            }
            this._config = options && options.config || BingApp.locator.config;
            this._date = options && options.date || null;
            var that = this;
            BingApp.Classes.XhrDataProvider.call(this,popVisualIdentity,popVisualCount,{
                urlOptions: function getUrlOptionsClosure()
                {
                    return that._getPopVisualUrl()
                },
                parser: function parseClosure(responseText)
                {
                    return that._parsePopVisualDocument(responseText)
                }
            })
        },{
            _getPopVisualUrl: getPopVisualUrl,
            _parsePopVisualDocument: parsePopVisualDocument
        })})
})()