﻿(function trendingControllerModule()
{
    "use strict";
    var localSettings = Windows.Storage.ApplicationData.current.localSettings;
    var cellWidth = 260;
    var cellHeight = 125;
    var rowSpacing = 10;
    var newsItemRows = 3;
    var heroItemRows = 3;
    var imageItemRows = 2;
    var textItemRows = 1;
    var newsRandomWeight = 2;
    var lastUpdateThreshold = 8;
    var storageKey = "BingApp.Classes.Trending";
    var viewStateKey = "BingApp.Classes.Trending.ViewData";
    var loadEvent = "Cl.TrendLoad";
    var lastUpdatedText = "";
    var refreshInterval = 21600;
    var textListShiftLength = 2;
    var trendsMetrics = null;
    var maxColumns = 8;
    var maxNewsItems = 3;
    var numDisplayedNews = 0;
    var trendingViewModel = null;
    var ellipses = WinJS.Resources.getString("trending_ellipses").value;
    var eventsBound = false;
    var showCopyright = false;
    var showImageCredit = false;
    var trendsViewState = {};
    var supportedNumberOfRows = [4,6];
    var trendsPageLayout = BingApp.Instrumentation.ClientPages.TrendsPageInstrumentation;
    var curRegion = Windows.System.UserProfile.GlobalizationPreferences.homeGeographicRegion;
    var viewStates = Windows.UI.ViewManagement.ApplicationViewState;
    var groupedList = null;
    var newsTitleMaxHeight = 105;
    var textItemMaxHeight = 125;
    var textSnapItemMaxHeight = 145;
    var searchBarIFrame;
    var searchBarIFrameHeight = 0;
    var lastKnownHeight = 0;
    var lastViewState;
    var newsItemThumbnailHtml;
    var trendIds = Object.freeze({
            msnTrends: "msnTrends",
            bingNews: "bingNews",
            popularNowText: "popularNowText",
            popularNowVisual: "popularNowVisual"
        });
    var TrendingViewRefreshListener = WinJS.Class.define(function constructor(trendingViewModelObject)
        {
            if(!(this instanceof BingApp.Classes.TrendingViewRefreshListener))
            {
                BingApp.traceWarning("TrendingViewRefreshListener.ctor: Attempted using TrendingViewRefreshListener ctor as function; redirecting to use 'new TrendingViewRefreshListener()'.");
                return new BingApp.Classes.TrendingViewRefreshListener(trendingViewModelObject)
            }
            if(!trendingViewModelObject)
                throw new BingApp.Classes.ErrorArgumentNullOrUndefined("trendingViewModelObject");
            if(!(trendingViewModelObject instanceof BingApp.Classes.ListViewModel))
                throw new BingApp.Classes.ErrorArgument("trendingViewModelObject","trendingViewModelObject is not an instance of BingApp.Classes.ListViewModel");
            if(!trendingViewModelObject.refresh || typeof trendingViewModelObject.refresh !== "function")
                throw new BingApp.Classes.ErrorArgument("trendingViewModelObject","trendingViewModelObject does not have a refresh function");
            this.trendingViewModel = trendingViewModelObject;
            this._refreshInProgress = null
        },{
            refresh: function()
            {
                var that = this;
                function refreshOperation()
                {
                    that._refreshInProgress = new WinJS.Promise(function init(complete, error)
                    {
                        BingApp.traceInfo("Trends View refresh start");
                        that.trendingViewModel.refresh().done(function onTrendingViewModelRefreshComplete()
                        {
                            BingApp.traceInfo("Trends View refresh completed");
                            var trendsListViewElement = document.querySelector("#trendsListView");
                            if(trendsListViewElement)
                                if(that.trendingViewModel.getList().length === 0)
                                {
                                    var goBack = function goBack()
                                        {
                                            BingApp.locator.navigationManager.goBack({navigatingBackInHistory: true})
                                        };
                                    BingApp.Utilities.showErrorDialogAsync("error_no_network",null,"trendingController.refresh","empty list").then(goBack,goBack)
                                }
                                else
                                {
                                    calcTrendsViewMetrics();
                                    lastUpdatedText = "";
                                    WinJS.Application.sessionState[viewStateKey] = null;
                                    if(trendsMetrics)
                                    {
                                        bindTrendingListView(that.trendingViewModel);
                                        localSettings.values[storageKey] = JSON.stringify({
                                            lastRefresh: new Date,
                                            refreshRegion: curRegion
                                        })
                                    }
                                    else
                                        BingApp.traceError("TrendingViewRefreshListener.refresh: failed to refresh trending UI. TrendsMetrics is null or undefined.")
                                }
                            complete()
                        },error)
                    })
                }
                if(this._refreshInProgress)
                    this._refreshInProgress.done(refreshOperation,function onTrendingViewModelRefreshError(err)
                    {
                        BingApp.traceError("TrendingViewRefreshListener.refresh: failed to refresh trending UI. Exact error message is {0}",err.message)
                    });
                else
                    refreshOperation()
            },
            callback: function refreshCallback()
            {
                this.refresh()
            }
        });
    WinJS.Namespace.define("BingApp.Classes",{TrendingViewRefreshListener: TrendingViewRefreshListener});
    function doBinding(mustRecalculateLayout)
    {
        trendsViewState.scrollPosition = 0;
        var recalculate = BingApp.Utilities.isNotNullOrUndefined(mustRecalculateLayout) ? mustRecalculateLayout : true;
        if(recalculate)
            calcTrendsViewMetrics();
        var config = BingApp.locator.config;
        var trendsSection = config.getConfig().trends;
        maxColumns = trendsSection.general.maxColumns;
        maxNewsItems = trendsSection.bingNews.maxItems;
        initListeners();
        initInstrumentation();
        if(!trendingViewModel)
            trendingViewModel = createTrendingViewModel();
        var doRefresh = false;
        var refreshData = localSettings.values[storageKey];
        if(refreshData)
        {
            try
            {
                var refreshInfo = JSON.parse(refreshData);
                var lastRefresh = new Date(refreshInfo.lastRefresh);
                if(refreshInfo.refreshRegion !== curRegion)
                    doRefresh = true
            }
            catch(e)
            {
                doRefresh = true
            }
            if(!doRefresh)
            {
                var curTime = new Date;
                var cacheHours = (curTime - lastRefresh) / 36e5;
                var formatString;
                if(cacheHours > lastUpdateThreshold)
                    if(cacheHours > 24)
                        if(cacheHours >= 48)
                        {
                            formatString = WinJS.Resources.getString("trending_lastupdated_days_ago").value;
                            lastUpdatedText = BingApp.Utilities.format(formatString,Math.floor(cacheHours / 24))
                        }
                        else
                            lastUpdatedText = WinJS.Resources.getString("trending_lastupdated_day_ago").value;
                    else
                    {
                        formatString = WinJS.Resources.getString("trending_lastupdated_hours_ago").value;
                        lastUpdatedText = BingApp.Utilities.format(formatString,Math.floor(cacheHours))
                    }
                refreshInterval = trendsSection.general.ttl;
                lastRefresh.setHours(lastRefresh.getHours() + refreshInterval / 3600);
                if(lastRefresh <= curTime)
                    doRefresh = true
            }
            if(trendingViewModel && trendingViewModel.getList() && trendingViewModel.getList().length > 0)
                bindTrendingListView(trendingViewModel);
            else
                doRefresh = true
        }
        else
            doRefresh = true;
        if(!BingApp.Classes.TrendingController.trendingViewRefreshListener)
            BingApp.Classes.TrendingController.trendingViewRefreshListener = new BingApp.Classes.TrendingViewRefreshListener(trendingViewModel);
        else
            BingApp.Classes.TrendingController.trendingViewRefreshListener.trendingViewModel = trendingViewModel;
        if(doRefresh)
            BingApp.Classes.TrendingController.trendingViewRefreshListener.refresh();
        if(!BingApp.locator.eventRelay.isRegisteredEventListener(BingApp.Classes.ConfigStore.events.updated,BingApp.Classes.TrendingController.trendingViewRefreshListener))
            BingApp.locator.eventRelay.addEventListener(BingApp.Classes.ConfigStore.events.updated,BingApp.Classes.TrendingController.trendingViewRefreshListener)
    }
    function initializeBindingLists()
    {
        if(!trendsViewState)
            trendsViewState = {};
        trendsViewState.lists = {};
        for(var i = 0; i < supportedNumberOfRows.length; i++)
        {
            var rowIndex = supportedNumberOfRows[i];
            trendsViewState.lists[rowIndex] = {};
            trendsViewState.lists[rowIndex].listData = new WinJS.Binding.List
        }
    }
    function isBindingListsValid(viewState)
    {
        if(!viewState)
            return false;
        if(!viewState.lists)
        {
            BingApp.traceWarning("trendingControllerModule.isBindingListsValid: Possible corruption for trends list data.");
            return false
        }
        for(var i = 0; i < supportedNumberOfRows.length; i++)
        {
            var rowIndex = supportedNumberOfRows[i];
            if(!viewState.lists.hasOwnProperty(rowIndex) || !viewState.lists[rowIndex] || !viewState.lists[rowIndex].listData || typeof viewState.lists[rowIndex].listData.forEach !== "function")
            {
                BingApp.traceWarning("trendingControllerModule.isBindingListsValid: Possible corruption for trends list data.");
                return false
            }
        }
        return true
    }
    function navigateToVertical(url)
    {
        if(!url)
        {
            BingApp.traceError("TrendingController.navigateToVertical: failed to launch search because item data does not have url.");
            return
        }
        url = BingApp.Utilities.upsertFormCode(url,BingApp.Classes.Shell.formCodes.fromTrendingSearch);
        var navigationUri = BingApp.Classes.Shell.getAppPathFromWebPath(url);
        if(navigationUri)
            BingApp.locator.navigationManager.navigateTo(navigationUri);
        else
            BingApp.Utilities.invokeURI(url)
    }
    function initListeners()
    {
        var eventName = "MSPointerDown";
        var trendsListViewElement = document.querySelector("#trendsListView");
        var copyrightLinkElement = document.querySelector("#copyrightLink");
        if(!eventsBound && trendsListViewElement && copyrightLinkElement)
        {
            window.addEventListener("resize",onWindowResize,false);
            trendsListViewElement.addEventListener("iteminvoked",onItemInvoked,false);
            trendsListViewElement.addEventListener(eventName,BingApp.Utilities.onMSPointerDown,false);
            trendsListViewElement.winControl.addEventListener("loadingstatechanged",onListViewLoadingStateChanged,false);
            lastViewState = Windows.UI.ViewManagement.ApplicationView.value;
            lastKnownHeight = window.outerHeight;
            copyrightLinkElement.innerText = WinJS.Resources.getString("trending_copyright_symbol").value;
            copyrightLinkElement.addEventListener("click",onCopyrightClicked,false);
            copyrightLinkElement.addEventListener("keydown",handleKeyDownForCopyright,false);
            copyrightLinkElement.addEventListener(eventName,BingApp.Utilities.onMSPointerDown,true);
            eventsBound = true
        }
    }
    function initInstrumentation()
    {
        trendsPageLayout = BingApp.Instrumentation.ClientPages.TrendsPageInstrumentation;
        trendsPageLayout.init();
        var copyrightLinkElement = document.querySelector("#copyrightLink");
        if(copyrightLinkElement)
            copyrightLinkElement.k = trendsPageLayout.KValues.kFooterStartingValue
    }
    function shouldPerformLayout()
    {
        var returnVal;
        var currentHeight = window.outerHeight;
        var currentViewState = Windows.UI.ViewManagement.ApplicationView.value;
        if(lastKnownHeight !== currentHeight)
        {
            returnVal = true;
            if(trendsViewState)
                trendsViewState.scrollPosition = 0
        }
        else if(currentViewState !== lastViewState)
            returnVal = true;
        else
            returnVal = false;
        lastViewState = currentViewState;
        lastKnownHeight = currentHeight;
        return returnVal
    }
    function onWindowResize()
    {
        if(getAvailableHeight() <= 0 || !shouldPerformLayout())
            return;
        calcTrendsViewMetrics();
        if(WinJS.Application.sessionState[viewStateKey])
            bindTrendingListView(trendingViewModel,{origin: "resize"});
        else
            doBinding(false)
    }
    function onItemInvoked(eventData)
    {
        if(!eventData)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("eventData");
        eventData.detail.itemPromise.done(function onComplete(invokedItem)
        {
            var eventName = "TrendClick";
            var itemData = invokedItem.data;
            var element = eventData.currentTarget;
            var sourceOfEvent = element ? element.evtSourceOrigin : "Unknown";
            if(itemData.type === "text")
            {
                trendsPageLayout.logCoreClick(eventName,invokedItem.index,"Text",itemData.id,itemData.k,sourceOfEvent);
                navigateToVertical(itemData.Url || itemData.link)
            }
            else if(itemData.type === "news")
                try
                {
                    trendsPageLayout.logCoreClick(eventName,invokedItem.index,"News",itemData.id,itemData.k,sourceOfEvent);
                    if(itemData.Url)
                        BingApp.Utilities.invokeURI(itemData.Url);
                    else
                        BingApp.traceError("TrendingController.onItemInvoked: failed to launch news because item data does not have news url.")
                }
                catch(e)
                {
                    BingApp.traceError("Launching Uri failed for uri {0}",itemData.url)
                }
            else if(itemData.type === "image" || itemData.type === "hero")
            {
                trendsPageLayout.logCoreClick(eventName,invokedItem.index,"Visual",itemData.id,itemData.k,sourceOfEvent);
                navigateToVertical(itemData.Url)
            }
        })
    }
    function onCopyrightClicked(eventData)
    {
        if(!eventData)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("eventData");
        if(eventData.srcElement)
            trendsPageLayout.logFooterClick("Attribution",eventData.srcElement.k,eventData.srcElement.evtSourceOrigin);
        toggleImageCredits()
    }
    function toggleImageCredits(show)
    {
        var trendsListViewElement = document.querySelector("#trendsListView");
        if(trendsListViewElement)
        {
            if(typeof show === "boolean")
                showImageCredit = show;
            else
                showImageCredit = !showImageCredit;
            var display = showImageCredit ? "block" : "none";
            var images = trendsListViewElement.querySelectorAll(".image-item.item.image-box");
            for(var i = 0; i < images.length; i++)
            {
                var credit = images[i].querySelector(".credit");
                if(credit)
                    credit.style.display = display
            }
        }
    }
    function handleKeyDownForCopyright(eventData)
    {
        if(!eventData)
            throw new BingApp.Classes.ErrorArgumentNullOrUndefined("eventData");
        if(BingApp.Utilities.isKeyDownAndSpaceOrEnterKey(eventData))
            onCopyrightClicked(eventData)
    }
    function onVisible()
    {
        calcTrendsViewMetrics();
        if(!WinJS.Application.sessionState[viewStateKey])
            doBinding()
    }
    function onUnload()
    {
        var trendsListViewElement = document.querySelector("#trendsListView");
        var copyrightLinkElement = document.querySelector("#copyrightLink");
        if(eventsBound && trendsListViewElement && copyrightLinkElement)
        {
            window.removeEventListener("resize",onWindowResize,false);
            trendsListViewElement.removeEventListener("iteminvoked",onItemInvoked,false);
            copyrightLinkElement.removeEventListener("click",onCopyrightClicked,false);
            copyrightLinkElement.removeEventListener("keydown",handleKeyDownForCopyright,false);
            trendsListViewElement.winControl.removeEventListener("loadingstatechanged",onListViewLoadingStateChanged,false);
            if(groupedList)
                groupedList.dispose();
            eventsBound = false
        }
    }
    function createTrendingViewModel()
    {
        var providers = [];
        var config = BingApp.locator.config;
        var trendsSection = config.getConfig().trends;
        var newsProvider = trendsSection.bingNews.enabled ? new BingApp.Classes.BingNewsDataProvider : null;
        if(newsProvider)
            providers.push(newsProvider);
        var imagesProvider = trendsSection.popularNowVisual.enabled ? new BingApp.Classes.PopVisualDataProvider : null;
        if(imagesProvider)
            providers.push(imagesProvider);
        var textProvider = trendsSection.popularNowText.enabled ? new BingApp.Classes.PopTextDataProvider : null;
        if(textProvider)
            providers.push(textProvider);
        var msnTextProvider = trendsSection.msnTrends.enabled ? new BingApp.Classes.MsnTrendingDataProvider : null;
        if(msnTextProvider)
            providers.push(msnTextProvider);
        var trendsProvider = null;
        if(imagesProvider)
            trendsProvider = new BingApp.Classes.PriorityAggregateDataProvider("bingsearch://data/trends",providers,[imagesProvider]);
        else
            trendsProvider = new BingApp.Classes.AggregateDataProvider("bingsearch://data/trends",providers);
        var cacheProvider = new BingApp.Classes.LocalStorageCacheProvider("bingsearch://cache/trends","id");
        return new BingApp.Classes.ListViewModel(trendsProvider,cacheProvider)
    }
    function bindTrendingListView(viewModel, hint)
    {
        var newsList = [];
        var imageList = [];
        var textList = [];
        var initialImageListLength = 0;
        var initialTextListLength = 0;
        var initialNewsListLength = 0;
        var foundImage = false;
        var i;
        numDisplayedNews = 0;
        showCopyright = false;
        var viewState = WinJS.Application.sessionState[viewStateKey];
        if(isBindingListsValid(viewState))
            trendsViewState = viewState;
        else
            initializeBindingLists();
        if(trendsViewState.lists[supportedNumberOfRows[0]].listData.length === 0)
        {
            var dataSourceList = viewModel.getList();
            for(i = 0; i < dataSourceList.length; i++)
            {
                var item = dataSourceList.getItem(i);
                if(!item.data)
                    continue;
                ellipsifyItem(item.data);
                if(item.data.type === "news")
                    newsList.push(item.data);
                else if(item.data.type === "image")
                    imageList.push(item.data);
                else if(item.data.type === "text")
                    textList.push(item.data)
            }
            initialImageListLength = imageList.length;
            initialTextListLength = textList.length;
            initialNewsListLength = newsList.length;
            var numColumns = 0;
            while((imageList.length > 0 || textList.length >= textListShiftLength || newsList.length > 0) && ++numColumns <= maxColumns)
                addTrendingColumnToBindingLists(newsList,imageList,textList);
            initInstrumentation();
            var kValue = trendsPageLayout.KValues.kCoreStartingValue;
            for(i = 0; i < trendsViewState.lists[supportedNumberOfRows[1]].listData.length; i++)
            {
                var curItem = trendsViewState.lists[supportedNumberOfRows[1]].listData.getAt(i);
                curItem.id = "trendsItems" + i;
                curItem.k = kValue++
            }
            var trendsBindingList = trendsViewState.lists[supportedNumberOfRows[1]].listData;
            trendsPageLayout.generateLayout(trendsBindingList);
            trendsViewState.scrollPosition = 0;
            trendsViewState.snapSnippetsGenerated = false;
            WinJS.Application.sessionState[viewStateKey] = trendsViewState
        }
        else if(Windows.UI.ViewManagement.ApplicationView.value === viewStates.snapped && !WinJS.Application.sessionState[viewStateKey].snapSnippetsGenerated)
            for(i = 0; i < supportedNumberOfRows.length; i++)
            {
                var rowIndex = supportedNumberOfRows[i];
                trendsViewState.lists[rowIndex].listData.forEach(function ellipsifyAndAppendListItem(itemObject)
                {
                    ellipsifyItem(itemObject)
                });
                WinJS.Application.sessionState[viewStateKey].snapSnippetsGenerated = true
            }
        var currentBindingList = trendsViewState.lists[trendsMetrics.numRows].listData;
        currentBindingList.forEach(function countImages(itemObject)
        {
            if(itemObject.type === "image")
            {
                initialImageListLength++;
                showCopyright = true
            }
        });
        var trendsTitleText = WinJS.Resources.getString("trending_trendingTitle").value;
        groupedList = currentBindingList.createGrouped(function groupKey(itemObject)
        {
            return trendsTitleText
        },function groupData(itemObject)
        {
            return{
                    title: trendsTitleText,
                    lastUpdated: lastUpdatedText
                }
        });
        var trendsListViewElement = document.querySelector("#trendsListView");
        if(trendsListViewElement)
        {
            trendsListViewElement.winControl.itemTemplate = trendsViewItemTemplate;
            trendsListViewElement.winControl.groupDataSource = groupedList.groups.dataSource;
            trendsListViewElement.winControl.itemDataSource = groupedList.dataSource;
            trendsListViewElement.winControl.indexOfFirstVisible = WinJS.Application.sessionState[viewStateKey].scrollPosition
        }
        if(!hint)
            BingApp.Instrumentation.updatePageIG(null);
        var instrumentationObject = {
                name: "TrendPageLoad",
                VTrend: initialImageListLength - imageList.length,
                TTrend: initialTextListLength - textList.length,
                NTrend: initialNewsListLength - newsList.length
            };
        BingApp.Instrumentation.log(instrumentationObject,loadEvent)
    }
    function onListViewLoadingStateChanged(eventData)
    {
        var listView = eventData.target;
        var listViewControl = listView.winControl;
        var trendsListViewElement = document.querySelector("#trendsListView");
        if(!trendsListViewElement || listViewControl && listViewControl._loadingState !== "complete")
            return;
        var groupHeader = trendsListViewElement.querySelector(".win-groupheader");
        if(groupHeader)
            groupHeader.id = "trendGroupHeader";
        var copyrightLinkElement = document.querySelector("#copyrightLink");
        if(copyrightLinkElement)
            if(showCopyright)
                copyrightLinkElement.style.display = "block";
            else
                copyrightLinkElement.style.display = "none";
        if(listViewControl && WinJS.Application.sessionState[viewStateKey])
            WinJS.Application.sessionState[viewStateKey].scrollPosition = listViewControl.indexOfFirstVisible
    }
    function getAvailableHeight()
    {
        if(!searchBarIFrame)
            searchBarIFrame = document.querySelector("#searchbariframe");
        if(!searchBarIFrameHeight && searchBarIFrame)
            searchBarIFrameHeight = searchBarIFrame.clientHeight;
        return window.outerHeight - searchBarIFrameHeight
    }
    function calcTrendsViewMetrics()
    {
        var trendsListViewElement = document.querySelector("#trendsListView");
        if(!trendsListViewElement)
        {
            trendsMetrics = null;
            return
        }
        trendsListViewElement.style.height = "";
        var viewHeight = getAvailableHeight();
        if(viewHeight <= 0)
        {
            trendsMetrics = null;
            return
        }
        for(var rowIndex = supportedNumberOfRows.length - 1; rowIndex >= 0; rowIndex--)
        {
            var listViewHeight = supportedNumberOfRows[rowIndex] * cellHeight + (supportedNumberOfRows[rowIndex] - 1) * rowSpacing;
            if(listViewHeight < viewHeight || rowIndex === 0)
            {
                trendsMetrics = {
                    numRows: supportedNumberOfRows[rowIndex],
                    listViewHeight: listViewHeight
                };
                break
            }
        }
        var clientHeight = window.outerHeight;
        var myViewState = Windows.UI.ViewManagement.ApplicationView.value;
        var newTopMargin;
        var newBottomMargin;
        var trendsListViewContainerElement = document.querySelector("#trendsListViewContainer");
        if(trendsListViewContainerElement)
            if(myViewState === viewStates.snapped)
            {
                trendsListViewContainerElement.style.display = "-ms-grid";
                trendsListViewElement.style.marginTop = null;
                trendsListViewElement.style.marginBottom = null;
                trendsListViewElement.style.height = null;
                WinJS.UI.setOptions(trendsListViewElement.winControl,{layout: new WinJS.UI.ListLayout({groupInfo: function()
                        {
                            return{
                                    cellWidth: cellWidth,
                                    cellHeight: cellHeight
                                }
                        }})})
            }
            else if(trendsMetrics)
            {
                trendsListViewContainerElement.style.display = "inline-block";
                newTopMargin = (clientHeight - trendsMetrics.listViewHeight) / 2;
                trendsListViewElement.style.marginTop = newTopMargin + "px";
                trendsListViewElement.style.height = clientHeight - newTopMargin + "px";
                WinJS.UI.setOptions(trendsListViewElement.winControl,{layout: new WinJS.UI.GridLayout({
                        maxRows: trendsMetrics.numRows,
                        groupInfo: function()
                        {
                            return{
                                    enableCellSpanning: true,
                                    cellWidth: cellWidth,
                                    cellHeight: cellHeight
                                }
                        }
                    })})
            }
    }
    function ellipsifyItem(item)
    {
        var myViewState = Windows.UI.ViewManagement.ApplicationView.value;
        if(item.type === "news")
        {
            var newsRuler = document.querySelector("#newsRuler");
            var title = newsRuler.querySelector(".title");
            var description = newsRuler.querySelector(".news-details");
            var source = newsRuler.querySelector(".source");
            if(!newsItemThumbnailHtml)
                newsItemThumbnailHtml = description.innerHTML;
            title.innerText = item.Title;
            description.innerHTML = newsItemThumbnailHtml + window.toStaticHTML(item.Snippet);
            source.innerText = item.Source;
            var maxDescriptionHeight;
            var snapDescriptionRuler = document.querySelector("#newsSnapDescriptionRuler");
            snapDescriptionRuler.innerText = item.Snippet;
            var maxSnapDescriptionHeight = 77;
            if(myViewState === viewStates.snapped)
            {
                BingApp.Utilities.ellipsify(snapDescriptionRuler,maxSnapDescriptionHeight,item.Snippet,ellipses);
                item.SnapSnippet = snapDescriptionRuler.innerText
            }
            else
            {
                BingApp.Utilities.ellipsify(title,newsTitleMaxHeight,item.Title,ellipses);
                item.Title = title.innerText;
                maxDescriptionHeight = newsRuler.clientHeight - source.clientHeight - (title.offsetTop + title.clientHeight) - parseInt(source.currentStyle.marginTop) * 2;
                BingApp.Utilities.ellipsify(description,maxDescriptionHeight,item.Snippet,ellipses,newsItemThumbnailHtml);
                item.Snippet = description.innerText
            }
        }
        else if(item.type === "text")
        {
            var textRuler;
            var caption;
            if(myViewState === viewStates.snapped)
            {
                textRuler = document.querySelector("#textSnapRuler");
                caption = textRuler.querySelector(".caption");
                caption.innerText = item.title;
                BingApp.Utilities.ellipsify(caption,textSnapItemMaxHeight,caption.innerText,ellipses);
                item.title = caption.innerText
            }
            else
            {
                textRuler = document.querySelector("#textRuler");
                caption = textRuler.querySelector(".caption");
                caption.innerText = item.title;
                BingApp.Utilities.ellipsify(caption,textItemMaxHeight,caption.innerText,ellipses);
                item.shortTitle = caption.innerText
            }
        }
    }
    function addTrendingColumnToBindingLists(newsList, imageList, textList)
    {
        var rowCount = 0;
        var minimalRowCount = supportedNumberOfRows[0];
        var bindingList4Rows = trendsViewState.lists[supportedNumberOfRows[0]].listData;
        var bindingList6Rows = trendsViewState.lists[supportedNumberOfRows[1]].listData;
        if(newsList.length > 0 && numDisplayedNews < maxNewsItems && Math.floor(Math.random() * newsRandomWeight) >= 1)
        {
            var randomNewsItem = getRandomListItem(newsList);
            var randomTextItem = getRandomListItem(textList);
            if(randomNewsItem && randomTextItem)
            {
                bindingList4Rows.push(randomNewsItem);
                bindingList6Rows.push(randomNewsItem);
                rowCount += newsItemRows;
                numDisplayedNews++;
                bindingList4Rows.push(randomTextItem);
                bindingList6Rows.push(randomTextItem);
                rowCount += textItemRows
            }
        }
        while(rowCount < minimalRowCount && (imageList.length > 0 || textList.length >= textListShiftLength))
            if(imageList.length > 0 && textList.length < textListShiftLength || imageList.length > 0 && Math.floor(Math.random() * 2) === 1)
            {
                var randomImageItem = getRandomListItem(imageList);
                if(randomImageItem)
                {
                    bindingList4Rows.push(randomImageItem);
                    bindingList6Rows.push(randomImageItem);
                    showCopyright = true;
                    rowCount += imageItemRows
                }
            }
            else if(textList.length >= textListShiftLength)
            {
                var randomItem1 = getRandomListItem(textList);
                var randomItem2 = getRandomListItem(textList);
                if(randomItem1 && randomItem2)
                {
                    bindingList4Rows.push(randomItem1);
                    bindingList4Rows.push(randomItem2);
                    bindingList6Rows.push(randomItem1);
                    bindingList6Rows.push(randomItem2);
                    rowCount += 2 * textItemRows
                }
            }
        if(imageList.length > 0 && textList.length < textListShiftLength || imageList.length > 0 && Math.floor(Math.random() * 2) === 1)
        {
            randomImageItem = getRandomListItem(imageList);
            if(randomImageItem)
            {
                bindingList6Rows.push(randomImageItem);
                showCopyright = true
            }
        }
        else if(textList.length >= textListShiftLength)
        {
            randomItem1 = getRandomListItem(textList);
            randomItem2 = getRandomListItem(textList);
            if(randomItem1 && randomItem2)
            {
                bindingList6Rows.push(randomItem1);
                bindingList6Rows.push(randomItem2)
            }
        }
    }
    function getRandomListItem(list)
    {
        var randomIndex = Math.floor(Math.random() * list.length);
        return list.splice(randomIndex,1)[0]
    }
    function trendsViewItemTemplate(itemPromise)
    {
        return itemPromise.then(function onComplete(item)
            {
                var renderedItem;
                var templateElement;
                if(Windows.UI.ViewManagement.ApplicationView.value === viewStates.snapped)
                    templateElement = document.querySelector("#" + item.data.type + "SnapTemplate");
                else
                    templateElement = document.querySelector("#" + item.data.type + "Template");
                if(templateElement)
                {
                    var template = templateElement.winControl;
                    renderedItem = template.render(item.data);
                    renderedItem.then(function onRenderItemComplete(itemHtmlFragment)
                    {
                        itemHtmlFragment.id = item.data.id;
                        var imageElement = itemHtmlFragment.querySelector("img");
                        if(showImageCredit && (templateElement.id === "imageTemplate" || templateElement.id === "imageSnapTemplate"))
                        {
                            var creditItem = itemHtmlFragment.querySelector(".credit");
                            if(creditItem)
                                creditItem.style.display = "block"
                        }
                        if(imageElement)
                        {
                            imageElement.setAttribute("aria-label",item.data.SearchTermDisplay || item.data.Title || "");
                            imageElement.onerror = function onError()
                            {
                                var errorMsg = "TrendingController.trendsViewItemTemplate: Error loading image";
                                var instObject = {
                                        url: this.src,
                                        ei: errorMsg
                                    };
                                BingApp.Instrumentation.logError(instObject);
                                this.src = "/views/start/images/trendsimageplaceholder.png"
                            }
                        }
                    })
                }
                else
                {
                    BingApp.traceVerbose("TrendingController.trendsViewItemTemplate: itemPromise completed after view has been unloaded from DOM; ignoring item generation request.");
                    renderedItem = document.createElement("div")
                }
                return renderedItem
            })
    }
    function trendsAvailableForCurrentMarket()
    {
        return BingApp.locator.config.getConfig().trends.general.enabled
    }
    WinJS.Namespace.define("BingApp.Classes.TrendingController",{
        doBinding: doBinding,
        onUnload: onUnload,
        onVisible: onVisible,
        trendsAvailableForCurrentMarket: trendsAvailableForCurrentMarket,
        trendIds: trendIds,
        trendingViewRefreshListener: null,
        navigateToVertical: navigateToVertical,
        toggleImageCredits: toggleImageCredits
    })
})()