﻿(function trendPageInstrumentationModule()
{
    "use strict";
    var trendPageLayout = null;
    var trendPageDs = null;
    var appNS = "BingApp";
    var coreReady = false;
    var footerReady = false;
    var headerReady = false;
    var pageName = "trends";
    var kTrendsPage = Object.freeze({
            kCoreStartingValue: 4e3,
            kFooterStartingValue: 4500,
            kHeaderStartingValue: 5e3
        });
    function createTrendsPageLayoutTemplate()
    {
        trendPageLayout = [{
                Region: "Header",
                L: [{
                        Region: "IdControl",
                        T: "L.Box",
                        L: [{
                                T: "L.Url",
                                K: BingApp.Instrumentation.KValues.kValueBack + .1,
                                AppNS: appNS
                            }]
                    },{
                        Region: "Searchbox",
                        T: "L.Box",
                        L: [{
                                T: "L.Form",
                                AppNS: appNS,
                                K: BingApp.Instrumentation.KValues.kValueSearch + .1
                            }]
                    },{
                        Region: "Hdrlogo",
                        T: "L.Box",
                        L: [{
                                T: "L.Url",
                                K: BingApp.Instrumentation.KValues.kValueLogo + .1,
                                AppNS: appNS
                            }]
                    }]
            },{
                Region: "Core",
                AppNS: appNS,
                T: "L.Box",
                L: []
            },{
                Region: "Footer",
                T: "L.Box",
                K: kTrendsPage.kFooterStartingValue + .1,
                AppNS: appNS,
                L: []
            }]
    }
    function createStartPageDsTemplate()
    {
        trendPageDs = [{
                AppNS: appNS,
                T: "D.Top",
                DS: [{
                        T: "D.Url",
                        K: BingApp.Instrumentation.KValues.kValueBack,
                        Url: "<<URL>>",
                        Tx: "Back"
                    },{
                        AppNS: appNS,
                        T: "D.Form",
                        K: BingApp.Instrumentation.KValues.kValueSearch,
                        Url: "/search"
                    },{
                        T: "D.Url",
                        Tx: "Bing",
                        Url: "<<URL>>",
                        K: BingApp.Instrumentation.KValues.kValueLogo
                    }]
            },{
                AppNS: appNS,
                T: "D.Top.RichBackground",
                Service: "Trends",
                Scenario: "BingApp",
                DS: [{
                        T: "D.Trends",
                        AppNS: appNS,
                        DS: []
                    }]
            },{
                AppNS: appNS,
                T: "D.Top",
                Scenario: "footer",
                Service: "config",
                DS: []
            }]
    }
    function populateCoreRegion(trendsData)
    {
        coreReady = false;
        trendPageLayout[1].L = [];
        trendPageDs[1].DS[0].DS = [];
        if(trendsData)
            for(var i = 0; i < trendsData.length; i++)
            {
                var item = trendsData.getAt(i);
                trendPageLayout[1].L.push({
                    T: "L.Url",
                    K: item.k + .1
                });
                trendPageDs[1].DS[0].DS.push({
                    T: "D.Url",
                    Url: item.Url,
                    K: item.k,
                    AppNS: appNS
                })
            }
        if(trendPageDs[1].DS[0].DS.length > 0 && trendPageLayout[1].L.length > 0)
            coreReady = true
    }
    function populateHeaderRegion(){}
    function populateFooterRegion()
    {
        footerReady = false;
        trendPageLayout[2].L = [];
        trendPageDs[2].DS = [];
        trendPageLayout[2].L.push({
            T: "L.Url",
            K: kTrendsPage.kFooterStartingValue + .1
        });
        trendPageDs[2].DS.push({
            T: "D.Url",
            K: kTrendsPage.kFooterStartingValue,
            N: "Copyright"
        });
        footerReady = true
    }
    function calcRowColumn(id)
    {
        var item = document.getElementById(id);
        if(item)
        {
            var column = 1;
            var parentObject = item.parentElement;
            var left = parentObject.style.left;
            var rowValue = 0;
            while(parentObject.style.left === left)
            {
                rowValue++;
                parentObject = parentObject.previousSibling;
                if(parentObject === null)
                    break
            }
            if(parentObject !== null)
            {
                column++;
                left = parentObject.style.left;
                while(parentObject !== null)
                {
                    if(parentObject.style.left !== left)
                    {
                        column++;
                        left = parentObject.style.left
                    }
                    parentObject = parentObject.previousSibling
                }
            }
            return"" + rowValue + "," + column
        }
        else
        {
            BingApp.traceError("TrendsPageInstrumention.calcRowColumn, id");
            return null
        }
    }
    function logCoreClick(page, index, trendType, id, k, type)
    {
        var position = calcRowColumn(id);
        var instrumentationObject = {
                name: page,
                trendPos: index,
                trendType: trendType,
                K: k + .1,
                position: position,
                Type: type
            };
        BingApp.Instrumentation.logClick(instrumentationObject)
    }
    function logFooterClick(page, k, origin)
    {
        if(!k || !origin)
            return;
        var instrumentationObject = {
                name: page,
                K: k + .1,
                Type: origin
            };
        BingApp.Instrumentation.logClick(instrumentationObject)
    }
    function logLayout()
    {
        if(footerReady && coreReady)
            BingApp.Instrumentation.logPartialImpression(pageName,trendPageLayout,trendPageDs);
        else
            BingApp.traceError("TrendsPageInstrumention.logLayout: Page Not Ready")
    }
    function generateLayout(trendsData)
    {
        populateHeaderRegion();
        populateCoreRegion(trendsData);
        populateFooterRegion()
    }
    function init()
    {
        createTrendsPageLayoutTemplate();
        createStartPageDsTemplate();
        BingApp.Instrumentation.registerLayoutHandler(pageName,logLayout)
    }
    function getLayoutRaw()
    {
        return trendPageLayout
    }
    function getDSRaw()
    {
        return trendPageDs
    }
    function reset()
    {
        trendPageLayout = null;
        trendPageDs = null
    }
    WinJS.Namespace.define("BingApp.Instrumentation.ClientPages.TrendsPageInstrumentation",{
        populateCoreRegion: populateCoreRegion,
        populateHeaderRegion: populateHeaderRegion,
        populateFooterRegion: populateFooterRegion,
        logCoreClick: logCoreClick,
        logFooterClick: logFooterClick,
        logLayout: logLayout,
        generateLayout: generateLayout,
        init: init,
        getLayoutRaw: getLayoutRaw,
        getDSRaw: getDSRaw,
        reset: reset,
        KValues: kTrendsPage
    })
})()