﻿(function voiceModule()
{
    "use strict";
    var selectors = Object.freeze({
            listening: ".listening",
            thinking: ".thinking",
            error: ".error"
        });
    var Voice = WinJS.Class.define(function constructor(options)
        {
            this._config = options && options.config || BingApp.locator.config;
            this._networkDetectionService = options && options.networkDetectionService || BingApp.locator.networkDetectionService;
            this._proxy = options && options.proxy || BingApp.locator.proxy;
            this._version = options && options.version || BingApp.Classes.Shell.fullVersion;
            this._voiceService = options && options.voiceService || Microsoft.Speech.VoiceService;
            this._voiceSurface = options && options.voiceSurface || document.querySelector(".voice");
            this._listeningVoiceMeterViewController = options && options.listeningVoiceMeterViewController || new BingApp.Classes.JSVoiceMeterViewController(new BingApp.Classes.JSMeterView("visualization-surface"));
            this._registerEvents()
        },{
            recognizeVoiceAsync: function()
            {
                if(this._context)
                {
                    BingApp.traceError("Voice.recognizeVoiceAsync: Cannot start voice recognition because there is already a request in progress");
                    return WinJS.Promise.wrapError(new WinJS.ErrorFromName("Voice.recognizeVoiceAsync.ErrorRequestInProgress","Cannot start voice recognition because there is already a request in progress"))
                }
                this._context = {};
                function initRecognizeVoicePromise(complete, error)
                {
                    this._context.complete = complete;
                    this._context.error = error;
                    this._initVoiceAsync()
                }
                return new WinJS.Promise(BingApp.Utilities.createClosure(this,initRecognizeVoicePromise))
            },
            _createVoiceRecognitionRequestConfigAsync: function()
            {
                function getMUIDCompleted(muid)
                {
                    var installId = muid ? this._getMuidAsGuid(muid) : this._getRandomGuid();
                    return this._createVoiceRecognitionRequestConfig(installId)
                }
                function getMUIDFailed(error)
                {
                    BingApp.traceWarning("Voice._createVoiceRecognitionRequestConfigAsync.getMUIDFailed: Error retrieving MUID: " + error.message);
                    return this._createVoiceRecognitionRequestConfig(this._getRandomGuid())
                }
                return this._proxy.getMUIDAsync().then(BingApp.Utilities.createClosure(this,getMUIDCompleted),BingApp.Utilities.createClosure(this,getMUIDFailed))
            },
            _createVoiceRecognitionRequestConfig: function(installId)
            {
                var voiceConfig = this._config.getConfig().voice;
                var requestConfig = this._voiceService.VoiceRecognitionRequestConfig.createSpeechboxRequest();
                requestConfig.appName = Windows.ApplicationModel.Package.current.id.name;
                requestConfig.appVersion = this._version;
                requestConfig.installId = installId;
                requestConfig.clientId = voiceConfig.general.clientId;
                requestConfig.grammar = this._voiceService.VoiceRecognitionGrammar.Search;
                requestConfig.maxResults = 1;
                requestConfig.url = new Windows.Foundation.Uri(voiceConfig.general.url);
                requestConfig.logUrl = new Windows.Foundation.Uri(voiceConfig.general.loggingUrl);
                if(voiceConfig.general.testMode)
                    requestConfig.test = true;
                return requestConfig
            },
            _initVoiceAsync: function()
            {
                this._context.timeToInitializeMic = Date.now();
                if(!this._networkDetectionService.isConnected())
                {
                    this._showErrorDialog(0,"voice_error_no_network");
                    return
                }
                function createRequestConfigCompleted(requestConfig)
                {
                    this._context.request = new this._voiceService.VoiceRecognitionRequest(requestConfig);
                    try
                    {
                        this._context.request.initAsync().done(BingApp.Utilities.createClosure(this,this._recognizeVoiceAsync),BingApp.Utilities.createClosure(this,this._handleInitVoiceError))
                    }
                    catch(err)
                    {
                        this._handleInitVoiceError(err)
                    }
                }
                function createRequestConfigFailed(error)
                {
                    BingApp.traceWarning("Voice._initVoiceAsync.createRequestConfigFailed: Error constructing request config: " + (error && error.message));
                    this._showErrorDialog(0,"voice_error_unknown")
                }
                this._createVoiceRecognitionRequestConfigAsync().done(BingApp.Utilities.createClosure(this,createRequestConfigCompleted),BingApp.Utilities.createClosure(this,createRequestConfigFailed))
            },
            _handleInitVoiceError: function(error)
            {
                BingApp.traceInfo("Voice._handleInitVoiceError: Error: {0}",error.number);
                this._context.timeToInitializeMic = Date.now() - this._context.timeToInitializeMic;
                var errorKey;
                switch(error.number)
                {
                    case-2147024894:
                        errorKey = "voice_error4";
                        break;
                    case-2147024891:
                        errorKey = "voice_error8";
                        break;
                    case-2147019873:
                        errorKey = "voice_error3";
                        break;
                    case-1072873854:
                        errorKey = "voice_error3";
                        break;
                    case-2146233029:
                        errorKey = "voice_error_no_network";
                        break;
                    case-2146233054:
                        errorKey = "voice_error9";
                        break
                }
                if(!errorKey && error.number >= -1072885196 && error.number <= -1072824321)
                    errorKey = "voice_error4";
                if(!errorKey)
                    errorKey = "voice_error3";
                this._showErrorDialog(error.number,errorKey)
            },
            _recognizeVoiceAsync: function()
            {
                this._context.timeToInitializeMic = Date.now() - this._context.timeToInitializeMic;
                this._context.promise = this._context.request.recognizeAsync().then(BingApp.Utilities.createClosure(this,this._handleRecognizeVoiceComplete),BingApp.Utilities.createClosure(this,this._handleRecognizeVoiceError),BingApp.Utilities.createClosure(this,this._handleRecognizeVoiceProgress));
                return this._context.promise
            },
            _handleRecognizeVoiceComplete: function(response)
            {
                this._context.serverRequestId = response.serverRequestId;
                this._context.timeToReceiveResponse = Date.now() - this._context.timeToReceiveResponse;
                if(response.status === this._voiceService.VoiceRecognitionResponseStatus.success)
                {
                    BingApp.traceInfo("Voice._handleRecognizeVoiceComplete: Status: success, serverRequestId: {0}, Confidence: {1}",this._context.serverRequestId,response.confidence);
                    var instObj = {
                            name: "voiceSuccess",
                            srid: this._context.serverRequestId,
                            rcl: response.confidence,
                            ttim: this._context.timeToInitializeMic,
                            ttcr: this._context.timeToReceiveResponse
                        };
                    BingApp.Instrumentation.log(instObj,BingApp.Instrumentation.appEvent);
                    this._invokeCompletePromise(response)
                }
                else
                {
                    var errorKey;
                    switch(response.status)
                    {
                        case this._voiceService.VoiceRecognitionResponseStatus.noMatch:
                            BingApp.traceInfo("Voice._handleRecognizeVoiceComplete: Status: noMatch");
                            errorKey = "voice_error_no_match";
                            break;
                        case this._voiceService.VoiceRecognitionResponseStatus.noSpeech:
                            BingApp.traceInfo("Voice._handleRecognizeVoiceComplete: Status: noSpeech");
                            errorKey = "voice_error_no_speech";
                            break;
                        default:
                            BingApp.traceInfo("Voice._handleRecognizeVoiceComplete: Status: unknown");
                            errorKey = "voice_error5";
                            break
                    }
                    this._showErrorDialog(-1,errorKey)
                }
            },
            _handleRecognizeVoiceProgress: function(progress)
            {
                this._tearDownVisualization();
                switch(progress.status)
                {
                    case this._voiceService.VoiceRecognitionRequestState.unknown:
                        BingApp.traceInfo("Voice._handleRecognizeVoiceProgress: unknown");
                        break;
                    case this._voiceService.VoiceRecognitionRequestState.listening:
                        BingApp.traceInfo("Voice._handleRecognizeVoiceProgress: listening");
                        this._showListeningDialog();
                        break;
                    case this._voiceService.VoiceRecognitionRequestState.thinking:
                        BingApp.traceInfo("Voice._handleRecognizeVoiceProgress: thinking");
                        this._context.timeToReceiveResponse = Date.now();
                        this._showThinkingDialog();
                        var thinkingEarcon = this._voiceSurface.querySelector("#thinking-earcon");
                        thinkingEarcon.currentTime = 0;
                        thinkingEarcon.play();
                        break;
                    case this._voiceService.VoiceRecognitionRequestState.complete:
                        BingApp.traceInfo("Voice._handleRecognizeVoiceProgress: complete");
                        this._voiceSurface.querySelector("#thinking-earcon").pause();
                        this._voiceSurface.querySelector("#results-earcon").play();
                        break;
                    case this._voiceService.VoiceRecognitionRequestState.cancelling:
                        BingApp.traceInfo("Voice._handleRecognizeVoiceProgress: cancelling");
                        break;
                    case this._voiceService.VoiceRecognitionRequestState.canceled:
                        BingApp.traceInfo("Voice._handleRecognizeVoiceProgress: canceled");
                        break
                }
            },
            _handleRecognizeVoiceError: function(error)
            {
                if(BingApp.Utilities.isPromiseCancellationError(error))
                    return;
                BingApp.traceInfo("Voice._handleRecognizeVoiceError: Error: {0}",error.number);
                var errorKey;
                switch(error.number)
                {
                    case-2147221499:
                        errorKey = "voice_error_no_network";
                        break;
                    case-2147024894:
                        errorKey = "voice_error4";
                        break;
                    case-2147024890:
                        errorKey = "voice_error3";
                        break;
                    case-2146233088:
                        errorKey = "voice_error_no_network";
                        break;
                    case-2146233086:
                        errorKey = "voice_error3";
                        break;
                    case-2146233079:
                    case-2146233033:
                        errorKey = "voice_error_no_network";
                        break;
                    case-2147024882:
                        errorKey = "voice_error3";
                        break;
                    case-2146233029:
                        errorKey = "voice_error_no_network";
                        break;
                    default:
                        errorKey = "voice_error5";
                        break
                }
                this._showErrorDialog(error.number,errorKey)
            },
            _invokeCompletePromise: function(response)
            {
                this._context.complete(response);
                this._dismissUI()
            },
            _invokeErrorPromise: function(error)
            {
                this._context.error(error);
                this._dismissUI()
            },
            _stopListening: function()
            {
                this._context.request.stopListeningAsync()
            },
            _cancelRecognizeVoice: function()
            {
                this._context.promise && this._context.promise.cancel();
                this._invokeErrorPromise()
            },
            _registerEvents: function()
            {
                this._eventHandlers = {};
                this._eventHandlers.stopListeningAsync = BingApp.Utilities.createClosure(this,this._stopListening);
                this._eventHandlers.cancelRecognizeVoice = BingApp.Utilities.createClosure(this,this._cancelRecognizeVoice);
                this._eventHandlers.initVoiceAsync = BingApp.Utilities.createClosure(this,this._initVoiceAsync);
                this._voiceSurface.querySelector(".shield").addEventListener("click",this._eventHandlers.cancelRecognizeVoice);
                this._voiceSurface.querySelector(".listening .go").addEventListener("click",this._eventHandlers.stopListeningAsync);
                this._voiceSurface.querySelector(".thinking .cancel").addEventListener("click",this._eventHandlers.cancelRecognizeVoice);
                this._voiceSurface.querySelector(".error .retry").addEventListener("click",this._eventHandlers.initVoiceAsync);
                this._voiceSurface.querySelector(".error .cancel").addEventListener("click",this._eventHandlers.cancelRecognizeVoice)
            },
            unregisterEvents: function()
            {
                this._voiceSurface.querySelector(".shield").removeEventListener("click",this._eventHandlers.cancelRecognizeVoice);
                this._voiceSurface.querySelector(".listening .go").removeEventListener("click",this._eventHandlers.stopListeningAsync);
                this._voiceSurface.querySelector(".thinking .cancel").removeEventListener("click",this._eventHandlers.cancelRecognizeVoice);
                this._voiceSurface.querySelector(".error .retry").removeEventListener("click",this._eventHandlers.initVoiceAsync);
                this._voiceSurface.querySelector(".error .cancel").removeEventListener("click",this._eventHandlers.cancelRecognizeVoice);
                this._eventHandlers = null
            },
            _showListeningDialog: function()
            {
                this._showDialog(selectors.listening);
                this._voiceSurface.querySelector(".listening .go").focus();
                this._setupListeningVisualization()
            },
            _showThinkingDialog: function()
            {
                this._showDialog(selectors.thinking);
                this._voiceSurface.querySelector(".thinking .cancel").focus();
                this._setupThinkingVisualization()
            },
            _showErrorDialog: function(errorNumber, errorKey)
            {
                this._voiceSurface.querySelector(".error .title").textContent = WinJS.Resources.getString(errorKey).value;
                this._showDialog(selectors.error);
                this._voiceSurface.querySelector(".error .retry").focus();
                var instObj = {
                        name: "voiceError",
                        srid: this._context.serverRequestId,
                        errc: errorNumber,
                        errk: errorKey,
                        ttim: this._context.timeToInitializeMic,
                        ttcr: this._context.timeToReceiveResponse
                    };
                BingApp.Instrumentation.logError(instObj)
            },
            _showDialog: function(dialogSelector)
            {
                this._voiceSurface.classList.remove("hidden");
                WinJS.Utilities.query(".container",this._voiceSurface).addClass("hidden");
                this._voiceSurface.querySelector(dialogSelector).classList.remove("hidden")
            },
            _dismissUI: function()
            {
                this._voiceSurface.classList.add("hidden")
            },
            _getMuidAsGuid: function(muid)
            {
                return[muid.substr(0,8),muid.substr(8,4),muid.substr(12,4),muid.substr(16,4),muid.substr(20,12)].join("-")
            },
            _getRandomGuid: function()
            {
                function hex(length)
                {
                    var array = [];
                    for(var i = 0; i < length; ++i)
                    {
                        var char = Math.floor(Math.random() * 16);
                        array.push(char.toString(16))
                    }
                    return array.join("")
                }
                return[hex(8),hex(4),hex(4),hex(4),hex(12)].join("-")
            },
            _setupListeningVisualization: function()
            {
                function addVolumeSample(sample)
                {
                    var volume = (sample.volume - sample.minVolume) / (sample.maxVolume - sample.minVolume);
                    this._listeningVoiceMeterViewController.setLevel(volume)
                }
                this._context.request.onvolumereported = BingApp.Utilities.createClosure(this,addVolumeSample);
                var visualization = this._voiceSurface.querySelector(".visualization-surface");
                this._voiceSurface.querySelector(".listening .visualization").appendChild(visualization);
                this._listeningVoiceMeterViewController.setMode(this._listeningVoiceMeterViewController.animationModes.kJSVoiceMeterViewController_ModeListening);
                this._listeningVoiceMeterViewController.startAnimating()
            },
            _setupThinkingVisualization: function()
            {
                var visualization = this._voiceSurface.querySelector(".visualization-surface");
                this._voiceSurface.querySelector(".thinking .visualization").appendChild(visualization);
                this._listeningVoiceMeterViewController.setMode(this._listeningVoiceMeterViewController.animationModes.kJSVoiceMeterViewController_ModeTransition);
                this._listeningVoiceMeterViewController.startAnimating()
            },
            _tearDownVisualization: function()
            {
                this._listeningVoiceMeterViewController.stopAnimating()
            }
        });
    WinJS.Namespace.define("BingApp.Classes",{Voice: Voice})
})()