﻿(function JSMeterViewModule()
{
    "use strict";
    /*!
     @note The constant values for box sizes, spacing, number of rows/columns, etc 
     are predetermined based on the expected view size 
    */
    var kJSMeterView_NumCols = 34;
    var kJSMeterView_NumRows = 10;
    var kJSMeterView_ColSpacing_px = 5;
    var kJSMeterView_RowSpacing_px = 5;
    var kJSMeterView_BoxWidth_px = 5;
    var kJSMeterView_BoxHeight_px = 5;
    var kJSMeterView_BoxColor = "rgba(153, 153, 153, 1.0)";
    var kJSMeterView_PeakColor = "rgba(255, 102, 0, 1.0)";
    var kJSMeterView_11Color = "rgba(53, 117, 135, 1.0)";
    /*!
     @class JSMeterView
     @brief Draws a grid of a orange/grey boxes
 
     This class draws a predetermined (10x15) grid of orange/grey 
     that represent a sort of "meter".  A few flags control which boxes
     are orange/grey, by default all boxes except those in the top row
     are grey.

     @note The mCanvas is assumed to already be sized at 280x77
 
     @todo support for stretching / configurable # of meters / etc
    */
    function JSMeterView(mCanvasElementId)
    {
        var mCanvas = document.getElementById(mCanvasElementId);
        var mContext = mCanvas.getContext("2d");
        var mColumns = new Array(kJSMeterView_NumCols);
        var mNumberOfPeaks = 1;
        var mShouldDrawPeaksOnTop = false;
        var mShouldDrawPeaksOnMax = false;
        this.columns = function()
        {
            return mColumns
        };
        this.setColumns = function(columns)
        {
            mColumns = columns.slice(0);
            this.drawRect()
        };
        this.numberOfPeaks = function()
        {
            return mNumberOfPeaks
        };
        this.setNumberOfPeaks = function(numberOfPeaks)
        {
            mNumberOfPeaks = numberOfPeaks
        };
        this.shouldDrawPeaksOnTop = function()
        {
            return mShouldDrawPeaksOnTop
        };
        this.setShouldDrawPeaksOnTop = function(shouldDrawPeaksOnTop)
        {
            mShouldDrawPeaksOnTop = shouldDrawPeaksOnTop
        };
        this.shouldDrawPeaksOnMax = function()
        {
            return mShouldDrawPeaksOnMax
        };
        this.setShouldDrawPeaksOnMax = function(shouldDrawPeaksOnMax)
        {
            mShouldDrawPeaksOnMax = shouldDrawPeaksOnMax
        };
        this.resetColumns = function(c)
        {
            for(var i = 0; i < mColumns.length; i++)
                mColumns[i] = c;
            this.drawRect()
        };
        this.drawRect = function()
        {
            mContext.clearRect(0,0,mCanvas.width,mCanvas.height);
            for(var col = 0; col < kJSMeterView_NumCols; ++col)
            {
                var x = col * kJSMeterView_BoxWidth_px + col * kJSMeterView_ColSpacing_px;
                for(var row = 0; row < mColumns[col]; ++row)
                {
                    var y = mCanvas.height - (row + 1) * kJSMeterView_BoxHeight_px - row * kJSMeterView_RowSpacing_px;
                    var bMax = row >= kJSMeterView_NumRows - mNumberOfPeaks;
                    var bTop = row >= mColumns[col] - mNumberOfPeaks;
                    if(mColumns[col] <= kJSMeterView_NumRows)
                        if(mShouldDrawPeaksOnMax && bMax || mShouldDrawPeaksOnTop && bTop && !bMax)
                        {
                            mContext.strokeStyle = kJSMeterView_PeakColor;
                            mContext.fillStyle = kJSMeterView_PeakColor
                        }
                        else
                        {
                            mContext.strokeStyle = kJSMeterView_BoxColor;
                            mContext.fillStyle = kJSMeterView_BoxColor
                        }
                    else if(row >= kJSMeterView_NumRows)
                    {
                        mContext.strokeStyle = kJSMeterView_11Color;
                        mContext.fillStyle = kJSMeterView_11Color
                    }
                    else
                    {
                        mContext.strokeStyle = kJSMeterView_BoxColor;
                        mContext.fillStyle = kJSMeterView_BoxColor
                    }
                    mContext.fillRect(x,y,kJSMeterView_BoxWidth_px,kJSMeterView_BoxHeight_px)
                }
            }
        }
    }
    JSMeterView.prototype = new Object;
    WinJS.Namespace.define("BingApp.Classes",{JSMeterView: JSMeterView})
})()