﻿(function JSVoiceMeterViewControllerModule()
{
    "use strict";
    var kJSMeterView_NumCols = 34;
    var kJSMeterView_NumRows = 10;
    var animationModes = {
            kJSVoiceMeterViewController_ModeFilling: 0,
            kJSVoiceMeterViewController_ModeDraining: 1,
            kJSVoiceMeterViewController_ModeListening: 2,
            kJSVoiceMeterViewController_ModeTransition: 3
        };
    var kJSVoiceMeterViewController_FillInterval = 96.15385;
    var kJSVoiceMeterViewController_DrainInterval = 100;
    var kJSVoiceMeterViewController_TransitionInterval = 50;
    var kJSVoiceMeterViewController_vuMeterShape = new Array(.53,.485,.522,.505,.501,.566,.502,.603,.629,.779,.811,.866,.92,.819,.779,.875,.561,.622,.72,.816,.806,.907,.688,.694,.618,.694,.787,.666,.743,.819,.772,.771,.871,.755,.751,.651,.498,.607,.352,.647,.536,.573,.433,.374,.32,.283,.301,.307,.289,.19,.135,.192,.119,.121,.14,.18,.231,.214,.137,.085);
    var kJSVoiceMeterViewController_thresholdTicksAt11 = 90;
    var kJSVoiceMeterViewController_peakVolumeLevel = .5;
    var kJSVoiceMeterViewController_silenceVolumeLevel = .03;
    function JSVoiceMeterViewController(meterView)
    {
        this.mMeterView = meterView;
        this.mColumns = new Array(kJSMeterView_NumCols);
        for(var i = 0; i < kJSMeterView_NumCols; i++)
            this.mColumns[i] = 0;
        this.mTimer;
        this.mLastUpdateTimeInterval;
        this.mIsAnimating;
        this.mCurrentMode;
        this.mActiveMeterCount;
        this.mLevel;
        this.mTicksAt11
    }
    JSVoiceMeterViewController.prototype = new Object;
    JSVoiceMeterViewController.prototype.meter = function()
    {
        return this.mMeterView
    };
    JSVoiceMeterViewController.prototype.setMeter = function(meterView)
    {
        this.mMeterView = meterView
    };
    JSVoiceMeterViewController.prototype.isAnimating = function()
    {
        return this.mIsAnimating
    };
    JSVoiceMeterViewController.prototype.level = function()
    {
        return this.mLevel
    };
    JSVoiceMeterViewController.prototype.setLevel = function(level)
    {
        this.mLevel = level
    };
    JSVoiceMeterViewController.prototype.mode = function()
    {
        return this.mCurrentMode
    };
    JSVoiceMeterViewController.prototype.setMode = function(newMode)
    {
        this.mCurrentMode = newMode;
        this.mLastUpdateTimeInterval = +new Date;
        if(animationModes.kJSVoiceMeterViewController_ModeDraining == this.mCurrentMode)
        {
            this.mActiveMeterCount = kJSMeterView_NumCols;
            this.mMeterView.resetColumns(kJSMeterView_NumRows);
            this.mMeterView.setNumberOfPeaks(1);
            this.mMeterView.setShouldDrawPeaksOnMax(true);
            this.mMeterView.setShouldDrawPeaksOnTop(true)
        }
        else if(animationModes.kJSVoiceMeterViewController_ModeFilling == this.mCurrentMode)
        {
            this.mActiveMeterCount = 0;
            this.mMeterView.resetColumns(0);
            this.mMeterView.setNumberOfPeaks(1);
            this.mMeterView.setShouldDrawPeaksOnMax(false);
            this.mMeterView.setShouldDrawPeaksOnTop(true)
        }
        else if(animationModes.kJSVoiceMeterViewController_ModeListening == this.mCurrentMode)
        {
            this.mActiveMeterCount = kJSMeterView_NumCols;
            this.mMeterView.resetColumns(0);
            this.mMeterView.setNumberOfPeaks(2);
            this.mMeterView.setShouldDrawPeaksOnMax(true);
            this.mMeterView.setShouldDrawPeaksOnTop(false)
        }
        else if(animationModes.kJSVoiceMeterViewController_ModeTransition == this.mCurrentMode)
        {
            this.mActiveMeterCount = kJSMeterView_NumCols;
            this.mMeterView.setNumberOfPeaks(1);
            this.mMeterView.setShouldDrawPeaksOnMax(true);
            this.mMeterView.setShouldDrawPeaksOnTop(true)
        }
    };
    JSVoiceMeterViewController.prototype.startAnimating = function()
    {
        console.log("startAnimating");
        if(this.mIsAnimating)
            return;
        this.mIsAnimating = true;
        var that = this;
        this.mTimer = setInterval(function()
        {
            that.updateMeterView()
        },33);
        this.mLastUpdateTimeInterval = +new Date
    };
    JSVoiceMeterViewController.prototype.stopAnimating = function()
    {
        this.mMeterView.resetColumns(0);
        if(!this.mIsAnimating)
            return;
        clearInterval(this.mTimer);
        this.updateMeterView();
        this.mIsAnimating = false
    };
    JSVoiceMeterViewController.prototype.updateMeterView = function()
    {
        if(animationModes.kJSVoiceMeterViewController_ModeFilling == this.mCurrentMode)
            this._animateFillMeters();
        else if(animationModes.kJSVoiceMeterViewController_ModeDraining == this.mCurrentMode)
            this._animateDrainMeters();
        else if(animationModes.kJSVoiceMeterViewController_ModeListening == this.mCurrentMode)
            this._animateListenMeters();
        else if(animationModes.kJSVoiceMeterViewController_ModeTransition == this.mCurrentMode)
            this._animateTransitionMeters();
        this.mMeterView.setColumns(this.mColumns)
    };
    JSVoiceMeterViewController.prototype._animateFillMeters = function()
    {
        var timestamp = +new Date;
        if(timestamp - (this.mLastUpdateTimeInterval + kJSVoiceMeterViewController_FillInterval) > 0)
        {
            this.mLastUpdateTimeInterval += kJSVoiceMeterViewController_FillInterval;
            if(this.mActiveMeterCount == 0 || this.mColumns[this.mActiveMeterCount - 1] == 3 && this.mActiveMeterCount < kJSMeterView_NumCols)
                this.mActiveMeterCount++;
            for(var i = 0; i < this.mActiveMeterCount; ++i)
                if(this.mColumns[i] < kJSMeterView_NumRows)
                    this.mColumns[i]++
        }
        if(this.mActiveMeterCount == kJSMeterView_NumCols && this.mColumns[kJSMeterView_NumCols - 1] == kJSMeterView_NumRows)
            this.setMode(animationModes.kJSVoiceMeterViewController_ModeDraining)
    };
    JSVoiceMeterViewController.prototype._animateDrainMeters = function()
    {
        var timestamp = +new Date;
        if(timestamp - (this.mLastUpdateTimeInterval + kJSVoiceMeterViewController_DrainInterval) > 0)
        {
            this.mLastUpdateTimeInterval += kJSVoiceMeterViewController_DrainInterval;
            for(var i = 0; i < kJSMeterView_NumCols; ++i)
                if(this.mColumns[i])
                    this.mColumns[i]--
        }
        if(!this.mColumns[0])
            this.setMode(animationModes.kJSVoiceMeterViewController_ModeFilling)
    };
    JSVoiceMeterViewController.prototype._animateListenMeters = function()
    {
        var barsAsPercentageOfPeak = (kJSMeterView_NumRows - 1) * this.mLevel / kJSVoiceMeterViewController_peakVolumeLevel;
        this.mTicksAt11 += this.mLevel > kJSVoiceMeterViewController_peakVolumeLevel ? 1 : -this.mTicksAt11;
        if(this.mTicksAt11 >= kJSVoiceMeterViewController_thresholdTicksAt11)
        {
            memset(this.mColumns,kJSMeterView_NumRows + 1,kJSMeterView_NumCols);
            return
        }
        for(var col = 0; col < kJSMeterView_NumCols; ++col)
        {
            if(this.mLevel < kJSVoiceMeterViewController_silenceVolumeLevel)
            {
                this.mColumns[col] = 0;
                continue
            }
            var rand = Math.random();
            var shapeIndex = col;
            var noise = rand & 3;
            if(noise)
                shapeIndex *= noise;
            var shape = kJSVoiceMeterViewController_vuMeterShape[shapeIndex];
            var bars = Math.round(barsAsPercentageOfPeak * shape + .25);
            noise = rand >> 3 & 3;
            if(noise)
                bars += noise - 2;
            if(bars < 0)
                bars = 0;
            if(bars > kJSMeterView_NumRows)
                bars = kJSMeterView_NumRows;
            var current = this.mColumns[col];
            var delta = Math.abs(bars - current);
            noise = rand >> 6 & 7;
            if(delta >= noise)
                this.mColumns[col] = bars
        }
    };
    JSVoiceMeterViewController.prototype._animateTransitionMeters = function()
    {
        var isFull = true;
        var timestamp = +new Date;
        if(timestamp - (this.mLastUpdateTimeInterval + kJSVoiceMeterViewController_TransitionInterval) > 0)
        {
            this.mLastUpdateTimeInterval += kJSVoiceMeterViewController_TransitionInterval;
            for(var i = 0; i < this.mActiveMeterCount; ++i)
            {
                if(this.mColumns[i] < kJSMeterView_NumRows)
                    this.mColumns[i]++;
                if(this.mColumns[i] != kJSMeterView_NumRows)
                    isFull = false
            }
        }
        else
            isFull = false;
        if(this.mActiveMeterCount == kJSMeterView_NumCols && isFull)
            this.setMode(animationModes.kJSVoiceMeterViewController_ModeDraining)
    };
    JSVoiceMeterViewController.prototype.animationModes = animationModes;
    WinJS.Namespace.define("BingApp.Classes",{JSVoiceMeterViewController: JSVoiceMeterViewController})
})()